#include <time.h>

#define Z_OK              0
#define Z_STREAM_END      1
#define Z_NEED_DICT       2
#define Z_ERRNO         (-1)
#define Z_STREAM_ERROR  (-2)
#define Z_DATA_ERROR    (-3)
#define Z_MEM_ERROR     (-4)
#define Z_BUF_ERROR     (-5)
#define Z_VERSION_ERROR (-6)

#define Z_NOMOREFILES     100
#define Z_INVALIDARG    (-100)
#define Z_STRINGCONV    (-101)
#define Z_BADPASSWORD   (-102)
#define Z_INSUFFBUFF    (-103)
#define Z_602STREAMERR  (-104)
#define Z_CRCDISAGR     (-105)

#ifdef __cplusplus
extern "C" {
#endif

#ifdef _WIN32
#ifdef ZIP602_STATIC
#define DllZip
#else
#define DllZip __declspec(dllimport)
#endif
typedef __int64 QWORD;
#else   // LINUX
#define DllZip
#define WINAPI
typedef int64_t QWORD;
typedef void *INT_PTR;
typedef unsigned char BYTE;
typedef uns16 WORD;
#ifndef DWORD
typedef uns32 DWORD;  
#endif
#endif


#pragma pack(1)

typedef struct ZIPINFO
{
    DWORD   Size;
    WORD    FileCount;          // Pocet souboru v archivu
    WORD    LocFileCount;       // Pocet souboru na tomto disku
    WORD    DiskNO;             // Cislo aktualniho disku
    WORD    DirDiskNO;          // Cislo disku s adresarem
    WORD    CommentSize;        // Velikost komentare
} ZIPINFO;

typedef struct ZIPENTRYINFO
{
    DWORD   Size;
    DWORD   CRC;                // CRC
    QWORD   ZipSize;            // Velikost zpakovaneho souboru
    QWORD   FileSize;           // Velikost originalniho souboru
    time_t  DateTime;           // Datum a cas posledni zmeny souboru
    DWORD   fAttr;              // Priznaky souboru, zavisi na OSMadeBy
    WORD    iAttr;              // Interni priznaky souboru
    WORD    FileNameSize;       // Delka jmena souboru    
    WORD    VerOS;              // Verze OS/FS, ktera archiv vytvorila
    WORD    VerMadeBy;          // Verze algoritmu, ktera archiv vytvorila
    WORD    VerNeedToExtr;      // Verze algoritmu, potrebna k rozbaleni
    WORD    Flags;              // Priznaky komprese a sifrovani
    WORD    Method;             // Metoda komprese
    WORD    hExtraSize;         // Delka Extra bytes, v hlavicce souboru
    WORD    dExtraSize;         // Delka Extra bytes, v adresari
    WORD    CommentSize;        // Delka komentare
    WORD    DiskNO;             // Cislo disku, na kterem soubor zacina
} ZIPENTRYINFO;

#pragma pack()

#if 0
typedef int   (WINAPI *LPZIP602READ)(INT_PTR Param, void *Buffer, int Size);
typedef bool  (WINAPI *LPZIP602WRITE)(INT_PTR Param, const void *Buffer, int Size);
typedef QWORD (WINAPI *LPZIP602SEEK)(INT_PTR Param, QWORD Offset, int Origin);

typedef struct CZip602Stream
{
    INT_PTR       Param;
    LPZIP602READ  Read;  
    LPZIP602WRITE Write;
    LPZIP602SEEK  Seek;
} CZip602Stream;
#endif

#ifndef __cplusplus
#define bool int
#define true 1
#define false 0
#endif

DllZip int WINAPI ZipCreate(const wchar_t *ZipName, const wchar_t *RootDir, INT_PTR *pHandle);
//DllZip int WINAPI ZipCreateS(const CZip602Stream *Stream, const wchar_t *RootDir, INT_PTR *pHandle);
DllZip int WINAPI ZipAdd(INT_PTR Handle, const wchar_t *FileName, bool Compress);
DllZip int WINAPI ZipAddM(INT_PTR Handle, const void *Data, int Size, const wchar_t *FileName, bool Compress);
//DllZip int WINAPI ZipAddS(INT_PTR Handle, const CZip602Stream *Stream, const wchar_t *FileName, bool Compress = true);
DllZip int WINAPI ZipAddFolder(INT_PTR Handle, const wchar_t *FolderName, bool Compress);
DllZip int WINAPI ZipAddMask(INT_PTR Handle, const wchar_t *Template, bool Compress);
DllZip int WINAPI ZipClose(INT_PTR Handle);
DllZip int WINAPI ZipCloseM(INT_PTR Handle, void *Zip, unsigned int Size);
DllZip int WINAPI ZipGetErrNo(INT_PTR Handle);
DllZip int WINAPI ZipSetPassword(INT_PTR Handle, const wchar_t *Password);
DllZip int WINAPI ZipGetErrNo(INT_PTR Handle);
DllZip int WINAPI ZipGetErrMsg(INT_PTR Handle, wchar_t *Msg, int Size);
DllZip int WINAPI ZipGetSize(INT_PTR Handle, QWORD *Size);

DllZip int WINAPI UnZipOpen(const wchar_t *ZipName, INT_PTR *pHandle);
DllZip int WINAPI UnZipOpenM(const BYTE *Zip, int ZipSize, INT_PTR *pHandle);
//DllZip int WINAPI UnZipOpenS(const CZip602Stream *Stream, INT_PTR *pHandle);
DllZip int WINAPI UnZipClose(INT_PTR Handle);
DllZip int WINAPI UnZipFile(INT_PTR Handle, INT_PTR EntryHandle, const wchar_t* FileName);
DllZip int WINAPI UnZipNextFile(INT_PTR Handle, const wchar_t* FileName);
DllZip int WINAPI UnZipFileM(INT_PTR Handle, INT_PTR EntryHandle, void *Buffer, int Size);
DllZip int WINAPI UnZipNextFileM(INT_PTR Handle, void *Buffer, int Size);
//DllZip int WINAPI UnZipFileS(INT_PTR Handle, INT_PTR EntryHandle, const CZip602Stream *Stream);
//DllZip int WINAPI UnZipNextFileS(INT_PTR Handle, const CZip602Stream *Stream);
DllZip int WINAPI UnZipAll(INT_PTR Handle, const wchar_t *Folder);
DllZip int WINAPI UnZipFindFirst(INT_PTR Handle, const wchar_t *Folder, bool Recurse, INT_PTR *pFindHandle, INT_PTR *pEntryHandle);
DllZip int WINAPI UnZipFindNext(INT_PTR FindHandle, INT_PTR *pEntryHandle);
DllZip int WINAPI UnZipIsFolder(INT_PTR Handle, INT_PTR EntryHandle);
DllZip int WINAPI UnZipIsEncrypted(INT_PTR Handle, INT_PTR EntryHandle);
DllZip int WINAPI UnZipGetInfo(INT_PTR Handle, ZIPINFO *Info);
DllZip int WINAPI UnZipGetComment(INT_PTR Handle, wchar_t *Comment, int Size);
DllZip int WINAPI UnZipGetFileInfo(INT_PTR Handle, INT_PTR EntryHandle, ZIPENTRYINFO *Info);
DllZip int WINAPI UnZipGetFileName(INT_PTR Handle, INT_PTR EntryHandle, wchar_t *FileName, int Size);
DllZip int WINAPI UnZipGetFileSize(INT_PTR Handle, INT_PTR EntryHandle, QWORD *Size);
DllZip int WINAPI UnZipGetFileComment(INT_PTR Handle, INT_PTR EntryHandle, wchar_t *Comment, int Size);
DllZip int WINAPI UnZipGetFileExtraBytes(INT_PTR Handle, INT_PTR EntryHandle, void *Buffer, int Size);
DllZip int WINAPI UnZipGetFileHdrExtraBytes(INT_PTR Handle, INT_PTR EntryHandle, void *Buffer, int Size);
DllZip int WINAPI UnZipSetPassword(INT_PTR Handle, const wchar_t *Password);
DllZip int WINAPI UnZipGetErrNo(INT_PTR Handle);
DllZip int WINAPI UnZipGetErrMsg(INT_PTR Handle, wchar_t *Msg, int Size);
#ifdef __cplusplus
}
#endif

