/*	602SQL PHP module
 *	Copyright (c) 2001-2009 Software602, a.s.
 *	see http://www.software602.cz
 *
 *	$Id: mod602sql.c,v 1.20 2010/02/25 10:39:24 pecuch Exp $ */

/*{{{ global defines which affect module functionality */

/* LOG_PHP_WB602
 * if defined then various info is written into log_filename file
 *	 (see logfile generation section below)
 * in production module have to be undefined!!! */
//#define LOG_PHP_WB602
/* USE_WB_DEBUG
 * if defined then module will write information into log_filename file
 *   only if log file usage is enabled by wb_debug() function
 * have to be used in conjunction with LOG_PHP_WB602 */
//#define USE_WB_DEBUG

/*
 * LOG_SERVER_MEMORY
 * If defined then all wb_XXX functions log the size of the memory used
 * by SQL server before and after the wb_XXX function is executed.
 * Logging occurs if and only if PHP.INI file contains key wb_log_server_memory=1
 * in the section of this module.
 * The log file is c:\php_602sql_memory.log or /tmp/php_602sql_memory.log by default
 * or the file which name is in wb_log_server_memory_file key in this module section
 * of the PHP.INI file. */
//#define LOG_SERVER_MEMORY

/* USE_PSQL
 * if defined PHP module will contain wb_psql_XXX functions which enable
 *   usage of prepared SQL statements */
#define USE_PSQL

/* USE_WIN32_DATETIME_CONVERSIONS
 * if defined PHP module for Windows will use Win32 functions to convert
 *   DATE and TIMESTAMP db. values to time_t
 * otherwise (i.e. on non-Windows platform or when isn't defined) will
 *   use mktime() function from C runtime library
 *
 * WARNING: This feature is purely experimetal. I was trying to improve
 * and/or purify conversions between 602SQL DATE and TIMESTAMP types
 * and time_t type. */
//#define USE_WIN32_DATETIME_CONVERSIONS

/* ENABLE_XML_IMPORT_EXPORT
 * if defined PHP module will contain functions that allow import
 * and export data to/from tables from/to XML files according to DAD
 * column-to-XML-element description
 * this feature could be enabled for 602SQL version 8.1 or higher
 *
 * WARNING: When we're compiling PHP module with this feature enabled and we
 * aren't compiling PHP module during PHP compilation on Linux we have to set
 * equally-named parameter in a makefile to 1 otherwise PHP module couldn't be
 * linked together. XML import/export functions are contained in 602xml8.dll
 * (602xml9.dll) on Windows or lib602xml8.so (lib602xml9.so) on Linux and this
 * parameter ensures that PHP module will be linked against corresponding XML
 * library.
 *
 * Makefiles:
 *   php_602sql.mak for Windows 
 *   php_602sql.lmk for Linux when PHP module is compiled standalone
 *
 * When we are compiling PHP module on Linux as part of PHP compilation
 * we don't have to set anything. In that case makefile automatically tests
 * existence and usability of lib602xmlN.so library and when it's present
 * and usable PHP module will be compiled against this library. */
#ifndef ENABLE_XML_IMPORT_EXPORT
#define ENABLE_XML_IMPORT_EXPORT
#endif

/* ENABLE_XML_RESPONSE
 * When enabled PHP module will contain wb_get_xml_response() function.
 * Warning: PHP module have to be linked with 602SQL XML library (see warning
 * message for ENABLE_XML_IMPORT_EXPORT #define). */
#ifndef ENABLE_XML_RESPONSE
#define ENABLE_XML_RESPONSE
#endif

/* USE_WB_SCRIPT_ENCODING */
#define USE_WB_SCRIPT_ENCODING

/* USE_XSD_PARSER */
#define USE_XSD_PARSER

/*{{{ dependencies of defines */
#if defined USE_WB_DEBUG && !defined LOG_PHP_WB602
#define LOG_PHP_WB602
#endif
/* WARNING: dependencies of defines that depend on WinBase602/602SQL version have to be put in "PHP includes" section further in this file */
/*}}}*/
/*}}}*/

/*{{{ PHP includes */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "php.h"
#include "php_ini.h"
#include "ext/standard/info.h"
#ifdef ZTS
#include "TSRM.h"
#endif
#ifndef WIN32
/* we need MAXPATHLEN value */
#include <sys/param.h>
/* we will use MAX_PATH anyway */
#ifndef MAX_PATH
#define MAX_PATH MAXPATHLEN
#endif
#endif
#include <math.h>
#include "mod602sql.h"

// alter global defines according to Winbase / 602SQL version
#if defined ENABLE_XML_IMPORT_EXPORT && WBVERS<810
#undef ENABLE_XML_IMPORT_EXPORT
#endif
#if defined ENABLE_XML_RESPONSE && WBVERS<950
#undef ENABLE_XML_RESPONSE
#endif
#if defined USE_WB_SCRIPT_ENCODING && WBVERS<800
#undef USE_WB_SCRIPT_ENCODING
#endif
#if defined USE_XSD_PARSER && WBVERS<1000
#undef USE_XSD_PARSER
#endif

#include "php_602sql.h"
#if defined ENABLE_XML_IMPORT_EXPORT && WBVERS<=900
// XML import/export functions are defined in wbprezen.h for 602SQL up to version 9.0
#include <wbprezen.h>
#endif
/*}}}*/

#ifdef HAVE_602SQL

/*{{{ additional includes & defines */
#include <ctype.h>
#include <assert.h>
#include <time.h>
#ifndef _WIN32
/* following define is needed by 602SQL headers to properly compile on Linux
 * we test _WIN32 existence because this is defined on Win32 platform only */
#ifndef LINUX
#define LINUX
#endif
/* 602SQL headers need next two includes */
//#include <sys/types.h>
//#include <sys/socket.h>
struct sockaddr_in;
/* redefine NONEBIGINT macro to comply with gcc style of long long constants */
#undef NONEBIGINT
#define NONEBIGINT ((sig64)0x8000000000000000LL)
// define stricmp() and strnicmp() functions
#ifndef stricmp
#define stricmp(a,b) strcasecmp(a,b)
#endif
#ifndef strnicmp
#define strnicmp(a,b,c) strncasecmp(a,b,c)
#endif
#endif
#if WBVERS>=800
DllKernel void WINAPI corefree( const void * x );
#endif

#ifdef PHP_WIN32
#define PHP_WB_API __declspec(dllexport)
#define callback WINAPI
#else
#define PHP_WB_API
#define callback /* */
#endif

#ifndef TSRMLS_FETCH
#define TSRMLS_FETCH() /* */
#endif

ZEND_DECLARE_MODULE_GLOBALS(wb)

#ifdef USE_PSQL
/* additional 602SQL API functions and defines
 * which we use in PSQL functions that aren't
 * in standard 602SQL API header files */
DllKernel BOOL WINAPI cd_Send_params(cdp_t cdp, uns32 stmt, DWORD len, void * value);
DllKernel BOOL WINAPI cd_SQL_get_param_info(cdp_t cdp, uns32 handle, void ** pars);
DllKernel BOOL WINAPI cd_Send_parameter(cdp_t cdp, uns32 stmt, sig16 parnum, DWORD len, DWORD offset, void * value);
DllKernel BOOL WINAPI cd_Pull_params(cdp_t cdp, uns32 handle, sig16 code, void ** answer);
DllKernel BOOL WINAPI cd_Pull_parameter(cdp_t cdp, uns32 handle, int parnum, int start, int length, char * buf);
#define SEND_PAR_PULL           -8
#define SEND_PAR_DROP           -2
#define SEND_PAR_DELIMITER      -1
#if WBVERS<800
DllKernel void WINAPI corefree( const void * x );
#endif
#endif

#if defined ENABLE_XML_IMPORT_EXPORT && WBVERS<950 && !defined xcdp_t
// uncomment following line if xcdp_t type isn't defined in standard 602SQL API header files
// typedef cdp_t xcdp_t;
#endif

#ifdef ENABLE_XML_RESPONSE
#define HTTP_REQOPT_PATH 0
#define HTTP_REQOPT_ACCEPT 1
#define HTTP_REQOPT_CONTTP 2
#define HTTP_REQOPT_HOST 3
#define HTTP_REQOPT_USERNAME 4
#define HTTP_REQOPT_PASSWORD 5
#define HTTP_REQOPT_COUNT 6

#define HTTP_METHOD_NONE  0
#define HTTP_METHOD_GET   1
#define HTTP_METHOD_POST  2
#define HTTP_METHOD_OTHER 3
#define HTTP_METHOD_RESP  10
#if WBVERS>=1000
DllExport int WINAPI get_xml_response(cdp_t cdp, int method, char ** options, const char * body, int bodysize, 
                      char ** resp, const char ** content_type, char * realm, int * resplen);
#else
DllExport int WINAPI get_xml_response(cdp_t cdp, int method, char ** options, const char * body, int bodysize, 
                      char ** resp, const char ** content_type, char * realm);
#endif
#endif

#if WBVERS>=800
#define attrspecif_t union t_specif
#define PTR_T_SPECIF_CAST(pspecif) (*((union t_specif*)(pspecif)))
#define T_SPECIF_CAST(specif) PTR_T_SPECIF_CAST(&(specif))
#else
#define	attrspecif_t uns16
#endif

#ifdef USE_WB_SCRIPT_ENCODING
/* superconv() function from 602SQL API isn't in standard SDK headers
 * so we will define it there
 * This function is used in estrdup_with_encoding(). */
#define SC_INCONVERTIBLE_WIDE_CHARACTER -1
#define SC_INCONVERTIBLE_INPUT_STRING   -2
#define SC_INPUT_STRING_TOO_LONG        -3
#define SC_VALUE_OUT_OF_RANGE           -4
#define SC_BAD_TYPE                     -5
DllKernel int WINAPI superconv(int stype, int dtype, const void * sbuf, char * dbuf, attrspecif_t sspec, attrspecif_t dspec, const char * string_format);
#endif

#ifdef USE_XSD_PARSER
/* copied from mpsql/dsparser.h
 * These functions aren't in standard SDK headers. */
enum DSPProp 
{
    DSP_SCHEMANAME,
    DSP_FOLDERNAME,
    DSP_DADNAME,
    DSP_TABLEPREFIX,
    DSP_DEFSTRLENGTH,
    DSP_UNICODE,
    DSP_IDTYPE,
    DSP_ERRCODE,
    DSP_ERRSOURCE,
    DSP_ERRMSG,
    DSP_ERRTYPEXML,
    DSP_ERRFILE,
    DSP_ERRLINE,
    DSP_ERRCOL,
    DSP_XMLFORMSOURCE,
    DSP_REFINT,
    DSP_XSD,
    DSP_ROOTTABLE,
    DSP_ROOTTABLEKEY,
    DSP_ROOTTABLESEQ,
    DSP_FORMID,
    DSP_FORMDATAROOTNAME,
    DSP_FORMDATANSPREFIX,
    DSP_FORMDATANSURI,
    DSP_SPECROOTSEQ
};

enum DSPErrSource
{
    DSPE_NONE,
    DSPE_SYSTEM,
    DSPE_SQL,
    DSPE_XML,
    DSPE_DOM,
    DSPE_SAX,
    DSPE_XSD,
    DSPE_ZIP,
    DSPE_OTHER
};
#if defined WINS && !defined HANDLE
// HANDLE is defined in general.h on Linux
typedef void * HANDLE;
#endif
#ifndef bool
typedef int bool;
#endif
DllImport BOOL WINAPI DSParserOpen(cdp_t cdp, HANDLE *pHandle);
DllImport void WINAPI DSParserClose(HANDLE Handle);
DllImport BOOL WINAPI DSParserSetStrProp(HANDLE Handle, enum DSPProp Prop, const char *Value);
DllImport BOOL WINAPI DSParserSetIntProp(HANDLE Handle, enum DSPProp Prop, int Value);
DllImport BOOL WINAPI DSParserGetStrProp(HANDLE Handle, enum DSPProp Prop, const char **Value);
DllImport BOOL WINAPI DSParserGetIntProp(HANDLE Handle, enum DSPProp Prop, int *Value);
DllImport BOOL WINAPI DSParserParse(HANDLE Handle, const char *Source);
DllImport BOOL WINAPI DSParserParseM(HANDLE Handle, const void *Source, int Size);
DllImport BOOL WINAPI DSParserGetObjectName(HANDLE Handle, int Index, char *Name, tcateg *Categ);
DllImport BOOL WINAPI DSParserGetColumnProps(HANDLE Handle, const char *TableName, int ColumnIndex, char *ColumnName, int *Type, union t_specif *Specif, int *Exclude
#if WBVERS>=1100
		, int *UnspecType
#endif
		);
DllImport BOOL WINAPI DSParserSetColumnProps(HANDLE Handle, const char *TableName, const char *ColumnName, int Type, union t_specif Specif, bool Exclude);
DllImport BOOL WINAPI DSParserCreateObjects(HANDLE Handle);
DllImport BOOL WINAPI DSParserGetLastError(HANDLE Handle, int *ErrCode, const char **ErrMsg);
DllImport BOOL WINAPI DSParserGetTableExcludeFlag(HANDLE Handle, const char *TableName, bool *Exclude);
DllImport BOOL WINAPI DSParserSetTableExcludeFlag(HANDLE Handle, const char *TableName, bool Exclude);
#if WBVERS>=1100 && defined LINUX
DllImport void prepare_charset_conversions(void);
#endif
#endif

#if WBVERS>=1000
DllImport char * WINAPI Merge_XML_formM(cdp_t cdp,const char *xml_form_ref,int xml_form_ref_size,const char *data,int *result_size,char **error_message);
typedef enum MFDState
{
    MFD_AGR,
    MFD_DISAGR,
    MFD_FORMREADFAIL,
    MFD_FORMPARSEFAIL,
    MFD_DADREADFAIL,
    MFD_DADPARSEFAIL,
    MFD_OTHER
} MFDState;
DllImport MFDState WINAPI MatchFormAndDAD(cdp_t cdp, const char *Form, const char *DAD);
DllImport MFDState WINAPI MatchFormAndDADM(cdp_t cdp, const void *Form, int Size, const char *DAD);
#endif

#if WBVERS>=1000
typedef void WINAPI t_client_error_callback2(cdp_t cdp, const char * text);
DllImport void WINAPI set_client_error_callback2(cdp_t cdp, t_client_error_callback2 * client_error_callback2In);
#endif

#if WBVERS>=1000
#include "Zip602.h"
#endif

/* wb_connect() error list */
#define WB_CONNECT_ERROR_OK 0
#define WB_CONNECT_ERROR_NO_MEMORY 1
#define WB_CONNECT_ERROR_CONNECT 2
#define WB_CONNECT_ERROR_PERSISTENT_RESOURCE_NOT_FOUND 3
#define WB_CONNECT_ERROR_TOO_MANY_CONNECTIONS 4
#define WB_CONNECT_ERROR_TOO_MANY_PERSISTENT_CONNECTIONS 5
#define WB_CONNECT_ERROR_GET_SERVER_VERSION 6
#define WB_CONNECT_ERROR_GET_SYSTEM_CHARSET 7
#define WB_CONNECT_ERROR_LOGIN 8
#define WB_CONNECT_ERROR_SET_PROGRESS_REPORT_MODULUS 9
#define WB_CONNECT_ERROR_SET_APPLICATION 10
#define WB_CONNECT_ERROR_STORE_CONNECTION_RESOURCE 11
#define WB_CONNECT_ERROR_RESOURCE_NOT_FOUND 12
/*}}}*/
/*{{{ debug logfile generation */
#ifdef LOG_PHP_WB602
#include <stdio.h>
#include <stdarg.h>

// filename of debug logfile
static char log_filename[MAX_PATH]=
#ifdef WIN32
  "c:\\php_602sql.log";
#else
  "/tmp/php_602sql.log";
#endif

#ifdef USE_WB_DEBUG
static int wb_debug_value=0;
#endif

#ifdef WIN32
static void print_line_start(FILE *f)
{
	SYSTEMTIME lt;
	GetLocalTime(&lt);
	fprintf(f,"[%lu] %d-%d-%d %02d:%02d:%02d.%d ",GetCurrentThreadId(),lt.wYear,lt.wMonth,lt.wDay,lt.wHour,lt.wMinute,lt.wSecond,lt.wMilliseconds);
}
#endif

static void wblog(char *format,...)
{
	FILE *f;
#ifdef USE_WB_DEBUG
  if( !wb_debug_value ) return;
#endif
	f=fopen(log_filename,"at");
	if( f!=NULL )
	{
		va_list ap;
		va_start(ap,format);
#ifdef WIN32
		print_line_start(f);
#endif
		vfprintf(f,format,ap);
		fclose(f);
	}
}
static void wblogs(char *str)
{
	FILE *f;
#ifdef USE_WB_DEBUG
  if( !wb_debug_value ) return;
#endif
	f=fopen(log_filename,"at");
	if( f!=NULL ) {
#ifdef WIN32
		print_line_start(f);
#endif
		fputs(str,f);
		fclose(f);
	}
}
static void wblogmem(char *message,char *ptr,size_t len)
{
	FILE *f;
#ifdef USE_WB_DEBUG
  if( !wb_debug_value ) return;
#endif
	f=fopen(log_filename,"at");
	if( f!=NULL ) {
	  size_t i;
#ifdef WIN32
		fprintf(f,"thread %lu: ",GetCurrentThreadId());
#endif
		fputs(message,f);
		fprintf(f,"\ndata size is %d bytes",len);
		for( i=0;i<len;i++ )
		{
			if( i%16==0 ) fprintf(f,"\n%8x: ",i/16);
			fprintf(f,"%02x ",(unsigned char)ptr[i]);
		}
		fputc('\n',f);
		fclose(f);
	}
}
static void wblogt_clivar(char *message,struct t_clivar *cv)
{
#ifdef USE_WB_DEBUG
	if( !wb_debug_value ) return;
#endif
	wblog("contents of t_clivar %s: name=\"%s\", mode=%d, wbtype=%d, specif=0x%x, buflen=%d, actlen=%d\n",
			message,cv->name,cv->mode,cv->wbtype,cv->specif,cv->buflen,cv->actlen);
	if( cv->actlen>0 ) wblogmem("buf",cv->buf,cv->actlen);
}
static void wblogt_clivar_array(char *message,struct t_clivar *cva,int cvcount)
{
	int i;
#ifdef USE_WB_DEBUG
	if( !wb_debug_value ) return;
#endif
	wblog("contents of t_clivar array %s, %d elements:\n",message,cvcount);
	for( i=0;i<cvcount;i++)
	{
		char buf[40];
		sprintf(buf,"%d. elem",i+1);
		wblogt_clivar(buf,&(cva[i]));
	}
	wblogs("end of t_clivar array\n");
}
#else // #ifdef LOG_PHP_WB602
#ifdef WIN32
#define wblog()
// disable warning "too many actual parameters for macro"
#pragma warning( disable: 4002 )
#else
// gcc can define variable-arguments macro
#define wblog(args...)
#endif
#define wblogs(str)
#define wblogmem(message,ptr,len)
#define wblogt_clivar(message,cv)
#define wblogt_clivar_array(message,cva,cvcount)
#endif
/*}}}*/
/*server memory log{{{*/
#ifdef LOG_SERVER_MEMORY
// this states from what server memory delta the SQL statement have to be logged
#define LSM_LOG_STATEMENT_DELTA (2048*1024)

#include <stdio.h>

#ifdef WIN32
#define WBLSM_DECLARE \
	cdp_t _lsm_cdp; \
  sig32 _lsm_start,_lsm_end; \
  char *_lsm_function_name,*_lsm_info; \
  SYSTEMTIME _lsm_time_start,_lsm_time_end;
#define WBLSM_START_EX(wb_function_name,paramcdp,info) \
	_lsm_cdp=paramcdp; \
	_lsm_start=cd_Used_memory(_lsm_cdp,FALSE); \
  _lsm_function_name=wb_function_name; \
  _lsm_info=info; \
  GetLocalTime(&_lsm_time_start);
#define WBLSM_START(wb_function_name,paramcdp) WBLSM_START_EX(wb_function_name,paramcdp,NULL)
#define WBLSM_END \
	_lsm_end=cd_Used_memory(_lsm_cdp,FALSE); \
  GetLocalTime(&_lsm_time_end); \
  wb_log_server_memory(_lsm_function_name,_lsm_start,_lsm_end,&_lsm_time_start,&_lsm_time_end,_lsm_info TSRMLS_CC);

void wb_log_server_memory(const char *wb_function_name,sig32 used_at_start,sig32 used_at_end,LPSYSTEMTIME time_start,LPSYSTEMTIME time_end,const char *info TSRMLS_DC)
{
	FILE *f=fopen(WBG(log_server_memory_file),"at");
	if( f!=NULL )
	{
		fprintf(f,"%lu, %d-%d-%d %02d:%02d:%02d.%d, ",GetCurrentThreadId(),time_start->wYear,time_start->wMonth,time_start->wDay,time_start->wHour,time_start->wMinute,time_start->wSecond,time_start->wMilliseconds);
		fprintf(f,"%d-%d-%d %02d:%02d:%02d.%d, ",time_end->wYear,time_end->wMonth,time_end->wDay,time_end->wHour,time_end->wMinute,time_end->wSecond,time_end->wMilliseconds);
		fprintf(f,"%s, %ld, %ld\n",wb_function_name,used_at_start,used_at_end);
		if( used_at_end-used_at_start>=LSM_LOG_STATEMENT_DELTA && info!=NULL )
		{
			fputs(info,f);
			fputc('\n',f);
		}
		fclose(f);
	}
}

#else
#include <time.h>

#define WBLSM_DECLARE \
	cdp_t _lsm_cdp; \
  sig32 _lsm_start,_lsm_end; \
  char *_lsm_function_name,*_lsm_info; \
  time_t _lsm_time_start,_lsm_time_end;
#define WBLSM_START_EX(wb_function_name,paramcdp,info) \
	_lsm_cdp=paramcdp; \
	_lsm_start=cd_Used_memory(_lsm_cdp,FALSE); \
  _lsm_function_name=wb_function_name; \
  _lsm_info=info; \
  time(&_lsm_time_start);
#define WBLSM_START(wb_function_name,paramcdp) WBLSM_START_EX(wb_function_name,paramcdp,NULL)
#define WBLSM_END \
	_lsm_end=cd_Used_memory(_lsm_cdp,FALSE); \
  time(&_lsm_time_end); \
  wb_log_server_memory(_lsm_function_name,_lsm_start,_lsm_end,_lsm_time_start,_lsm_time_end,_lsm_info TSRMLS_CC);

void wb_log_server_memory(const char *wb_function_name,sig32 used_at_start,sig32 used_at_end,time_t time_start,time_t time_end,const char *info TSRMLS_DC)
{
	FILE *f=fopen(WBG(log_server_memory_file),"at");
	if( f!=NULL )
	{
		char tstart[50],tend[50];
		*tstart=*tend='\0';
		ctime_r(_lsm_time_start,tstart);
		ctime_r(_lsm_time_end,tend);
		fprintf(f,"%s, %s, %s, %ld, %ld\n",tstart,tend,wb_function_name,used_at_start,used_at_end);
		if( used_at_end-used_at_start>=LSM_LOG_STATEMENT_DELTA && info!=NULL )
		{
			fputs(f,info);
			fputc(f,'\n');
		}
		fclose(f);
	}
}

#endif
#else
#define WBLSM_DECLARE
#define WBLSM_START(wb_function_name,paramcdp)
#define WBLSM_START_EX(wb_function_name,paramcdp,info)
#define WBLSM_END
#endif
/*}}}*/
/*{{{ module's internal structures and error handling functions */
/*{{{ Connection information. */
/* connection status */
typedef enum wb_connection_status { 
	
	WB_CS_UNINITIALIZED=0, /* connection is allocated, isn't connected to SQL
														server, isn't used by any thread */
	WB_CS_DISCONNECTED,		/* disconnected & not used by any thread */
	WB_CS_CONNECTED_IN_USE,/* connected & used by some thread */
	WB_CS_CONNECTED_UNUSED /* connected & not used by any thread */
}	wb_connection_status;

/* connection info struct */
typedef struct wb_conn {
#ifdef USE_STRUCT_NAME
	char struct_name[20];
#endif
	cd_t cd; /* 602SQL internal stuff - 602SQL client data structure */ 
	int usage; /* number of cursors using cdp */ 
	int	is_persistent; /* 1 if this is persistent connection, otherwise 0 */
#if WBVERS>=800
 	wb_encoding encoding; /* client encoding used by this connection */
#endif
#ifdef USE_WB_SCRIPT_ENCODING
	wb_encoding system_encoding; /* system encoding */
#endif
 	wb_connection_status status; /* connection status */
#if WBVERS<900
 	sig32 selected_language_id; /* selected language ID or -1 if no language is
													 selected language ID is in fact ordinal number of
													 column in "_multiling" table where translated
													 strings for that language are stored language ID is
													 always greater than zero (with except for -1 if no
													 languages is selected) */
#endif
#if WBVERS>=1000
	unsigned long client_error_level; /* E_USER_xxx error level which have to be used
																			 to log client errors, or 0 which means not to
																			 log errors through PHP error log mechanism */
	char *client_error_callback; /* name of the PHP user function that have to be
																	invoked on client error (emalloc-ed buffer) or
																	NULL which means not to invoke any function */
	uns32 server_version[4]; /* version of the SQL server we are connected to
                           server_version[0] is major version number (OP_GI_VERSION_1)
												   server_version[1] is minor (OP_GI_VERSION_2)
												   server_version[2] is patch (OP_GI_VERSION_3)
												   server_version[3] is month and day (OP_GI_VERSION_4)	*/
#endif
} wb_conn;
#define CDP(conn) (&((conn)->cd))

#define IS_CONNECTION_PERSISTENT(conn) ((conn)->is_persistent)
#if WBVERS>=800
#define ENCODING(conn) ((conn)->encoding)
#endif
#ifdef USE_WB_SCRIPT_ENCODING
#define IS_STRING_CONVERSION_REQUIRED(source_encoding,target_encoding) ((source_encoding)!=WB_ENC_UNKNOWN && (target_encoding)!=WB_ENC_UNKNOWN && (source_encoding)!=(target_encoding))
#define IS_STRING_SCRIPT2SYSTEM_REQUIRED(script_encoding,conn) ((script_encoding)!=WB_ENC_UNKNOWN && (conn)->system_encoding!=(script_encoding))
#endif

#if WBVERS>=1000
/* SQL server version number test macros */
#define IS_SERVER_VERSION_9_UP(conn) ((conn)->server_version[0]>=9)
#define IS_SERVER_VERSION_9_5_UP(conn) ((conn)->server_version[0]>=9 && (conn)->server_version[1]>=5)
#endif
/*}}}*/
/*{{{ Cursor & LOB information. */
/* wb_fetch_into() result_type parameter values
 * bitmask xxxxxANN
 *   NN _NUM0 or _NUM1
 *   A  _ASSOC */
#define WB_FETCH_NUM0 1
#define WB_FETCH_NUM1 2
#define WB_FETCH_ASSOC 4
#define WB_FETCH_BOTH0 (WB_FETCH_NUM0+WB_FETCH_ASSOC)
#define WB_FETCH_BOTH1 (WB_FETCH_NUM1+WB_FETCH_ASSOC)

/* LOB manipulation behaviour */
typedef enum wb_lob_mode {
	WB_LOBMODE_DIRECT=1, /* LOBs' values will be readed according to wb_binmode and wb_longreadlen */
	WB_LOBMODE_RESOURCE=2 /* LOB resources will be created for LOBs */
} wb_lob_mode;

/* Cursor information.	The result is stored in a structure of type struct
 * sql_answer. It contains the cursor number returned by the server, but also
 * few other informations about names and types of columns of the table,
 * number of rows, and some parameters store per-cursor.	The additional
 * information about the table is requested from server at the time of
 * creation of the cursor; actually, particulary number of rows may take a
 * little while to count. If this would appear to be an issue, more
 * sophisticated strategy could be done: when cursor is created, assign an
 * invalid value to rec_cnt and compute the proper value when it is needed.
 * The affected routines would be wb_rec_cnt and the final statement of
 * wb_fetch_row; the latter could check if result is used and delay
 * calculating of number of rows if it is not. */
#define MAX_RETURN_FIELDS 10
typedef struct sql_answer{
#ifdef USE_STRUCT_NAME
	char struct_name[20];
#endif
	tcursnum cursor;
	struct wb_conn* conn; /* connection that belongs to the answer */
	int attrs, total_attrs; /* number of attributes, total includes system attributes */
	uns8 *types; /* types of attributes */
	tattrib *attr_col; /* translation from PHP columns to server columns */
	char **names; /* names of attributes */
#if WBVERS>=800
	attrspecif_t *specif; /* additional info about columns' types */
#endif
	trecnum rec_cnt; /* Number of records in cursor */
	int usage; /* Number of LOB resources than are using this cursor */
	wb_lob_mode lob_mode; /* LOB manipulation behaviour used for this cursor */
	int is_opened; /* =1 if cursor is opened, =0 if cursor is closed */
	/*<Parameters for connection with global defaults>:*/
	binmode_t binmode;
	int longreadlen; /* Used size of the long type */
	/*Internal sql_answer parameters:*/
	/* Fetching row by row.	Another mechanism of retrieving results is
	 * to call wb_fetch_row, and then wb_result. The former fetches
	 * next (or the one given by second parameter) row of the result, and
	 * returns true if suceeded, false when there are no more rows.
	 * The latter retrieves the value of the column specified by second
	 * attribute (as a column number when a numeric type, or as a column
	 * name when string type) in the current row. */
	trecnum current_record;
} sql_answer;
#define CONN_P_FROM_ANSWER_P(answerp) (answerp)->conn
#define CDP_FROM_ANSWER_P(answerp) CDP(CONN_P_FROM_ANSWER_P(answerp))

/* LOB information
 * */
typedef enum lob_status_t { LOB_STATUS_OPENED, LOB_STATUS_CLOSED } lob_status_t;
typedef struct lob_info_t
{
#ifdef USE_STRUCT_NAME
	char struct_name[20];
#endif 
	sql_answer *answer; /* cursor resource which was used to create this LOB */
	tattrib attrib; /* LOB's column order in cursor */
	trecnum recnum; /* LOB's record order in cursor */
	lob_status_t status; /* status of LOB (opened|closed) */
} lob_info_t;
#define ANSWER_P_FROM_LOB_P(lobp) (lobp)->answer
#define CDP_FROM_LOB_P(lobp) CDP_FROM_ANSWER_P(ANSWER_P_FROM_LOB_P(lobp))
#if WBVERS>=1100
#define IS_LOB_NCLOB(lobp) (((lob)->answer->types[(lob)->attrib-1]==ATT_TEXT || (lob)->answer->types[(lob)->attrib-1]==ATT_EXTCLOB) && (lob)->answer->specif[(lob)->attrib-1].wide_char)
#define IS_LOB_CLOB(lobp) (((lob)->answer->types[(lob)->attrib-1]==ATT_TEXT || (lob)->answer->types[(lob)->attrib-1]==ATT_EXTCLOB) && !((lob)->answer->specif[(lob)->attrib-1].wide_char))
#else
#define IS_LOB_NCLOB(lobp) ((lob)->answer->types[(lob)->attrib-1]==ATT_TEXT && (lob)->answer->specif[(lob)->attrib-1].wide_char)
#define IS_LOB_CLOB(lobp) ((lob)->answer->types[(lob)->attrib-1]==ATT_TEXT && !((lob)->answer->specif[(lob)->attrib-1].wide_char))
#endif
/*}}}*/
#ifdef USE_PSQL
/*{{{ PSQL struct - prepared SQL statement */
/* status of prepared SQL statement */
typedef enum psql_status_t {
	PSQL_STATUS_PREPARED=1, /* statement is prepared, none IN param is sent to server yet */
	PSQL_STATUS_SENDING_PARAMS, /* statement is prepared, at least one IN param is sent to server */
	PSQL_STATUS_EXECUTED, /* statement is executed */
	PSQL_STATUS_CLOSED /* prepared statement was closed through wb_close() */
} psql_status_t;
/* prepared SQL statement description */
typedef struct psql_t {
#ifdef _DEBUG
	char struct_name[20];
#endif
	struct wb_conn *conn; /* connection */
	psql_status_t status; /* status of statement execution */
	uns32 psql_handle; /* handle of prepared SQL statement (result of cd_SQL_prepare() 602SQL API function) */
	sig32 *param_info; /* various info about parameters of SQL statement (result of cd_SQL_get_param_info() 602SQL API function) */
	char *outparam_info; /* info about OUT parameters (result of cd_Pull_params() 602SQL API function) */
	sig16 outparam_count; /* number of OUT parameters, or 0
                         * this value is computed after cd_Pull_params() execution */
#ifdef LOG_SERVER_MEMORY
	char *sql_statement; /* SQL statement source */
#endif
} psql_t;
#define CONN_P_FROM_PSQL_P(psql) (psql)->conn
#define CDP_FROM_PSQL_P(psql) CDP(CONN_P_FROM_PSQL_P(psql))
/* description of parameter */
typedef struct psql_param_info_t {
  uns8 type_id;
  attrspecif_t specif;
  enum t_parmode parmode;
  int position;
} param_info_t;
#define PSQL_HANDLE(psql) (psql)->psql_handle
#define PSQL_INPARAM_COUNT(psql) ( ((psql)->param_info==NULL) ? 0 : *((psql)->param_info) )
#define PSQL_INPARAM(psql,paridx) (((struct psql_param_info_t*)((psql)->param_info+1))+(paridx))
#define OUTPARAM_SIZE_LIMIT 256
static sig16 internal_psql_out_param_count(struct psql_t *psql)
{
	char *parptr=psql->outparam_info;
	sig16 result=0;
	if( parptr==NULL ) return 0;
	while( *(sig16*)parptr!=SEND_PAR_DELIMITER ) {
		uns32 length=*(uns32*)(parptr+sizeof(sig16));
		parptr+=sizeof(sig16)+sizeof(uns32);
		if( length<=OUTPARAM_SIZE_LIMIT )
		{
			// data jsou v bufferu parptr
			parptr+=length;
		}
		else
		{
			// data musi byt prectena pomoci cd_Pull_parameter()
		}
		result++;
	}
	return result;
}
/*}}}*/
#endif // USE_PSQL
#ifdef USE_XSD_PARSER
/*{{{ xsdp struct - XSD parser */
typedef enum xsdp_status_t
{
  XSDP_STATUS_OPENED=1,
	XSDP_STATUS_PARSED,
	XSDP_STATUS_OBJECTS_CREATED,
	XSDP_STATUS_CLOSED
} xsdp_status_t;
/* XSD parser struct */
typedef struct xsdp_t {
#ifdef _DEBUG
	char struct_name[20];
#endif
	struct wb_conn *conn; /* connection */
	xsdp_status_t status;
	HANDLE xsdp_handle; /* handle of XSD parser instance */
} xsdp_t;
#define CONN_P_FROM_XSDP_P(xsdp) (xsdp)->conn
#define CDP_FROM_XSDP_P(xsdp) CDP(CONN_P_FROM_XSDP_P(xsdp))
#define XSDP_HANDLE(xsdp) ((xsdp)->xsdp_handle)
/*}}}*/
#endif // USE_XSD_PARSER
#if WBVERS>=1000
/*{{{ odbc_t struct - ODBC connections */
/* ODBC connection struct */
typedef struct odbc_t {
#ifdef _DEBUG
	char struct_name[20];
#endif
	t_pconnection odbc_conn; /* ODBC connection */
} odbc_t;
#define T_PCONNECTION_FROM_ODBC_P(odbc_p) (odbc_p)->odbc_conn
/*}}}*/
#endif
/*{{{ Error handling.
 * Too often something goes wrong. From the view of a PHP programmer, there
 * are two ways of handling a problem; the most simple is to do nothing and
 * let the module issues PHP warnings as it fails. The other is to use PHP
 * function error_reporting to turn off warnings, check for return values
 * of our functions, and after failure call wb_err_str.
 *
 * How are the error messages created?	602SQL offers some preformated error
 * messages, but its mechanism requires creating buffer. Following function
 * and macro should make life easier. It is supposed that the macro is called
 * after failed 602SQL API call, with cdp being connection. Its parameter
 * should be name of the function that failed.

 * The OUT_OF_TABLE error is a kind of ``soft error''; usually it is mere
 * indication of end of table that should be handled internally. Namely, in
 * wb_result_all and wb_fetch_row, we make iteration until an
 * error is returned, and we do not want the user to see the warning. */
static char *estrdup_with_encoding(const char *source,wb_encoding source_encoding,wb_encoding dest_encoding);

static int report_err_from_number(struct wb_conn *conn, int res, const char *where TSRMLS_DC)
{
	char buffer[256],*errmsgptr;
	cdp_t cdp;
#ifdef USE_WB_SCRIPT_ENCODING
	char *emalloc_string;
#endif
	cdp=CDP(conn);
	if( !cdp ) return 0;
	errmsgptr=NULL;
#ifdef USE_WB_SCRIPT_ENCODING
	emalloc_string=NULL;
#endif
	if (res==OUT_OF_TABLE)
	{
		wblogs("report_err_from_number: OUT_OF_TABLE detected\n");
		return OUT_OF_TABLE;
	}
	Get_error_num_text(cdp, res, buffer, sizeof(buffer));
#ifdef USE_WB_SCRIPT_ENCODING
	if( IS_STRING_CONVERSION_REQUIRED(conn->system_encoding,WBG(script_encoding)) )
	{
		emalloc_string=estrdup_with_encoding(buffer,conn->system_encoding,WBG(script_encoding));
		errmsgptr=emalloc_string;
	}
	else
	{
		errmsgptr=buffer;
	}
#else
	errmsgptr=buffer;
#endif
	php_error(E_WARNING, "602SQL error in %s: %s", where, errmsgptr);
#ifdef LOG_PHP_WB602
	wblog("602SQL error in %s: %s\n",where,errmsgptr);
#endif
#ifdef USE_WB_SCRIPT_ENCODING
	if( emalloc_string!=NULL ) efree(emalloc_string);
#endif
	return res;
}
#define CONN_REPORT_ERR(conn,where) report_err_from_number(conn, cd_Sz_error(CDP(conn)), where TSRMLS_CC)
static void report_out_of_memory_error(const char *reason)
{
	php_error(E_WARNING, "602SQL error: Out of memory. %s",reason);
}
/*}}}*/
/*}}}*/
/*{{{ module's resources (destructors and get_{restype}_from_resource functions)
 */
/* Resources.
In PHP, resource is a structure hidden from PHP application. The
application can access it only by a handler that is created by appropriate
function call, and only thing it can do with it is to pass it as a parameter
to other functions (Oh yes, and it may print out that this is "Resource #1").
At the moment that all references to particular resource are lost,
corresponding shutdown function is called; the shutdown function is determined
at the point of resource creation.

Our module makes use of four resource classes; we have connection resources,
created by wb_connect, persistent connection resources, created by wb_pconnect,
result resources, created by wb_exec, and LOB resources which one can use to
read or write large objects' data. The corresponding shutdown functions clean
the data in the structure and free allocated memory.

Later, we provide a function wb_close that closes cursor or connection or LOB
resource described by the handle; to avoid double closing, it invalidates the
structures. For a connection it means setting usage to 0, while for cursors it
is setting types to NULL (after freeing). */

/* Every shutdown function must be registered; registration generates unique
 * handler that is later used for creating resources of the particular class.
 * We shall keep the handlers in variables below. They should be initialized
 * as soon as the module is loaded into memory. When a resource class is
 * registered, resources can be created by a call to ZEND_REGISTER_RESOURCE
 * function; see wb_connect for an example. */
#define WBRES_PCONN_NAME "602SQL link persistent"
#define WBRES_CONN_NAME "602SQL link"
#define WBRES_CURSOR_NAME "602SQL cursor"
#define WBRES_LOB_NAME "602SQL LOB"
#ifdef USE_PSQL
#define WBRES_PSQL_NAME "602SQL prepared statement"
static int wbres_psql;
#endif
#ifdef USE_XSD_PARSER
#define WBRES_XSDP_NAME "602SQL XSD parser"
static int wbres_xsdp;
#endif
#if WBVERS>=1000
#define WBRES_ODBC_NAME "602SQL ODBC connection"
static int wbres_odbc;
#endif
static int wbres_pconn,wbres_conn,wbres_cursor,wbres_lob;

/* headers of destructor functions */
static int cdp_destructor(wb_conn *conn,int do_close_connection TSRMLS_DC);
static int cursor_destructor(sql_answer *answer,int do_close_cursor,int call_cdp_destructor TSRMLS_DC );
#ifdef USE_PSQL
static int psql_destructor(struct psql_t *psql,int call_cdp_destructor TSRMLS_DC);
#endif
static void lob_destructor(struct lob_info_t *lob,int call_cursor_destructor TSRMLS_DC);
#ifdef USE_XSD_PARSER
static int xsdp_destructor(struct xsdp_t *xsdp,int call_cdp_destructor TSRMLS_DC);
#endif
#if WBVERS>=1000
static int odbc_destructor(struct odbc_t *odbc,int do_close_connection TSRMLS_DC);
#endif

/* this function is called after destruction of other types resources
 * which use connection
 * this function isn't called during destruction of wb_conn structure
 * function decreases conn->usage counter and optionally disconnects from
 * SQL server in case of conn->usage==0 */
static int cdp_destructor(wb_conn *conn,int do_close_connection TSRMLS_DC)
{
	cdp_t cdp;

	wblog("cdp_destructor: [%d]\n",1);
	cdp=CDP(conn);
	wblog("cdp_destructor: [%d]\n",2);
	wblog("cdp_destructor enter, conn=%p cdp=%p, conn->usage=%d, do_close_connection=%d\n",conn,cdp,conn->usage,do_close_connection);
	wblog("cdp_destructor: [%d]\n",3);
	if( --(conn->usage)>0 && !do_close_connection ) {
		wblog("cdp_destructor leave, not destroyed, conn->usage=%d\n",conn->usage);
		return 0; /* Someone still needs me */
	}
	wblog("cdp_destructor: [%d]\n",4);
	if (conn->usage>=0)	/* Not explicitly closed before */
	{
		// if there are other resources which use this connection
		if( conn->usage>0 )
		{ // close them before disconnecting
			int i,nument,type;
			void *ptr;
			wblog("cdp_destructor: trying to close %d resources which are using this connection\n",conn->usage);
			nument = zend_hash_next_free_element(&EG(regular_list));
			for(i = 1; i < nument; i++) {
				ptr = zend_list_find(i, &type);
				/* if this resource isn't NULL and uses conn connection
				 * close this resource but don't destroy it */
				if( ptr!=NULL )
				{
					if( type==wbres_cursor )
					{
						if( CONN_P_FROM_ANSWER_P((sql_answer*)ptr)==conn )
						{
							wblog("cdp_destructor: closing cursor %p\n",ptr);
							cursor_destructor((sql_answer*)ptr,1/*do close cursor*/,0/*don't call cdp_destructor*/ TSRMLS_CC);
							conn->usage--; // we have to explicitly do this because cdp_destructor() wasn't called
						}
					}
#ifdef USE_PSQL
					else if( type==wbres_psql )
					{
						if( CONN_P_FROM_PSQL_P((psql_t*)ptr)==conn )
						{
							wblog("cdp_destructor: closing PSQL %p\n",ptr);
							psql_destructor((psql_t*)ptr,0/*don't call cdp_destructor*/ TSRMLS_CC);
							conn->usage--; // we have to explicitly do this because cdp_destructor() wasn't called
						}
					}
#endif
#ifdef USE_XSD_PARSER
					else if( type==wbres_xsdp )
					{
						if( CONN_P_FROM_XSDP_P((xsdp_t*)ptr)==conn )
						{
							wblog("cdp_destructor: closing XSDP 0x%p\n",ptr);
							xsdp_destructor((xsdp_t*)ptr,0/*don't call cdp_destructor*/ TSRMLS_CC);
							conn->usage--; // we have to explicitly do this because cdp_destructor() wasn't called
						}
					}
#endif
				}
			}
		}
		wblog("cdp_destructor: [%d]\n",5);
#if WBVERS>=1000
		if( conn->client_error_level!=0 || conn->client_error_callback!=NULL )
		{
			wblog("cdp_destructor: before set_client_error_callback2: conn->client_error_level=%d conn->client_error_callback=0x%p, conn->status=%lu\n",conn->client_error_level,conn->client_error_callback,conn->status);
			set_client_error_callback2(cdp,NULL);
			wblogs("cdp_destructor: after set_client_error_callback2\n");
			if( conn->client_error_level!=0 ) conn->client_error_level=0;
			wblogs("cdp_destructor: after conn->client_error_level=0\n");
			if( conn->client_error_callback!=NULL ) { efree(conn->client_error_callback); wblogs("cdp_destructor: after efree(conn->client_error_callback)\n"); conn->client_error_callback=NULL; }
			wblogs("cdp_destructor: after conn->client_error_callback=NULL;\n");
		}
#endif
		if( conn->status==WB_CS_DISCONNECTED )
		{
			wblogs("cdp_destructor: already disconnected - status=WB_CS_DISCONNECTED\n");
		}
		else
		{
			wblogs("cdp_destructor: before cd_assign_to_thread()\n");
			cd_assign_to_thread(cdp);
			wblogs("cdp_destructor: after cd_assign_to_thread()\n");
			wblogs("cdp_destructor: disconnecting from SQL server\n");
			cd_disconnect(cdp); // disconnect from 602SQL Server
			conn->status=WB_CS_DISCONNECTED;
			// update number of opened connections
			if( IS_CONNECTION_PERSISTENT(conn) ) WBG(num_persistent)--;
			WBG(num_conn)--;
			wblogs("cdp_destructor: disconnected from SQL server\n");
		}
	}
	wblog("cdp_destructor: [%d]\n",6);
	wblogs("cdp_destructor leave, destroyed\n");
	return 1;
}

/* destruction of sql_answer structure */
static int cursor_destructor(sql_answer *answer,int do_close_cursor,int call_cdp_destructor TSRMLS_DC )
{
	wblog("cursor_destructor enter, answer=%p, answer->usage=%d, answer->is_opened=%d, answer->conn=%p, do_close_cursor=%d, call_cdp_destructor=%d\n",answer,answer->usage,answer->is_opened,answer->conn,do_close_cursor,call_cdp_destructor);
	if( --(answer->usage)>0 && !do_close_cursor )
	{
		wblogs("cursor_destructor leave, not closed, opened LOB resource exists\n");
		return 0;
	}
	wblog("cursor_destructor: [%d]\n",1);
	/* if sql_answer isn't needed any more
	 * or we have to close cursor */
	if( answer->usage>=0 )
	{
		wblog("cursor_destructor: [%d]\n",2);
		// if there are other resources which utilize this cursor
		if( answer->usage>0 )
		{ // close these resources before closing
			int i,nument,type;
			void *ptr;
			wblog("cursor_destructor: trying to close %d resources which are using this cursor\n",answer->usage);
			nument = zend_hash_next_free_element(&EG(regular_list));
			for(i = 1; i < nument; i++) {
				ptr = zend_list_find(i, &type);
				/* if this resource isn't NULL and uses "asnwer" cursor
				 * close this resource but don't destroy it */
				if( ptr!=NULL && type==wbres_lob )
				{
					if( ANSWER_P_FROM_LOB_P((lob_info_t*)ptr)==answer )
					{
						wblog("cursor_destructor: closing LOB %p\n",ptr);
						lob_destructor((lob_info_t*)ptr,0/*don't call cursor destructor*/ TSRMLS_CC);
						answer->usage--; // we have to explicitly do this because cursor_destructor() wasn't called
					}
				}
			}
		}
		wblog("cursor_destructor: [%d]\n",3);
		if( answer->is_opened )
		{
			cdp_t cdp;
			wblog("cursor_destructor: [%d]\n",4);
			/* It's crucial not to call 602SQL API functions before we know that cursor is opened yet.
			 * When connection is destroyed first during PHP request shutdown cursor is already closed
			 * and any call of 602SQL API function could lead to access violation. */
			cdp=CDP_FROM_ANSWER_P(answer);
			cd_assign_to_thread(cdp);
			/* close cursor */
			wblogs("cursor_destructor: closing cursor\n");
			if (cd_Close_cursor(cdp, answer->cursor))
			{
				CONN_REPORT_ERR(CONN_P_FROM_ANSWER_P(answer),"Close_cursor");
				wblogs("cursor_destructor leave, cd_Close_cursor failed\n");
				return 0;
			}
			answer->is_opened=0;
			wblogs("cursor_destructor: cursor closed\n");
		}
		wblog("cursor_destructor: [%d]\n",5);
		if( call_cdp_destructor ) cdp_destructor(answer->conn,0/*don't force to close connection*/ TSRMLS_CC);
		wblog("cursor_destructor: [%d]\n",6);
	} 
	else { wblogs("	cursor was closed by wb_close()\n"); }
	wblogs("cursor_destructor leave\n");
	return 1;
}

#ifdef USE_PSQL
/* destructor of psql_t structure */
static int psql_destructor(struct psql_t *psql,int call_cdp_destructor TSRMLS_DC)
{
	wblog("psql_destructor enter, psql=%p, call_cdp_destructor=%d\n",psql,call_cdp_destructor);
	if( psql->status!=PSQL_STATUS_CLOSED )
	{
		cdp_t cdp=CDP_FROM_PSQL_P(psql); 
		cd_assign_to_thread(cdp);
		psql->status=PSQL_STATUS_CLOSED;
		if( cd_SQL_drop(cdp,PSQL_HANDLE(psql)) )
		{
			CONN_REPORT_ERR(CONN_P_FROM_PSQL_P(psql),"SQL_drop");
			wblogs("psql_destructor leave, cd_SQL_drop failed\n");
			return 0;
		}
		if( psql->param_info!=NULL )
		{
			corefree(psql->param_info);
			psql->param_info=NULL;
		}
		if( psql->outparam_info!=NULL )
		{
			corefree(psql->outparam_info);
			psql->outparam_info=NULL;
		}
#ifdef LOG_SERVER_MEMORY
		if( psql->sql_statement!=NULL )
		{
			efree(psql->sql_statement);
			psql->sql_statement=NULL;
		}
#endif
		wblogs("  psql closed\n");
		if( call_cdp_destructor ) cdp_destructor(psql->conn,0/*don't force to close connection*/ TSRMLS_CC);
	}
	else { wblogs(" psql was closed by wb_close()\n"); }
	wblogs("psql_destructor leave\n");
	return 1;
}
#endif

static void lob_destructor(struct lob_info_t *lob,int call_cursor_destructor TSRMLS_DC)
{
	wblog("lob_destructor enter, lob=%p, call_cursor_destructor=%d\n",lob,call_cursor_destructor);
	if( lob->status!=LOB_STATUS_CLOSED )
	{
		lob->status=LOB_STATUS_CLOSED;
		wblogs("lob_destructor: LOB closed\n");
		if( call_cursor_destructor ) cursor_destructor(lob->answer,0,1 TSRMLS_CC);
	}
	else { wblogs("	 LOB is already closed\n"); }
	wblogs("lob_destructor leave\n");
} 

#ifdef USE_XSD_PARSER
/* destructor of xsdp_t structure */
static int xsdp_destructor(struct xsdp_t *xsdp,int call_cdp_destructor TSRMLS_DC)
{
	wblog("xsdp_destructor enter, xsdp=%p, call_cdp_destructor=%d\n",xsdp,call_cdp_destructor);
	if( xsdp->status!=XSDP_STATUS_CLOSED )
	{
		cdp_t cdp=CDP_FROM_XSDP_P(xsdp); 
		cd_assign_to_thread(cdp);
		xsdp->status=XSDP_STATUS_CLOSED;
		DSParserClose(XSDP_HANDLE(xsdp));
		wblogs("  xsdp closed\n");
		if( call_cdp_destructor ) cdp_destructor(CONN_P_FROM_XSDP_P(xsdp),0/*don't force to close connection*/ TSRMLS_CC);
	}
	else { wblogs(" xsdp was closed by wb_close()\n"); }
	wblogs("xsdp_destructor leave\n");
	return 1;
}
#endif

#if WBVERS>=1000
/* destructor of odbc_t structure */
static int odbc_destructor(struct odbc_t *odbc,int do_close_connection TSRMLS_DC)
{
	wblog("odbc_destructor enter, odbc=%p, do_close_connection=%d\n",odbc,do_close_connection);
	if( do_close_connection )
	{
		if( T_PCONNECTION_FROM_ODBC_P(odbc)!=NULL )
		{
			ODBC_disconnect(T_PCONNECTION_FROM_ODBC_P(odbc));
			T_PCONNECTION_FROM_ODBC_P(odbc)=NULL;
			wblogs("  odbc connection closed\n");
		}
		else { wblogs("  odbc connection is already NULL\n"); }
	}
	wblogs("odbc_destructor leave\n");
	return 1;
}
#endif

/* this is called for wbres_pconn on end of request */
static void wbres_commit_pconn(zend_rsrc_list_entry *rsrc TSRMLS_DC)
{
	wb_conn *conn=(wb_conn*)rsrc->ptr;
	wblog("wbres_commit_pconn enter, conn=%p\n",conn);
	if( conn!=NULL )
	{
		wblog("wbres_commit_pconn: conn=%p current status is %d\n",conn,conn->status);
		cd_assign_to_thread(CDP(conn));
		cd_Commit(CDP(conn));
#if WBVERS>=1000
		if( conn->client_error_level!=0 || conn->client_error_callback!=NULL )
		{
			set_client_error_callback2(CDP(conn),NULL);
			if( conn->client_error_level!=0 ) conn->client_error_level=0;
			if( conn->client_error_callback!=NULL ) { efree(conn->client_error_callback); conn->client_error_callback=NULL; }
		}
#endif
		conn->status=WB_CS_CONNECTED_UNUSED;
		wblog("wbres_commit_pconn: status of conn %p changed to WB_CS_CONNECTED_UNUSED (%d)\n",conn,WB_CS_CONNECTED_UNUSED);
	}
	wblogs("wbres_commit_pconn leave\n");
}

/* this is called for wbres_pconn on end of PHP process */
static void wbres_close_pconn(zend_rsrc_list_entry *rsrc TSRMLS_DC)
{
	wb_conn *conn=(wb_conn*)rsrc->ptr;
	wblog("wbres_close_pconn enter, conn=%p\n",conn);
	if( conn!=NULL )
	{
		// set rsrc->ptr to NULL
		rsrc->ptr=NULL;
		wblogs("setting rsrc->ptr=NULL\n");
		// call destructor
		cdp_destructor(conn,1/*do close connection*/ TSRMLS_CC);
		// free allocated memory
		free(conn); // see internal_create_connection() comments - conn is malloc-ed or emalloc-ed
		wblogs("wbres_close_pconn: conn destroyed\n");
	} 
	wblogs("wbres_close_pconn leave\n");
}

/* destroys wbres_conn on end of request */
static void wbres_close_conn(zend_rsrc_list_entry *rsrc TSRMLS_DC)
{
	wb_conn *conn=(wb_conn*)rsrc->ptr;
	wblog("wbres_close_conn enter, conn=%p\n",conn);
	if( conn!=NULL )
	{ 
		// set rsrc->ptr to NULL
		rsrc->ptr=NULL;
		wblogs("setting rsrc->ptr=NULL\n");
		// call destructor
		cdp_destructor(conn,1/*do close connection*/ TSRMLS_CC);
		// free allocated memory
		efree(conn); // see internal_create_connection() comments - conn is malloc-ed or emalloc-ed
		wblogs("wbres_close_conn: conn destroyed\n");
	} 
	wblogs("wbres_close_conn leave\n");
}

/* destroys wbres_cursor on end of request */
static void wbres_close_cursor(zend_rsrc_list_entry *rsrc TSRMLS_DC)
{
	struct sql_answer *answer=(struct sql_answer*)rsrc->ptr;
	wblog("wbres_close_cursor enter, answer=%p\n",answer);
	if( answer!=NULL )
	{
		rsrc->ptr=NULL;
		wblogs("setting rsrc->ptr=NULL\n");
		cursor_destructor(answer,1/*do close cursor*/,1/*do call cdp_destructor*/ TSRMLS_CC);
  	efree(answer->types);
#if WBVERS>=800
		efree(answer->specif);
#endif
		efree(answer);
	} 
	wblogs("wbres_close_cursor leave\n");
}

/* destroys LOB resource on end of request */
static void wbres_close_lob(zend_rsrc_list_entry *rsrc TSRMLS_DC)
{
	struct lob_info_t *lob=(struct lob_info_t*)rsrc->ptr;
	wblog("wbres_close_lob enter, lob=%p\n",lob);
	if( lob!=NULL )
	{
		rsrc->ptr=NULL;
		wblogs("setting rsrc->ptr=NULL\n");
		lob_destructor(lob,1/*do call cursor destructor*/ TSRMLS_CC);
		efree(lob);
	}
	wblogs("wbres_close_lob leave\n");
}

#ifdef USE_PSQL
/* destructor of prepared SQL statement resource */
static void wbres_close_psql(zend_rsrc_list_entry *rsrc TSRMLS_DC)
{
	struct psql_t *psql=(struct psql_t*)rsrc->ptr;
	wblog("wbres_close_psql enter, psql=%p\n",psql);
	if( psql!=NULL )
	{
		rsrc->ptr=NULL;
		wblogs("setting rsrc->ptr=NULL\n");
		psql_destructor(psql,1/*do call cdp destructor*/ TSRMLS_CC);
  	efree(psql);
	} 
	wblogs("wbres_close_psql leave\n");
}
#endif

#ifdef USE_XSD_PARSER
/* destructor of XSD parser resource */
static void wbres_close_xsdp(zend_rsrc_list_entry *rsrc TSRMLS_DC)
{
	struct xsdp_t *xsdp=(struct xsdp_t*)rsrc->ptr;
	wblog("wbres_close_xsdp enter, xsdp=%p\n",xsdp);
	if( xsdp!=NULL )
	{
		rsrc->ptr=NULL;
		wblogs("setting rsrc->ptr=NULL\n");
		xsdp_destructor(xsdp,1/*do call cdp destructor*/ TSRMLS_CC);
  	efree(xsdp);
	} 
	wblogs("wbres_close_xsdp leave\n");
}
#endif

#if WBVERS>=1000
/* destructor of ODBC connection resource */
static void wbres_close_odbc(zend_rsrc_list_entry *rsrc TSRMLS_DC)
{
	struct odbc_t *odbc=(struct odbc_t*)rsrc->ptr;
	wblog("wbres_close_odbc enter, odbc=%p\n",odbc);
	if( odbc!=NULL )
	{
		rsrc->ptr=NULL;
		wblogs("setting rsrc->ptr=NULL\n");
		odbc_destructor(odbc,1/*do close ODBC connection*/ TSRMLS_CC);
		efree(odbc);
	}
	wblogs("wbres_close_odbc leave\n");
}
#endif

static wb_conn *get_conn_from_resource(INTERNAL_FUNCTION_PARAMETERS, zval **conn_id)
{
	wb_conn *resource=NULL;
	
	wblogs("get_conn_from_resource enter\n");
	/* Next 6 lines are modified ZEND_FETCH_RESOURCE2 macro expansion
	 * ZEND_FETCH_RESOURCE2(resource, wb_conn *, conn_id, -1, WBRES_CONN_NAME, wbres_conn, wbres_pconn); 
	 * we want to return NULL when resource isn't found. */
	resource = (wb_conn*) zend_fetch_resource(conn_id TSRMLS_CC, -1, WBRES_CONN_NAME, NULL, 2, wbres_conn, wbres_pconn);	\
	if(!resource) {
		zend_error(E_WARNING,"602SQL: Resource not found");
		wblogs("get_conn_from_resource leave, resource not found\n");
		return NULL;
	}
	wblogs("get_conn_from_resource leave, OK\n");
	return resource;
}

/* Not a resourceResource is not a database connectionSimilar method is used to get
 * answer from a cursor resource. Function returns NULL whenever a problem occurs. */
static struct sql_answer *get_answer_from_result(INTERNAL_FUNCTION_PARAMETERS, zval **result_id)
{
	struct sql_answer *answer;
	wblogs("get_answer_from_result enter\n");
  /* expansion of macro
	 * ZEND_FETCH_RESOURCE(answer, struct sql_answer *, result_id, -1, WBRES_CURSOR_NAME, wbres_cursor);
	 */
	answer = (struct sql_answer *) zend_fetch_resource(result_id TSRMLS_CC, -1, WBRES_CURSOR_NAME, NULL, 1, wbres_cursor);
	if(!answer) {
		zend_error(E_WARNING,"602SQL: Resource not found");
		wblogs("get_answer_from_result leave, resource not found\n");
		return NULL;
	}
	/* end of expansion */
	wblog("get_answer_from_result leave, result=%p\n",answer);
	return answer;
}

static struct lob_info_t *get_lob_from_resource(INTERNAL_FUNCTION_PARAMETERS, zval **lob_id)
{
	struct lob_info_t *lob;
	wblogs("get_lob_from_resource enter\n");
  /* expansion of macro
	 * ZEND_FETCH_RESOURCE(lob, struct lob_info_t*, lob_id, -1, WBRES_LOB_NAME, wbres_lob);
	 */
	lob = (struct lob_info_t*) zend_fetch_resource(lob_id TSRMLS_CC, -1, WBRES_LOB_NAME, NULL, 1, wbres_lob);
	if(!lob) {
		zend_error(E_WARNING,"602SQL: Resource not found");
		wblogs("get_lob_from_resource leave, resource not found\n");
		return NULL;
	}
	/* end of expansion */
	wblog("get_lob_from_resource leave, result=%p\n",lob);
	return lob;
}

#ifdef USE_PSQL
static struct psql_t *get_psql_from_resource(INTERNAL_FUNCTION_PARAMETERS, zval **psql_id)
{
	struct psql_t *psql;
	wblogs("get_psql_from_resource enter\n");
  /* expansion of macro
	 * ZEND_FETCH_RESOURCE(psql, struct psql_t*, psql_id, -1, WBRES_PSQL_NAME, wbres_psql);
	 */
	psql = (struct psql_t*) zend_fetch_resource(psql_id TSRMLS_CC, -1, WBRES_PSQL_NAME, NULL, 1, wbres_psql);
	if(!psql) {
		zend_error(E_WARNING,"602SQL: Resource not found");
		wblogs("get_psql_from_resource leave, resource not found\n");
		return NULL;
	}
	/* end of expansion */
	wblog("get_psql_from_resource leave, result=%p\n",psql);
	return psql;
}
#endif

#ifdef USE_XSD_PARSER
static struct xsdp_t *get_xsdp_from_resource(INTERNAL_FUNCTION_PARAMETERS, zval **xsdp_id)
{
	struct xsdp_t *xsdp;
	wblogs("get_xsdp_from_resource enter\n");
  /* expansion of macro
	 * ZEND_FETCH_RESOURCE(xsdp, struct xsdp_t*, xsdp_id, -1, WBRES_XSDP_NAME, wbres_xsdp);
	 */
	xsdp = (struct xsdp_t*) zend_fetch_resource(xsdp_id TSRMLS_CC, -1, WBRES_XSDP_NAME, NULL, 1, wbres_xsdp);
	if(!xsdp) {
		zend_error(E_WARNING,"602SQL: Resource not found");
		wblogs("get_xsdp_from_resource leave, resource not found\n");
		return NULL;
	}
	/* end of expansion */
	wblog("get_xsdp_from_resource leave, result=%p\n",xsdp);
	return xsdp;
}
#endif

#if WBVERS>=1000
static struct odbc_t *get_odbcp_from_resource(INTERNAL_FUNCTION_PARAMETERS, zval **odbc_id)
{
	struct odbc_t *odbc;
	wblogs("get_odbc_from_resource enter\n");
  /* expansion of macro
	 * ZEND_FETCH_RESOURCE(odbc, struct odbc_t*, odbc_id, -1, WBRES_ODBC_NAME, wbres_odbc);
	 */
	odbc = (struct odbc_t*) zend_fetch_resource(odbc_id TSRMLS_CC, -1, WBRES_ODBC_NAME, NULL, 1, wbres_odbc);
	if(!odbc) {
		zend_error(E_WARNING,"602SQL: Resource not found");
		wblogs("get_odbc_from_resource leave, resource not found\n");
		return NULL;
	}
	/* end of expansion */
	wblog("get_odbc_from_resource leave, result=%p\n",odbc);
	return odbc;
}
#endif
/*}}}*/
/*{{{ INI entries */
/* entries in php.ini which affect this module */
#if defined ZEND_ENGINE_2 && !defined OnUpdateInt
#define OnUpdateInt OnUpdateLong
#endif
PHP_INI_BEGIN()
	STD_PHP_INI_ENTRY("wb_longreadlen", "4096", PHP_INI_ALL, OnUpdateInt, longreadlen, zend_wb_globals, wb_globals)
	STD_PHP_INI_ENTRY("wb_binmode", "2", PHP_INI_ALL, OnUpdateInt, binmode, zend_wb_globals, wb_globals)
	STD_PHP_INI_ENTRY("wb_datestring", "0", PHP_INI_ALL, OnUpdateInt, datestring, zend_wb_globals, wb_globals)
	STD_PHP_INI_ENTRY("wb_max_links", "-1", PHP_INI_SYSTEM, OnUpdateInt, max_conn, zend_wb_globals, wb_globals)
	STD_PHP_INI_ENTRY("wb_max_persistent", "-1", PHP_INI_SYSTEM, OnUpdateInt, max_persistent, zend_wb_globals, wb_globals)
	STD_PHP_INI_ENTRY("wb_allow_persistent", "1", PHP_INI_SYSTEM, OnUpdateInt, allow_persistent, zend_wb_globals, wb_globals)
#ifdef LOG_SERVER_MEMORY
	STD_PHP_INI_ENTRY("wb_log_server_memory", "0", PHP_INI_SYSTEM, OnUpdateInt, log_server_memory, zend_wb_globals, wb_globals)
#ifdef WIN32
	STD_PHP_INI_ENTRY("wb_log_server_memory_file", "c:\\php_602sql_memory.log", PHP_INI_ALL, OnUpdateString, log_server_memory_file, zend_wb_globals, wb_globals)
#else
	STD_PHP_INI_ENTRY("wb_log_server_memory_file", "/tmp/php_602sql_memory.log", PHP_INI_ALL, OnUpdateString, log_server_memory_file, zend_wb_globals, wb_globals)
#endif
#endif
PHP_INI_END()

/*}}}*/
/*{{{ PHP module magic (MINIT, MSHUTDOWN, etc.) */
/* Having defined all the functions, we may finally place the interface tables. */
/*Table of functions that module exports to PHP:*/
/* All the PHP functions provided by the module must be registered in wb_functions table.
Short comments are meant only for orientation; detailed description can be found before
actual definition of each of the functions (you can use index to look it up). */

#ifdef ZEND_ENGINE_2
ZEND_BEGIN_ARG_INFO(a2_force_ref, 0)
	ZEND_ARG_PASS_INFO(0)
	ZEND_ARG_PASS_INFO(1)
ZEND_END_ARG_INFO()
ZEND_BEGIN_ARG_INFO(a3_force_ref, 0)
	ZEND_ARG_PASS_INFO(0)
	ZEND_ARG_PASS_INFO(0)
	ZEND_ARG_PASS_INFO(1)
ZEND_END_ARG_INFO()
ZEND_BEGIN_ARG_INFO(a5_force_ref, 0)
	ZEND_ARG_PASS_INFO(0)
	ZEND_ARG_PASS_INFO(0)
	ZEND_ARG_PASS_INFO(0)
	ZEND_ARG_PASS_INFO(0)
	ZEND_ARG_PASS_INFO(1)
ZEND_END_ARG_INFO()
#else
static unsigned char a2_force_ref[]={2, BYREF_NONE, BYREF_FORCE};
static unsigned char a3_force_ref[]={3, BYREF_NONE, BYREF_NONE, BYREF_FORCE};
static unsigned char a5_force_ref[]={5, BYREF_NONE, BYREF_NONE, BYREF_NONE, BYREF_NONE, BYREF_FORCE};
#endif

zend_function_entry wb_functions[] =	{
	ZEND_FE(wb_connect,	a5_force_ref)/* Connect to base. */
	ZEND_FE(wb_pconnect, a5_force_ref)/* Persistently connect to base. */
	ZEND_FE(wb_exec,		NULL)/* Exec SQL statement */
	ZEND_FALIAS(wb_query, wb_exec, NULL) /* Alias */
	ZEND_FE(wb_fetch_into,	a2_force_ref)/* Fetch record into array */
	ZEND_FE(wb_field_name,	NULL)/* Get field name by column number */
	ZEND_FE(wb_field_num, NULL)/* Get field number by field name */
	ZEND_FE(wb_num_fields,	NULL)/* Get number of attributes */
	ZEND_FE(wb_num_rows,	NULL)/* Get number of records */
	ZEND_FE(wb_fetch_row,	NULL)/* Set current record */
	ZEND_FE(wb_result_all,	NULL)/* Display result as a table */
	ZEND_FE(wb_result,	NULL)/* Get value of an attribute in current record */
	ZEND_FE(wb_binmode,	NULL)/* Specify how binary and long types are used */
	ZEND_FE(wb_longreadlen,	NULL)/* Specify how long types are used */
	ZEND_FE(wb_close,	NULL)/* Close a connection or cursor */
	ZEND_FALIAS(wb_free_results, wb_close, NULL) /* Other name */
	ZEND_FE(wb_err_str,	NULL)/* Get string with last 602SQL error */
	ZEND_FE(wb_import_file,	NULL)/* Read data from a file to table */
	ZEND_FE(wb_data_seek, NULL)/* Seek to specified record in cursor. Next
							wb_fetch_row or wb_fetch_into call will
							read this record. */
	ZEND_FE(wb_field_type, NULL)/* Get name of SQL type of field specified by column number */
	ZEND_FE(wb_field_type_num, NULL)/* Get numeric constant (ATT_XXX) which represents SQL type of field specified by column number*/
#if WBVERS>=800
	ZEND_FE(wb_client_encoding, NULL)/* Get string host variables encoding number - one of WB_ENC_XXX constant */
	ZEND_FE(wb_set_client_encoding, NULL)/* Change string host variables encoding for specified connection */
#endif
#if WBVERS<900
	ZEND_FE(wb_select_language, NULL)/* select language */
	ZEND_FE(wb_get_selected_language_id, NULL)/* get selected language ID */
	ZEND_FE(wb_get_selected_language_name, NULL)/* get selected language name */
	ZEND_FE(wb_metastring, NULL)/* get metastring text in selected language */
	ZEND_FE(wb_get_languages, NULL)/* get languages defined in this application */
#endif
	ZEND_FE(wb_set_lob_mode, NULL)/* set new LOB manipulation mode for specified cursor */
	ZEND_FE(wb_lob_write, NULL)/* write segment of LOB value */
	ZEND_FE(wb_get_lob_mode, NULL)/* get LOB manipulation mode valid for specified cursor */
	ZEND_FALIAS(wb_lob_close, wb_close, NULL)/* alias */
	ZEND_FE(wb_lob_length, NULL)/* get size of LOB value */
	ZEND_FE(wb_lob_read, NULL)/* read segment of LOB value */
	ZEND_FE(wb_lob_truncate, NULL)/* truncate LOB value to specified size */
#ifdef USE_PSQL
	ZEND_FE(wb_psql_prepare, NULL)
	ZEND_FE(wb_psql_execute, NULL)
	ZEND_FE(wb_psql_param_count, NULL)
	ZEND_FE(wb_psql_param_info, NULL)
	ZEND_FE(wb_psql_drop_params, NULL)
	ZEND_FE(wb_psql_send_param, NULL)
	ZEND_FE(wb_psql_out_param_count, NULL)
	ZEND_FE(wb_psql_get_out_param, NULL)
	ZEND_FE(wb_psql_out_param_length, NULL)
	ZEND_FALIAS(wb_psql_close, wb_close, NULL) /* alias */
#endif
#ifdef ENABLE_XML_IMPORT_EXPORT
	ZEND_FE(wb_export_to_xml, NULL)
	ZEND_FE(wb_export_to_xml_buffer, NULL)
	ZEND_FE(wb_import_from_xml, NULL)
	ZEND_FE(wb_import_from_xml_buffer, NULL)
#endif
#if WBVERS>=950
	ZEND_FE(wb_data_transport, NULL)
#endif
	ZEND_FE(wb_err_num, NULL)
	ZEND_FE(wb_select_db, NULL)
#ifdef ENABLE_XML_RESPONSE
	ZEND_FE(wb_get_xml_response, NULL)
#endif
#ifdef USE_WB_SCRIPT_ENCODING
	ZEND_FE(wb_script_encoding, NULL)
	ZEND_FE(wb_set_script_encoding, NULL)
#endif
#ifdef USE_XSD_PARSER
	ZEND_FE(wb_xsdp_open, NULL)
	ZEND_FALIAS(wb_xsdp_close, wb_close, NULL)
	ZEND_FE(wb_xsdp_set_prop, NULL)
	ZEND_FE(wb_xsdp_get_prop, a3_force_ref)
	ZEND_FE(wb_xsdp_parse, a3_force_ref)
	ZEND_FE(wb_xsdp_create_objects, a2_force_ref)
	ZEND_FE(wb_xsdp_get_columns, a3_force_ref)
	ZEND_FE(wb_xsdp_set_column_type, NULL)
	ZEND_FE(wb_xsdp_exclude_table, NULL)
#endif
#if WBVERS>=1000
	ZEND_FE(wb_get_xml_form, NULL)
	ZEND_FE(wb_match_form_and_dad, NULL)
#endif
#if WBVERS>=1000
	ZEND_FE(wb_set_client_error_callback, NULL)
	ZEND_FE(wb_odbc_connect, NULL)
	ZEND_FALIAS(wb_odbc_disconnect, wb_close, NULL)
#endif
	ZEND_FE(wb_escape_string, NULL)
#if WBVERS>=1000
	ZEND_FE(wb_zfo_to_fo, NULL)
	ZEND_FE(wb_fo_to_zfo, NULL)
#endif
#if defined LOG_PHP_WB602 && defined USE_WB_DEBUG
	ZEND_FE(wb_debug, NULL)/* affect debug logfile generation */
#endif
	{NULL, NULL, NULL}	/* Must be the last line in wb_functions[] */
};

/*Functions for initialization and cleanup:*/
/* Request init function - if we don't initialize module's globals in wb_globals_ctor
 * we have to do it there. */
PHP_RINIT_FUNCTION(wb)
{
	wblogs("PHP_RINIT_FUNCTION(wb) enter\n");
	WBG(num_conn)=WBG(num_persistent);
	wblogs("PHP_RINIT_FUNCTION(wb) leave\n");
	return SUCCESS;
}

/* Request shutdown */
PHP_RSHUTDOWN_FUNCTION(wb)
{
	wblogs("PHP_RSHUTDOWN_FUNCTION(wb) enter\n");
	wblogs("PHP_RSHUTDOWN_FUNCTION(wb) leave\n");
	return SUCCESS;
}

/* Module shutdown */
PHP_MSHUTDOWN_FUNCTION(wb)
{
	wblogs("PHP_MSHUTDOWN_FUNCTION(wb) enter\n");
	UNREGISTER_INI_ENTRIES();
	wblogs("PHP_MSHUTDOWN_FUNCTION(wb) leave\n");
	return SUCCESS;
}

static void wb_globals_ctor(zend_wb_globals *wb_globals)
{
	wblog("wb_globals_ctor enter, wb_globals=%p\n",wb_globals);
	wb_globals->num_persistent=0;
#ifdef USE_WB_SCRIPT_ENCODING
	wb_globals->script_encoding=WB_ENC_UNKNOWN;
#endif
#ifdef LOG_SERVER_MEMORY
	wb_globals->log_server_memory=0;
	wb_globals->log_server_memory_file=NULL;
#endif
	wblogs("wb_globals_ctor_leave\n");
}

/* Module init */
PHP_MINIT_FUNCTION(wb)
{
	wblogs("PHP_MINIT_FUNCTION(wb) enter\n");
	ZEND_INIT_MODULE_GLOBALS(wb, wb_globals_ctor, NULL);
	REGISTER_INI_ENTRIES();

	/*Register resource classes:*/
	wbres_pconn=zend_register_list_destructors_ex(wbres_commit_pconn,wbres_close_pconn,WBRES_PCONN_NAME,module_number);
	wbres_conn=zend_register_list_destructors_ex(wbres_close_conn,NULL,WBRES_CONN_NAME,module_number);
	wbres_cursor=zend_register_list_destructors_ex(wbres_close_cursor,NULL,WBRES_CURSOR_NAME,module_number);
	wbres_lob=zend_register_list_destructors_ex(wbres_close_lob,NULL,WBRES_LOB_NAME,module_number);
#ifdef USE_PSQL
	wbres_psql=zend_register_list_destructors_ex(wbres_close_psql,NULL,WBRES_PSQL_NAME,module_number);
#endif
#ifdef USE_XSD_PARSER
	wbres_xsdp=zend_register_list_destructors_ex(wbres_close_xsdp,NULL,WBRES_XSDP_NAME,module_number);
#endif
#if WBVERS>=1000
	wbres_odbc=zend_register_list_destructors_ex(wbres_close_odbc,NULL,WBRES_ODBC_NAME,module_number);
#endif

	/*Register constants for PHP:*/
	/*	- WB_PASSTHRU, WB_RETURN, WB_CONVERT
	We do not want force the PHP programmer to remember the numeric values, so we create
	constants for her. */
	REGISTER_LONG_CONSTANT("WB_PASSTHRU", PASSTHRU, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("WB_RETURN", RETURN, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("WB_CONVERT", CONVERT, CONST_CS | CONST_PERSISTENT);
	/* ATT_XXX constants - these values are returned by wb_field_type_num */
	REGISTER_LONG_CONSTANT("ATT_BOOLEAN",ATT_BOOLEAN,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("ATT_CHAR",ATT_CHAR,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("ATT_INT16",ATT_INT16,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("ATT_INT32",ATT_INT32,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("ATT_MONEY",ATT_MONEY,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("ATT_FLOAT",ATT_FLOAT,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("ATT_STRING",ATT_STRING,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("ATT_BINARY",ATT_BINARY,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("ATT_DATE",ATT_DATE,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("ATT_TIME",ATT_TIME,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("ATT_TIMESTAMP",ATT_TIMESTAMP,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("ATT_PTR",ATT_PTR,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("ATT_BIPTR",ATT_BIPTR,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("ATT_AUTOR",ATT_AUTOR,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("ATT_DATIM",ATT_DATIM,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("ATT_HIST",ATT_HIST,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("ATT_RASTER",ATT_RASTER,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("ATT_TEXT",ATT_TEXT,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("ATT_NOSPEC",ATT_NOSPEC,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("ATT_SIGNAT",ATT_SIGNAT,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("ATT_INT8",ATT_INT8,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("ATT_INT64",ATT_INT64,CONST_CS|CONST_PERSISTENT);
#if WBVERS>=1100
	REGISTER_LONG_CONSTANT("ATT_EXTCLOB",ATT_EXTCLOB,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("ATT_EXTBLOB",ATT_EXTBLOB,CONST_CS|CONST_PERSISTENT);
#endif
#if WBVERS>=800
	/* client and script encoding */
	REGISTER_LONG_CONSTANT("WB_ENC_UNKNOWN",WB_ENC_UNKNOWN,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("WB_ENC_ASCII",WB_ENC_ASCII,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("WB_ENC_WIN1250",WB_ENC_WIN1250,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("WB_ENC_WIN1252",WB_ENC_WIN1252,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("WB_ENC_ISO_8859_2",WB_ENC_ISO_8859_2,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("WB_ENC_UTF8",WB_ENC_UTF8,CONST_CS|CONST_PERSISTENT);
#endif
	/* LOBs manipulation behaviour */
	REGISTER_LONG_CONSTANT("WB_LOBMODE_DIRECT",WB_LOBMODE_DIRECT,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("WB_LOBMODE_RESOURCE",WB_LOBMODE_RESOURCE,CONST_CS|CONST_PERSISTENT);
#ifdef USE_PSQL
	/* MODE_xxx constants */
	REGISTER_LONG_CONSTANT("MODE_IN",MODE_IN,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("MODE_OUT",MODE_OUT,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("MODE_INOUT",MODE_INOUT,CONST_CS|CONST_PERSISTENT);
#endif
	/* module version information */
	REGISTER_LONG_CONSTANT("PHP_602SQL_VERSION_MAJOR",MODVERS_1,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("PHP_602SQL_VERSION_MINOR",MODVERS_2,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("PHP_602SQL_VERSION_PATCH",MODVERS_3,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("PHP_602SQL_VERSION_PATCH2",MODVERS_4,CONST_CS|CONST_PERSISTENT);
#if WBVERS>=950
	REGISTER_LONG_CONSTANT("IMPEXP_FORMAT_WINBASE",IMPEXP_FORMAT_WINBASE,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("IMPEXP_FORMAT_TEXT_COLUMNS",IMPEXP_FORMAT_TEXT_COLUMNS,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("IMPEXP_FORMAT_TEXT_CSV",IMPEXP_FORMAT_TEXT_CSV,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("IMPEXP_FORMAT_DBASE",IMPEXP_FORMAT_DBASE,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("IMPEXP_FORMAT_FOXPRO",IMPEXP_FORMAT_FOXPRO,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("IMPEXP_FORMAT_ODBC",IMPEXP_FORMAT_ODBC,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("IMPEXP_FORMAT_CURSOR",IMPEXP_FORMAT_CURSOR,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("IMPEXP_FORMAT_ODBCVIEW",IMPEXP_FORMAT_ODBCVIEW,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("IMPEXP_FORMAT_TABLE",IMPEXP_FORMAT_TABLE,CONST_CS|CONST_PERSISTENT);
#endif
	/* wb_fetch_into() result_type parameter values */
	REGISTER_LONG_CONSTANT("WB_FETCH_NUM0",WB_FETCH_NUM0,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("WB_FETCH_NUM1",WB_FETCH_NUM1,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("WB_FETCH_ASSOC",WB_FETCH_ASSOC,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("WB_FETCH_BOTH0",WB_FETCH_BOTH0,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("WB_FETCH_BOTH1",WB_FETCH_BOTH1,CONST_CS|CONST_PERSISTENT);
#ifdef USE_XSD_PARSER
	// enum DSPProp
	REGISTER_LONG_CONSTANT("DSP_SCHEMANAME",DSP_SCHEMANAME,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DSP_FOLDERNAME",DSP_FOLDERNAME,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DSP_DADNAME",DSP_DADNAME,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DSP_TABLEPREFIX",DSP_TABLEPREFIX,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DSP_DEFSTRLENGTH",DSP_DEFSTRLENGTH,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DSP_UNICODE",DSP_UNICODE,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DSP_IDTYPE",DSP_IDTYPE,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DSP_ERRCODE",DSP_ERRCODE,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DSP_ERRSOURCE",DSP_ERRSOURCE,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DSP_ERRMSG",DSP_ERRMSG,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DSP_ERRTYPEXML",DSP_ERRTYPEXML,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DSP_ERRFILE",DSP_ERRFILE,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DSP_ERRLINE",DSP_ERRLINE,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DSP_ERRCOL",DSP_ERRCOL,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DSP_XMLFORMSOURCE",DSP_XMLFORMSOURCE,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DSP_REFINT",DSP_REFINT,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DSP_XSD",DSP_XSD,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DSP_ROOTTABLE",DSP_ROOTTABLE,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DSP_ROOTTABLEKEY",DSP_ROOTTABLEKEY,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DSP_ROOTTABLESEQ",DSP_ROOTTABLESEQ,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DSP_FORMID",DSP_FORMID,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DSP_FORMDATAROOTNAME",DSP_FORMDATAROOTNAME,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DSP_FORMDATANSPREFIX",DSP_FORMDATANSPREFIX,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DSP_FORMDATANSURI",DSP_FORMDATANSURI,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DSP_SPECROOTSEQ",DSP_SPECROOTSEQ,CONST_CS|CONST_PERSISTENT);
	// enum DSPErrSource
  REGISTER_LONG_CONSTANT("DSPE_NONE",DSPE_NONE,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DSPE_SYSTEM",DSPE_SYSTEM,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DSPE_SQL",DSPE_SQL,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DSPE_XML",DSPE_XML,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DSPE_DOM",DSPE_DOM,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DSPE_SAX",DSPE_SAX,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DSPE_XSD",DSPE_XSD,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DSPE_ZIP",DSPE_ZIP,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("DSPE_OTHER",DSPE_OTHER,CONST_CS|CONST_PERSISTENT);
#endif
	/* CATEG_xxx database objects categories */
  REGISTER_LONG_CONSTANT("CATEG_TABLE",CATEG_TABLE,CONST_CS|CONST_PERSISTENT); //  0  tabulka 
  REGISTER_LONG_CONSTANT("CATEG_USER",CATEG_USER,CONST_CS|CONST_PERSISTENT); //  1  uzivatel 
  REGISTER_LONG_CONSTANT("CATEG_VIEW",CATEG_VIEW,CONST_CS|CONST_PERSISTENT); //  2  pohled 
  REGISTER_LONG_CONSTANT("CATEG_CURSOR",CATEG_CURSOR,CONST_CS|CONST_PERSISTENT); //  3  dotaz 
  REGISTER_LONG_CONSTANT("CATEG_PGMSRC",CATEG_PGMSRC,CONST_CS|CONST_PERSISTENT); //  4  text programu 
  REGISTER_LONG_CONSTANT("CATEG_PGMEXE",CATEG_PGMEXE,CONST_CS|CONST_PERSISTENT); //  5  prelozeny program 
  REGISTER_LONG_CONSTANT("CATEG_MENU",CATEG_MENU,CONST_CS|CONST_PERSISTENT); //  6  menu 
  REGISTER_LONG_CONSTANT("CATEG_APPL",CATEG_APPL,CONST_CS|CONST_PERSISTENT); //  7  aplikace 
  REGISTER_LONG_CONSTANT("CATEG_PICT",CATEG_PICT,CONST_CS|CONST_PERSISTENT); //  8  obrazek 
  REGISTER_LONG_CONSTANT("CATEG_GROUP",CATEG_GROUP,CONST_CS|CONST_PERSISTENT); //  9  skupina 
  REGISTER_LONG_CONSTANT("CATEG_ROLE",CATEG_ROLE,CONST_CS|CONST_PERSISTENT); //  10  role 
  REGISTER_LONG_CONSTANT("CATEG_CONNECTION",CATEG_CONNECTION,CONST_CS|CONST_PERSISTENT); //  11  ODBC spojeni 
  REGISTER_LONG_CONSTANT("CATEG_RELATION",CATEG_RELATION,CONST_CS|CONST_PERSISTENT); //  12  relace 
  REGISTER_LONG_CONSTANT("CATEG_DRAWING",CATEG_DRAWING,CONST_CS|CONST_PERSISTENT); //  13  nakres 
  REGISTER_LONG_CONSTANT("CATEG_GRAPH",CATEG_GRAPH,CONST_CS|CONST_PERSISTENT); //  14  graf 
  REGISTER_LONG_CONSTANT("CATEG_REPLREL",CATEG_REPLREL,CONST_CS|CONST_PERSISTENT); //  15  replikacni vztah 
  REGISTER_LONG_CONSTANT("CATEG_PROC",CATEG_PROC,CONST_CS|CONST_PERSISTENT); //  16  rutina 
  REGISTER_LONG_CONSTANT("CATEG_TRIGGER",CATEG_TRIGGER,CONST_CS|CONST_PERSISTENT); //  17  trigger 
  REGISTER_LONG_CONSTANT("CATEG_WWW",CATEG_WWW,CONST_CS|CONST_PERSISTENT); //  18  WWW objekt 
  REGISTER_LONG_CONSTANT("CATEG_FOLDER",CATEG_FOLDER,CONST_CS|CONST_PERSISTENT); //  19  slozka 
  REGISTER_LONG_CONSTANT("CATEG_SEQ",CATEG_SEQ,CONST_CS|CONST_PERSISTENT); //  20  sekvence 
  REGISTER_LONG_CONSTANT("CATEG_INFO",CATEG_INFO,CONST_CS|CONST_PERSISTENT); //  21  informacni objekt 
#if WBVERS>=800
  REGISTER_LONG_CONSTANT("CATEG_DOMAIN",CATEG_DOMAIN,CONST_CS|CONST_PERSISTENT); //  22  domena 
#if WBVERS>=1000
  REGISTER_LONG_CONSTANT("CATEG_STSH",CATEG_STSH,CONST_CS|CONST_PERSISTENT); //  23  style sheet 
  REGISTER_LONG_CONSTANT("CATEG_XMLFORM",CATEG_XMLFORM,CONST_CS|CONST_PERSISTENT); //  24  XML form 
#endif
  REGISTER_LONG_CONSTANT("CATEG_KEY",CATEG_KEY,CONST_CS|CONST_PERSISTENT); //  25  verejny klic uzivatele 
  REGISTER_LONG_CONSTANT("CATEG_SERVER",CATEG_SERVER,CONST_CS|CONST_PERSISTENT); //  26  replikacni server 
#endif
#if WBVERS>=1000
	// wb_match_form_and_dad results
  REGISTER_LONG_CONSTANT("MFD_AGR",MFD_AGR,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("MFD_DISAGR",MFD_DISAGR,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("MFD_FORMREADFAIL",MFD_FORMREADFAIL,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("MFD_FORMPARSEFAIL",MFD_FORMPARSEFAIL,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("MFD_DADREADFAIL",MFD_DADREADFAIL,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("MFD_DADPARSEFAIL",MFD_DADPARSEFAIL,CONST_CS|CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("MFD_OTHER",MFD_OTHER,CONST_CS|CONST_PERSISTENT);
#endif
	// wb_connect and wb_pconnect error codes
  REGISTER_LONG_CONSTANT("WB_CONNECT_ERROR_OK",WB_CONNECT_ERROR_OK,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("WB_CONNECT_ERROR_NO_MEMORY",WB_CONNECT_ERROR_NO_MEMORY,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("WB_CONNECT_ERROR_CONNECT",WB_CONNECT_ERROR_CONNECT,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("WB_CONNECT_ERROR_PERSISTENT_RESOURCE_NOT_FOUND",WB_CONNECT_ERROR_PERSISTENT_RESOURCE_NOT_FOUND,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("WB_CONNECT_ERROR_TOO_MANY_CONNECTIONS",WB_CONNECT_ERROR_TOO_MANY_CONNECTIONS,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("WB_CONNECT_ERROR_TOO_MANY_PERSISTENT_CONNECTIONS",WB_CONNECT_ERROR_TOO_MANY_PERSISTENT_CONNECTIONS,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("WB_CONNECT_ERROR_GET_SERVER_VERSION",WB_CONNECT_ERROR_GET_SERVER_VERSION,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("WB_CONNECT_ERROR_GET_SYSTEM_CHARSET",WB_CONNECT_ERROR_GET_SYSTEM_CHARSET,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("WB_CONNECT_ERROR_LOGIN",WB_CONNECT_ERROR_LOGIN,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("WB_CONNECT_ERROR_SET_PROGRESS_REPORT_MODULUS",WB_CONNECT_ERROR_SET_PROGRESS_REPORT_MODULUS,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("WB_CONNECT_ERROR_SET_APPLICATION",WB_CONNECT_ERROR_SET_APPLICATION,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("WB_CONNECT_ERROR_STORE_CONNECTION_RESOURCE",WB_CONNECT_ERROR_STORE_CONNECTION_RESOURCE,CONST_CS|CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("WB_CONNECT_ERROR_RESOURCE_NOT_FOUND",WB_CONNECT_ERROR_RESOURCE_NOT_FOUND,CONST_CS|CONST_PERSISTENT);

#if WBVERS>=1100 && defined USE_XSD_PARSER && defined LINUX
	// initialize string encoding conversions
	prepare_charset_conversions();
#endif

	wblogs("PHP_MINIT_FUNCTION(wb) leave\n"); 
	return SUCCESS;
}

/* Administration table for PHP: */
/* Stuff needed for handling of the module.
Most of the stuff generated here was (at least partially) autogenerated. */
zend_module_entry wb_module_entry = {
#if PHP_API_VERSION >=20010901
	STANDARD_MODULE_HEADER,
#endif
#if WBVERS >= 800
	"602sql", /* module name */
#else
	"wb602",
#endif
	wb_functions, /* table of functions */
	PHP_MINIT(wb), /* startup stuff */
	PHP_MSHUTDOWN(wb), /* shutdown stuff */
	PHP_RINIT(wb),		/* There's nothing to do at request start */
	PHP_RSHUTDOWN(wb),	/* There's nothing to do at request end */
	PHP_MINFO(wb),
#if PHP_API_VERSION >=20010901
	VERSION, /* A string containing version for phpinfo */
#endif
	STANDARD_MODULE_PROPERTIES
};

#if COMPILE_DL_602SQL
ZEND_GET_MODULE(wb)
#endif

PHP_MINFO_FUNCTION(wb)
{
	char buf[32];
	wblogs("PHP_MINFO_FUNCTION(wb) enter\n");
	php_info_print_table_start();
	php_info_print_table_header(2,
#if WBVERS >= 800
			"602SQL module"
#else
			"WB602 module"
#endif
			, "Loaded");
	php_info_print_table_row(2, "Compiled",__DATE__ " "	__TIME__);
	php_info_print_table_row(2, "Version", VERSION );
	sprintf(buf,"%d",WBG(num_persistent));
	php_info_print_table_row(2, "Active Persistent Links", buf);
	sprintf(buf,"%d",WBG(num_conn));
	php_info_print_table_row(2, "Active Links", buf);
#ifdef LOG_PHP_WB602
	php_info_print_table_row(2, "LOG_PHP_WB602: Filename of Debug Logfile", log_filename);
	php_info_print_table_row(2, "USE_WB_DEBUG: Debug logfile generation can be controlled by wb_debug() function",
#ifdef USE_WB_DEBUG
			"yes"
#else
			"no"
#endif
			);
#endif
	php_info_print_table_row(2, "USE_PSQL: Prepared SQL functions are present",
#ifdef USE_PSQL
			"yes"
#else
			"no"
#endif
			);
	php_info_print_table_row(2, "ENABLE_XML_IMPORT_EXPORT: XML import/export functions are present",
#ifdef ENABLE_XML_IMPORT_EXPORT
			"yes"
#else
			"no"
#endif
			);
	php_info_print_table_row(2, "ENABLE_XML_RESPONSE: wb_get_xml_response() function is present",
#ifdef ENABLE_XML_RESPONSE
			"yes"
#else
			"no"
#endif
			);
	php_info_print_table_row(2, "USE_WB_SCRIPT_ENCODING: script encoding functions are present",
#ifdef USE_WB_SCRIPT_ENCODING
			"yes"
#else
			"no"
#endif
			);
	php_info_print_table_row(2, "USE_XSD_PARSER: wb_xsdp_XXX functions are present",
#ifdef USE_XSD_PARSER
			"yes"
#else
			"no"
#endif
			);
	php_info_print_table_end();

	DISPLAY_INI_ENTRIES();
	wblogs("PHP_MINFO_FUNCTION(wb) leave\n");
}
/*}}}*/
#if defined LOG_PHP_WB602 && defined USE_WB_DEBUG
/*{{{ boolean wb_debug(int debug_value,string debug_log_filename) */
ZEND_FUNCTION(wb_debug)
{
	zval **dbg_value,**dbg_filename=NULL;

	switch(ZEND_NUM_ARGS()){
		case 1: /* debug_value only */
			if(zend_get_parameters_ex(1,&dbg_value)!=SUCCESS) WRONG_PARAM_COUNT;
			break;
		case 2: /* debug_value and debug_log_filename */
			if(zend_get_parameters_ex(2,&dbg_value,&dbg_filename)!=SUCCESS) WRONG_PARAM_COUNT;
			break;
		default: WRONG_PARAM_COUNT;
	}

	if( dbg_filename )
	{
		convert_to_string_ex(dbg_filename);
		strncpy(log_filename,Z_STRVAL_PP(dbg_filename),MAX_PATH);
		log_filename[MAX_PATH]='\0';
	}
	convert_to_long_ex(dbg_value);
	wb_debug_value=Z_LVAL_PP(dbg_value);
	RETURN_TRUE;
}
/*}}}*/
#endif
/*{{{ internal helper functions that are used module-wide */
const char wb_encoding_name[WB_ENC_LAST+1][16] = /*{{{*/
{
	"unknown",
	"ASCII",
	"Windows1250",
	"Windows1252",
	"ISO-8859-2",
	"UTF8"
};/*}}}*/
/* sql602_charset2wb_encoding {{{*/
static wb_encoding sql602_charset2wb_encoding(uns8 sql602_charset)
{
	switch( sql602_charset )
	{
		case 0: return WB_ENC_ASCII;
		case 1:
		case 2: return WB_ENC_WIN1250;
		case 3:
		case 4:
		case 5: return WB_ENC_WIN1252;
		case 7: return WB_ENC_UTF8;
		case 129:
		case 130: return WB_ENC_ISO_8859_2;
		case 131:
		case 132:
		case 133: return WB_ENC_ASCII;
		default:
			return WB_ENC_ASCII;
	}
} 
/*}}}*/
/* wb_encoding2sql602_charset {{{*/
static uns8 wb_encoding2sql602_charset(wb_encoding encoding)
{
	switch( encoding )
	{
		case WB_ENC_ASCII: return 0;
		case WB_ENC_WIN1250: return 1;
		case WB_ENC_WIN1252: return 3;
		case WB_ENC_ISO_8859_2: return 129;
		case WB_ENC_UTF8: return 7; // see comm/wbdefs.h
		default: return 0;
	}
}
/*}}}*/
/* string conversion functions {{{*/
#define IS_UCS2_EOS(ucs2strptr) ((uns8)(((char*)(ucs2strptr))[0])==0x0 || ((uns8)(((char*)(ucs2strptr))[0])==0xff && (uns8)(((char*)(ucs2strptr))[1])==0xff))
#define IS_UTF8_EOS(utf8strptr) ((uns8)(((char*)(utf8strptr))[0])==0x0)
static size_t UCS2bytelen(char *str) /*{{{
* returns size of UCS-2 string str in bytes */
{
	size_t ret=0;
	if( str==NULL ) return ret;
	for( ;; )
	{
		if( IS_UCS2_EOS(str) )
		{
			return ret;
		}
		str+=2; ret+=2;
	}
}
/*}}}*/
static int charUCS2toUTF8(uns16 ucs2char,uns8 *utf8str) /*{{{
* converts UCS-2 char to UTF-8 char
* Parameters: ucs2char .. UCS-2 char
*             utf8str  .. pointer to buffer where function have to store UTF-8 character
*                         buffer must be sufficiently long (function will write at most four bytes to buffer)
*                         pointer can be NULL, in that case nothing will be written to buffer
*                         however number of bytes that would be written will be returned
* Return value: 0 on error, or nuber of bytes which were (or would be) written to buffer
* */
{
	if( ucs2char < 0x80 )
	{
		if( utf8str!=NULL ) utf8str[0]=(uns8)ucs2char;
		return 1;
	}
	else if( ucs2char < 0x800 )
 	{
		if( utf8str!=NULL )
		{
		  utf8str[0]=(uns8)(0xC0 | (ucs2char >>  6));
			utf8str[1]=(uns8)(0x80 | (ucs2char & 0x3F));
		}
		return 2;
	}
	else
 	{
		if( utf8str!=NULL )
		{
		  utf8str[0]=(uns8)(0xE0 | (ucs2char >> 12));
		  utf8str[1]=(uns8)(0x80 | ((ucs2char >>  6) & 0x3F));
			utf8str[2]=(uns8)(0x80 | (ucs2char & 0x3F));
		}
		return 3;
	} 
} 
/*}}}*/
/* static unsigned int sgetu8(unsigned char *strptr, int *chars) {{{
 * function sgetu8() obtained from libutf-8 library
 *
 * Copyright (c) 1999 G. Adam Stanislav
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	sget.utf-8.c v.1
 */
#define	sgetu8_INVALID	0x80000000

#define	get(c)	c = *strptr++; \
	if (chars) (*chars)++; \
	if ((c) == 0) return (unsigned int)EOF

static unsigned int sgetu8(unsigned char *strptr, int *chars) {
	unsigned int c;
	int i, iterations;
	unsigned int ch;

	if (chars) *chars = 0;

	if (strptr == NULL)
		return (unsigned int)EOF;

	get(c);

	if ((c & 0xFE) == 0xFC) {
		c &= 0x01;
		iterations = 5;
	}
	else if ((c & 0xFC) == 0xF8) {
		c &= 0x03;
		iterations = 4;
	}
	else if ((c & 0xF8) == 0xF0) {
		c &= 0x07;
		iterations = 3;
	}
	else if ((c & 0xF0) == 0xE0) {
		c &= 0x0F;
		iterations = 2;
	}
	else if ((c & 0xE0) == 0xC0) {
		c &= 0x1F;
		iterations = 1;
	}
	else if ((c & 0x80) == 0x80)
		return sgetu8_INVALID;
	else return c;

	for (i = 0; i < iterations; i++) {
		get(ch);
		if ((ch & 0xC0) != 0x80)
			return sgetu8_INVALID;
		c <<= 6;
		c |= ch & 0x3F;
	}

	return c;
}
#undef get
/*}}}*/
static int charUTF8toUCS2(uns8 *utf8char,uns16 *ucs2char) /*{{{
* converts UTF-8 char to UCS-2 char
* Parameters: utf8char .. pointer to UTF-8 string buffer which points to start of UTF-8 that have to be converted
*             ucs2char .. pointer to buffer into which function have to write UCS-2 char
*                         or NULL, in that case nothing will be converted but functions returns correct return value
* Return value: 0 on error, number of bytes which were read from UTF-8 string (i.e. length of UTF-8 char in bytes) on success */
{
	int chars_readed;
	unsigned int ucs4char;
	wblog("charUTF8toUCS2 enter, UTF-8 char 0x%lx\n",*utf8char);
	ucs4char=sgetu8((unsigned char*)utf8char,&chars_readed);
	wblog("ucs4char=0x%lx\n",ucs4char);
	if( ucs4char==sgetu8_INVALID )
	{
		char buf[200];
		sprintf(buf,"602SQL: Can't convert UTF-8 string to UCS-2 string, failed on UTF-8 char 0x%x.",*utf8char);
		php_error(E_WARNING,buf);
		wblog("charUTF8toUCS2 leave: ERROR, sgetu8() failed for UTF-8 char 0x%x\n",*utf8char);
		return 0;
	}
	if( ucs4char!=(uns16)ucs4char )
	{
		char buf[200];
		wblog("charUTF8toUCS2 leave: ERROR, non-UCS-2 char 0x%x for UTF-8 char 0x%x\n",ucs4char,*utf8char);
		sprintf(buf,"602SQL: Can't convert UTF-8 string to UCS-2 string because it contains UTF-8 char 0x%x which can't be converted to UCS-2 char (it is UCS-4 char).",*utf8char);
	  php_error(E_WARNING,buf);
		return 0;
	}
	if( ucs2char!=NULL ) *ucs2char=(uns16)ucs4char;
	wblog("charUTF8toUCS2 leave, result is %d\n",chars_readed);
	return chars_readed;
}
/*}}}*/
static char *strUCS2toUTF8(unsigned char *ucs2str,uns32 *string_length) /*{{{
* converts UCS-2 string to UTF-8 string
* Parameters: ucs2str .. pointer to start of UCS-2 string, string have to be ended with '\0' character
* string_length .. length of ucs2str in bytes(!) on input, length of UTF-8 string (i.e. return value) on output
* Return value: emalloc-ed buffer with UTF-8 string (caller have to efree buffer) on success or NULL on error
* Function doesn't trigger any PHP error (with php_error() or zend_error()). */
{
	int utf8str_bytelen=0;
	uns16 *ucs2ptr;
	char *utf8str;
	uns8 *utf8ptr;
	uns32 ucs2strlen;
	wblogs("strUCS2toUTF8 enter\n");
	// compute UTF-8 string byte length
	ucs2ptr=(uns16*)ucs2str;
	ucs2strlen=0;
	while( ucs2strlen<*string_length )
	{
		int tmp=charUCS2toUTF8(*ucs2ptr,NULL);
		if( tmp<=0 )
		{ 
			wblog("strUCS2toUTF8 leave: ERROR - charUCS2toUTF8() failed for 0x%x UCS2 character while computing UTF-8 string byte length, utf8str_bytelen=%d\n",*ucs2ptr,utf8str_bytelen);
			return NULL;
		}
		utf8str_bytelen+=tmp;
		ucs2ptr++;
		ucs2strlen+=2; // UCS-2 char is 2 bytes long
	}
#ifdef _DEBUG
	wblog("UCS-2 string length is %d UCS-2 chars, UTF-8 string length will be %d bytes\n",ucs2ptr-(uns16*)ucs2str,utf8str_bytelen);
#endif
	// allocate buffer for UTF-8 string
	utf8str=(char*)emalloc(utf8str_bytelen+1);
	if( utf8str==NULL )
	{ 
		php_error(E_WARNING, "602SQL: Can not allocate internal structure. Out of memory. ");
		wblog("strUCS2toUTF8 leave: ERROR, out of memory while emalloc-ing utf8str, utf8str_bytelen=%d\n",utf8str_bytelen);
		return NULL;
	}
	// convert UCS-2 string to UTF-8
	ucs2ptr=(uns16*)ucs2str;
	utf8ptr=(uns8*)utf8str;
	ucs2strlen=0;
	while( ucs2strlen<*string_length )
	{
		int tmp=charUCS2toUTF8(*ucs2ptr,utf8ptr);
		if( tmp<=0 )
		{ 
			wblog("strUCS2toUTF8 leave: ERROR - charUCS2toUTF8() failed for 0x%x UCS2 character while converting string\n",*ucs2ptr);
			efree(utf8str);
			return NULL;
		}
		utf8ptr+=tmp;
		ucs2ptr++;
		ucs2strlen+=2; // UCS-2 char is 2 bytes long
	}
	*utf8ptr=0x0; // append EOS
	*string_length=utf8str_bytelen; // set output parameter to UTF-8 string byte length
	wblogmem("strUCS2toUTF8 leave: OK, UTF-8 string is",utf8str,utf8str_bytelen+1);
	return utf8str;
} 
/*}}}*/
static char *strUTF8toUCS2(unsigned char *utf8str,int utf8charlen,int *ucs2bytelenptr) /*{{{
* converts UTF-8 string to UCS-2 string
* Parameters: utf8str .. pointer to UTF-8 string which have to be converted
*             utf8charlen .. number of UTF-8 chars which have to be converted or 0 (which means "convert whole string up to ending '\0' char")
*             ucs2bytelenptr .. pointer to variable into which function have to store length of UCS-2 string in bytes or NULL (in that case length of UCS-2 string will not be stored)
* Return value: emalloc-ed buffer with UCS-2 string (caller have to efree buffer) on success or NULL on error
* Function doesn't trigger any PHP error (with php_error() or zend_error()). */
{
	unsigned char *utf8ptr;
	char *ucs2str;
	uns16 *ucs2ptr;
	int ucs2bytelen,utf8char_counter;
	wblogs("strUTF8toUCS2 enter\n");
	// compute UCS-2 string byte length
	utf8ptr=utf8str;
	ucs2bytelen=0;
	utf8char_counter=0;
	while( !IS_UTF8_EOS(utf8ptr) )
	{
		int tmp=charUTF8toUCS2(utf8ptr,NULL);
		if( tmp==0 )
		{
			wblog("strUTF8toUCS2 leave: ERROR, charUTF8toUCS2() failed for 0x%lx UTF-8 char while computing byte length of UCS-2 string\n",*(uns32*)utf8ptr);
			return NULL;
		}
		ucs2bytelen+=2;
		utf8ptr+=tmp;
		utf8char_counter++;
		if( utf8charlen>0 && utf8char_counter==utf8charlen ) break;
	}
	// allocate buffer for UCS-2 string
	ucs2str=(char*)emalloc(ucs2bytelen+2);
  if( ucs2str==NULL )
	{ 
		php_error(E_WARNING, "602SQL: Can not allocate internal structure. Out of memory. ");
		wblog("strUTF8toUCS2 leave: ERROR, out of memory while emalloc-ing ucs2str, ucs2bytelen=%d\n",ucs2bytelen);
		return NULL;
	}
	// convert
	utf8ptr=utf8str;
	ucs2ptr=(uns16*)ucs2str;
	utf8char_counter=0;
	while( !IS_UTF8_EOS(utf8ptr) )
	{
		int tmp=charUTF8toUCS2(utf8ptr,ucs2ptr);
		if( tmp==0 )
		{
			wblog("strUTF8toUCS2 leave: ERROR, charUTF8toUCS2() failed for 0x%lx UTF-8 char while converting string\n",*(uns32*)utf8ptr);
			efree(ucs2str);
			return NULL;
		}
		ucs2ptr++;
		utf8ptr+=tmp;
		utf8char_counter++;
		if( utf8charlen>0 && utf8char_counter==utf8charlen ) break;
	}
	*ucs2ptr=0; // append EOS
	wblogmem("strUTF8toUCS2 leave: OK, UCS-2 string is",ucs2str,ucs2bytelen+2);
	if( ucs2bytelenptr!=NULL ) *ucs2bytelenptr=ucs2bytelen;
	return ucs2str;
}
/*}}}*/
/*{{{ static char *UTF8phpval2UCS2buf(zval *phpval,size_t *buflen)
 * converts PHP string phpval with UTF-8 encoded text to UCS-2 encoded text
 * returns UCS-2 string in emalloc-allocated buffer on success or NULL on error
 * caller have to efree returned buffer */
static char *UTF8phpval2UCS2buf(zval *phpval,size_t *buflen)
{
	char *out;
	wblog("UTF8phpval2UCS2buf enter, phpval=%p, buflen=%p\n",phpval,buflen);
  out=strUTF8toUCS2((unsigned char*)Z_STRVAL_P(phpval),0,NULL);
	if( out==NULL )
	{
		zend_error(E_WARNING,"602SQL: failed to convert UTF-8 string to UCS-2 string");
		wblogs("UTF8phpval2UCS2buf leave, strUTF8toUCS2() failed\n");
		return NULL;
	}
	wblogs("UTF8phpval2UCS2buf leave: OK\n");
	return out;
}
/*}}}*/
static char *strASCIItoUCS2(char *asciistr,int asciistrlen,int *ucs2bytelenptr) /*{{{
* function converts 7-bit ASCII, 8-bit ASCII or ISO-8859-1 string to UCS-2 string
* Parameters: asciistr .. pointer to ASCII string which have to be converted
*             asciistrlen .. length of ASCII string or 0 (which means "convert whole string up to ending '\0'")
*             ucs2bytelenptr .. pointer to variable into which function have to store length of UCS-2 string in bytes
*                               or NULL, in that case length of UCS-2 string will not be stored
* Return value: emalloc-ed buffer with UCS-2 string (caller have to efree buffer) on success or NULL on error */
{
	char *ret;
	int i;
	wblog("strASCIItoUCS2 enter: asciistr=%s, asciistrlen=%d\n",asciistr,asciistrlen);
	if( asciistrlen==0 ) asciistrlen=strlen(asciistr);
	ret=(char*)emalloc(2*asciistrlen+2);
  if( ret==NULL )
	{ 
		php_error(E_WARNING, "602SQL: Can not allocate internal structure. Out of memory. ");
		wblog("strASCIItoUCS2 leave: ERROR, out of memory while emalloc-ing ucs2str, ucs2bytelen=%d\n",2*asciistrlen);
		return NULL;
	}
	for( i=0;i<asciistrlen;i++ )
	{
		ret[2*i]=asciistr[i];
		ret[2*i+1]='\0';
	}
	ret[2*asciistrlen]=ret[2*asciistrlen+1]='\0';
	if( ucs2bytelenptr!=NULL ) *ucs2bytelenptr=2*asciistrlen;
	wblogmem("strASCIItoUCS2 leave: OK, UCS-2 string is",ret,2*asciistrlen+2);
	return ret;
}
/*}}}*/
/* conversion tables - tabXXXtoUCS2 {{{*/
static uns16 tabCP1250toUCS2[256] = {
0x0000, 0x0001, 0x0002, 0x0003, 0x0004, 0x0005, 0x0006, 0x0007, 0x0008, 0x0009, 0x000A, 0x000B, 0x000C, 0x000D, 0x000E, 0x000F,
0x0010, 0x0011, 0x0012, 0x0013, 0x0014, 0x0015, 0x0016, 0x0017, 0x0018, 0x0019, 0x001A, 0x001B, 0x001C, 0x001D, 0x001E, 0x001F,
0x0020, 0x0021, 0x0022, 0x0023, 0x0024, 0x0025, 0x0026, 0x0027, 0x0028, 0x0029, 0x002A, 0x002B, 0x002C, 0x002D, 0x002E, 0x002F,
0x0030, 0x0031, 0x0032, 0x0033, 0x0034, 0x0035, 0x0036, 0x0037, 0x0038, 0x0039, 0x003A, 0x003B, 0x003C, 0x003D, 0x003E, 0x003F,
0x0040, 0x0041, 0x0042, 0x0043, 0x0044, 0x0045, 0x0046, 0x0047, 0x0048, 0x0049, 0x004A, 0x004B, 0x004C, 0x004D, 0x004E, 0x004F,
0x0050, 0x0051, 0x0052, 0x0053, 0x0054, 0x0055, 0x0056, 0x0057, 0x0058, 0x0059, 0x005A, 0x005B, 0x005C, 0x005D, 0x005E, 0x005F,
0x0060, 0x0061, 0x0062, 0x0063, 0x0064, 0x0065, 0x0066, 0x0067, 0x0068, 0x0069, 0x006A, 0x006B, 0x006C, 0x006D, 0x006E, 0x006F,
0x0070, 0x0071, 0x0072, 0x0073, 0x0074, 0x0075, 0x0076, 0x0077, 0x0078, 0x0079, 0x007A, 0x007B, 0x007C, 0x007D, 0x007E, 0x007F,
0x0080, 0x0081, 0x201A, 0x0083, 0x201E, 0x2026, 0x2020, 0x2021, 0x0088, 0x2030, 0x0160, 0x2039, 0x015A, 0x0164, 0x017D, 0x0179,
0x0090, 0x2018, 0x2019, 0x201C, 0x201D, 0x2022, 0x2013, 0x2014, 0x0098, 0x2122, 0x0161, 0x203A, 0x015B, 0x0165, 0x017E, 0x017A,
0x00A0, 0x02C7, 0x02D8, 0x0141, 0x00A4, 0x0104, 0x00A6, 0x00A7, 0x00A8, 0x00A9, 0x015E, 0x00AB, 0x00AC, 0x00AD, 0x00AE, 0x017B,
0x00B0, 0x00B1, 0x02DB, 0x0142, 0x00B4, 0x00B5, 0x00B6, 0x00B7, 0x00B8, 0x0105, 0x015F, 0x00BB, 0x013D, 0x02DD, 0x013E, 0x017C,
0x0154, 0x00C1, 0x00C2, 0x0102, 0x00C4, 0x0139, 0x0106, 0x00C7, 0x010C, 0x00C9, 0x0118, 0x00CB, 0x011A, 0x00CD, 0x00CE, 0x010E,
0x0110, 0x0143, 0x0147, 0x00D3, 0x00D4, 0x0150, 0x00D6, 0x00D7, 0x0158, 0x016E, 0x00DA, 0x0170, 0x00DC, 0x00DD, 0x0162, 0x00DF,
0x0155, 0x00E1, 0x00E2, 0x0103, 0x00E4, 0x013A, 0x0107, 0x00E7, 0x010D, 0x00E9, 0x0119, 0x00EB, 0x011B, 0x00ED, 0x00EE, 0x010F,
0x0111, 0x0144, 0x0148, 0x00F3, 0x00F4, 0x0151, 0x00F6, 0x00F7, 0x0159, 0x016F, 0x00FA, 0x0171, 0x00FC, 0x00FD, 0x0163, 0x02D9
};
static uns16 tabISO8859_2toUCS2[256] = {
0x0000, 0x0001, 0x0002, 0x0003, 0x0004, 0x0005, 0x0006, 0x0007, 0x0008, 0x0009, 0x000A, 0x000B, 0x000C, 0x000D, 0x000E, 0x000F,
0x0010, 0x0011, 0x0012, 0x0013, 0x0014, 0x0015, 0x0016, 0x0017, 0x0018, 0x0019, 0x001A, 0x001B, 0x001C, 0x001D, 0x001E, 0x001F,
0x0020, 0x0021, 0x0022, 0x0023, 0x0024, 0x0025, 0x0026, 0x0027, 0x0028, 0x0029, 0x002A, 0x002B, 0x002C, 0x002D, 0x002E, 0x002F,
0x0030, 0x0031, 0x0032, 0x0033, 0x0034, 0x0035, 0x0036, 0x0037, 0x0038, 0x0039, 0x003A, 0x003B, 0x003C, 0x003D, 0x003E, 0x003F,
0x0040, 0x0041, 0x0042, 0x0043, 0x0044, 0x0045, 0x0046, 0x0047, 0x0048, 0x0049, 0x004A, 0x004B, 0x004C, 0x004D, 0x004E, 0x004F,
0x0050, 0x0051, 0x0052, 0x0053, 0x0054, 0x0055, 0x0056, 0x0057, 0x0058, 0x0059, 0x005A, 0x005B, 0x005C, 0x005D, 0x005E, 0x005F,
0x0060, 0x0061, 0x0062, 0x0063, 0x0064, 0x0065, 0x0066, 0x0067, 0x0068, 0x0069, 0x006A, 0x006B, 0x006C, 0x006D, 0x006E, 0x006F,
0x0070, 0x0071, 0x0072, 0x0073, 0x0074, 0x0075, 0x0076, 0x0077, 0x0078, 0x0079, 0x007A, 0x007B, 0x007C, 0x007D, 0x007E, 0x007F,
0x0080, 0x0081, 0x0082, 0x0083, 0x0084, 0x0085, 0x0086, 0x0087, 0x0088, 0x0089, 0x008A, 0x008B, 0x008C, 0x008D, 0x008E, 0x008F,
0x0090, 0x0091, 0x0092, 0x0093, 0x0094, 0x0095, 0x0096, 0x0097, 0x0098, 0x0099, 0x009A, 0x009B, 0x009C, 0x009D, 0x009E, 0x009F,
0x00A0, 0x0104, 0x02D8, 0x0141, 0x00A4, 0x013D, 0x015A, 0x00A7, 0x00A8, 0x0160, 0x015E, 0x0164, 0x0179, 0x00AD, 0x017D, 0x017B,
0x00B0, 0x0105, 0x02DB, 0x0142, 0x00B4, 0x013E, 0x015B, 0x02C7, 0x00B8, 0x0161, 0x015F, 0x0165, 0x017A, 0x02DD, 0x017E, 0x017C,
0x0154, 0x00C1, 0x00C2, 0x0102, 0x00C4, 0x0139, 0x0106, 0x00C7, 0x010C, 0x00C9, 0x0118, 0x00CB, 0x011A, 0x00CD, 0x00CE, 0x010E,
0x0110, 0x0143, 0x0147, 0x00D3, 0x00D4, 0x0150, 0x00D6, 0x00D7, 0x0158, 0x016E, 0x00DA, 0x0170, 0x00DC, 0x00DD, 0x0162, 0x00DF,
0x0155, 0x00E1, 0x00E2, 0x0103, 0x00E4, 0x013A, 0x0107, 0x00E7, 0x010D, 0x00E9, 0x0119, 0x00EB, 0x011B, 0x00ED, 0x00EE, 0x010F,
0x0111, 0x0144, 0x0148, 0x00F3, 0x00F4, 0x0151, 0x00F6, 0x00F7, 0x0159, 0x016F, 0x00FA, 0x0171, 0x00FC, 0x00FD, 0x0163, 0x02D9
};
static uns16 tabCP1252toUCS2[256] = {
0x0000, 0x0001, 0x0002, 0x0003, 0x0004, 0x0005, 0x0006, 0x0007, 0x0008, 0x0009, 0x000A, 0x000B, 0x000C, 0x000D, 0x000E, 0x000F,
0x0010, 0x0011, 0x0012, 0x0013, 0x0014, 0x0015, 0x0016, 0x0017, 0x0018, 0x0019, 0x001A, 0x001B, 0x001C, 0x001D, 0x001E, 0x001F,
0x0020, 0x0021, 0x0022, 0x0023, 0x0024, 0x0025, 0x0026, 0x0027, 0x0028, 0x0029, 0x002A, 0x002B, 0x002C, 0x002D, 0x002E, 0x002F,
0x0030, 0x0031, 0x0032, 0x0033, 0x0034, 0x0035, 0x0036, 0x0037, 0x0038, 0x0039, 0x003A, 0x003B, 0x003C, 0x003D, 0x003E, 0x003F,
0x0040, 0x0041, 0x0042, 0x0043, 0x0044, 0x0045, 0x0046, 0x0047, 0x0048, 0x0049, 0x004A, 0x004B, 0x004C, 0x004D, 0x004E, 0x004F,
0x0050, 0x0051, 0x0052, 0x0053, 0x0054, 0x0055, 0x0056, 0x0057, 0x0058, 0x0059, 0x005A, 0x005B, 0x005C, 0x005D, 0x005E, 0x005F,
0x0060, 0x0061, 0x0062, 0x0063, 0x0064, 0x0065, 0x0066, 0x0067, 0x0068, 0x0069, 0x006A, 0x006B, 0x006C, 0x006D, 0x006E, 0x006F,
0x0070, 0x0071, 0x0072, 0x0073, 0x0074, 0x0075, 0x0076, 0x0077, 0x0078, 0x0079, 0x007A, 0x007B, 0x007C, 0x007D, 0x007E, 0x007F,
0x0080, 0x0081, 0x201A, 0x0192, 0x201E, 0x2026, 0x2020, 0x2021, 0x02C6, 0x2030, 0x0160, 0x2039, 0x0152, 0x008D, 0x008E, 0x008F,
0x0090, 0x2018, 0x2019, 0x201C, 0x201D, 0x2022, 0x2013, 0x2014, 0x02DC, 0x2122, 0x0161, 0x203A, 0x0153, 0x009D, 0x009E, 0x0178,
0x00A0, 0x00A1, 0x00A2, 0x00A3, 0x00A4, 0x00A5, 0x00A6, 0x00A7, 0x00A8, 0x00A9, 0x00AA, 0x00AB, 0x00AC, 0x00AD, 0x00AE, 0x00AF,
0x00B0, 0x00B1, 0x00B2, 0x00B3, 0x00B4, 0x00B5, 0x00B6, 0x00B7, 0x00B8, 0x00B9, 0x00BA, 0x00BB, 0x00BC, 0x00BD, 0x00BE, 0x00BF,
0x00C0, 0x00C1, 0x00C2, 0x00C3, 0x00C4, 0x00C5, 0x00C6, 0x00C7, 0x00C8, 0x00C9, 0x00CA, 0x00CB, 0x00CC, 0x00CD, 0x00CE, 0x00CF,
0x00D0, 0x00D1, 0x00D2, 0x00D3, 0x00D4, 0x00D5, 0x00D6, 0x00D7, 0x00D8, 0x00D9, 0x00DA, 0x00DB, 0x00DC, 0x00DD, 0x00DE, 0x00DF,
0x00E0, 0x00E1, 0x00E2, 0x00E3, 0x00E4, 0x00E5, 0x00E6, 0x00E7, 0x00E8, 0x00E9, 0x00EA, 0x00EB, 0x00EC, 0x00ED, 0x00EE, 0x00EF,
0x00F0, 0x00F1, 0x00F2, 0x00F3, 0x00F4, 0x00F5, 0x00F6, 0x00F7, 0x00F8, 0x00F9, 0x00FA, 0x00FB, 0x00FC, 0x00FD, 0x00FE, 0x00FF
};
/*}}}*/
static uns16 *strTABtoUCS2(uns16 *tab,char *genericstr,int genericstrlen,int *ucs2bytelenptr) /*{{{
* function converts string in generic encoding to UCS-2 string
* Parameters: tab .. conversion table from the encoding of the string to UCS-2
*             genericstr .. pointer to string which have to be converted
*             genericstrlen .. length of string or 0 (which means "convert whole string up to ending '\0'")
*             ucs2bytelenptr .. pointer to variable into which function have to store length of UCS-2 string in bytes
*                               or NULL, in that case length of UCS-2 string will not be stored
* Return value: emalloc-ed buffer with UCS-2 string (caller have to efree buffer) on success or NULL on error */
{
	uns16 *ret;
	int i;
	wblog("strTABtoUCS2 enter: tab=%p, genericstr=%s, genericstrlen=%d\n",tab,genericstr,genericstrlen);
	if( genericstrlen==0 ) genericstrlen=strlen(genericstr);
	ret=(uns16*)emalloc(2*genericstrlen+2);
  if( ret==NULL )
	{ 
		php_error(E_WARNING, "602SQL: Can not allocate internal structure. Out of memory. ");
		wblog("strTABtoUCS2 leave: ERROR, out of memory while emalloc-ing ucs2str, ucs2bytelen=%d\n",2*genericstrlen);
		return NULL;
	}
	for( i=0;i<genericstrlen;i++ )
	{
		wblog("0x%02x -> 0x%04x\n",(unsigned char)((unsigned char)(genericstr[i])),tab[(unsigned char)(genericstr[i])]);
		ret[i]=tab[(unsigned char)(genericstr[i])];
	}
	ret[genericstrlen]=0;
	if( ucs2bytelenptr!=NULL ) *ucs2bytelenptr=2*genericstrlen;
	wblogmem("strTABtoUCS2 leave: OK, UCS-2 string is",(char*)ret,2*genericstrlen+2);
	return ret;
}
/*}}}*/
static uns16 *strCP1250toUCS2(char *cp1250str,int cp1250strlen,int *ucs2bytelenptr) /*{{{
* function converts Windows CP 1250 string to UCS-2 string
* Parameters: cp1250str .. pointer to Win1250 string which have to be converted
*             cp1250strlen .. length of Win1250 string or 0 (which means "convert whole string up to ending '\0'")
*             ucs2bytelenptr .. pointer to variable into which function have to store length of UCS-2 string in bytes
*                               or NULL, in that case length of UCS-2 string will not be stored
* Return value: emalloc-ed buffer with UCS-2 string (caller have to efree buffer) on success or NULL on error */
{
	uns16 *ret;
	wblog("strCP1250toUCS2 enter: cp1250str=%s, cp1250strlen=%d\n",cp1250str,cp1250strlen);
	ret=strTABtoUCS2(tabCP1250toUCS2,cp1250str,cp1250strlen,ucs2bytelenptr);
#ifdef USE_WB_DEBUG
	if( ret==NULL ) wblogs("strCP1250toUCS2 leave: ERROR\n");
	else wblogmem("strCP1250toUCS2 leave: OK, UCS-2 string is",(char*)ret,2*cp1250strlen+2);
#endif
	return ret;
}
/*}}}*/
static uns16 *strISO8859_2toUCS2(char *isostr,int isostrlen,int *ucs2bytelenptr) /*{{{
* function converts ISO-8859-2 string to UCS-2 string
* Parameters: isostr .. pointer to ISO-8859-2 string which have to be converted
*             isostrlen .. length of ISO-8859-2 string or 0 (which means "convert whole string up to ending '\0'")
*             ucs2bytelenptr .. pointer to variable into which function have to store length of UCS-2 string in bytes
*                               or NULL, in that case length of UCS-2 string will not be stored
* Return value: emalloc-ed buffer with UCS-2 string (caller have to efree buffer) on success or NULL on error */
{
	uns16 *ret;
	wblog("strISO8859_2toUCS2 enter: isostr=%s, isostrlen=%d\n",isostr,isostrlen);
	ret=strTABtoUCS2(tabISO8859_2toUCS2,isostr,isostrlen,ucs2bytelenptr);
#ifdef USE_WB_DEBUG
	if( ret==NULL ) wblogs("strISO8859_2toUCS2 leave: ERROR\n");
	else wblogmem("strISO8859_2toUCS2 leave: OK, UCS-2 string is",(char*)ret,2*isostrlen+2);
#endif
	return ret;
}
/*}}}*/
static uns16 *strCP1252toUCS2(char *cp1252str,int cp1252strlen,int *ucs2bytelenptr) /*{{{
* function converts Windows CP 1252 string to UCS-2 string
* Parameters: cp1252str .. pointer to Win1252 string which have to be converted
*             cp1252strlen .. length of Win1252 string or 0 (which means "convert whole string up to ending '\0'")
*             ucs2bytelenptr .. pointer to variable into which function have to store length of UCS-2 string in bytes
*                               or NULL, in that case length of UCS-2 string will not be stored
* Return value: emalloc-ed buffer with UCS-2 string (caller have to efree buffer) on success or NULL on error */
{
	uns16 *ret;
	wblog("strCP1252toUCS2 enter: cp1252str=%s, cp1252strlen=%d\n",cp1252str,cp1252strlen);
	ret=strTABtoUCS2(tabCP1252toUCS2,cp1252str,cp1252strlen,ucs2bytelenptr);
#ifdef USE_WB_DEBUG
	if( ret==NULL ) wblogs("strCP1252toUCS2 leave: ERROR\n");
	else wblogmem("strCP1252toUCS2 leave: OK, UCS-2 string is",(char*)ret,2*cp1252strlen+2);
#endif
	return ret;
}
/*}}}*/
static char *getnthUTF8charptr(char *utf8str,int n) /*{{{*/
/* Computes and returns pointer to n-th UTF-8 char in utf8str UTF-8 string.
 * Returns NULL when utf8str string doesn't have n UTF-8 chars.
 * First UTF-8 char has index 0, second has index 1 etc. */
{
	if( utf8str==NULL || n<0 ) return NULL;
	if( n==0 ) return utf8str;
	for( ;n>0;n-- )
	{
		int c=*utf8str;
		int utf8charlen_expected,i;
		if( (c&0x80)==0 )
		{ // one byte UTF-8 char
			utf8charlen_expected=0;
		}
		else if( (c&0xe0)==0xc0 )
		{ // two bytes UTF-8 char
			utf8charlen_expected=2;
		}
		else if( (c&0xf0)==0xe0 )
		{ // three bytes UTF-8 char
			utf8charlen_expected=3;
		}
		else if( (c&0xf8)==0xf0 )
		{ // four bytes UTF-8 char
			utf8charlen_expected=4;
		}
		else if( (c&0xfc)==0xf8 )
		{
			utf8charlen_expected=5;
		}
		else if( (c&0xfe)==0xfc )
		{
			utf8charlen_expected=6;
		}
		else return NULL; // invalid first byte of UTF-8 char
		utf8str++; // skip first byte
		for( i=0;i<utf8charlen_expected;i++ )
		{
			c=*utf8str;
			// test if byte is 10xxxxxx and for second byte if at least one 'x' bit is nonzero
			if( (c&0xc0)==0x80 && (c&0x3f)!=0 ) return NULL;
			utf8str++; // skip that byte
		}
	}
	return utf8str;
}
/*}}}*/
/*}}}*/
#ifdef USE_WB_SCRIPT_ENCODING
/* estrdup_with_encoding {{{*/
static char *estrdup_with_encoding(const char *source,wb_encoding source_encoding,wb_encoding dest_encoding)
{
	size_t source_len;
  int cnv_result;
	char *result=NULL;
	attrspecif_t sspecif,dspecif;
	wblog("estrdup_with_encoding() enter: source=\"%s\",source_encoding=%d,dest_encoding=%d\n",source,source_encoding,dest_encoding);
	if( source==NULL ) { wblogs("estrdup_with_encoding() leave: result is NULL (source is NULL)\n"); return NULL; }
	if( !IS_VALID_ENCODING(source_encoding) )
	{
		php_error(E_WARNING,"602SQL: Invalid source encoding specified while converting a string (in estrdup_with_encoding() function).");
		wblog("estrdup_with_encoding() leave: ERROR, invalid source_encoding %d\n",source_encoding);
		return NULL;
	}
	if( !IS_VALID_ENCODING(dest_encoding) )
	{
		php_error(E_WARNING,"602SQL: Invalid destination encoding specified while converting a string (in estrdup_with_encoding() function).");
		wblog("estrdup_with_encoding() leave: ERROR, invalid dest_encoding %d\n",dest_encoding);
		return NULL;
	}
	if( source_encoding==dest_encoding // the same encoding
			|| source_encoding==WB_ENC_UNKNOWN || dest_encoding==WB_ENC_UNKNOWN // source or destination encoding isn't set
			|| (source_encoding==WB_ENC_ASCII && IS_8BIT_ENCODING(dest_encoding)) // conversion from ASCII to any 8-bit encoding
			|| (IS_8BIT_ENCODING(source_encoding) && dest_encoding==WB_ENC_ASCII) // conversion from any 8-bit encoding to ASCII
			|| (source_encoding==WB_ENC_ASCII && dest_encoding==WB_ENC_UTF8) // conversion from ASCII to UTF-8
		)
	{
		char *ret=estrdup(source);
		if( ret==NULL )
		{
			report_out_of_memory_error("Can't estrdup() string in estrdup_with_encoding() function.");
		}
		wblog("estrdup_with_encoding() leave: result is \"%s\"\n",ret);
		return ret;
	}
	source_len=strlen(source);
	if( source_encoding==WB_ENC_UTF8 || (IS_8BIT_ENCODING(source_encoding) && IS_8BIT_ENCODING(dest_encoding)) )
	{
		result=(char*)emalloc(sizeof(char)*source_len+1);
	}
	else if( dest_encoding==WB_ENC_UTF8 )
	{
		result=(char*)emalloc(sizeof(char)*source_len*4+1);
	}
	else
	{
		wblog("empty else when allocating result: source_encoding=%d,dest_encoding=%d,source_len=%d\n",source_encoding,dest_encoding,source_len);
	}
	if( result==NULL )
	{
		report_out_of_memory_error("Can't allocate result buffer in estrdup_with_encoding().");
		wblogs("estrdup_with_encoding() leave: ERROR - result==NULL\n");
		return NULL;
	}
	sspecif.length=(uns16)source_len; sspecif.charset=wb_encoding2sql602_charset(source_encoding); sspecif.ignore_case=0; sspecif.wide_char=0;
	dspecif.length=(uns16)source_len; dspecif.charset=wb_encoding2sql602_charset(dest_encoding); dspecif.ignore_case=0; dspecif.wide_char=0;
	if( (cnv_result=superconv(ATT_STRING,ATT_STRING,source,result,sspecif,dspecif,NULL))<0 )
	{
		php_error(E_WARNING,"602SQL: Can't convert string from %s encoding to %s encoding (in estrdup_with_encoding() function).",wb_encoding_name[source_encoding],wb_encoding_name[dest_encoding]);
		wblog("estrdup_with_encoding ERROR: source_encoding=%s (%d), dest_encoding=%s (%d), superconv result=%d\n",wb_encoding_name[source_encoding],source_encoding,wb_encoding_name[dest_encoding],dest_encoding,cnv_result);
		efree(result);
		wblogs("estrdup_with_encoding() leave: ERROR, can't convert\n");
		return NULL;
	}
	wblog("estrdup_with_encoding() leave: result is \"%s\"\n",result);
	return result;
}
/*}}}*/
/* estrndup_with_encoding {{{*/
static char *estrndup_with_encoding(const char *source,uns32 source_length,wb_encoding source_encoding,wb_encoding dest_encoding)
{
	size_t source_len;
  int cnv_result;
	char *result=NULL;
	attrspecif_t sspecif,dspecif;
	wblog("estrndup_with_encoding() enter: source=\"%s\",source_length=%lu,source_encoding=%d,dest_encoding=%d\n",source,source_length,source_encoding,dest_encoding);
	if( source==NULL ) { wblogs("estrndup_with_encoding() leave: result is NULL (source is NULL)\n"); return NULL; }
	if( !IS_VALID_ENCODING(source_encoding) )
	{
		php_error(E_WARNING,"602SQL: Invalid source encoding specified while converting a string (in estrndup_with_encoding() function).");
		wblog("estrndup_with_encoding() leave: ERROR, invalid source_encoding %d\n",source_encoding);
		return NULL;
	}
	if( !IS_VALID_ENCODING(dest_encoding) )
	{
		php_error(E_WARNING,"602SQL: Invalid destination encoding specified while converting a string (in estrndup_with_encoding() function).");
		wblog("estrndup_with_encoding() leave: ERROR, invalid dest_encoding %d\n",dest_encoding);
		return NULL;
	}
	if( source_encoding==dest_encoding // the same encoding
			|| source_encoding==WB_ENC_UNKNOWN || dest_encoding==WB_ENC_UNKNOWN // source or destination encoding isn't set
			|| (source_encoding==WB_ENC_ASCII && IS_8BIT_ENCODING(dest_encoding)) // conversion from ASCII to any 8-bit encoding
			|| (IS_8BIT_ENCODING(source_encoding) && dest_encoding==WB_ENC_ASCII) // conversion from any 8-bit encoding to ASCII
			|| (source_encoding==WB_ENC_ASCII && dest_encoding==WB_ENC_UTF8) // conversion from ASCII to UTF-8
		)
	{
		char *ret=emalloc(source_length+1);
		if( ret==NULL )
		{
			report_out_of_memory_error("Can't allocate result buffer in estrndup_with_encoding().");
			wblogs("estrndup_with_encoding() leave: ERROR - result==NULL\n");
			return NULL;
		}
		memcpy(ret,source,source_length);
		ret[source_length]='\0';
		wblog("estrndup_with_encoding() leave: result is \"%s\"\n",ret);
		return ret;
	}
	source_len=(size_t)source_length;
	if( source_encoding==WB_ENC_UTF8 || (IS_8BIT_ENCODING(source_encoding) && IS_8BIT_ENCODING(dest_encoding)) )
	{
		result=(char*)emalloc(sizeof(char)*source_len+1);
	}
	else if( dest_encoding==WB_ENC_UTF8 )
	{
		result=(char*)emalloc(sizeof(char)*source_len*4+1);
	}
	else
	{
		wblog("empty else when allocating result: source_encoding=%d,dest_encoding=%d,source_len=%d\n",source_encoding,dest_encoding,source_len);
	}
	if( result==NULL )
	{
		report_out_of_memory_error("Can't allocate result buffer in estrndup_with_encoding().");
		wblogs("estrndup_with_encoding() leave: ERROR - result==NULL\n");
		return NULL;
	}
	sspecif.length=(uns16)source_len; sspecif.charset=wb_encoding2sql602_charset(source_encoding); sspecif.ignore_case=0; sspecif.wide_char=0;
	dspecif.length=(uns16)source_len; dspecif.charset=wb_encoding2sql602_charset(dest_encoding); dspecif.ignore_case=0; dspecif.wide_char=0;
	if( (cnv_result=superconv(ATT_STRING,ATT_STRING,source,result,sspecif,dspecif,NULL))<0 )
	{
		php_error(E_WARNING,"602SQL: Can't convert string from %s encoding to %s encoding (in estrndup_with_encoding() function).",wb_encoding_name[source_encoding],wb_encoding_name[dest_encoding]);
		wblog("estrndup_with_encoding ERROR: source_encoding=%s (%d), dest_encoding=%s (%d), superconv result=%d\n",wb_encoding_name[source_encoding],source_encoding,wb_encoding_name[dest_encoding],dest_encoding,cnv_result);
		efree(result);
		wblogs("estrndup_with_encoding() leave: ERROR, can't convert\n");
		return NULL;
	}
	wblog("estrndup_with_encoding() leave: result is \"%s\"\n",result);
	return result;
}
/*}}}*/
#define CONN_ESTRDUP_SCRIPT2SYSTEM(script_encoding,conn,source) estrdup_with_encoding(source,script_encoding,(conn)->system_encoding)
static int prepare_string_script2system(wb_encoding script_encoding,wb_conn *conn,char **tmp_string_p,char **emalloc_string_p,char *source_string) /*{{{*/
{
	wblog("prepare_string_script2system() enter: script_encoding=%d,source_string=\"%s\"\n",(int)script_encoding,source_string);
	if( source_string!=NULL && IS_STRING_SCRIPT2SYSTEM_REQUIRED(script_encoding,conn) )
	{
		if( (*tmp_string_p=*emalloc_string_p=CONN_ESTRDUP_SCRIPT2SYSTEM(script_encoding,conn,source_string))==NULL )
		{
			wblogs("prepare_string_script2system() leave: ERROR, CONN_ESTRDUP_SCRIPT2SYSTEM() failed\n");
			return 0;
		}
	}
	else
	{
		*emalloc_string_p=NULL;
		*tmp_string_p=source_string;
	}
	wblog("prepare_string_script2system() leave: OK, *emalloc_string_p=%p, *tmp_string_p=\"%s\"\n",*emalloc_string_p,*tmp_string_p);
	return 1;
} /*}}}*/
#define PREPARE_STRING_SCRIPT2SYSTEM(script_encoding,conn,tmp_string_p,emalloc_string_p,source_string) prepare_string_script2system(script_encoding,conn,tmp_string_p,emalloc_string_p,source_string)
#define EFREE_EMALLOC_STRING(emalloc_string) if( (emalloc_string)!=NULL ) { efree(emalloc_string); emalloc_string=NULL; }
#else //  !USE_WB_SCRIPT_ENCODING
#define PREPARE_STRING_SCRIPT2SYSTEM(script_encoding,conn,tmp_string_p,emalloc_string_p,source_string) ( ( *(tmp_string_p)=(source_string) ), 1 )
#define EFREE_EMALLOC_STRING(emalloc_string)
#endif
#if WBVERS>=900
/*{{{ static sig64 mypower(const sig64 number,sig32 power) */
static sig64 mypower(const sig64 number,sig32 power)
{
	sig64 result=1;
	if( power<0 ) power=-power;
	while( power>0 ) { result*=number; power--; }
	return result;
}
/*}}}*/
#endif
static double myround(double number)
{
	double decimal_fraction=number-(long)number;
	if( decimal_fraction==0 ) return (long)number;
	if( (decimal_fraction>0 && decimal_fraction<=0.5) ) return floor(number);
	return ceil(number);
}
/*{{{ internal_phpvalue2wbvalue() 
 * converts PHP variable's value to 602SQL value
 * 602SQL type into which we want convert PHP value is described by wbvalue_type and wbvalue_specif parameters
 * returns 0 on error and reports error to PHP core
 *      or 1 on success */
static int internal_phpvalue2wbvalue(
#if WBVERS>=800
		wb_encoding client_encoding, // client encoding
#endif
		uns8 wbvalue_type, // 602SQL type of output value
		attrspecif_t wbvalue_specif, // more accurate informatins about 602SQL type of output value
		zval **phpvar, // PHP variable which contains value of parameter
		uns32 phpval_offset, // optional offset of value to send in PHP variable's value or 0
		uns32 phpval_size, // optional size of value to send or 0
		void **pvalue, // outparam: pointer to value which has to be sent to SQL server
		uns32 *pvalue_offset, // outparam: offset of value to send in pvalue buffer
		uns32 *pvalue_size, // outparam: size of value to send
		int *is_emalloced // outparam: if is 0 then pvalue points inside PHP variable's value - i.e. caller mustn't efree pvalue buffer
		                  //           if is 1 then pvalue is emalloc-ed buffer and caller have to efree it
	)
{
#define phpvar2wbvalue_return_if_NULL(pvalue) \
			if( (pvalue)==NULL ) { \
				report_out_of_memory_error("Can't allocate temporary data structure."); \
				wblogs("internal_phpvalue2wbvalue leave, ERROR, out of memory\n"); \
				return 0; \
			}
#define phpvar2wbvalue_alloc_pvalue(size) \
			*pvalue=emalloc(size); \
			phpvar2wbvalue_return_if_NULL(*pvalue); \
			*is_emalloced=1; \
			*pvalue_size=size; \
			*pvalue_offset=0;
	wblog("internal_phpvalue2wbvalue enter, target 602SQL type: %d (specif: 0x%x)\n",wbvalue_type,wbvalue_specif);
	*pvalue=NULL;
	*pvalue_offset=0;
	*pvalue_size=0;
	*is_emalloced=0;
	switch( wbvalue_type ) {
		case ATT_BOOLEAN:
			phpvar2wbvalue_alloc_pvalue(sizeof(wbbool));
			if( Z_TYPE_PP(phpvar)==IS_NULL ) *((wbbool*)*pvalue)=NONEBOOLEAN;
			else {
				convert_to_boolean_ex(phpvar);
				*((wbbool*)*pvalue)=zval_is_true(*phpvar)?wbtrue:wbfalse;
			}
			break;
		case ATT_CHAR:
			if( Z_TYPE_PP(phpvar)==IS_NULL ) {
				phpvar2wbvalue_alloc_pvalue(sizeof(char));
				*((char*)*pvalue)=NONECHAR;
			}
			else {
				convert_to_string_ex(phpvar);
				*pvalue=Z_STRVAL_PP(phpvar);
				*pvalue_offset=0;
				*pvalue_size=sizeof(char);
			}
			break;
		case ATT_INT8:
			phpvar2wbvalue_alloc_pvalue(sizeof(sig8));
			if( Z_TYPE_PP(phpvar)==IS_NULL ) {
				*((sig8*)*pvalue)=NONETINY;
			}
#if WBVERS>=900
			else if( wbvalue_specif.scale!=0 )
			{
				sig64 pow;
				double tmp;
				convert_to_double_ex(phpvar);
				pow=mypower(10,wbvalue_specif.scale);
				tmp=Z_DVAL_PP(phpvar)*pow;
				if( (sig64)tmp!=(sig8)tmp )
				{
					php_error(E_WARNING,"602SQL error: value %f exceeds scaled TINYINT type",Z_DVAL_PP(phpvar));
					wblog("internal_phpvalue2wbvalue leave, false, ERROR: value %f exceeds scaled TINYINT width\n",Z_DVAL_PP(phpvar));
					efree(*pvalue);
					return 0;
				}
				*((sig8*)*pvalue)=(sig8)myround(tmp);
			}
#endif
			else {
				convert_to_long_ex(phpvar);
				if( Z_LVAL_PP(phpvar)!=(sig8)Z_LVAL_PP(phpvar) ) {
					php_error(E_WARNING,"602SQL error: value %ld exceeds TINYINT type",Z_LVAL_PP(phpvar));
					wblog("internal_phpvalue2wbvalue leave, false, ERROR: value %ld exceeds TINYINT width\n",Z_LVAL_PP(phpvar));
					efree(*pvalue);
					return 0;
				}
				*((sig8*)*pvalue)=(sig8)Z_LVAL_PP(phpvar);
			}
			break;
		case ATT_INT16:
			phpvar2wbvalue_alloc_pvalue(sizeof(sig16));
			if( Z_TYPE_PP(phpvar)==IS_NULL ) {
				*((sig16*)*pvalue)=NONESHORT;
			}
#if WBVERS>=900
			else if( wbvalue_specif.scale!=0 )
			{
				sig64 pow;
				double tmp;
				convert_to_double_ex(phpvar);
				pow=mypower(10,wbvalue_specif.scale);
				tmp=Z_DVAL_PP(phpvar)*pow;
				if( (sig64)tmp!=(sig16)tmp )
				{
					php_error(E_WARNING,"602SQL error: value %f exceeds scaled SMALLINT type",Z_DVAL_PP(phpvar));
					wblog("internal_phpvalue2wbvalue leave, false, ERROR: value %f exceeds scaled SMALLINT width\n",Z_DVAL_PP(phpvar));
					efree(*pvalue);
					return 0;
				}
				*((sig16*)*pvalue)=(sig16)myround(tmp);
			}
#endif
			else {
				convert_to_long_ex(phpvar);
				if( Z_LVAL_PP(phpvar)!=(sig16)Z_LVAL_PP(phpvar) ) {
					php_error(E_WARNING,"602SQL error: value %ld exceeds SMALLINT type",Z_LVAL_PP(phpvar));
					wblog("internal_phpvalue2wbvalue leave, false, ERROR: value %ld exceeds SMALLINT width\n",Z_LVAL_PP(phpvar));
					efree(*pvalue);
					return 0;
				}
				*((sig16*)*pvalue)=(sig16)Z_LVAL_PP(phpvar);
			}
			break;
		case ATT_INT32:
			if( Z_TYPE_PP(phpvar)==IS_NULL ) {
				phpvar2wbvalue_alloc_pvalue(sizeof(sig32));
				*((sig32*)*pvalue)=NONEINTEGER;
			}
#if WBVERS>=900
			else if( wbvalue_specif.scale!=0 )
			{
				sig64 pow;
				double tmp;
				phpvar2wbvalue_alloc_pvalue(sizeof(sig32));
				convert_to_double_ex(phpvar);
				pow=mypower(10,wbvalue_specif.scale);
				tmp=Z_DVAL_PP(phpvar)*pow;
				if( (sig64)tmp!=(sig32)tmp )
				{
					php_error(E_WARNING,"602SQL error: value %f exceeds scaled INT type",Z_DVAL_PP(phpvar));
					wblog("internal_phpvalue2wbvalue leave, false, ERROR: value %f exceeds scaled INT width\n",Z_DVAL_PP(phpvar));
					efree(*pvalue);
					return 0;
				}
				*((sig32*)*pvalue)=(sig32)myround(tmp);
			}
#endif
			else {
				convert_to_long_ex(phpvar);
				*pvalue=&Z_LVAL_PP(phpvar);
				*pvalue_offset=0;
				*pvalue_size=sizeof(Z_LVAL_PP(phpvar));
			}
			break;
		case ATT_INT64:
			phpvar2wbvalue_alloc_pvalue(sizeof(sig64));
			if( Z_TYPE_PP(phpvar)==IS_NULL ) {
				*((sig64*)*pvalue)=NONEBIGINT;
			}
#if WBVERS>=900
			else if( wbvalue_specif.scale!=0 )
			{
				sig64 pow;
				double tmp;
				convert_to_double_ex(phpvar);
				pow=mypower(10,wbvalue_specif.scale);
				tmp=Z_DVAL_PP(phpvar)*pow;
				if( (sig64)tmp!=(sig64)tmp )
				{
					php_error(E_WARNING,"602SQL error: value %f exceeds scaled BIGINT type",Z_DVAL_PP(phpvar));
					wblog("internal_phpvalue2wbvalue leave, false, ERROR: value %f exceeds scaled BIGINT width\n",Z_DVAL_PP(phpvar));
					efree(*pvalue);
					return 0;
				}
				*((sig64*)*pvalue)=(sig64)myround(tmp);
			}
#endif
			else {
				convert_to_long_ex(phpvar);
				*((sig64*)*pvalue)=Z_LVAL_PP(phpvar);
			}
			break;
		case ATT_MONEY:
			phpvar2wbvalue_alloc_pvalue(sizeof(monstr));
			if( Z_TYPE_PP(phpvar)==IS_NULL ) {
				((monstr*)*pvalue)->money_lo2=0x80;
				((monstr*)*pvalue)->money_hi4=0;
			}
			else {
				convert_to_double_ex(phpvar);
				if( !real2money(Z_DVAL_PP(phpvar),(monstr*)*pvalue) ) {
					php_error(E_WARNING,"602SQL error: value %f exceeds NUMERIC(14,2) type",Z_DVAL_PP(phpvar));
					wblog("internal_phpvalue2wbvalue leave, false, ERROR value %f exceeds NUMERIC(14,2) type\n",Z_DVAL_PP(phpvar));
					efree(*pvalue);
					return 0;
				}
			}
			break;
		case ATT_FLOAT:
			if( Z_TYPE_PP(phpvar)==IS_NULL ) {
				phpvar2wbvalue_alloc_pvalue(sizeof(double));
				*((double*)*pvalue)=NONEREAL;
			}
			else {
				convert_to_double_ex(phpvar);
				*pvalue=&Z_DVAL_PP(phpvar);
				*pvalue_offset=0;
				*pvalue_size=sizeof(double);
			}
			break;
		case ATT_STRING:
			if( Z_TYPE_PP(phpvar)==IS_NULL ) {
				phpvar2wbvalue_alloc_pvalue(sizeof(char));
				*((char*)*pvalue)='\0';
			}
			else {
				convert_to_string_ex(phpvar);
#if WBVERS>=800
				if( T_SPECIF_CAST(wbvalue_specif).wide_char ) {
					int tmp_size;
					switch( client_encoding ) {
						case WB_ENC_ASCII:
							*pvalue=strASCIItoUCS2(Z_STRVAL_PP(phpvar)+phpval_offset,phpval_size,&tmp_size);
							if( *pvalue==NULL ) return 0;
							*pvalue_size=(uns32)tmp_size;
							*pvalue_offset=0;
							*is_emalloced=1;
							break;
						case WB_ENC_WIN1250:
							*pvalue=(char*)strCP1250toUCS2(Z_STRVAL_PP(phpvar)+phpval_offset,phpval_size,&tmp_size);
							if( *pvalue==NULL ) return 0;
							*pvalue_size=(uns32)tmp_size;
							*pvalue_offset=0;
							*is_emalloced=1;
							break;
						case WB_ENC_ISO_8859_2:
							*pvalue=(char*)strISO8859_2toUCS2(Z_STRVAL_PP(phpvar)+phpval_offset,phpval_size,&tmp_size);
							if( *pvalue==NULL ) return 0;
							*pvalue_size=(uns32)tmp_size;
							*pvalue_offset=0;
							*is_emalloced=1;
							break;
						case WB_ENC_WIN1252:
							*pvalue=(char*)strCP1252toUCS2(Z_STRVAL_PP(phpvar)+phpval_offset,phpval_size,&tmp_size);
							if( *pvalue==NULL ) return 0;
							*pvalue_size=(uns32)tmp_size;
							*pvalue_offset=0;
							*is_emalloced=1;
							break;
						case WB_ENC_UTF8:
							{ 
								char *tmp;
								if( (tmp=getnthUTF8charptr(Z_STRVAL_PP(phpvar),phpval_offset))==NULL )
								{
									php_error(E_WARNING,"602SQL error: invalid UTF-8 string or UTF-8 string isn't long enough");
									wblogs("internal_phpvalue2wbvalue leave, false: ERROR invalid UTF-8 string or UTF-8 string isn't long enough");
									return 0;
								}
								*pvalue=(char*)strUTF8toUCS2(tmp,(int)phpval_size,&tmp_size);
								if( *pvalue==NULL ) return 0;
								*pvalue_size=(uns32)tmp_size;
								*pvalue_offset=0;
								*is_emalloced=1;
							}
							break;
						default:
							php_error(E_WARNING,"602SQL error: unsupported encoding no. %d for assigning strings to UNICODE string parameters",client_encoding);
							wblog("internal_phpvalue2wbvalue leave, false, ERROR unsupported encoding no. %d for assigning strings to UNICODE string parameters\n",client_encoding);
							efree(*pvalue);
							return 0;
					}
				}
				else {
#endif
					*pvalue=Z_STRVAL_PP(phpvar)+phpval_offset;
					*pvalue_offset=0;
					*pvalue_size=(phpval_size>0&&phpval_size<=(uns32)Z_STRLEN_PP(phpvar))?phpval_size:(uns32)Z_STRLEN_PP(phpvar);
#if WBVERS>=800
				}
#endif
			}
			break;
		case ATT_TEXT:
#if WBVERS>=1100
		case ATT_EXTCLOB:
#endif
			if( Z_TYPE_PP(phpvar)==IS_NULL ) {
				phpvar2wbvalue_alloc_pvalue(sizeof(char));
				*((char*)*pvalue)='\0';
			}
			else {
				convert_to_string_ex(phpvar);
#if WBVERS>=800
				if( T_SPECIF_CAST(wbvalue_specif).wide_char ) {
					int tmp_size;
					switch( client_encoding ) {
						case WB_ENC_ASCII:
							*pvalue=strASCIItoUCS2(Z_STRVAL_PP(phpvar)+phpval_offset,phpval_size,&tmp_size);
							if( *pvalue==NULL ) return 0;
							*pvalue_size=(uns32)tmp_size;
							*pvalue_offset=0;
							*is_emalloced=1;
							break;
						case WB_ENC_WIN1250:
							*pvalue=(char*)strCP1250toUCS2(Z_STRVAL_PP(phpvar)+phpval_offset,phpval_size,&tmp_size);
							if( *pvalue==NULL ) return 0;
							*pvalue_size=(uns32)tmp_size;
							*pvalue_offset=0;
							*is_emalloced=1;
							break;
						case WB_ENC_ISO_8859_2:
							*pvalue=(char*)strISO8859_2toUCS2(Z_STRVAL_PP(phpvar)+phpval_offset,phpval_size,&tmp_size);
							if( *pvalue==NULL ) return 0;
							*pvalue_size=(uns32)tmp_size;
							*pvalue_offset=0;
							*is_emalloced=1;
							break;
						case WB_ENC_UTF8:
							{ 
								char *tmp;
								if( (tmp=getnthUTF8charptr(Z_STRVAL_PP(phpvar),phpval_offset))==NULL )
								{
									php_error(E_WARNING,"602SQL error: invalid UTF-8 string or UTF-8 string isn't long enough");
									wblogs("internal_phpvalue2wbvalue leave, false: ERROR invalid UTF-8 string or UTF-8 string isn't long enough");
									return 0;
								}
								*pvalue=(char*)strUTF8toUCS2(tmp,phpval_size,&tmp_size);
								if( *pvalue==NULL ) return 0;
								*pvalue_size=(uns32)tmp_size;
								*pvalue_offset=0;
								*is_emalloced=1;
							}
							break;
						default:
							php_error(E_WARNING,"602SQL error: unsupported encoding no. %d for assigning strings to UNICODE string parameters",client_encoding);
							wblog("internal_phpvalue2wbvalue leave, false, ERROR unsupported encoding no. %d for assigning strings to UNICODE string parameters\n",client_encoding);
							efree(*pvalue);
							return 0;
					}
				}
				else {
#endif
					*pvalue=Z_STRVAL_PP(phpvar)+phpval_offset;
					*pvalue_offset=0;
					*pvalue_size=(phpval_size>0&&phpval_size<=(uns32)Z_STRLEN_PP(phpvar))?phpval_size:(uns32)Z_STRLEN_PP(phpvar); 
#if WBVERS>=800
				}
#endif
			}
			break;
		case ATT_NOSPEC:
#if WBVERS>=1100
		case ATT_EXTBLOB:
#endif
			if( Z_TYPE_PP(phpvar)==IS_NULL ) {
				phpvar2wbvalue_alloc_pvalue(sizeof(char));
				*((char*)*pvalue)='\0';
			}
			else {
				convert_to_string_ex(phpvar);
				*pvalue=Z_STRVAL_PP(phpvar)+phpval_offset;
				*pvalue_offset=0;
				*pvalue_size=(phpval_size>0&&phpval_size<=(uns32)Z_STRLEN_PP(phpvar))?phpval_size:(uns32)Z_STRLEN_PP(phpvar); 
			}
			break;
			// FIXME mozna ATT_BINARY a ATT_RASTER
		case ATT_DATE:
		  phpvar2wbvalue_alloc_pvalue(sizeof(uns32));
			switch( Z_TYPE_PP(phpvar) ) {
				case IS_NULL:
				  *((uns32*)(*pvalue))=NONEDATE;
				  break;
				case IS_LONG:
					{
						struct tm tmp;
					  /* phpvalue contains Unix time (number of seconds since the Unix Epoch) */
						if( php_localtime_r((time_t *)(&Z_LVAL_PP(phpvar)),&tmp)==NULL ) {
							php_error(E_WARNING,"602SQL error: can't convert calendar time %u to struct tm",(uns32)Z_LVAL_PP(phpvar));
							wblog("internal_phpvalue2wbvalue leave, false, ERROR can't convert time_t %u to struct tm\n",(uns32)Z_LVAL_PP(phpvar));
							efree(*pvalue);
							return 0;
						}
						wblog("date: day=%d month=%d year=%d hour=%d min=%d sec=%d\n",tmp.tm_mday,tmp.tm_mon,tmp.tm_year,tmp.tm_hour,tmp.tm_min,tmp.tm_sec);
					  *((uns32*)(*pvalue))=Make_date(tmp.tm_mday,tmp.tm_mon+1,tmp.tm_year+1900);
					}
					break;
				case IS_STRING:
					if( !str2date(Z_STRVAL_PP(phpvar),(uns32*)(*pvalue)) ) {
						php_error(E_WARNING,"602SQL error: can't convert \"%s\" to DATE",Z_STRVAL_PP(phpvar));
						wblog("internal_phpvalue2wbvalue leave, false, ERROR can't convert \"%s\" to DATE\n",Z_STRVAL_PP(phpvar));
						efree(*pvalue);
						return 0;
					}
					break;
				default:
					efree(*pvalue);
					convert_to_string_ex(phpvar);
					php_error(E_WARNING,"602SQL error: can't convert \"%s\" to DATE",Z_STRVAL_PP(phpvar));
  				wblog("internal_phpvalue2wbvalue leave, false, ERROR can't convert \"%s\" to DATE\n",Z_STRVAL_PP(phpvar));
					return 0;
			}
			break;
		case ATT_TIME:
			phpvar2wbvalue_alloc_pvalue(sizeof(uns32));
			if( Z_TYPE_PP(phpvar)==IS_NULL ) {
				*((uns32*)(*pvalue))=NONETIME;
			}
			else {
				convert_to_string_ex(phpvar);
				if( !str2time(Z_STRVAL_PP(phpvar),(uns32*)(*pvalue)) ) {
					efree(*pvalue);
					php_error(E_WARNING,"602SQL error: can't convert \"%s\" to TIME",Z_STRVAL_PP(phpvar));
					wblog("internal_phpvalue2wbvalue leave, false, ERROR can't convert \"%s\" to TIME\n",Z_STRVAL_PP(phpvar));
					return 0;
				}
			}
			break;
		case ATT_TIMESTAMP:
			phpvar2wbvalue_alloc_pvalue(sizeof(uns32));
			wblog("Z_TYPE_PP(phpvar)=%d\n",Z_TYPE_PP(phpvar));
			switch( Z_TYPE_PP(phpvar) ) {
				case IS_NULL:
					*((uns32*)(*pvalue))=NONETIMESTAMP;
					break;
				case IS_LONG:
					{
						struct tm tmp;
					  /* phpvalue contains Unix time (number of seconds since the Unix Epoch) */
						if( php_localtime_r((time_t *)(&Z_LVAL_PP(phpvar)),&tmp)==NULL ) {
							php_error(E_WARNING,"602SQL error: can't convert calendar time %u to struct tm",(uns32)Z_LVAL_PP(phpvar));
							wblog("internal_phpvalue2wbvalue leave, false, ERROR can't convert time_t %u to struct tm\n",(uns32)Z_LVAL_PP(phpvar));
							efree(*pvalue);
							return 0;
						}
					  *((uns32*)(*pvalue))=datetime2timestamp(Make_date(tmp.tm_mday,tmp.tm_mon+1,tmp.tm_year+1900),Make_time(tmp.tm_hour,tmp.tm_min,tmp.tm_sec,0));
					}
					break;
				case IS_STRING:
					wblog("converting %s to TIMESTAMP\n",Z_STRVAL_PP(phpvar));
					if( !str2timestamp(Z_STRVAL_PP(phpvar),(uns32*)(*pvalue)) ) {
						efree(*pvalue);
						php_error(E_WARNING,"602SQL error: can't convert \"%s\" to TIMESTAMP",Z_STRVAL_PP(phpvar));
						wblog("internal_phpvalue2wbvalue leave, false, ERROR can't convert \"%s\" to TIMESTAMP\n",Z_STRVAL_PP(phpvar));
						return 0;
					}
					break;
				default:
					efree(*pvalue);
					convert_to_string_ex(phpvar);
					php_error(E_WARNING,"602SQL error: can't convert \"%s\" to TIMESTAMP",Z_STRVAL_PP(phpvar));
  				wblog("internal_phpvalue2wbvalue leave, false, ERROR can't convert \"%s\" to TIMESTAMP\n",Z_STRVAL_PP(phpvar));
					return 0;
			}
			break;

		default:
			php_error(E_WARNING,"602SQL error: parameter type %d isn't supported",wbvalue_type);
			wblog("internal_phpvalue2wbvalue leave, false, ERROR parameter type %d isn't supported\n",wbvalue_type);
			return 0;
	} // switch( wbvalue_type )
	wblogs("internal_phpvalue2wbvalue leave, true\n");
	return 1;
}
/*}}}*/
/*}}}*/
/*{{{ general functions (connect, exec, result etc.) */
/*{{{ int wb_num_rows(int result_id) */
ZEND_FUNCTION(wb_num_rows) 
{
	WBLSM_DECLARE
	zval **result_id;
	struct sql_answer *answer;
	long result;

	wblogs("wb_num_rows enter\n");

	if (ZEND_NUM_ARGS()!=1||zend_get_parameters_ex(1, &result_id)!=SUCCESS) {
		wblogs("wb_num_rows leave, wrong param count\n");
		WRONG_PARAM_COUNT;
	}

	if( (answer=get_answer_from_result(INTERNAL_FUNCTION_PARAM_PASSTHRU,result_id))==NULL )
	{
		wblogs("wb_num_rows leave, FALSE\n");
		RETURN_FALSE;
	}
	WBLSM_START("wb_num_rows",CDP_FROM_ANSWER_P(answer));
	if( answer->rec_cnt==(trecnum)-1 && cd_Rec_cnt(CDP_FROM_ANSWER_P(answer), answer->cursor, &(answer->rec_cnt)) )
	{
		WBLSM_END;
		zend_error(E_WARNING, "Can not get number of records!");
	}
	result=(answer&&answer->types)?answer->rec_cnt:(unsigned)-1;

	wblog("wb_num_rows leave, result=%ld\n",result);
	WBLSM_END;
	RETURN_LONG(result);
}
/*}}}*/
/*{{{ bool wb_close(int connection_id) */
/* Resource is not an open cursor1Closing cursors and connections.	PHP
 * provides its own memory management; whenever a resource is no longer used
 * (usually, when no variable contains it), its destructor is sooner or later
 * invoked automatically, and in the process, corresponding connection or
 * cursor is also closed.	However, for those who demand control, there is
 * also an explicit function.	It closes the connection or cursor
 * corresponding to passed resource, but does not free the resource itself.
 *
 * It is not an error to close one resource several times. */
ZEND_FUNCTION(wb_close)
{
	WBLSM_DECLARE
	zval **resource_id;
	void *ptr;
	int type;
	
	wblogs("wb_close enter\n");
	if (ZEND_NUM_ARGS()!=1 || zend_get_parameters_ex(1, &resource_id)!=SUCCESS)
		WRONG_PARAM_COUNT;
	
	if (Z_TYPE_PP(resource_id)!=IS_RESOURCE) {
		wblogs("wb_close leave, FALSE - parameter isn't a resource\n");
		zend_error(E_WARNING, "602SQL: Parameter is not a resource");
		RETURN_FALSE;
	}
	
	ptr=zend_list_find(Z_RESVAL_PP(resource_id), &type);
	if( ptr==NULL ) {
		wblogs("wb_close leave, ERROR, zend_list_find() returned NULL\n");
		zend_error(E_WARNING, "602SQL: Resource not found in resource list");
		RETURN_FALSE;
	}
	if( type==wbres_pconn )
	{ /*Attempt to close persistent connection - always return TRUE and do nothing */
		wblogs("wb_close leave, TRUE, persistent connection not closed\n");
		RETURN_TRUE;
	}
	else if (type==wbres_conn)
	{ /*Attemp to disconnect from server; return TRUE on success, FALSE otherwise:*/
		if( cdp_destructor((wb_conn*)ptr,0/*don't force to close connection*/ TSRMLS_CC) )
		{
			wblogs("wb_close leave, TRUE, connection disconnected\n");
			RETURN_TRUE;
		}
		else
		{
			zend_error(E_WARNING, "602SQL: Attempt to close a connection which is being used");
			wblogs("wb_close leave, FALSE, attempt to close a connection which is being used \n");
			/* We have to increase usage counter of connection. cdp_destructor() decreased it as if connection
			 * had been closed. However there exists at least one resource (cursor, LOB or PSQL resource) created
			 * from this connection therefore connection is still opened and we have to keep usage counter
			 * consistent with this situation. */
			((wb_conn*)ptr)->usage++;
			RETURN_FALSE;
		}
	}
	else if (type==wbres_cursor)
	{ /*Close open cursor and return TRUE; if fails, return FALSE:*/
		WBLSM_START("wb_close",CDP_FROM_ANSWER_P((struct sql_answer*)ptr));
		if( cursor_destructor((struct sql_answer*)ptr,0/*don't force to close cursor*/,1/*do call cdp destructor*/ TSRMLS_CC) )
		{ // if cursor was destroyed
			wblogs("wb_close leave, TRUE, cursor closed\n");
			WBLSM_END;
			RETURN_TRUE;
		}
	  else
		{
			zend_error(E_WARNING, "602SQL: Attempt to close a cursor which is being used");
			wblogs("wb_close leave, FALSE, attempt to close a cursor which is being used \n");
			/* We have to increase usage counter of cursor. cursor_destructor() decreased it as if cursor
			 * had been closed. However there exists at least one LOB resource created from this cursor
			 * therefore cursor is still opened and we have to keep usage counter consistent with this
			 * situation. */
			((struct sql_answer*)ptr)->usage++;
			WBLSM_END;
			RETURN_FALSE;
		}
	}
	else if (type==wbres_lob)
	{
		WBLSM_START("wb_close",CDP_FROM_LOB_P((struct lob_info_t*)ptr));
		lob_destructor((struct lob_info_t*)ptr,1/*do call cursor destructor*/ TSRMLS_CC);
		wblogs("wb_close leave, TRUE, LOB closed\n");
		WBLSM_END;
		RETURN_TRUE;
	}
#ifdef USE_PSQL
	else if( type==wbres_psql )
	{
		WBLSM_START("wb_close",CDP_FROM_PSQL_P((struct psql_t*)ptr));
		psql_destructor((struct psql_t*)ptr,1/*do call cdp destructor*/ TSRMLS_CC);
		wblogs("wb_close leave, TRUE, PSQL closed\n");
		WBLSM_END;
		RETURN_TRUE;
	}
#endif
#ifdef USE_XSD_PARSER
	else if( type==wbres_xsdp )
	{
		WBLSM_START("wb_close",CDP_FROM_XSDP_P((struct xsdp_t*)ptr));
		xsdp_destructor((struct xsdp_t*)ptr,1/*do call cdp destructor*/ TSRMLS_CC);
		wblogs("wb_close leave, TRUE, XSDP close\n");
		WBLSM_END;
		RETURN_TRUE;
	}
#endif
#if WBVERS>=1000
	else if( type==wbres_odbc )
	{
		odbc_destructor((struct odbc_t*)ptr,1/*do close ODBC connection*/ TSRMLS_CC);
		wblogs("wb_close leave, TRUE, ODBC close\n");
		RETURN_TRUE;
	}
#endif
	zend_error(E_WARNING, "602SQL: Attempt to close strange resource");
	RETURN_FALSE;
}
/*}}}*/
/*{{{ string wb_err_str(int connection_id) */
/* The PHP function wb_err_str allows access to the error message of
 * 602SQL. It takes a connection as parameter and returns a string with
 * error, or FALSE when the function itself fails. */
ZEND_FUNCTION(wb_err_str)
{
	WBLSM_DECLARE
	zval ** conn_id;
	cdp_t cdp;
	char buffer[256];
	struct wb_conn *conn;

	if (ZEND_NUM_ARGS()!=1 || zend_get_parameters_ex(1, &conn_id)!=SUCCESS)
		WRONG_PARAM_COUNT;

	conn=get_conn_from_resource(INTERNAL_FUNCTION_PARAM_PASSTHRU,conn_id);
	if( conn==NULL ) RETURN_FALSE;
	cdp=CDP(conn);
	if (!cdp) RETURN_FALSE;
	WBLSM_START("wb_err_str",cdp);
	Get_error_num_text(cdp, cd_Sz_error(cdp), buffer, sizeof(buffer));
#ifdef USE_WB_SCRIPT_ENCODING
	if( IS_STRING_CONVERSION_REQUIRED(conn->system_encoding,WBG(script_encoding)) )
	{
		char *emalloc_string=estrdup_with_encoding(buffer,conn->system_encoding,WBG(script_encoding));
		WBLSM_END;
		RETURN_STRING(emalloc_string,FALSE/* return_value takes ownership of emalloc_string buffer */);
	}
	else
	{
		WBLSM_END;
		RETURN_STRING(buffer, TRUE);
	}
#else
	WBLSM_END;
	RETURN_STRING(buffer, TRUE);
#endif
}
/*}}}*/
/*{{{ int wb_err_num(int connection_id) */
/* The PHP function wb_err_num allows access to the error code of
 * 602SQL. It takes a connection as parameter and returns an integer with
 * error, or FALSE when the function itself fails. */
ZEND_FUNCTION(wb_err_num)
{
	WBLSM_DECLARE
	zval ** conn_id;
	cdp_t cdp;
	struct wb_conn *conn;
	int errnum;

	if (ZEND_NUM_ARGS()!=1 || zend_get_parameters_ex(1, &conn_id)!=SUCCESS)
		WRONG_PARAM_COUNT;

	conn=get_conn_from_resource(INTERNAL_FUNCTION_PARAM_PASSTHRU,conn_id);
	if( conn==NULL ) RETURN_FALSE;
	cdp=CDP(conn);
	if (!cdp) RETURN_FALSE;
	WBLSM_START("wb_err_num",cdp);
	errnum=cd_Sz_error(cdp);
	WBLSM_END;
	RETURN_LONG(errnum);
}
/*}}}*/
/*{{{ wb_connect & wb_pconnect */
/*{{{ static char *internal_create_hashed_details(zval **srv,zval **apl,zval **user,zval **pass) */
/* Allocates buffer for hashed details and fills buffer with parameters' values.
 * Returns emalloc-ed hashed details on success, or NULL on error.
 * Caller have to free return value with efree()! */
#define SQL602_HASHED_DETAILS "sql602_%s_%s_%s_%s"
#define SAFE_Z_STRVAL_PP(s) (((s)==NULL)?"":Z_STRVAL_PP(s))
static char *internal_create_hashed_details(zval **srv,zval **apl,zval **user,zval **pass)
{
	int len=0;
	char *ret;
	wblog("internal_create_hashed_details enter, srv=%p, apl=%p, user=%p, pass=%p\n",srv,apl,user,pass);
	if( srv!=NULL ) len+=Z_STRLEN_PP(srv);
	if( apl!=NULL ) len+=Z_STRLEN_PP(apl);
	if( user!=NULL ) len+=Z_STRLEN_PP(user);
	if( pass!=NULL ) len+=Z_STRLEN_PP(pass);
	ret=(char*)emalloc(len+strlen(SQL602_HASHED_DETAILS)+1);
	if( ret==NULL ) return ret;
	sprintf(ret,SQL602_HASHED_DETAILS,SAFE_Z_STRVAL_PP(srv),SAFE_Z_STRVAL_PP(apl),SAFE_Z_STRVAL_PP(user),SAFE_Z_STRVAL_PP(pass));
	wblog("internal_create_hashed_details leave, result is %s\n",ret);
	return ret;
}
/*}}}*/
#define INTERNAL_WB_CONNECT_SET_ERROR_INFO(error_info_wb_connect,error_info_server)	\
		if( error_info!=NULL ) \
		{ \
			add_assoc_long(*error_info,"wb_connect",error_info_wb_connect); \
			add_assoc_long(*error_info,"server",error_info_server); \
		}
/*{{{ static wb_conn *internal_create_connection(INTERNAL_FUNCTION_PARAMETERS, char *srv,char *apl,char *user,char *pass,int persistent,zval **error_info) */
/* Internal function for emalloc-ing and creating fresh new connection to SQL server.
 * Return value is NULL on error, and pointer to malloc-ed or emalloc-ed structure wb_conn on success,
 * in this case caller have to efree returned structure.
 * "persistent" parameter is 1 while to create persistent connection or 0 to create common connection
 * if persistent=1 then structure wb_conn is malloc-ed (and persistent member of this structure is 1),
 * otherwise the structure is emalloc-ed */
static wb_conn *internal_create_connection(INTERNAL_FUNCTION_PARAMETERS, char *srv,char *apl,char *user,char *pass,int persistent,zval **error_info) 
{
	cdp_t cdp;
	wb_conn *conn;
	int res;
#ifdef USE_WB_SCRIPT_ENCODING
	char *emalloc_user=NULL,*emalloc_pass=NULL,*emalloc_apl=NULL;
#endif
	char *tmp_user,*tmp_pass,*tmp_apl;
	wblog("internal_create_connection enter: persistent=%d\n",persistent);
	conn=(wb_conn*) (persistent)?malloc(sizeof(wb_conn)):emalloc(sizeof(wb_conn));
	if (NULL==conn){
		INTERNAL_WB_CONNECT_SET_ERROR_INFO(WB_CONNECT_ERROR_NO_MEMORY,0);
		php_error(E_WARNING, "602SQL: Can not allocate internal structure. ");
		wblogs("internal_create_connection leave, result=NULL, can not allocate internal structure.\n");
		return NULL;
	}
#ifdef USE_STRUCT_NAME
	strcpy(conn->struct_name,"wb_conn");
#endif 
	conn->status=WB_CS_UNINITIALIZED;
#if WBVERS>=1000
	// initialize client error callback values
	conn->client_error_level=0;
	conn->client_error_callback=NULL;
#endif
	cdp=CDP(conn);
	cdp_init(cdp);
	res=cd_connect(cdp, srv, -1);
	if (res){
		INTERNAL_WB_CONNECT_SET_ERROR_INFO(WB_CONNECT_ERROR_CONNECT,res);
		report_err_from_number(conn, res, "connect" TSRMLS_CC);
		(persistent)?free(conn):efree(conn);
		wblog("internal_create_connection leave, result=NULL, can't cd_connect() to SQL server, cd_connect() result is %d\n",res);
		return NULL;
	}
#if WBVERS>=1000
	/* get SQL server version number */
	if( cd_Get_server_info(cdp,OP_GI_VERSION_1,&(conn->server_version[0]),sizeof(conn->server_version[0]))
	 || cd_Get_server_info(cdp,OP_GI_VERSION_2,&(conn->server_version[1]),sizeof(conn->server_version[1]))
	 || cd_Get_server_info(cdp,OP_GI_VERSION_3,&(conn->server_version[2]),sizeof(conn->server_version[2]))
	 || cd_Get_server_info(cdp,OP_GI_VERSION_4,&(conn->server_version[3]),sizeof(conn->server_version[3])) )
	{
		int sqlerrnum=CONN_REPORT_ERR(conn,"Get_server_info (detecting SQL server version)");
		INTERNAL_WB_CONNECT_SET_ERROR_INFO(WB_CONNECT_ERROR_GET_SERVER_VERSION,sqlerrnum);
		cd_disconnect(cdp);
		(persistent)?free(conn):efree(conn);
		wblogs("internal_create_connection leave, result=NULL, cd_Get_server_info() failed\n");
		return NULL;
	}
#endif
	/* system encoding */
#ifdef USE_WB_SCRIPT_ENCODING
	{
		uns8 sysenc;
#if WBVERS>=1000
	  if( IS_SERVER_VERSION_9_5_UP(conn) ) // system charset was introduced in version 9.5
		{
#endif
#if WBVERS>=950 // system charset was introduced in version 9.5
			if( cd_Get_server_info(cdp,OP_GI_SYS_CHARSET,&sysenc,sizeof(sysenc)) )
			{
				int sqlerrnum=CONN_REPORT_ERR(conn,"Get_server_info (detecting system encoding)");
				INTERNAL_WB_CONNECT_SET_ERROR_INFO(WB_CONNECT_ERROR_GET_SYSTEM_CHARSET,sqlerrnum);
				cd_disconnect(cdp);
				(persistent)?free(conn):efree(conn);
				wblogs("internal_create_connection leave, result=NULL, cd_Get_server_info() failed\n");
				return NULL;
			}
#else
			sysenc=1; // system charset of older versions is Windows CP 1250
#endif
#if WBVERS>=1000
		}
		else
		{ // system charset of older versions is Windows CP 1250
			sysenc=1;
		}
#endif
		conn->system_encoding=sql602_charset2wb_encoding(sysenc);
		wblog("system encoding is %s (%d)\n",wb_encoding_name[conn->system_encoding],conn->system_encoding);
	}
#endif
	if( !PREPARE_STRING_SCRIPT2SYSTEM(WBG(script_encoding),conn,&tmp_user,&emalloc_user,user) )
	{
		INTERNAL_WB_CONNECT_SET_ERROR_INFO(WB_CONNECT_ERROR_NO_MEMORY,0);
		cd_disconnect(cdp);
		(persistent)?free(conn):efree(conn);
		wblogs("internal_create_connection leave, result=NULL, PREPARE_STRING_SCRIPT2SYSTEM(user) failed\n");
		return NULL;
	}
	if( !PREPARE_STRING_SCRIPT2SYSTEM(WBG(script_encoding),conn,&tmp_pass,&emalloc_pass,pass) )
	{
		INTERNAL_WB_CONNECT_SET_ERROR_INFO(WB_CONNECT_ERROR_NO_MEMORY,0);
		EFREE_EMALLOC_STRING(emalloc_user);
		cd_disconnect(cdp);
		(persistent)?free(conn):efree(conn);
		wblogs("internal_create_connection leave, result=NULL, PREPARE_STRING_SCRIPT2SYSTEM(pass) failed\n");
		return NULL;
	}
	res=cd_Login(cdp,(user)?tmp_user:"Anonymous",(user&&pass)?tmp_pass:"");
	EFREE_EMALLOC_STRING(emalloc_user);
	EFREE_EMALLOC_STRING(emalloc_pass);
	if (res){
		int sqlerrnum=CONN_REPORT_ERR(conn,"Login");
		INTERNAL_WB_CONNECT_SET_ERROR_INFO(WB_CONNECT_ERROR_LOGIN,sqlerrnum);
		cd_disconnect(cdp);
		(persistent)?free(conn):efree(conn);
		wblogs("internal_create_connection leave, result=NULL, cd_Login() failed\n");
		return NULL;
	}
	if( cd_Set_progress_report_modulus(cdp,0) )
	{
		int sqlerrnum=CONN_REPORT_ERR(conn,"Set_progress_report_modulus");
		INTERNAL_WB_CONNECT_SET_ERROR_INFO(WB_CONNECT_ERROR_SET_PROGRESS_REPORT_MODULUS,sqlerrnum);
		cd_disconnect(cdp);
		(persistent)?free(conn):efree(conn);
		wblogs("internal_create_connection leave, result=NULL, cd_Set_progress_report_modulus() failed\n");
		return NULL;
	}
	if (apl)
	{
		if( !PREPARE_STRING_SCRIPT2SYSTEM(WBG(script_encoding),conn,&tmp_apl,&emalloc_apl,apl) )
		{
			INTERNAL_WB_CONNECT_SET_ERROR_INFO(WB_CONNECT_ERROR_NO_MEMORY,0);
			EFREE_EMALLOC_STRING(emalloc_apl);
			cd_disconnect(cdp);
			(persistent)?free(conn):efree(conn);
			wblogs("internal_create_connection leave, result=NULL, PREPARE_STRING_SCRIPT2SYSTEM(apl) failed\n");
			return NULL;
		}
		res=cd_Set_application_ex(cdp,tmp_apl,2);
		EFREE_EMALLOC_STRING(emalloc_apl);
		if (res){
			int sqlerrnum=CONN_REPORT_ERR(conn,"Set_application");
			INTERNAL_WB_CONNECT_SET_ERROR_INFO(WB_CONNECT_ERROR_SET_APPLICATION,sqlerrnum);
			cd_disconnect(cdp);
			(persistent)?free(conn):efree(conn);
			wblogs("internal_create_connection leave, result=NULL, cd_Set_application_ex() failed\n");
			return NULL;
		}
	}
	/* initialize new connection params */
	conn->usage=1;
	conn->is_persistent=persistent;
	conn->status=WB_CS_CONNECTED_IN_USE;
#if WBVERS>=800
	ENCODING(conn)=WB_ENC_ASCII; // default encoding
#endif
#if WBVERS<900
	conn->selected_language_id=-1; // no language selected
#endif
	wblog("internal_create_connection leave, result=%p\n",conn);
	return conn;
}
/*}}}*/
/*{{{ static void internal_wb_connect(INTERNAL_FUNCTION_PARAMETERS, int persistent) */
/* This function is called by both wb_connect and wb_pconnect functions
 * and performs all the work.
 * Parameter persistent states if we have to make persistent connection or not. */
static void internal_wb_connect(INTERNAL_FUNCTION_PARAMETERS, int persistent)
{
	zval **srv, **apl=NULL, **user=NULL, **pass=NULL, **error_info=NULL;
	wb_conn *conn=NULL;
	char *hashed_details;
	int hashed_details_length;

	wblog("internal_wb_connect enter, perisistent=%d\n",persistent);

	switch(ZEND_NUM_ARGS()){
	case 1: /* Server name only */
					if (zend_get_parameters_ex(1, &srv) != SUCCESS) WRONG_PARAM_COUNT;
					break;
	case 2: /* Server name and application */
					if (zend_get_parameters_ex(2, &srv, &apl) != SUCCESS) WRONG_PARAM_COUNT;
					break;
	case 3: /* server, application, user */
					if (zend_get_parameters_ex(3, &srv, &apl, &user) != SUCCESS) WRONG_PARAM_COUNT;
					break;
	case 4: /* all parameters */
					if (zend_get_parameters_ex(4, &srv, &apl, &user, &pass) != SUCCESS) WRONG_PARAM_COUNT;
					break;
	case 5: /* and error_info too */
					if (zend_get_parameters_ex(5, &srv, &apl, &user, &pass, &error_info) != SUCCESS) WRONG_PARAM_COUNT;
					break;
	default:
					WRONG_PARAM_COUNT;
	}

	convert_to_string_ex(srv);
	if (apl) convert_to_string_ex(apl);
	if (user) convert_to_string_ex(user);
	if (pass) convert_to_string_ex(pass);
	if( error_info!=NULL && Z_TYPE_PP(error_info)!=IS_ARRAY && array_init(*error_info)!=SUCCESS )
	{
		zend_error(E_WARNING, "602SQL: Can not create error_info array");
		wblogs("internal_wb_connect leave, FALSE, array_init() failed\n");
		RETURN_FALSE;
	}
	/* we can use	INTERNAL_WB_CONNECT_SET_ERROR_INFO macro now */

	wblog("internal_wb_connect: srv=\"%s\", apl=\"%s\", user=\"%s\", pass=\"%s\"\n",Z_STRVAL_PP(srv),(apl)?Z_STRVAL_PP(apl):"",(user)?Z_STRVAL_PP(user):"",(pass)?Z_STRVAL_PP(pass):"");

	hashed_details=internal_create_hashed_details(srv,apl,user,pass);
	if( hashed_details==NULL ) {
		INTERNAL_WB_CONNECT_SET_ERROR_INFO(WB_CONNECT_ERROR_NO_MEMORY,0);
		php_error(E_WARNING,"602SQL: Out of memory");
		RETURN_FALSE;
	}
	hashed_details_length=strlen(hashed_details);

	if( persistent )
	{
		/* Provide perisistent database connection: */
		zend_rsrc_list_entry *le;
		int open_new_connection = 1;
		
		if (zend_hash_find(&EG(persistent_list), hashed_details, hashed_details_length+1, (void **)&le) != FAILURE) {
			uns32 tmp_sqloption;
			wblog("internal_wb_connect: found list entry (resource) %p\n",le);
			if (Z_TYPE_P(le) != wbres_pconn) {
				INTERNAL_WB_CONNECT_SET_ERROR_INFO(WB_CONNECT_ERROR_PERSISTENT_RESOURCE_NOT_FOUND,0);
				efree(hashed_details);
				wblog("internal_wb_connect leave, resource %p isn't wbres_pconn type\n",le);
				RETURN_FALSE;
			}
			/* Check if connection has timed out */
			conn = (wb_conn *) le->ptr;
			wblog("internal_wb_connect: conn (le->ptr) is %p\n",conn);
			if( conn->status==WB_CS_CONNECTED_UNUSED )
			{
				wblog("internal_wb_connect: conn %p is unused\n",conn);
				/* Claim this connection for use in this thread */
				cd_assign_to_thread(CDP(conn));
				/* In fact we don't want to read current SQL options
				 * we need to communicate with SQL server to ensure connection validity.
				 * So we perform fast function from 602SQL API which doesn't affect
				 * any acpect of core database functionality (i.e. data change,
				 * transaction change etc.). We chose cd_Get_sql_option() but there are
				 * more such functions in 602SQL API. */
				if ( !cd_Get_sql_option(CDP(conn),&tmp_sqloption) ) {
					/* connection is OK */
					open_new_connection = 0;
					conn->status=WB_CS_CONNECTED_IN_USE;
					wblog("internal_wb_connect: conn %p will be returned by this wb_pconnect() call\n",conn);
				}
				else
				{
					wblog("internal_wb_connect: cd_Get_sql_option() failed on conn %p\n",conn);
				}
			}
		}
		
		/* There was no previous connection to use or it has timed out */
		if (open_new_connection) {
			zend_rsrc_list_entry new_le;
			
			wblogs("internal_wb_connect: opening new connection\n");
			if( WBG(max_conn)>0 && WBG(num_conn)>=WBG(max_conn) )
			{
				INTERNAL_WB_CONNECT_SET_ERROR_INFO(WB_CONNECT_ERROR_TOO_MANY_CONNECTIONS,0);
				php_error(E_WARNING, "602SQL: Too many open connections (%d)", WBG(num_conn));
				efree(hashed_details);
				wblogs("internal_wb_connect leave, too many open connections\n");
				RETURN_FALSE;
			}
			if( WBG(max_persistent)>0 && WBG(num_persistent)>=WBG(max_persistent) )
			{
				INTERNAL_WB_CONNECT_SET_ERROR_INFO(WB_CONNECT_ERROR_TOO_MANY_PERSISTENT_CONNECTIONS,0);
				php_error(E_WARNING, "602SQL: Too many open persistent connections (%d)", WBG(num_persistent));
				efree(hashed_details);
				wblogs("internal_wb_connect leave, too many open persistent connections\n");
				RETURN_FALSE;
			}

			/* create the connection */
			if( (conn=internal_create_connection(INTERNAL_FUNCTION_PARAM_PASSTHRU,(srv)?Z_STRVAL_PP(srv):NULL,(apl)?Z_STRVAL_PP(apl):NULL,(user)?Z_STRVAL_PP(user):NULL,(pass)?Z_STRVAL_PP(pass):NULL,1/*persistent*/,error_info))==NULL )
			{
				efree(hashed_details);
				wblogs("internal_wb_connect leave, result=FALSE internal_create_connection() failed\n");
				RETURN_FALSE;
			}

			/* hash it up */
			Z_TYPE(new_le) = wbres_pconn;
			new_le.ptr = conn;
			if (zend_hash_update(&EG(persistent_list), hashed_details, hashed_details_length+1, (void *) &new_le, sizeof(zend_rsrc_list_entry), NULL)==FAILURE) {
				INTERNAL_WB_CONNECT_SET_ERROR_INFO(WB_CONNECT_ERROR_STORE_CONNECTION_RESOURCE,0);
				efree(hashed_details);
				free(conn); // conn was malloc-ed (see internal_create_connection() comments)
				wblogs("internal_wb_connect leave, result=FALSE zend_hash_update() failed\n");
				RETURN_FALSE;
			}

			WBG(num_conn)++;
			WBG(num_persistent)++;

		}

		ZEND_REGISTER_RESOURCE(return_value, conn, wbres_pconn);
	}
	else
	{
		/* Allocate and initialize plain (nonpersistent) database connection: */
		/* If password is not set, it is supposed to be empty. If
		 * user name is not set, we do not make login (i.e. we will act as
		 * anonymous user). If the application is not set, we work without
		 * any application. There are not many object we can access without
		 * application set, but we can still create new applications and
		 * access system tables. */
		zend_rsrc_list_entry *index_ptr, new_index_ptr;
		
		/* first we check the hash for the hashed_details key.	if it exists,
		 * it should point us to the right offset where the actual ib_link sits.
		 * if it doesn't, open a new ib_link, add it to the resource list,
		 * and add a pointer to it with hashed_details as the key.
		 */
		if (zend_hash_find(&EG(regular_list), hashed_details, hashed_details_length+1, (void **)&index_ptr)==SUCCESS) {
			int type, xlink;
			void *ptr;
			if (Z_TYPE_P(index_ptr) != le_index_ptr) {
				INTERNAL_WB_CONNECT_SET_ERROR_INFO(WB_CONNECT_ERROR_RESOURCE_NOT_FOUND,0);
				efree(hashed_details);
				RETURN_FALSE;
			}
			xlink = (int) index_ptr->ptr;
			ptr = zend_list_find(xlink, &type);	 /* check if the xlink is still there */
			if (ptr && (type==wbres_conn || type==wbres_pconn)) {
				zend_list_addref(xlink);
				Z_LVAL_P(return_value) = xlink;
				Z_TYPE_P(return_value) = IS_RESOURCE;
				efree(hashed_details);
				return;
			} else {
				zend_hash_del(&EG(regular_list), hashed_details, hashed_details_length+1);
			}
		}

		if( WBG(max_conn)>0 && WBG(num_conn)>=WBG(max_conn) )
		{
			INTERNAL_WB_CONNECT_SET_ERROR_INFO(WB_CONNECT_ERROR_TOO_MANY_CONNECTIONS,0);
			php_error(E_WARNING, "602SQL: Too many open connections (%d)", WBG(num_conn));
			efree(hashed_details);
			RETURN_FALSE;
		}

		/* connect to 602SQL server */
		if( (conn=internal_create_connection(INTERNAL_FUNCTION_PARAM_PASSTHRU,(srv)?Z_STRVAL_PP(srv):NULL,(apl)?Z_STRVAL_PP(apl):NULL,(user)?Z_STRVAL_PP(user):NULL,(pass)?Z_STRVAL_PP(pass):NULL,0/*nonpersistent*/,error_info))==NULL )
		{
			efree(hashed_details);
			wblogs("internal_wb_connect leave, result=FALSE\n");
			RETURN_FALSE;
		}
		// conn was emalloc-ed (see internal_create_connection() comments)

		ZEND_REGISTER_RESOURCE(return_value, conn, wbres_conn);

		/* add it to the hash */
		new_index_ptr.ptr = (void *) Z_LVAL_P(return_value);
		Z_TYPE(new_index_ptr) = le_index_ptr;
		if (zend_hash_update(&EG(regular_list), hashed_details, hashed_details_length+1, (void *) &new_index_ptr, sizeof(zend_rsrc_list_entry), NULL)==FAILURE) {
			INTERNAL_WB_CONNECT_SET_ERROR_INFO(WB_CONNECT_ERROR_STORE_CONNECTION_RESOURCE,0);
			efree(hashed_details);
			RETURN_FALSE;
		}
		WBG(num_conn)++;
	}

	wblog("internal_wb_connect leave, result=%ld\n",return_value);
	INTERNAL_WB_CONNECT_SET_ERROR_INFO(WB_CONNECT_ERROR_OK,0);
}
/*}}}*/
/*{{{ mixed wb_connect(string server [,string application [,string user [,string passwd [,array error_info]]]]) */
/* Connecting to the server.	The four parameters are server name, application
 * name, user name and password. Only server name is obligatory. Return value
 * is the resource that can be used, e.g., for wb_exec, or FALSE on
 * error. We provide persistent connections too. */
ZEND_FUNCTION(wb_connect)
{
	internal_wb_connect(INTERNAL_FUNCTION_PARAM_PASSTHRU,0);
}
/*}}}*/
/*{{{ mixed wb_pconnect(string server [,string application [,string user [,string passwd [,array error_info]]]]) */
ZEND_FUNCTION(wb_pconnect)
{
	internal_wb_connect(INTERNAL_FUNCTION_PARAM_PASSTHRU,1);
}
/*}}}*/
/*}}}*/
/*{{{ wb_select_db */
/* Selects another schema (i.e. application in 602SQL terminology).
 * Returns true on success, false on error. */
ZEND_FUNCTION(wb_select_db)
{
	WBLSM_DECLARE
	zval **conn_id,**db_name;
	cdp_t cdp;
	struct wb_conn *conn;

	if (ZEND_NUM_ARGS()!=2 || zend_get_parameters_ex(2, &conn_id, &db_name)!=SUCCESS)
		WRONG_PARAM_COUNT;

	conn=get_conn_from_resource(INTERNAL_FUNCTION_PARAM_PASSTHRU,conn_id);
	if( conn==NULL ) RETURN_FALSE;
	cdp=CDP(conn);
	if (!cdp) RETURN_FALSE;
	WBLSM_START("wb_select_db",cdp);
	convert_to_string_ex(db_name);
	if( cd_Set_application_ex(cdp,Z_STRVAL_PP(db_name),FALSE) )
	{
		WBLSM_END;
		zend_error(E_WARNING,"602SQL: function wb_select_db failed because database application \"%s\" was not found",Z_STRVAL_PP(db_name));
		RETURN_FALSE;
	}
	WBLSM_END;
	RETURN_TRUE;
}
/*}}}*/
/*{{{ db_item_to_php + support functions */
/* Getting data from SQL results.
The data as returned by SQL may be of many types; in PHP, any variable may
contain any type. The translation between those is one of the most boring (and
important) part of the module.*/

/* Most typed are handled in expected way, with some exceptions. Huge integer
 * is treated as integer or as real, if it is too large. Money type is
 * translated to double; other possibilities would be string or integer value
 * after multiplication by 100; the latter would suffer same problems as the
 * huge integer. Date and time are returned as strings in format like
 * dd.mm.yy, and hh:mm:ss.ssss, resp hh:mm:ss, if there are no
 * fractions of seconds, while timestamp is returned as UNIX timestamp,
 * which can be handled and converted easily by PHP. Two of the watch
 * attributes, as well as pointers between tables are omitted completely. The
 * signature is handled as any binary data, so it is almost unusable.

 * We provide a function db_item_to_php that performs the task. It does not
 * handle multiattributes, as well as some extremely rare types it would be
 * difficult to deal with. Function returns FALSE when something failed,
 * TRUE if all went smoothly.
 *
 * In case of LOB values it uses special function db_read_lob_value for LOB
 * values reading and conversion. */
/* sql602date2time_t() and sql602timestamp2time_t() {{{*/
#define SQL602TOTIME_T_ERROR ((time_t)-1)
/* Converts 602SQL DATE value to time_t value with local timezone correction.
 * Returns Unix timestamp of first second of the given date on success
 * or (time_t)-1 on error. */
static time_t sql602date2time_t(uns32 date)
{
	wblog("sql602date2time_t enter: date=%lu\n",date);
	if( date==NONEDATE ) { wblogs("sql602date2time_t leave: error, NONEDATE\n"); return SQL602TOTIME_T_ERROR; }
#if defined PHP_WIN32 && defined USE_WIN32_DATETIME_CONVERSIONS
	{
		SYSTEMTIME st;
		FILETIME lft,ft;
		ULONGLONG ll;
		st.wYear=Year(date);
		st.wMonth=Month(date);
		st.wDay=Day(date);
		st.wHour=0;
		st.wMinute=0;
		st.wSecond=0;
		st.wMilliseconds=0;
		if( !SystemTimeToFileTime(&st,&lft) )
		{
			wblogs("sql602date2time_t() leave: error, SystemTimeToFileTime() failed\n");
			return SQL602TOTIME_T_ERROR;
		}
		if( !LocalFileTimeToFileTime(&lft,&ft) )
		{
			wblogs("sql602date2time_t() leave: error, LocalFileTimeToFileTime() failed\n");
			return SQL602TOTIME_T_ERROR;
		}
		ll=(((((ULONGLONG)ft.dwHighDateTime)<<32)+ft.dwLowDateTime)-(ULONGLONG)116444736000000000)/10000000;
		if( (time_t)ll!=ll )
		{
			char buf[100];
			_ui64toa(ll,buf,10);
			wblog("sql602date2time_t() leave: error, (time_t)ll=%ld != ll=%s\n  ft.dwHighDateTime=%lu ft.dwLowDateTime=%lu\n",(time_t)ll,buf,ft.dwHighDateTime,ft.dwLowDateTime);
			return SQL602TOTIME_T_ERROR;
		}
		return (time_t)ll;
	}
#else
	{
		struct tm timeval;
		int tmp;
		time_t result;
		timeval.tm_sec=0;
		timeval.tm_min=0;
		timeval.tm_hour=0;
		timeval.tm_mday=Day(date);
		tmp=Year(date);
		timeval.tm_year=(tmp>=1900)?tmp-1900:tmp;
		timeval.tm_mon=Month(date)-1; /* 0 = january */
		timeval.tm_isdst=-1;
		wblog("year=%d, month=%d, day=%d\n",timeval.tm_year,timeval.tm_mon,timeval.tm_mday);
		result=mktime(&timeval);
		wblog("sql602date2time_t leave: result is %d\n",result);
		return result;
	}
#endif
} 
/* Converts 602SQL TIMESTAMP value to time_t value with local timezone correction.
 * Returns Unix timestamp of the given timestamp on success
 * or (time_t)-1 on error. */
static time_t sql602timestamp2time_t(uns32 timestamp)
{
	uns32 date, time;
	if( timestamp==NONETIMESTAMP ) { wblogs("sql602timestamp2time_t leave: error, NONETIMESTAMP\n"); return SQL602TOTIME_T_ERROR; }
	date=timestamp2date(timestamp);
	time=timestamp2time(timestamp);
#if defined PHP_WIN32 && defined USE_WIN32_DATETIME_CONVERSIONS
	{
		SYSTEMTIME st;
		FILETIME ft,lft;
		ULONGLONG ll;
		st.wYear=Year(date);
		st.wMonth=Month(date);
		st.wDay=Day(date);
		st.wHour=Hours(time);
		st.wMinute=Minutes(time);
		st.wSecond=Seconds(time);
		st.wMilliseconds=0;
		if( !SystemTimeToFileTime(&st,&lft) ) return SQL602TOTIME_T_ERROR;
		if( !LocalFileTimeToFileTime(&lft,&ft) ) return SQL602TOTIME_T_ERROR;
		ll=(((((ULONGLONG)ft.dwHighDateTime)<<32)+ft.dwLowDateTime)-(ULONGLONG)116444736000000000)/10000000;
		if( (time_t)ll!=ll ) return SQL602TOTIME_T_ERROR;
		return (time_t)ll;
	}
#else
	{
		struct tm timeval;
		int tmp;
		time_t result;
		timeval.tm_sec=Seconds(time);
		timeval.tm_min=Minutes(time);
		timeval.tm_hour=Hours(time);
		timeval.tm_mday=Day(date);
		tmp=Year(date);
		timeval.tm_year=(tmp>=1900)?tmp-1900:tmp;
		timeval.tm_mon=Month(date)-1; /* 0 = january */
		timeval.tm_isdst=-1;
		wblog("year=%d, month=%d, day=%d, hour=%d, minute=%d, second=%d\n",timeval.tm_year,timeval.tm_mon,timeval.tm_mday,timeval.tm_hour,timeval.tm_min,timeval.tm_sec);
		result=mktime(&timeval);
		wblog("sql602timestamp2time_t leave: result is %d\n",result);
		return result;
	}
#endif
}
/*}}}*/
/*{{{ convert_binary_to_hexa
The buff array is supposed to be capable to hold 2*buffsize+1 bytes,
as every byte in buffsize gets converted to two characters, and one cell is
needed for terminating NULL. Conversion starts at the end of buffer. */
static const char hdigit[]="0123456789abcdef";
static void convert_binary_to_hexa(unsigned char *buff, uns32 *buffsize)
{
	wblog("convert_binary_to_hexa enter: buff=0x%p buffsize=%u\n",buff,*buffsize);
	if( *buffsize>0 ) {
		unsigned char *in=buff+*buffsize;
		unsigned char *out=buff+*buffsize*2;
		*buffsize=*buffsize*2;
		*out--='\0';
		while (in-->buff) {
			*out--=hdigit[*in & 0xf];
			*out--=hdigit[*in >> 4];
		}
	}
	wblogs("convert_binary_to_hexa leave: OK\n");
}
/*}}}*/
/*{{{ smallwbvalue2phpvalue
 * converts values of types ATT_BOOLEAN, ATT_INT8, ATT_INT16, ATT_INT32,
 * ATT_INT64, ATT_MONEY, ATT_FLOAT, ATT_DATE, ATT_TIME, ATT_TIMESTAMP
 * returns 1 on success, 0 on error */
#define CASE_smallwbvalue2phpvalue \
  case ATT_BOOLEAN:\
	case ATT_INT8:\
	case ATT_INT16:\
	case ATT_INT32:\
	case ATT_INT64:\
	case ATT_MONEY:\
	case ATT_FLOAT:\
	case ATT_DATE:\
	case ATT_TIME:\
	case ATT_TIMESTAMP:
#if WBVERS<900
static int smallwbvalue2phpvalue(uns8 wbv_type,void *wbv_ptr,zval *phpvalue)
#else
static int smallwbvalue2phpvalue(uns8 wbv_type,attrspecif_t wbv_specif,void *wbv_ptr,zval *phpvalue)
#endif
{
#if WBVERS>=900
	wblog("smallwbvalue2phpvalue enter, wbv_type=%u, wbv_specif=0x%x, wbv_ptr=%p, phpvalue=%p\n",wbv_type,wbv_specif,wbv_ptr,phpvalue);
#else
	wblog("smallwbvalue2phpvalue enter, wbv_type=%u, wbv_ptr=%p, phpvalue=%p\n",wbv_type,wbv_ptr,phpvalue);
#endif
	switch( wbv_type )
	{
		/* Grand switch over small types: */
		/*		 - att_int8, att_int16, att_int32
             The integer types are very similar; they differ only in size and value of NONE */
#if WBVERS<900
#define MAKE_LONG_OR_NULL(type, none) \
				if( *((type *)wbv_ptr)==none ) { ZVAL_NULL(phpvalue) } \
				else { ZVAL_LONG(phpvalue,*((type *)wbv_ptr)) } \
				break;
#else
#define MAKE_LONG_OR_NULL(type, none) \
				if( *((type *)wbv_ptr)==none ) { ZVAL_NULL(phpvalue) } \
				else if( wbv_specif.scale==0 ) { ZVAL_LONG(phpvalue,*((type *)wbv_ptr)) } \
				else { ZVAL_DOUBLE(phpvalue,((double)(*((type *)wbv_ptr)))/mypower(10,wbv_specif.scale)) } \
				break;
#endif
		case ATT_INT8:
			MAKE_LONG_OR_NULL(sig8,NONETINY)
		case ATT_INT16:
			MAKE_LONG_OR_NULL(sig16,NONESHORT)
		case ATT_INT32:
			MAKE_LONG_OR_NULL(sig32,NONEINTEGER)

		/*		 - att_int64
						 64 bit integers have no corresponging type in PHP. A warning message is issued if you
						 use them in a size that cannot be handled. */
		case ATT_INT64:
			if( *((sig64*)wbv_ptr)==NONEBIGINT ) { ZVAL_NULL(phpvalue) }
			else if( *((sig64*)wbv_ptr)==(sig64)(signed int)*((sig64*)wbv_ptr)
#if WBVERS>=900
					  && wbv_specif.scale==0
#endif
				 	)
		  {
			 	ZVAL_LONG(phpvalue,(signed int)*((sig64*)wbv_ptr));
			}
#if WBVERS>=900
			else if( wbv_specif.scale!=0 )
			{
				ZVAL_DOUBLE(phpvalue,((double)(*((sig64 *)wbv_ptr)))/mypower(10,wbv_specif.scale))
			}
#endif
		 	else {
				zend_error(E_NOTICE, "602SQL: Big integer converted to float");
				ZVAL_DOUBLE(phpvalue,(double)*((sig64*)wbv_ptr));
			}
			break;

			/*		 - att_money
							 Money is converted to floats, thus well written applications probably should not
							 rely on its accuracy. */
			case ATT_MONEY:
				if( ((monstr*)wbv_ptr)->money_hi4==NONEINTEGER && ((monstr*)wbv_ptr)->money_lo2==0 ) {
					ZVAL_NULL(phpvalue)
				}
			 	else {
				 	ZVAL_DOUBLE(phpvalue,money2real((monstr*)wbv_ptr));
				}
				break;

				/*		 - att_float */
				case ATT_FLOAT:
					if( *((double*)wbv_ptr)==NONEREAL ) { ZVAL_NULL(phpvalue) }
					else { ZVAL_DOUBLE(phpvalue,*((double*)wbv_ptr)); }
					break;

				/*		 - att_boolean */
				case ATT_BOOLEAN:
					if( *((wbbool*)wbv_ptr)==NONEBOOLEAN ) { ZVAL_NULL(phpvalue) }
					else { ZVAL_BOOL(phpvalue,*((wbbool*)wbv_ptr)==wbtrue);
					break;

				/*		 - att_timestamp
								 Timestamp is returned as UNIX timestamp, i.e., long integer containing number of
								 seconds between the Unix Epoch (January 1. 1970) and given time. There are
								 functions in PHP that can handle those, and the format is widely used.

								 Timestamps that cannot be expressed this way are returned as -1; use 602SQL SQL
								 function to convert it to date and time if you expect them. */
				case ATT_TIMESTAMP:
				case ATT_DATIM:
					if( *((uns32*)wbv_ptr)==NONETIMESTAMP ) { ZVAL_NULL(phpvalue) }
					else {
						ZVAL_LONG(phpvalue,sql602timestamp2time_t(*((uns32*)wbv_ptr)));
					}
					break;

				/*		 - att_date
								 Date is converted to number of seconds between the Unix Epoch and start of the given date. */
				case ATT_DATE:
					if( *((uns32*)wbv_ptr)==NONEDATE ) { ZVAL_NULL(phpvalue) }
					else {
						ZVAL_LONG(phpvalue,sql602date2time_t(*((uns32*)wbv_ptr)));
					}
					break;

				/*		 - att_time
				 *		   Time is printed as string. */
				case ATT_TIME:
					if( *((uns32*)wbv_ptr)==NONETIME ) { ZVAL_NULL(phpvalue) }
					else {
						char buffer[20];
						int msecs;
						msecs=Sec1000(*((uns32*)wbv_ptr));
						snprintf(buffer, sizeof(buffer),
							(msecs)?"%02d:%02d:%02d.%04d":"%02d:%02d:%02d",
							Hours(*((uns32*)wbv_ptr)), Minutes(*((uns32*)wbv_ptr)),
							Seconds(*((uns32*)wbv_ptr)), msecs);
						ZVAL_STRING(phpvalue,buffer,TRUE);
					}
					break;

				/*		 - other types (not handled so far)
								 New types may appear; before we implement them here, string value is returned. */
				default:
					ZVAL_STRING(phpvalue, "Type not handled yet", TRUE);
			}
	}
	wblogs("smallwbvalue2phpvalue leave, 1\n");
	return 1;
}
/*}}}*/
/* wbstring2emalloc_buffer {{{
 * converts string value from database encoding to PHP script encoding
 * returns 1 on success (and string value in emalloc-ed buffer '*buffer' too and actual string length in *string_length too), or 0 on error */
#if WBVERS<800
static int wbstring2emalloc_buffer(uns8 wbv_type,void *wbv_ptr,uns32 *string_length,char **buffer)
#else
static int wbstring2emalloc_buffer(
#ifdef USE_WB_SCRIPT_ENCODING
		wb_encoding script_encoding,
#endif
		uns8 wbv_type,attrspecif_t wbv_specif,void *wbv_ptr,uns32 *string_length,char **buffer)
#endif
{
	char *string;
#if WBVERS>=800
	wblog("wbstring2emalloc_buffer enter, wbv_type=%u, wbv_specif=0x%x, wbv_ptr=%p, string_length=%lu\n",wbv_type,wbv_specif,wbv_ptr,*string_length);
#else
	wblog("wbstring2emalloc_buffer enter, wbv_type=%u, wbv_ptr=%p, string_length=%lu\n",wbv_type,wbv_ptr,*string_length);
#endif
	/* Convert from SQL to PHP: Strings: */
	/*		- common code for string types (att_char, att_string, ...)
	String could probably be handled in the loop above, too. We do not do it
	only because the buffer required for it is much larger than for the other
	fixed width types. */
  string=(char*)wbv_ptr;
	if( *string_length==0 ) 
	{ *buffer=NULL; }
#if WBVERS>=800
	else if( wbv_specif.wide_char )
	{
		char *utf8str=strUCS2toUTF8((unsigned char*)string,string_length);
		if( utf8str==NULL )
		{
			zend_error(E_WARNING,"602SQL: Can't convert UNICODE string to UTF-8 string");
			wblogs("wbstring2emalloc_buffer leave result=0, UTF-16 to UTF-8 conversion failed\n");
			return 0;
		}
		*buffer=utf8str; // utf8str is emalloc-ed
#ifdef _DEBUG
		wblog("wbstring2emalloc_buffer: length of utf8str is %lu\n",*string_length);
#endif
	}
#endif
	else
	{
#ifdef USE_WB_SCRIPT_ENCODING
		if( IS_STRING_CONVERSION_REQUIRED(sql602_charset2wb_encoding(wbv_specif.charset),script_encoding) )
		{
			char *emalloc_string=estrndup_with_encoding(string,*string_length,sql602_charset2wb_encoding(wbv_specif.charset),script_encoding);
			if( emalloc_string==NULL )
			{ 
				wblogs("wbstring2emalloc_buffer leave, ERROR in estrdup_with_encoding()\n");
				return 0;
			}
			*buffer=emalloc_string;
			*string_length=strlen(*buffer); /* we have to determine length of converted string 
																				 Warning: When source string contains ASCII zero code chars
																				 we will report invalid length of string by this command. */
		}
		else
		{
			*buffer=emalloc(*string_length+1);
			if( *buffer==NULL )
			{
				report_out_of_memory_error("Can not allocate temporary buffer.");
				wblogs("wbstring2emalloc_buffer() leave: ERROR out of memory\n");
				return 0;
			}
			memcpy(*buffer,string,*string_length);
			(*buffer)[*string_length]='\0';
			// string length is unchanged
		}
#else
		*buffer=emalloc(*string_length+1);
		if( *buffer==NULL )
		{
			report_out_of_memory_error("Can not allocate temporary buffer.");
			wblogs("wbstring2emalloc_buffer() leave: ERROR out of memory\n");
			return 0;
		}
		memcpy(*buffer,string,*string_length);
		(*buffer)[*string_length]='\0';
		// string length is unchanged
#endif
	}
	wblog("wbstring2emalloc_buffer leave, 1, result: string_length=%lu, string=\"%s\"\n",*string_length,*buffer);
	return 1;
} /*}}}*/
/*{{{ wbstring2phpvalue
 * This function assigns wbv_ptr value to phpvalue. wbv_ptr is pointer to
 * buffer with string value which was read from SQL server.
 * For 602SQL 8.0 and higher function converts UNICODE string to UTF-8 encoding too.
 * Returns 1 on succes, 0 on error.
 * wbv_ptr buffer have to be ended with \0 char.
 * Function can convert value of ATT_TEXT and ATT_EXTCLOB types too even if this type isn't metioned in CASE #define. */
#if WBVERS<800
#define CASE_wbstring2phpvalue \
  case ATT_CHAR:\
	case ATT_STRING:\
	case ATT_CSSTRING:\
	case ATT_CSISTRING:\
	case ATT_AUTOR:
#else
#define CASE_wbstring2phpvalue \
  case ATT_CHAR:\
	case ATT_STRING:\
	case ATT_AUTOR:
#endif
#if WBVERS<800
static int wbstring2phpvalue(uns8 wbv_type,void *wbv_ptr,zval *phpvalue)
#else
static int wbstring2phpvalue(
#ifdef USE_WB_SCRIPT_ENCODING
		wb_encoding script_encoding,
#endif
		uns8 wbv_type,attrspecif_t wbv_specif,void *wbv_ptr,zval *phpvalue)
#endif
{
	char *buffer;
	uns32 string_length;
	// determine string length
#if WBVERS<800
	string_length=(uns32)strlen((char*)wbv_ptr);
#else
	if( wbv_specif.wide_char ) string_length=(uns32)UCS2bytelen((char*)wbv_ptr);
	else string_length=(uns32)strlen((char*)wbv_ptr);
#endif
	// convert string
#if WBVERS<800
	if( !wbstring2emalloc_buffer(wbv_type,wbv_ptr,&string_length,&buffer) )
#else
	if( !wbstring2emalloc_buffer(
#ifdef USE_WB_SCRIPT_ENCODING 
				script_encoding,
#endif
				wbv_type,wbv_specif,wbv_ptr,&string_length,&buffer) )
#endif
	{
		return 0;
	}
	if( buffer==NULL ) { ZVAL_NULL(phpvalue); }
	else { ZVAL_STRING(phpvalue,buffer,FALSE/*buffer is owned by phpvalue now*/); }
	return 1;
}
/*}}}*/
/*{{{ db_read_lob_value
 * reads value of LOB from cursor to buffer *buff
 *
 * buffer *buff have to be at least size+2 bytes long and have to be emalloc-ed
 * returns 1 on success (and in readed returns size of readed data), 0 on error
 * convert_to_hexa is 1 in case that readed data have to be converted to string with hexadecimal
 *
 * WARNING: Buffer *buff can be realloced inside this function and especially SHORTENED!
 *          Don't rely on *buff size after execution of this function.  */
static int db_read_lob_value(wb_conn *conn,tcursnum cursor,trecnum record,tattrib attr,uns32 start,uns32 *buffsize,uns32 *readed,uns32 *readed_bytes,char **buff,int convert_to_hexa,uns8 attrtype
#if WBVERS>=800
		,attrspecif_t specif
#endif
#ifdef USE_WB_SCRIPT_ENCODING
		,wb_encoding script_encoding
#endif
		TSRMLS_DC)
{

	wblog("db_read_lob_value enter: cursor=%d, record=%d, attr=%d, start=%lu, *buffsize=%lu, convert_to_hexa=%d, attrtype=%d\n",cursor,record,attr,start,*buffsize,convert_to_hexa,attrtype);
	*readed=*readed_bytes=0;
	if (cd_Read_var(CDP(conn), cursor, record, attr, NOINDEX, start, (*buffsize), *buff, readed_bytes))
	{
		CONN_REPORT_ERR(conn,"Read_var");
		wblogs("db_read_lob_value leave, error in cd_Read_var()\n");
		return 0;
	}
	*readed=*readed_bytes;
	wblog("db_read_lob_value: LOB cd_Read_var() readed %u bytes\n",*readed);
	if( (*readed)>0 ) { wblog("	last byte is 0x%02x\n",(*buff)[(*readed)-1]); }
	if( (*readed)>1 ) { wblog("	second last byte is 0x%02x\n",(*buff)[(*readed)-2]); }
	(*buff)[(*readed)]=(*buff)[(*readed)+1]='\0';

	if( 
#if WBVERS>=1100
		(attrtype==ATT_TEXT || attrtype==ATT_EXTCLOB)
#else
		attrtype==ATT_TEXT
#endif
		&& *readed>0
	)
	{
		char *buffer;
#if WBVERS<800
		if( !wbstring2emalloc_buffer(attrtype,*buff,readed,&buffer) )
#else
		if( !wbstring2emalloc_buffer(
#ifdef USE_WB_SCRIPT_ENCODING 
					script_encoding,
#endif
					attrtype,specif,*buff,readed,&buffer) )
#endif
		{
			wblogs("db_read_lob_value leave, wbstring2emalloc_buffer() failed\n");
			return 0;
		}
		efree(*buff);
		*buff=buffer;
		(*buffsize)=*readed;

	}
	else
	{
		wblogmem("readed:",*buff,*readed);
	}
	if( convert_to_hexa )
	{
		wblogmem("buff:",*buff,*readed);
		convert_binary_to_hexa((unsigned char*)(*buff), readed);
	}
	wblogs("db_read_lob_value leave, OK\n");
	return 1;
}
/*}}}*/
/*{{{ db_item_to_php
 */
static int db_item_to_php(
#ifdef USE_WB_SCRIPT_ENCODING
		wb_encoding script_encoding,
#endif
		struct sql_answer *answer, trecnum record,int col, zval *new_element, int ignore_passthru TSRMLS_DC)
{
	tcurstab cursor=answer->cursor;
	uns8 attrtype=answer->types[col];
	tattrib attr=answer->attr_col[col];
#if WBVERS>=800
	attrspecif_t specif=answer->specif[col];
#endif
	cdp_t cdp=CDP_FROM_ANSWER_P(answer);

	wblog("db_item_to_php enter, cursor=%d record=%d attr=%d attrtype=%d\n",cursor,record,attr,attrtype);

	switch (attrtype)
	{
		CASE_wbstring2phpvalue
		  {
#if WBVERS>=800
				char string[MAX_FIXED_STRING_LENGTH+2];
#else
				char string[256];
#endif
				if (cd_Read(cdp, cursor, record, attr, NULL, string))
				{
					CONN_REPORT_ERR(CONN_P_FROM_ANSWER_P(answer),"Read (string)");
					wblogs("db_item_to_php leave result=FALSE\n");
					return FALSE;
				}
				if (attrtype==ATT_CHAR) string[1]=0;
#if WBVERS>=800
				if( !wbstring2phpvalue(
#ifdef USE_WB_SCRIPT_ENCODING
							script_encoding,
#endif
							attrtype,specif,string,new_element) )
#else
				if( !wbstring2phpvalue(attrtype,string,new_element) )
#endif
				{
					wblogs("db_item_to_php leave, ERROR, wbstring2phpvalue failed\n");
					return FALSE;
				}
			}
			break;
		case ATT_TEXT:
#if WBVERS>=1100
		case ATT_EXTCLOB:
		case ATT_EXTBLOB:
#endif
		case ATT_NOSPEC:
		case ATT_RASTER:
		case ATT_SIGNAT:
			if( answer->lob_mode==WB_LOBMODE_RESOURCE )
			{ /* Create LOB resource for this data: */
				struct lob_info_t *lob=(struct lob_info_t*)emalloc(sizeof(struct lob_info_t));
				if( lob==NULL )
				{
					php_error(E_WARNING, "602SQL: Can not allocate internal structure. Out of memory. ");
					return FALSE;
				}
#ifdef USE_STRUCT_NAME
				strcpy(lob->struct_name,"lob_info_t");
#endif
				lob->answer=answer;
				lob->attrib=attr;
				lob->recnum=record;
				lob->status=LOB_STATUS_OPENED;
				//id=zend_list_insert(lob, wbres_lob);
				ZEND_REGISTER_RESOURCE(new_element,lob,wbres_lob);
				answer->usage++;
				wblog("db_item_to_php: result is LOB resource - ptr=%p resid=%ld\n",lob,Z_RESVAL_P(new_element));
				//ZVAL_RESOURCE(new_element,id);
			}
			else
			{ /* Convert from SQL to PHP: Long types: */
				/* Here is the code that actually converts a long type into PHP. In the case of
				wb_fetch_into function, pass thru in fact returns empty string; ignoring the value has
				some inner logic, and besides, this is the way the ODBC module works. This detail is
				controlled by a parameter ignore_passthru. */
				uns32 buffsize,buffsize_to_emalloc;
				size_t start=0; /* from where we read */
				char *buff;
				int longreadlen=answer->longreadlen;
				binmode_t binmode=(longreadlen)?answer->binmode:PASSTHRU;
				if (ignore_passthru&&binmode==PASSTHRU)
				{
					ZVAL_EMPTY_STRING(new_element);
					break;
				}
				/* Set buffsize and allocate buff: */
				/* if longreadlen is zeno and binmode isn't PASSTHRU then we have to read whole LOB
				 * so we find length of the LOB and update longreadlen value */
				wblog("longreadlen=%d\n",longreadlen);
				if( longreadlen==-1 && binmode!=PASSTHRU )
				{
					uns32 len;
					if( cd_Read_len(cdp,cursor,record,attr,NOINDEX,&len) )
					{
						CONN_REPORT_ERR(CONN_P_FROM_ANSWER_P(answer),"Read_len");
						wblogs("db_item_to_php leave, error in cd_Read_len()\n");
						return FALSE;
					}
					longreadlen=(int)len;
					wblog("LOB length is %u, longreadlen=%d\n",len,longreadlen);
				}
				/* Size of the buffer will be 4096 if the data are passed thru; it would be mistake to use
					 longreadlen in that case, as it is allowed to be zero. */

				buffsize_to_emalloc=buffsize=(longreadlen>0 && binmode!=PASSTHRU)?longreadlen:4096;


				if( binmode==CONVERT && attrtype!=ATT_TEXT
#if WBVERS>=1100
						&& attrtype!=ATT_EXTCLOB
#endif
						)
				{
					buffsize=buffsize/2-1; /* 2 chars per byte, ending '\0' */
				}
				else if( 
#if WBVERS>=1100
						(attrtype==ATT_TEXT || attrtype==ATT_EXTCLOB)
#else
						attrtype==ATT_TEXT
#endif
#if WBVERS>=800
					&& !specif.wide_char
#endif
				) buffsize--; /* for final '\0' */

				/* If binmode is PASSTHRU, print out content of attribute in blocks of buffsize,
				 * otherwise read first up to buffsize bytes into buff and set buffsize to actual
				 * number of read bytes: */
				buff=NULL;

				do {
					uns32 len,bytelen;
					/* We have to efree() and emalloc() buff because db_read_lob_value() can realloc (shorten) this buffer
					 * and we need to guarrantee that buff is buffsize_to_emalloc+2 bytes long.*/
					if( buff!=NULL ) { efree(buff); }
					buff=(char*)emalloc(buffsize_to_emalloc+2);
					if( buff==NULL )
					{
						report_out_of_memory_error("Can not allocate internal structure for cursor.");
						wblogs("db_item_to_php leave, FALSE, emalloc() failed\n");
						return FALSE;
					}

					buffsize=buffsize_to_emalloc;
					wblog("buff=0x%p\n",buff);
					if( !db_read_lob_value(CONN_P_FROM_ANSWER_P(answer),cursor,record,attr,start,&buffsize,&len,&bytelen,&buff,
								(binmode==CONVERT && attrtype!=ATT_TEXT
#if WBVERS>=1100
									&& attrtype!=ATT_EXTCLOB
#endif
								),attrtype
#if WBVERS>=800
								,specif
#endif
#ifdef USE_WB_SCRIPT_ENCODING
								,script_encoding
#endif
								TSRMLS_CC) )
					{
						efree(buff);
						wblogs("db_item_to_php leave, FALSE, db_read_lob_value() failed\n");
						return FALSE;
					}
					wblog("len=%d, buffsize after db_read_lob_value() is %u, buff=0x%p\n",len,buffsize,buff);
					start+=bytelen;
					if (len<buffsize)
					{

						buffsize=len;
						start=0; /* stop reading */
					}
					if (binmode==PASSTHRU) { ZEND_WRITE(buff, buffsize); }
				} while (binmode==PASSTHRU && start);
				/* Make new_element empty string if binmode is PASSTHRU, content of buffer
				 * (possibly converted) otherwise: */
				if (binmode==PASSTHRU)
				{
					ZVAL_EMPTY_STRING(new_element);
					efree(buff);
				}
				else
				{
					wblog("db_item_to_php: LOB buffsize=%u\n",buffsize);
					ZVAL_STRINGL(new_element, buff, buffsize, FALSE);
				}
			}
			break;
		case ATT_BINARY:
			{ /* Convert from SQL to PHP: Binary type: */
				/* Having dealt with variable binary types, it is quite easy now to work with
					 ordinary binary types, too. The only thing that is worthy noticing is that we
					 must get the length of the binary type from the server, as this information is
					 not stored in our sql_answer structure. */

				tattrib unused;
				uns8 unused2;
				uns32 size=0;
				char *buff;
				if( !cd_Attribute_info(cdp, cursor, answer->names[col], &unused, &unused2, &unused2,(uns16 *)&size) )
				{
					CONN_REPORT_ERR(CONN_P_FROM_ANSWER_P(answer),"Attribute_info");
					return FALSE;
				}
				buff=(char*)emalloc(((answer->binmode==CONVERT)?size*2:size)+1);
				if (cd_Read(cdp, cursor, record, attr, NULL, buff))
				{
					CONN_REPORT_ERR(CONN_P_FROM_ANSWER_P(answer),"Read (binary)");
					efree(buff);
					wblogs("db_item_to_php leave, result=FALSE\n");
					return FALSE;
				}
				if (answer->binmode==CONVERT) convert_binary_to_hexa((unsigned char*)buff, &size);
				ZVAL_STRINGL(new_element, buff, size, FALSE);
			}
			break;
		case ATT_HIST:
			ZVAL_STRING(new_element, "Type not handled", TRUE);
			break;
		default:
			{ /* Convert small type of fixed width from SQL: */
				/*		- common code for small (64 bits and below) types
				The rest are comparably small types with fixed width; they are all read in same way
				and only their actual length, representation in PHP and value representing NONE varies. */

				char value[8];
				if (cd_Read(cdp, cursor, record,
					attr, NULL, &value)){
					CONN_REPORT_ERR(CONN_P_FROM_ANSWER_P(answer),"Read (small type)");
					wblogs("db_item_to_php leave, result=FALSE\n");
					return FALSE;
				}
				/* Small types are handled in separate function: */
#if WBVERS<900
				if( !smallwbvalue2phpvalue(attrtype,value,new_element) )
#else
				if( !smallwbvalue2phpvalue(attrtype,specif,value,new_element) )
#endif
				{
					wblogs("db_item_to_php leave, result=FALSE, smallwbvalue2phpvalue failed\n");
					return FALSE;
				}
			}
	}
	wblogs("db_item_to_php leave, result=TRUE\n");
	return TRUE;
}
/*}}}*/
/*}}}*/
/*{{{ wb_exec */
/*{{{ struct attr_alloc */
/* In order to get filled types and names of the sql_answer structure
 * right, we call two callback function. The first one only counts space
 * required for these data, while the other one actually fills the data into
 * already allocated slots. The amount of allocated data is $2\tilde
 * c+v(1+c)+\sum_{i=1}^{c-1} s_i$, where $c$ is number of attributes advanced
 * by one, $v$ is size of a void *, and $s_i$ is the length of the $i$th
 * attribute name; a tilde denotes rounding up to the integer size. If the
 * allocated memory starts at base, then $types\equiv base$,
 * $attr\_col\equiv base+\tilde c$, $names\equiv base+2\tilde c$, and actual
 * names of attributes start at $base+2\tilde c+v(1+c)$. The one redundant
 * char * is provided for easier handling in fix_types_and_names, and is
 * located between column attributes (with indices between 0 and attrs-1)
 * and system attributes (with indices between attrs+1 and total_attrs).
 * However, at the end of the initialization the system attributes are fixed
 * so that since then it is possible to make a loop for (i=0;
 * i<answer->total_attrs; i++) when one wants to cycle through all attributes. */

struct attr_alloc{
	int c; /* number of attributes */
	int s; /* accumulated size */
};
/*}}}*/
/*{{{ static BOOL WINAPI callback fill_attr_alloc(char * attrname, uns8 attrtype, uns8 attrmult, attrspecif_t attrspecif, void *alloc_v) */
static BOOL WINAPI callback fill_attr_alloc(char * attrname, uns8 attrtype, uns8 attrmult,
	attrspecif_t attrspecif, void *alloc_v)
{
	struct attr_alloc *alloc=(struct attr_alloc*) alloc_v;
	alloc->c++;
	alloc->s+=1+strlen(attrname);
	return TRUE;
}
/*}}}*/
/*{{{ static BOOL callback fix_types_and_names(char *attrname, uns8 attrtype, uns8 attrmult, attrspecif_t attrspecif, void *answer_v) */
/* At the point of first call to fix_types_and_names, values of types,
 * names, and names[0] are supposed to be properly initialized, attrs is
 * zero, and total_attrs points after the last attribute. Later, whenever we
 * get there, attrs is expected to be number of the expected attribute
 * (starting at 0), total_attrs point at the next system attribute to use,
 * and names[attrs] to the free string memory pool. */
static BOOL callback fix_types_and_names(char *attrname, uns8 attrtype, uns8 attrmult,
	attrspecif_t attrspecif, void *answer_v)
{
	struct sql_answer *answer=(struct sql_answer *)answer_v;
	char *next=answer->names[answer->attrs]+1+strlen(attrname);
	answer->attr_col[answer->attrs]++; /* now it has current attr column number */
		/* Attributes with names starting "_W5_" contain system informations
	 * kept by the server. In spite of the fact that it gives them to us,
	 * we shall not pass them to the PHP program unless explicitly asked.
	 * */
	if (!strncmp("_W5_", attrname, 4)){
		answer->attr_col[answer->total_attrs]=answer->attr_col[answer->attrs];
		answer->types[answer->total_attrs]=attrtype;
#if WBVERS>=800
	  answer->specif[answer->total_attrs]=attrspecif;
#endif
		strcpy(answer->names[answer->attrs], attrname);
		answer->names[answer->total_attrs--]=answer->names[answer->attrs];
		answer->names[answer->attrs]=next;
	} else {
		answer->attr_col[answer->attrs+1]=answer->attr_col[answer->attrs];
		answer->types[answer->attrs]=attrtype;
#if WBVERS>=800
	  answer->specif[answer->attrs]=attrspecif;
#endif
		strcpy(answer->names[answer->attrs++], attrname);
		answer->names[answer->attrs]=next;
	}
#if WBVERS>=800
	wblog("cursor column: name=\"%s\", type=%d, specif=0x%x (length=%d, charset=%d, wide_char=%d, ignore_case=%d | scale=%d, precision=%d)\n",attrname,attrtype,attrspecif,attrspecif.length,attrspecif.charset,attrspecif.wide_char,attrspecif.ignore_case,attrspecif.scale,attrspecif.precision);
#else
	wblog("cursor column: name=\"%s\", type=%d\n",attrname,attrtype);
#endif
	return TRUE;
}
/*}}}*/
/*{{{ static int create_cursor_resource(struct wb_conn *conn,uns32 result,struct sql_answer **ppanswer TSRMLS_DC) */
/* Creates and fills sql_answer structure with informations about opened cursor whose handle is in result parameter.
 * Returns 1 on success (and pointer to emalloc-ated sql_answer structure in ppanswer)
 *         0 on error
 * This function is used in wb_exec and in wb_psql_execute. */
#if WBVERS>=800
#define CURSOR(result) result
#define IS_CURSOR(result) IS_CURSOR_NUM(result)
#else
#define CURSOR(result) ((tcursnum)(result & 0xffff)) /* cursor number from a result */
#define IS_CURSOR(result) (((result)!=0xffffffff) && (((result) &0xffff0000)==0xffff0000))
#endif
static int create_cursor_resource(struct wb_conn *conn,uns32 result,struct sql_answer **ppanswer TSRMLS_DC)
{
	struct sql_answer *answer;
	cdp_t cdp=CDP(conn);

	wblog("create_cursor_resource enter, conn=%p, result=%u\n",conn,result);
	answer = (struct sql_answer *)emalloc(sizeof(struct sql_answer));
	if (NULL==answer)
	{
		report_out_of_memory_error("Can not allocate internal structure for cursor.");
		wblogs("create_cursor_resource leave, 0 error can't allocate sql_answer struct\n");
		return 0;
	}
	*ppanswer=answer;
	answer->conn=conn;
	answer->cursor=CURSOR(result);
	
	/* Allocate place for types and attribute names of the result and fill them: */
	{
		struct attr_alloc alloc={0, 0};
		size_t needed_space;
		int rounded_c;
		if (!cd_Enum_attributes_ex(cdp, answer->cursor, fill_attr_alloc, (void*)&alloc))
		{
			php_error(E_WARNING, "Enum_attributes failed");
			efree(answer);
			wblogs("create_cursor_resource leave, 0 error cd_Enum_attributes_ex failed\n");
			return 0;
		}
		rounded_c=sizeof(int)*(1+(alloc.c)/sizeof(int)); /* this is at least $c+1$ */
		needed_space =
		   sizeof(uns8)*rounded_c     // answer->types[]
			+sizeof(tattrib)*rounded_c  // answer->attr_col[]
			+(1+alloc.c)*sizeof(char *) // answer->names[]
			+alloc.s;                   // space for column names
		answer->types=(uns8 *)emalloc(needed_space);
#if WBVERS>=800
		answer->specif=(attrspecif_t *)emalloc((1+alloc.c)*sizeof(attrspecif_t));
#endif
		if (answer->types==NULL
#if WBVERS>=800
			|| answer->specif==NULL
#endif
			) 
		{
			report_out_of_memory_error("Can't allocate structures for cursor description.");
			efree(answer);
#if WBVERS>=800
			wblogs("create_cursor_resource leave, 0 error emalloc for answer->types or answer->specif failed\n");
#else
			wblogs("create_cursor_resource leave, 0 error emalloc for answer->types failed\n");
#endif
			return 0;
		}
		answer->attr_col=(tattrib*)(((char*)(answer->types))+rounded_c*sizeof(uns8));
		answer->names=(char **)(((char*)(answer->attr_col))+rounded_c*sizeof(tattrib));
		answer->names[0]=(char *)(1+answer->names+alloc.c);
		answer->attrs=0;
		answer->total_attrs=alloc.c;
		answer->attr_col[0]=0;
		if( !cd_Enum_attributes_ex(cdp, answer->cursor, fix_types_and_names, answer) ) 
		{
			CONN_REPORT_ERR(CONN_P_FROM_ANSWER_P(answer),"Enum_attributes");
			efree(answer);
			wblogs("create_cursor_resource leave, 0 error cd_Enum_attributes_ex failed\n");
			return 0;
		}
		answer->names[answer->attrs]=answer->names[alloc.c]; /* allow simple for */
		answer->types[answer->attrs]=answer->types[alloc.c];
		answer->attr_col[answer->attrs]=answer->attr_col[alloc.c];
		answer->total_attrs=alloc.c;
		answer->usage=1; // while none LOB resources opened from this cursor yet we assign one for cursor itself
		answer->lob_mode=WB_LOBMODE_DIRECT; // default LOB behaviour
		answer->is_opened=1; // cursor is opened
	}

	/* Set internal cursor parameters to the default: */
	/*		- Default values of record counters
	Records are internally numbered starting at 0, but initial fetch_row is
	expected. Note that while our counting is compatible with the one of 602SQL,
	it is shifted by one from the PHP's; the convention of the ODBC driver
	expects that first row has number 1. The wb_fetch_row must take that into
	account. */
	answer->current_record=-1;
	answer->rec_cnt=-1;								 // we don't know number of records in cursor
	/* following series of statements we would use if we wanted to compute records immediately:
	if( cd_Rec_cnt(cdp,answer->cursor,&(answer->rec_cnt)) ) 
	{
		CONN_REPORT_ERR(CONN_P_FROM_ANSWER_P(answer),"Rec_cnt");
		efree(answer);
		wblogs("create_cursor_resource leave, 0 error cd_Rec_cnt failed\n");
		return 0;
	}
	*/
#ifdef USE_STRUCT_NAME
	strcpy(answer->struct_name,"sql_answer");
#endif
 

	/*		- per-cursor values of binmode and longreadlen
	These parameters are set for every cursor; the global values are their default. */

	answer->binmode=WBG(binmode);
	answer->longreadlen=WBG(longreadlen);
	conn->usage++;
	wblogs("create_cursor_resource leave, 1\n");
	return 1;
} 
/*}}}*/
/*{{{ int wb_exec(int connection_id,string SQL_statement) */
/* Executing SQL command.
 * The wb_exec sends a SQL command passed as the second parameter at the
 * server identified by the first parameter. If the command created a cursor,
 * it creates a new resource that contains data about the cursor; otherwise,
 * it returns a long with number of affected record. On error, FALSE is
 * returned.

 * Server returns up to MAX_RETURN_FIELDS in passed buffer; every value
 * corresponds to result of one SQL command---there may be more SQL
 * commands in one string. We ignore all the results except the first one.

 * Note the SEPARATE_ZVAL in the code; we may change the value of command
 * while cutting off long client variable names, but we do not want these
 * changes to be seen from outside. */
ZEND_FUNCTION(wb_exec)
{
	WBLSM_DECLARE
	zval **connhandle;
	zval **command=NULL;
	cdp_t cdp;
	wb_conn *conn;
	uns32 result[MAX_RETURN_FIELDS]; /* results of SQL as returned by server */
	char *tmp_command;
#ifdef USE_WB_SCRIPT_ENCODING
	char *emalloc_command=NULL;
#endif

	/* Variables needed for using clivars: */
	/* Using client variables in SQL commands.	It is also possible to
	 * use PHP variables in the SQL commands. The variables in a SQL
	 * command preceded by a colon and optional '>', '<' or both are
	 * mapped to PHP variables in current namespace.

	 * The 602SQL implementation of the feature is based on client variables,
	 * or clivars. They must be initialized before use, so a quick scan of
	 * the passed SQL command is necessary. Its implementation is based on
	 * the similar task solved in the 602SQL Internet clients. */
	struct t_clivar * clivar_table=NULL; /* Table of client variables */
	int clivar_tablecount=0; /* Number of passed clivars */
	zval *** bound_variables=NULL; /* table of bound variables */
	zval *** current_zval; /* current bound variable */
	char *name=NULL; /* name of current clivar */

	wblogs("wb_exec enter\n");

	if (2!=ZEND_NUM_ARGS() ||
			zend_get_parameters_ex(2, &connhandle, &command) != SUCCESS)
		WRONG_PARAM_COUNT;

	SEPARATE_ZVAL(command); /* Use local copy of the SQL command */
	convert_to_string_ex(command);
	conn=get_conn_from_resource(INTERNAL_FUNCTION_PARAM_PASSTHRU,connhandle);
	cdp=CDP(conn);
	if (!conn) RETURN_FALSE; /* error in get_conn_from_resource */
	WBLSM_START_EX("wb_exec",cdp,Z_STRVAL_PP(command));

	/* get command */
	if( !PREPARE_STRING_SCRIPT2SYSTEM(WBG(script_encoding),conn,&tmp_command,&emalloc_command,Z_STRVAL_PP(command)) )
	{
		WBLSM_END;
		RETURN_FALSE;
	}
	/* Scan command for client variables and initialize clivar_table: */
	/* Scan command for client variable...
	 * There are several types of comments to ignore, namely those going
	 * from // or -- to the end of line, Pascal style comment in
	 * curly brackets, and C style comments delimited by / * and
	 * * /. We also have to look for strings, which are delimited by
	 * ' and " . */
  wblog("SQL statement to scan for host variables: %s\n",tmp_command);
	{
		char *ptr=tmp_command;
		int clivar_tablemaxcount=0;
		const int clivar_tablegrow=5;
		clivar_table=NULL;
		while(*ptr)
		{
			switch( *ptr )
			{
				case '\"':
				case '\'':
					{/* skip strings */
						/* Advance ptr behind string: */
						int delim=*ptr;
						wblog("  skipping string which starts at %s\n",ptr);
						for( ptr++;*ptr!='\0';ptr++ )
						{
							if( *ptr==delim )
							{
								if( ptr[1]==delim ) ptr++; /* two delimiters are a char in string */
								else break; /* while only one is end of string */
							}
						}
						if( *ptr==delim ) ptr++; /* end of string found */
						wblog("  skipped to %s\n",ptr);
					}
					break;
				case '-':
				case '/': /* May be comment starting with --, // or in C style */
					if(ptr[1]==*ptr)
					{ /* Advance ptr to next line: */
						wblog("  skipping one line comment which starts at %s\n",ptr);
						if( ptr[1]==*ptr )
							for( ptr+=2;*ptr!='\0';ptr++ )
							{
								if( *ptr=='\r' )
								{
									ptr++;
									if( *ptr=='\n' ) ptr++;
									break; /* end of comment */
								}
								else if( *ptr=='\n' ) { ptr++; break; }
							}
						else ptr++;
						wblog("  skipped to %s\n",ptr);
					}
					else if (*ptr=='/' && ptr[1]=='*') 
					{ /* Advance ptr to end of C-style comment: */
						wblog("  skipping C-style comment which starts at %s\n",ptr);
						for( ptr+=2;*ptr!='\0';ptr++ )
						{
							if( *ptr=='*'&&ptr[1]=='/' ) { ptr+=2; break; } /* end of comment */
						}
						wblog("  skipped to %s\n",ptr);
					}
					else ptr++;
					break;
				case '{': /* Comment in Pascal style */
					/*Advance ptr to end of Pascal-style coment: */
					wblog("  skipping Pascal-style comment which starts at %s\n",ptr);
					for( ptr++;*ptr!='}'&&*ptr!='\0';ptr++ ) ;
					if( *ptr=='}' ) ptr++;
					wblog("  skipped to %s\n",ptr);
					break;
				case ':': /* host var, < > <> >< or identifier may follow */
					{ /* Create and initialize new client variable; move ptr behind it: */
						/* After a colon, optional mode specifiers <, >, <>, or ><) may appear, and
						then comes name of the client variable. The client variable is bound to a PHP
						variable with the name exactly (full length and not ignoring case) as written
						in the SQL, and then its first NAMELEN characters are copied into the
						t_clivar structure; they are uppercased in the process. Unused characters are also
						deleted (i.e., replaced by spaces) from the SQL command. */
						zval * current_pzval;
						char *pom=ptr;
						int i;
						tname wbname;

						/* Read mode of client variable: */
						/* Default mode is MODE_INOUT. Only if there is just one of modifiers < > the mode
						is changed. */
						enum t_parmode parmode=MODE_INOUT;

						wblog("  colon found at %s\n",ptr);
						ptr++;
						if( *ptr=='<' )
						{
							if( ptr[1]=='>' ) { ptr+=2; }
							else { parmode=MODE_IN; ptr++; }
						}
						else if( *ptr=='>' )
						{
							if( ptr[1]=='<' ) { ptr+=2; }
							else { parmode=MODE_OUT; ptr++; }
						}
						/* Read name of client variable into emalloc-ed name; break if fails: */
						pom=ptr;
						if( *pom>='0' && *pom<='9' )
						{
							wblogs("  it's time or timestamp constant\n");
							break; /* time or timestamp, not identifier */
						}
						while( isalnum(*pom) || *pom=='_' || (*pom>='\200' && *pom<='\377') ) pom++;
						if( pom==ptr )
						{
							wblog("  no identifier found at %s\n",ptr);
							break; /* no identifier */
						}
						name=(char*)emalloc(pom-ptr+1);
						strncpy(name,ptr,pom-ptr);
						name[pom-ptr]='\0';
			/* Prepare 602SQL name of current client variable into wbname - max. NAMELEN long and uppercased: */
						{
							char *out=wbname;
							char *in=pom;
							while (in>ptr+NAMELEN) --in;
							out+=(in-ptr);
							*out='\0';
							while (in>ptr){
								*--out=toupper(*--in);
							}
						}
						wblog("  identifier %s found\n",wbname);
						/* If this client variable was read earlier don't add it to the list: */
						for( i=0;i<clivar_tablecount;i++ )
						{
							if( strcmp(wbname,clivar_table[i].name)==0 )
							{ // this variable was found earlier
								wblogs("  this variable was found earlier\n");
								if( parmode!=clivar_table[i].mode ) // if this occurence of variable has different mode than earlier one
									clivar_table[i].mode=MODE_INOUT; // change mode to INOUT
								break;
							}
						}
						if( i>=clivar_tablecount ) // if variable isn't already in list add it
						{
							wblogs("  adding variable to list of host vars\n");
							/* If there is more client variables than expected, allocate more space: */
							if(clivar_tablecount>=clivar_tablemaxcount)
							{
								void *tmp;
								wblogs("  resizing clivar_table and bound_variables arrays .. ");
								clivar_tablemaxcount+=clivar_tablegrow;
								tmp=erealloc(clivar_table, sizeof(struct t_clivar)*clivar_tablemaxcount);
								if( tmp==NULL ) goto cleanup;
								clivar_table=(struct t_clivar *)tmp;
								tmp = erealloc(bound_variables, sizeof(zval)*clivar_tablemaxcount);
								if( tmp==NULL ) goto cleanup;
								bound_variables=(zval ***)tmp;
								wblogs("OK\n");
							}
							/* Bind corresponding PHP variable and add it to bound_variables: */
							/* To create variables that can be seen ``from the outside'' by executing script, one
							needs to allocate new zval, fill it with meaningful values, and then introduce it to
							Zend's internal symbol table.

							Presented approach is probably not the optimal one. */

							current_zval=bound_variables+clivar_tablecount;
#if defined PHP_VERSION_ID && PHP_VERSION_ID>=50300
							if (!EG(active_symbol_table)) { zend_rebuild_symbol_table(TSRMLS_C); }
#endif
							if (SUCCESS==zend_hash_find(EG(active_symbol_table), name, strlen(name)+1,(void **)current_zval)) { }
							else {
								MAKE_STD_ZVAL(current_pzval);
								ZVAL_NULL(current_pzval);
								ZEND_SET_SYMBOL(EG(active_symbol_table), name, current_pzval);
								zend_hash_find(EG(active_symbol_table), name, strlen(name)+1,(void **)current_zval);
							}
							current_pzval=**current_zval;

							efree(name); name=NULL;
							/* Add clivar to clivar_table: */
							/* Client variables has to have assigned their type; unfortunately, there is no
							declaration of types in PHP. The problem is dealt with this way: if the
							variable has assigned type, the type is used. Otherwise, if the variable is
							used only for output, it is supposed to be a string of size longreadlen.
							Otherwise, error is issued.

							If a string variable is intended for an output, buffer is always resized to be
							at least longreadlen. */

							/* Copy name of variable to clivar: */
							/* Now, pom points behind the end of the name string, while ptr point to the string. */

							{
								char *in=pom;
								strcpy(clivar_table[clivar_tablecount].name,wbname);
								while (in>ptr+NAMELEN) *--in=' '; /* cut off trailing chars in SQL */
								ptr=pom;
							}
							wblog("clivar no. %d name is '%s'\n",clivar_tablecount,clivar_table[clivar_tablecount].name);
							clivar_table[clivar_tablecount].mode=parmode;
#if WBVERS>=800
							clivar_table[clivar_tablecount].specif=0;
#endif
							switch( Z_TYPE_P(current_pzval) )
							{
								case IS_LONG:
									clivar_table[clivar_tablecount].wbtype=ATT_INT32;
									clivar_table[clivar_tablecount].buf=&(current_pzval->value.lval);
									clivar_table[clivar_tablecount].buflen=sizeof(int);
									wblog("clivar no. %d is ATT_INT32\n",clivar_tablecount);
									break;
								case IS_NULL:
									if (parmode!=MODE_OUT){
										zend_error(E_WARNING, "602SQL: Attempt to use unassigned variable '%s' as a host",clivar_table[clivar_tablecount].name);
										goto cleanup;
									}
									ZVAL_STRINGL(current_pzval, NULL, 0, FALSE);
									wblog("clivar no. %d NULL\n",clivar_tablecount);
									/* no break! */
								case IS_STRING:
									/* If the variable is for output, resize buffer for at least longreadlen bytes: */
									if (parmode!=MODE_IN && WBG(longreadlen) > current_pzval->value.str.len)
									{
										char *tmp;
										if (0!=current_pzval->value.str.len)
										{
											tmp=(char*)erealloc(current_pzval->value.str.val, WBG(longreadlen));
										}
										else
										{
											tmp=(char*)emalloc(WBG(longreadlen));
											if(tmp) *tmp='\0';
										}
										if (tmp)
										{
											current_pzval->value.str.val=tmp;
											current_pzval->value.str.len=WBG(longreadlen);
											wblog("PHP string variable for clivar no. %d expanded to %ld bytes\n",clivar_tablecount,WBG(longreadlen));
										}
										else zend_error(E_NOTICE, "602SQL: Failed to expand buffer of host variable '%s'; using old one",clivar_table[clivar_tablecount].name);
									}
									if (WBG(datestring))
									{ /* Try to handle the string as date, time, or timestamp; break if it is so: */
										/* When datestring is set, strings passed as host variables are tested
										against being of date, time, or timestamp format. If they do match, we suppose
										that they are of appropriate type. */

										if (*current_pzval->value.str.val)
										{ /* null string */
											uns32 *value=(uns32*)emalloc(sizeof(uns32));
											clivar_table[clivar_tablecount].buf=value;
											clivar_table[clivar_tablecount].buflen=sizeof(uns32);
											if (str2date(current_pzval->value.str.val, value))
											{
												clivar_table[clivar_tablecount].wbtype=ATT_DATE;
												wblog("clivar no. %d is ATT_DATE\n",clivar_tablecount);
												break;
											}
											else if (str2time(current_pzval->value.str.val, value))
											{
												clivar_table[clivar_tablecount].wbtype=ATT_TIME;
												wblog("clivar no. %d is ATT_TIME\n",clivar_tablecount);
												break;
											}
											else if (str2timestamp(current_pzval->value.str.val, value))
											{
												clivar_table[clivar_tablecount].wbtype=ATT_TIMESTAMP;
												wblog("clivar no. %d is ATT_TIMESTAMP\n",clivar_tablecount);
												break;
											}
											else efree(value);
										}
									}
									clivar_table[clivar_tablecount].wbtype=ATT_TEXT;
									wblog("clivar no. %d is ATT_TEXT\n",clivar_tablecount);
#if WBVERS<800
									//clivar_table[clivar_tablecount].specif=0;
									clivar_table[clivar_tablecount].actlen=(Z_STRVAL_P(current_pzval)!=NULL)?strlen(Z_STRVAL_P(current_pzval)):0;
									clivar_table[clivar_tablecount].buf=Z_STRVAL_P(current_pzval);
									clivar_table[clivar_tablecount].buflen=Z_STRLEN_P(current_pzval)+1; // buflen have to include space for ending '\0' char too
#else
									clivar_table[clivar_tablecount].specif=0;
									switch( ENCODING(conn) )
									{
										case WB_ENC_ASCII:
											clivar_table[clivar_tablecount].buf=Z_STRVAL_P(current_pzval);
											clivar_table[clivar_tablecount].buflen=Z_STRLEN_P(current_pzval)+1; // buflen have to include space for ending '\0' char too
											clivar_table[clivar_tablecount].actlen=(Z_STRVAL_P(current_pzval)!=NULL)?strlen(Z_STRVAL_P(current_pzval)):0;
											break;
										case WB_ENC_WIN1250:
											T_SPECIF_CAST(clivar_table[clivar_tablecount].specif).charset=1;
											clivar_table[clivar_tablecount].buf=Z_STRVAL_P(current_pzval);
											clivar_table[clivar_tablecount].buflen=Z_STRLEN_P(current_pzval)+1; // buflen have to include space for ending '\0' char too
											clivar_table[clivar_tablecount].actlen=(Z_STRVAL_P(current_pzval)!=NULL)?strlen(Z_STRVAL_P(current_pzval)):0;
											break;
										case WB_ENC_WIN1252:
											T_SPECIF_CAST(clivar_table[clivar_tablecount].specif).charset=3;
											clivar_table[clivar_tablecount].buf=Z_STRVAL_P(current_pzval);
											clivar_table[clivar_tablecount].buflen=Z_STRLEN_P(current_pzval)+1; // buflen have to include space for ending '\0' char too
											clivar_table[clivar_tablecount].actlen=(Z_STRVAL_P(current_pzval)!=NULL)?strlen(Z_STRVAL_P(current_pzval)):0;
											break;
										case WB_ENC_ISO_8859_2:
											T_SPECIF_CAST(clivar_table[clivar_tablecount].specif).charset=129;
											clivar_table[clivar_tablecount].buf=Z_STRVAL_P(current_pzval);
											clivar_table[clivar_tablecount].buflen=Z_STRLEN_P(current_pzval)+1; // buflen have to include space for ending '\0' char too
											clivar_table[clivar_tablecount].actlen=(Z_STRVAL_P(current_pzval)!=NULL)?strlen(Z_STRVAL_P(current_pzval)):0;
											break;
										case WB_ENC_UTF8:
											T_SPECIF_CAST(clivar_table[clivar_tablecount].specif).wide_char=1;
											{ // convert current_pzval value from UTF-8 to UCS-2
												size_t tmplen;
												char *tmp=UTF8phpval2UCS2buf(current_pzval,&tmplen);
												if( tmp==NULL )
												{
													goto cleanup;
												}
												// set clivar buf to UCS-2 string
												clivar_table[clivar_tablecount].buf=tmp;
												clivar_table[clivar_tablecount].buflen=tmplen+2; // buflen have to include space for ending '\0\0' UCS-2 char too
												clivar_table[clivar_tablecount].actlen=UCS2bytelen(tmp);
												// current_pzval value is preserved
												wblog("host var %s converted from UTF-8 to UCS-2:\n",clivar_table[clivar_tablecount].name);
												wblog("buflen=%d actlen=%d string=%s\n",clivar_table[clivar_tablecount].buflen,clivar_table[clivar_tablecount].actlen,Z_STRVAL_P(current_pzval));
												wblogmem("UTF-8 string:",Z_STRVAL_P(current_pzval),tmplen);
												wblogmem("UCS-2 string:",tmp,tmplen*2+2);
											}
											break;
										default:
											zend_error(E_WARNING,"602SQL: invalid client encoding number %d",ENCODING(conn));
											goto cleanup;
									}
#endif
									break;
								case IS_BOOL:
									clivar_table[clivar_tablecount].wbtype=ATT_BOOLEAN;
									wblog("clivar no. %d is ATT_BOOLEAN, value is 0x%x\n",clivar_tablecount,current_pzval->value.lval);
									clivar_table[clivar_tablecount].buf=&(current_pzval->value.lval);
									clivar_table[clivar_tablecount].buflen=sizeof(current_pzval->value.lval);
									break;
								case IS_DOUBLE:
									clivar_table[clivar_tablecount].wbtype=ATT_FLOAT;
									wblog("clivar no. %d is ATT_FLOAT\n",clivar_tablecount);
									clivar_table[clivar_tablecount].buf=&(current_pzval->value.dval);
									clivar_table[clivar_tablecount].buflen=sizeof(current_pzval->value.dval);
									break;
								default:
									zend_error(E_WARNING,
										"602SQL: You attempt to use a complex type variable '%s' as host",clivar_table[clivar_tablecount].name);
									goto cleanup;

							}
							clivar_tablecount++;
						}
					}
					break;
				default: ptr++;
			}
		}
	}
	/* Execute SQL command in command and store the answer to result: */
	/* 602SQL API offers cd_SQL_host_execute for SQL command execution,
	 * and it expects an array of 32 bit integers passed as last argument,
	 * the length of array being at least the lesser of (number of SQL
	 * commands, 10). It fills this array with results, which may be
	 * number of affected records (all 32 bits), 16 bit long number of
	 * cursor, padded with 0xffff, or, in cases of object creation,
	 * object identifier. We shall suppose only first two cases are of
	 * importance; as the user usually know what passes	to the server, it
	 * should not make much harm. Only first result is used; other cursors
	 * are closed and any other resutls ignored.	For simplicity, we
	 * expect only one SQL command is performed for now. */
	{
		uns32 *ptr;
		result[0]=0;
		for (ptr=result+1; ptr<result+MAX_RETURN_FIELDS; ptr++) *ptr=0xffffffff;
		wblog("clivar count=%d\n",clivar_tablecount);
		if(cd_SQL_host_execute(cdp, tmp_command, result,
			clivar_table, clivar_tablecount))
		{
			CONN_REPORT_ERR(conn,"SQL_execute");
			goto cleanup;
		}
		EFREE_EMALLOC_STRING(emalloc_command);
		for (ptr=result+1; ptr<result+MAX_RETURN_FIELDS; ptr++)
			if (IS_CURSOR(*ptr))
				if (cd_Close_cursor(cdp, CURSOR(*ptr)))
					CONN_REPORT_ERR(conn,"Close_cursor");
		/* If there were client variables, fix their return values: */
		/* Few fixes of client variables are necessary. First, strings are sometime
		returned from 602SQL without terminating NULL. Also their actual length must
		be passed to the PHP.

	In case that current connection uses UTF8 encoding for string client variables
	we have to convert values of OUT and INOUT variables from UCS-2 to UTF8 encoding.

		Another chapter date and time functions, that must be converted from 602SQL
		native representation to a string. */

		if (clivar_tablecount)
		{		
			int i;
			zval ***current_zval=bound_variables;
			for (i=0; i<clivar_tablecount; i++, current_zval++)
			{
				zval *current_pzval=**current_zval;
				switch (clivar_table[i].wbtype)
				{
					case ATT_TEXT:
#if WBVERS>=1100
					case ATT_EXTCLOB:
#endif
#if WBVERS>=800
				if( T_SPECIF_CAST(clivar_table[i].specif).wide_char )
				{
					if( clivar_table[i].mode==MODE_IN )
					{
						// free emalloc-ed buffer
						if(	clivar_table[i].buf!=NULL )
						{
							efree(clivar_table[i].buf);
							clivar_table[i].buf=NULL;
							clivar_table[i].actlen=clivar_table[i].buflen=0;
						}
					}
					else
					{
						if( !wbstring2phpvalue(
#ifdef USE_WB_SCRIPT_ENCODING
									WBG(script_encoding),
#endif
									(uns8)clivar_table[i].wbtype,T_SPECIF_CAST(clivar_table[i].specif),(void*)(clivar_table[i].buf),current_pzval) )
						{
							wblogs("wbstring2phpvalue() failed\n");
							goto cleanup;
						}
						efree(clivar_table[i].buf); // buffer was emalloc-ed during SQL statement client variables preparation earlier in this function
						clivar_table[i].buf=NULL;
						clivar_table[i].actlen=clivar_table[i].buflen=0;
						wblog("finished copying value of host var %s to PHP var\n",clivar_table[i].name);
					}
				}
				else
#endif
				{
					current_pzval->value.str.len=clivar_table[i].actlen;
					current_pzval->value.str.val[clivar_table[i].actlen]=0;
				}
							break;
					case ATT_TIME:
					case ATT_DATE:
					case ATT_TIMESTAMP:
						if (clivar_table[i].mode!=MODE_IN)
						{
							uns32 value=*(uns32 *)(clivar_table[i].buf);
							switch(clivar_table[i].wbtype)
							{
								case ATT_TIMESTAMP:
									timestamp2str(value,current_pzval->value.str.val, 5);
									current_pzval->value.str.len=strlen(current_pzval->value.str.val);
									break;
								case ATT_TIME:
									time2str(value, current_pzval->value.str.val, 2);
									current_pzval->value.str.len=strlen(current_pzval->value.str.val);
									break;
								case ATT_DATE:
									date2str(value, current_pzval->value.str.val, 1);
									current_pzval->value.str.len=
									strlen(current_pzval->value.str.val);
									break;
							}
						}
						efree(clivar_table[i].buf);
						break;
			case ATT_BOOLEAN:
			wblog("clivar no. %i is ATT_BOOLEAN, value is 0x%x\n",i,current_pzval->value.lval);
			// convert value to PHP's boolean type
			// 602SQL sets least significant byte of current_pzval->value.lval only
			if( (current_pzval->value.lval&0xff)==NONEBOOLEAN ) { ZVAL_NULL(current_pzval); }
			else if( current_pzval->value.lval&0xff ) { ZVAL_TRUE(current_pzval); }
			else { ZVAL_FALSE(current_pzval); }
			wblog("	value changed to 0x%x\n",current_pzval->value.lval);
			break;
					default:
						break;
				}
			}
			efree(clivar_table);
			efree(bound_variables);
		}
	}
	
	if (IS_CURSOR(*result))
	{
		struct sql_answer* answer;
		int id;
		if( !create_cursor_resource(conn,result[0],&answer TSRMLS_CC) ) {
			WBLSM_END;
			RETURN_FALSE;
		}
		id=zend_list_insert(answer, wbres_cursor
#if PHP_VERSION_ID>50400
				TSRMLS_CC
#endif
				);
		wblog("wb_exec leave, result=%ld (cursor)\n",id);
		WBLSM_END;
		RETURN_RESOURCE(id);
	} 
	else 
	{
		wblog("wb_exec leave, result=%ld\n",result[0]);
		WBLSM_END;
		RETURN_LONG((result[0]==0)?-1:(int)result[0])
	}
cleanup: /* used when search for clivars fails */
	/* Cleanup for failed clivar search: */
	/* There are many points when work with client variables may fail, from scanning the
	string to error in the execution at the server. All these failures go here.
	The PHP memory allocation mechanism does not allow freeing NULL for now. */

	wblogs("wb_exec leave, result=FALSE\n");
	EFREE_EMALLOC_STRING(emalloc_command);
	if (bound_variables) efree(bound_variables);
	if (clivar_table) efree(clivar_table);
	if (name) efree(name);
	WBLSM_END;
	RETURN_FALSE;
}
/*}}}*/
/*}}}*/
/*{{{ wb_fetch_into */
ZEND_FUNCTION(wb_fetch_into)
{
	WBLSM_DECLARE
	zval **result_id; /* cursor resource */
	zval **rownumber; /* starts from 1 */
	zval **resarray; /* result array */
	zval **restype; /* result type */
	tcursnum cursor;
	int rownum; /* from 0, -1 means not determined yet */

	struct sql_answer* answer; /* cursor number or number of	*/
	int i,result_type;

	wblogs("wb_fetch_into enter\n");

	switch(ZEND_NUM_ARGS())
	{
		case 4:
			if (zend_get_parameters_ex(4, &result_id, &resarray, &rownumber, &restype) != SUCCESS)
				WRONG_PARAM_COUNT;
			convert_to_long_ex(rownumber);
			if( Z_LVAL_PP(rownumber)>0 ) rownum=Z_LVAL_PP(rownumber)-1;
			else rownum=-1;
			convert_to_long_ex(restype);
			result_type=Z_LVAL_PP(restype);
			break;
		case 3:
			if (zend_get_parameters_ex(3, &result_id, &resarray, &rownumber) != SUCCESS)
				WRONG_PARAM_COUNT;
			convert_to_long_ex(rownumber);
			if( Z_LVAL_PP(rownumber)>0 ) rownum=Z_LVAL_PP(rownumber)-1;
			else rownum=-1;
			result_type=WB_FETCH_BOTH1; // default result_type
			break;
		case 2: /* Parameter rownumber may be ommited */
			if (zend_get_parameters_ex(2, &result_id, &resarray) != SUCCESS)
				WRONG_PARAM_COUNT;
			rownum=-1; /* will be retrieved later */
			result_type=WB_FETCH_BOTH1; // default result_type
			break;
		default:
			WRONG_PARAM_COUNT;
	}
	wblog("wb_fetch_into: rownum=%d\n",rownum);
	/* Check whether third parameter is a reference: */
	/* Reading whole record.
	The wb_fetch_into PHP function reads whole row (specified by the second argument) of
	the table given as first argument into an array created in the third argument.

	The third parameter must be passed as a referrence.
	We use a macro for a check, because it may come handy later. Macro for
	checking this disappeared in new versions of PHP API. */

#if 0
	if (!ParameterPassedByReference(ht, 2)){
		zend_error(E_WARNING, "Parameter not passed by reference");
		RETURN_FALSE;
	}
#endif
	if( (answer=get_answer_from_result(INTERNAL_FUNCTION_PARAM_PASSTHRU,result_id))==NULL )
	{
		wblogs("wb_fetch_into leave, FALSE, get_answer_from_result() failed\n");
		RETURN_FALSE;
	}
	WBLSM_START("wb_fetch_into",CDP_FROM_ANSWER_P(answer));
	if (rownum==-1) rownum=++(answer->current_record);
	else answer->current_record=rownum;
	if ((unsigned)rownum >= (unsigned)answer->rec_cnt)
	{
		wblog("wb_fetch_into leave, FALSE, (unsigned)rownum >= (unsigned)answer->rec_cnt i.e. %u>=%u\n",(unsigned)rownum,(unsigned)answer->rec_cnt);
		WBLSM_END;
		RETURN_FALSE;
	}
	cursor=answer->cursor;

	/* Create an array of values of all columns in SQL result: */
	/*		 - Create an array of values.. */

	if( Z_TYPE_PP(resarray)!=IS_ARRAY && array_init(*resarray)!=SUCCESS )
	{
		zend_error(E_WARNING, "602SQL: Can not create output array");
		wblogs("wb_fetch_into leave, FALSE, array_init() failed\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	for (i=0; i<answer->attrs; i++)
	{
		zval *new_element;
		MAKE_STD_ZVAL(new_element);
		if (!db_item_to_php(
#ifdef USE_WB_SCRIPT_ENCODING
					WBG(script_encoding),
#endif
					answer, rownum, i, new_element, 1 TSRMLS_CC))
		{
			FREE_ZVAL(new_element);
			ZVAL_NULL(*resarray);
			wblogs("wb_fetch_into leave, result=FALSE\n");
			WBLSM_END;
			RETURN_FALSE;
		}
		/* Add column value into array */
		if( result_type&WB_FETCH_ASSOC )
		{
			//zend_hash_update(Z_ARRVAL_PP(resarray),answer->names[i],strlen(answer->names[i])+1,(void *)&new_element,sizeof(zval*),NULL);
			if( Z_TYPE_P(new_element)==IS_RESOURCE )
			{
				add_assoc_resource(*resarray,answer->names[i],Z_RESVAL_P(new_element));
			}
			else add_assoc_zval(*resarray,answer->names[i],new_element);
			wblogs("wb_fetch_into: after zend_hash_update\n");
		}
		if( result_type&(WB_FETCH_NUM0|WB_FETCH_NUM1) )
		{
			//zend_hash_index_update(Z_ARRVAL_PP(resarray),(result_type&WB_FETCH_NUM1)?i+1:i,(void *)&new_element,sizeof(zval*),NULL);
			if( Z_TYPE_P(new_element)==IS_RESOURCE )
			{
				add_index_resource(*resarray,(result_type&WB_FETCH_NUM1)?i+1:i,Z_RESVAL_P(new_element));
			}
			else
			{
				add_index_zval(*resarray,(result_type&WB_FETCH_NUM1)?i+1:i,new_element);
				if( result_type&WB_FETCH_ASSOC )
				{
#ifdef Z_ADDREF_P
					// this macro is defined in PHP 5.3.0
					Z_ADDREF_P(new_element);
#else
				 	new_element->refcount++;
#endif
				}
			}
			wblogs("wb_fetch_into: after zend_hash_index_update\n");
		}
		// when new_element is resource (i.e. LOB resource) increase its reference counter
		if( Z_TYPE_P(new_element)==IS_RESOURCE ) zend_list_addref(Z_RESVAL_P(new_element));
		/* slightly older version of column value addition:
		wblogs("wb_fetch_into: after db_item_to_php\n");
		add_index_zval(*resarray, 1+i, new_element);
		wblogs("wb_fetch_into: after add_index_zval\n");
		add_assoc_zval(*resarray,answer->names[i], new_element);
		wblogs("wb_fetch_into: after add_assoc_zval\n");
		*/
	}
	wblog("wb_fetch_into leave, result=%ld\n",answer->attrs);
	WBLSM_END;
	RETURN_LONG(answer->attrs);
}
/*}}}*/
/*{{{ wb_field_name
In the direct analogy to odbc_field_name, the wb_field_name ``will return the name
of the field occupying the given column number in the given ODBC result identifier.
Field numbering starts at 1. FALSE is returned on error.''

The implementation is straightforward, as we have built up the column table in wb_exec. */
ZEND_FUNCTION(wb_field_name)
{
	WBLSM_DECLARE
	zval **result_id, **field_number;
	struct sql_answer *answer;

	wblogs("wb_field_name enter\n");

	if (ZEND_NUM_ARGS()!=2||zend_get_parameters_ex(2, &result_id, &field_number)!=SUCCESS) WRONG_PARAM_COUNT;

	if( (answer=get_answer_from_result(INTERNAL_FUNCTION_PARAM_PASSTHRU,result_id))==NULL )
	{
		wblogs("wb_field_name leave, FALSE\n");
		RETURN_FALSE;
	}
	WBLSM_START("wb_field_name",CDP_FROM_ANSWER_P(answer));
	convert_to_long_ex(field_number);
	if (answer->attrs<Z_LVAL_PP(field_number))
	{
		zend_error(E_WARNING, "Column number too high!");
		wblogs("wb_field_name leave, FALSE\n");
		WBLSM_END;
		RETURN_FALSE;
	}

	wblogs("wb_field_name leave\n");

	if (answer->types)
	{
		wblog("wb_field_name leave, result=%s\n",answer->names[Z_LVAL_PP(field_number)-1]);
#ifdef USE_WB_SCRIPT_ENCODING
		if( IS_STRING_CONVERSION_REQUIRED(CONN_P_FROM_ANSWER_P(answer)->system_encoding,WBG(script_encoding)) )
		{
			char *emalloc_string=estrdup_with_encoding(answer->names[Z_LVAL_PP(field_number)-1],CONN_P_FROM_ANSWER_P(answer)->system_encoding,WBG(script_encoding));
			WBLSM_END;
			RETURN_STRING(emalloc_string,FALSE/* return_value takes ownership of emalloc_string buffer */);
		}
		else
		{
			WBLSM_END;
			RETURN_STRING(answer->names[Z_LVAL_PP(field_number)-1], TRUE);
		}
#else
		WBLSM_END;
		RETURN_STRING(answer->names[Z_LVAL_PP(field_number)-1], TRUE);
#endif
	}
	wblogs("wb_field_name leave, FALSE (answer->types is NULL)\n");
	WBLSM_END;
	RETURN_FALSE;
}
/*}}}*/
/*{{{ wb_field_num */
ZEND_FUNCTION(wb_field_num)
{
	WBLSM_DECLARE
	zval **result_id, **field_name, **silent_arg;
	struct sql_answer *answer;
	int silent,column;
	char *tmp_field_name;
#ifdef USE_WB_SCRIPT_ENCODING
	char *emalloc_field_name=NULL;
#endif

	wblogs("wb_field_num enter\n");

	switch(ZEND_NUM_ARGS())
	{
		case 2:
			if( zend_get_parameters_ex(2, &result_id, &field_name)!=SUCCESS ) WRONG_PARAM_COUNT;
			silent=0;
			break;
		case 3:
			if( zend_get_parameters_ex(3,&result_id,&field_name,&silent_arg)!=SUCCESS ) WRONG_PARAM_COUNT;
			convert_to_boolean_ex(silent_arg);
			silent=Z_BVAL_PP(silent_arg);
			break;
		default:
			WRONG_PARAM_COUNT;
	}

	if( (answer=get_answer_from_result(INTERNAL_FUNCTION_PARAM_PASSTHRU,result_id))==NULL )
	{
		wblogs("wb_field_num leave, FALSE\n");
		RETURN_FALSE;
	}
	WBLSM_START("wb_field_num",CDP_FROM_ANSWER_P(answer));
	convert_to_string_ex(field_name);
	if( !PREPARE_STRING_SCRIPT2SYSTEM(WBG(script_encoding),CONN_P_FROM_ANSWER_P(answer),&tmp_field_name,&emalloc_field_name,Z_STRVAL_PP(field_name)) )
	{
		wblogs("wb_field_num leave, ERROR in PREPARE_STRING_SCRIPT2SYSTEM()\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	Upcase(tmp_field_name); // column name have to be uppercased
	for (column=0; column < answer->total_attrs; column++)
	{
  	if( strcmp(answer->names[column],tmp_field_name)==0 ) break;
	}
	EFREE_EMALLOC_STRING(emalloc_field_name);
	if (column==answer->total_attrs) { /* failed to find */
		if( !silent ) zend_error(E_WARNING, "602SQL: Column \"%s\" not present in the table",Z_STRVAL_PP(field_name));
		wblogs("wb_field_num leave, FALSE\n");
		WBLSM_END;
		RETURN_FALSE;
	}

	if( answer->attrs<column )
	{
		zend_error(E_WARNING, "Column number too high!");
		wblogs("wb_field_num leave, FALSE\n");
		WBLSM_END;
		RETURN_FALSE;
	}

	wblogs("wb_field_num leave\n");

	if (answer->types)
	{
		wblog("wb_field_num leave, result=%d\n",column+1);
		WBLSM_END;
		RETURN_LONG(column+1);
	}
	wblogs("wb_field_num leave, FALSE (answer->types is NULL)\n");
	WBLSM_END;
	RETURN_FALSE;
}
/*}}}*/
/*{{{ wb_field_type */
/*{{{ static const char *get_SQL_type_name(uns8 wbtype,attrspecif_t specif) */
static const char *get_SQL_type_name(uns8 wbtype
#if WBVERS>=800
		,attrspecif_t specif
#endif
		)
{
#if WBVERS<900
#define return_integer_type_name(inttypename) return inttypename;
#else
#define return_integer_type_name(inttypename) if( specif.scale==0 ) return inttypename; else return "NUMERIC";
#endif
	wblog("get_SQL_type_name() enter, wbtype=%d\n",wbtype);
	switch( wbtype )
	{
	case ATT_BOOLEAN: return "BOOLEAN";
	case ATT_CHAR: return "CHAR";
	case ATT_INT8: return_integer_type_name("TINYINT");
	case ATT_INT16: return_integer_type_name("SMALLINT");
	case ATT_INT32: return_integer_type_name("INT");
	case ATT_INT64: return_integer_type_name("BIGINT");
	case ATT_MONEY: return "NUMERIC(14,2)";
	case ATT_FLOAT: return "REAL";
	case ATT_BINARY: return "BINARY";
#if WBVERS<800
	case ATT_CSSTRING:
	case ATT_CSISTRING:
#endif
	case ATT_STRING:
#if WBVERS>=800
					 if( specif.wide_char ) return "NCHAR";
#endif
					 return "VARCHAR";
	case ATT_DATE: return "DATE";
	case ATT_TIME: return "TIME";
	case ATT_TIMESTAMP: return "TIMESTAMP";
	case ATT_RASTER: return "VARBINARY";
	case ATT_TEXT:
#if WBVERS>=800
					 if( specif.wide_char ) return "NCLOB";
#endif
					 return "CLOB";
	case ATT_NOSPEC: return "BLOB";
#if WBVERS>=1100
	case ATT_EXTCLOB:
						if( specif.wide_char ) return "EXTNCLOB";
						else return "EXTCLOB";
	case ATT_EXTBLOB:
						return "EXTBLOB";
#endif
	default: return NULL;
	}
}
/*}}}*/
ZEND_FUNCTION(wb_field_type)
{
	WBLSM_DECLARE
	zval **result_id, **field;
	struct sql_answer *answer;
	int column;

	wblogs("wb_field_type enter\n");

	if (ZEND_NUM_ARGS()!=2||zend_get_parameters_ex(2, &result_id, &field)!=SUCCESS) WRONG_PARAM_COUNT;

	if( (answer=get_answer_from_result(INTERNAL_FUNCTION_PARAM_PASSTHRU,result_id))==NULL )
	{
		wblogs("wb_field_type leave, FALSE: answer is NULL\n");
		RETURN_FALSE;
	}
	WBLSM_START("wb_field_type",CDP_FROM_ANSWER_P(answer));
	if (Z_TYPE_PP(field)==IS_LONG) column=Z_LVAL_PP(field)-1;
	else {
		char *tmp_field;
#ifdef USE_WB_SCRIPT_ENCODING
		char *emalloc_field=NULL;
#endif
		/* Get column number from column name: */
		/* We simply traverse all the columns to find the right one. Hopefuly
		 * there are not hundreds of them. */

		convert_to_string_ex(field);
		if( !PREPARE_STRING_SCRIPT2SYSTEM(WBG(script_encoding),CONN_P_FROM_ANSWER_P(answer),&tmp_field,&emalloc_field,Z_STRVAL_PP(field)) )
		{
			wblogs("wb_field_type leave, FALSE: PREPARE_STRING_SCRIPT2SYSTEM() failed\n");
			WBLSM_END;
			RETURN_FALSE;
		}
		wblog("column name=\"%s\"\n",tmp_field);
  	Upcase(tmp_field); // column name have to be uppercased
		for (column=0; column < answer->total_attrs; column++)
		{
		  if (strcmp(answer->names[column],tmp_field)==0 ) break;
		}
		EFREE_EMALLOC_STRING(emalloc_field);
		if (column==answer->total_attrs) { /* failed to find */
			zend_error(E_WARNING, "602SQL: Column \"%s\" not present in the table",Z_STRVAL_PP(field));
			wblogs("wb_field_type leave, FALSE\n");
			WBLSM_END;
			RETURN_FALSE;
		}
	}
	if (answer->attrs<column)
	{
		zend_error(E_WARNING, "Column number too high!");
		wblogs("wb_field_type leave, FALSE\n");
		WBLSM_END;
		RETURN_FALSE;
	}

	if (answer->types)
	{
		const char *ptr=get_SQL_type_name(answer->types[column]
#if WBVERS>=800
			,answer->specif[column]
#endif
			);
		if( ptr==NULL )
		{
			zend_error(E_WARNING,"602SQL: Name of type of column \"%s\" is unknown, type number is %d",answer->names[column],answer->types[column]);
			wblogs("wb_field_type leave, FALSE: get_SQL_type_name() result is NULL\n");
			WBLSM_END;
			RETURN_FALSE;
		}
		else
		{
			wblog("wb_field_type leave, %s\n",ptr);
			WBLSM_END;
			RETURN_STRING((char*)ptr,TRUE);
		}
	}
	wblogs("wb_field_type leave, FALSE: answer->types is NULL\n");
	WBLSM_END;
	RETURN_FALSE;
}
/*}}}*/
/*{{{ wb_field_type_num */
ZEND_FUNCTION(wb_field_type_num)
{
	WBLSM_DECLARE
	zval **result_id, **field;
	struct sql_answer *answer;
	int column;

	wblogs("wb_field_type_num enter\n");

	if (ZEND_NUM_ARGS()!=2||zend_get_parameters_ex(2, &result_id,
				&field)!=SUCCESS) WRONG_PARAM_COUNT;

	if( (answer=get_answer_from_result(INTERNAL_FUNCTION_PARAM_PASSTHRU,result_id))==NULL )
	{
		wblogs("wb_field_type_num leave, FALSE\n");
		RETURN_FALSE;
	}
	WBLSM_START("wb_field_type_num",CDP_FROM_ANSWER_P(answer));
	if (Z_TYPE_PP(field)==IS_LONG) column=Z_LVAL_PP(field)-1;
	else {
		char *tmp_field;
#ifdef USE_WB_SCRIPT_ENCODING
		char *emalloc_field=NULL;
#endif
		/* Get column number from column name: */
		/* We simply traverse all the columns to find the right one. Hopefuly
		 * there are not hundreds of them. */

		convert_to_string_ex(field);
		if( !PREPARE_STRING_SCRIPT2SYSTEM(WBG(script_encoding),CONN_P_FROM_ANSWER_P(answer),&tmp_field,&emalloc_field,Z_STRVAL_PP(field)) )
		{
			wblogs("wb_field_type_num leave, ERROR: PREPARE_STRING_SCRIPT2SYSTEM(field) failed\n");
			WBLSM_END;
			RETURN_FALSE;
		}
  	Upcase(tmp_field); // column name have to be uppercased
		for (column=0; column < answer->total_attrs; column++)
		{
			if( strcmp(answer->names[column],tmp_field)==0 ) break;
		}
		EFREE_EMALLOC_STRING(emalloc_field);
		if (column==answer->total_attrs) { /* failed to find */
			zend_error(E_WARNING, "602SQL: Column \"%s\" not present in the table",Z_STRVAL_PP(field));
			wblogs("wb_field_type_num leave, FALSE\n");
			WBLSM_END;
			RETURN_FALSE;
		}
	}
	if (answer->attrs<column)
	{
		zend_error(E_WARNING, "Column number too high!");
		wblogs("wb_field_type_num leave, FALSE\n");
		WBLSM_END;
		RETURN_FALSE;
	}

	wblog("wb_field_type_num leave, %ld\n",(answer->types)?answer->types[column]:0);
  WBLSM_END;
	RETURN_LONG((answer->types)?answer->types[column]:0);
}
/*}}}*/
/*{{{ wb_num_fields
Another simple and direct analog; wb_num_fields(int result_id) ``will return the
number of fields (columns) in a result. This function will return -1 on error''. */
ZEND_FUNCTION(wb_num_fields)
{
	WBLSM_DECLARE
	zval **result_id;
	struct sql_answer *answer;
	int result;

	wblogs("wb_num_fields enter\n");
	if (ZEND_NUM_ARGS()!=1||zend_get_parameters_ex(1, &result_id)!=SUCCESS)
		WRONG_PARAM_COUNT;
	if( (answer=get_answer_from_result(INTERNAL_FUNCTION_PARAM_PASSTHRU,result_id))==NULL )
	{
		wblogs("wb_num_fields leave, FALSE\n");
		RETURN_FALSE;
	}
	WBLSM_START("wb_num_fields",CDP_FROM_ANSWER_P(answer));
	result=(answer && answer->types)?answer->attrs:-1;
	wblog("wb_num_fields leave, result is %ld\n",result);
	WBLSM_END;
	RETURN_LONG(result);
}
/*}}}*/
/*{{{ wb_fetch_row */
ZEND_FUNCTION(wb_fetch_row)
{
	WBLSM_DECLARE
	int argc=ZEND_NUM_ARGS();
	zval **result_id, **row_number;
	struct sql_answer *answer;

	wblogs("wb_fetch_row enter\n");

	if (argc>2||argc<1||
		zend_get_parameters_ex(argc, &result_id, &row_number)!=SUCCESS)
			WRONG_PARAM_COUNT;

	if( (answer=get_answer_from_result(INTERNAL_FUNCTION_PARAM_PASSTHRU,result_id))==NULL )
	{
		wblogs("wb_fetch_row leave, FALSE\n");
		RETURN_FALSE;
	}
	WBLSM_START("wb_fetch_row",CDP_FROM_ANSWER_P(answer));

	wblog("old value of answer->current_record=%d\n",answer->current_record);
	if (argc==1) answer->current_record++;
	else {
		convert_to_long_ex(row_number);
		answer->current_record=Z_LVAL_PP(row_number)-1;
	}
	wblog("new value of answer->current_record=%d\n",answer->current_record);
	if (answer->rec_cnt!=(trecnum)-1) {
		int result=answer->rec_cnt==(trecnum)-1 || answer->current_record<answer->rec_cnt;
		wblog("wb_fetch_row leave, result=%d\n",result);
		WBLSM_END;
		RETURN_BOOL(result)
	}
	else {
		uns32 size;
		int result=1;
		cdp_t cdp=CDP_FROM_ANSWER_P(answer);
		if( cd_Read_ind(cdp, answer->cursor, answer->current_record, 0, NOINDEX, &size) ) {
			int err;
			if( (err=cd_Sz_error(cdp))==OUT_OF_TABLE ) {
				/* answer->current_record record doesn't exist in cursor */
				if( answer->rec_cnt==(trecnum)-1 ) {
					/* set the number of records in cursor if it wasn't already done */
					answer->rec_cnt=answer->current_record; /* first record has 0 number and current_record
																									 * is first record number which is missing in cursor */
				}
			}
			else {
				report_err_from_number(CONN_P_FROM_ANSWER_P(answer),err,"Read_ind" TSRMLS_CC);
			}
			result=0;
		}
		wblog("wb_fetch_row leave, (cd_Read_ind() executed) result=%d\n",result);
		WBLSM_END;
		RETURN_BOOL(result);
	}
}
/*}}}*/
/*{{{ bool wb_data_seek(result_id,row_number)
 * Analogous to MySQL mysql_data_seek function, moves internal result pointer of result_id
 * result to value specified by row_number parameter.
 * Returns TRUE on success, FALSE on error. */
ZEND_FUNCTION(wb_data_seek)
{
	WBLSM_DECLARE
	zval **result_id, **row_number;
	struct sql_answer *answer;

	wblogs("wb_data_seek enter\n");

	if( ZEND_NUM_ARGS()!=2||zend_get_parameters_ex(2, &result_id, &row_number)!=SUCCESS )
			WRONG_PARAM_COUNT;

	if( (answer=get_answer_from_result(INTERNAL_FUNCTION_PARAM_PASSTHRU,result_id))==NULL )
	{
		wblogs("wb_data_seek leave, FALSE\n");
		RETURN_FALSE;
	}
	WBLSM_START("wb_data_seek",CDP_FROM_ANSWER_P(answer));

	convert_to_long_ex(row_number);

	wblog("answer->rec_cnt=%d row_number=%d\n",answer->rec_cnt,Z_LVAL_PP(row_number));
	if( Z_LVAL_PP(row_number)<1 || (answer->rec_cnt!=(trecnum)-1 && (trecnum)(Z_LVAL_PP(row_number))>answer->rec_cnt) ) {
		zend_error(E_WARNING,"Column number too high!");
		wblogs("wb_data_seek leave, result=FALSE\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	/* Set current_record such a way that the next call of wb_fetch_row will read
	 * the (*row_number)->value.lval-1 record. */
	answer->current_record=Z_LVAL_PP(row_number)-2;

	wblogs("wb_data_seek leave, result=TRUE\n");
	WBLSM_END;
	RETURN_TRUE;
}
/*}}}*/
/*{{{ wb_result
Field indices start at 1. An error occurs if a column number parameter is less than
one or exceeds the number of columns in the current record (we do not check it now, but
602SQL does). */
ZEND_FUNCTION(wb_result)
{
	WBLSM_DECLARE
	zval **result_id, **field;
	struct sql_answer *answer;
	int column;

	wblogs("wb_result enter\n");

	if (ZEND_NUM_ARGS()!=2|| zend_get_parameters_ex(2, &result_id, &field)!=SUCCESS)
			WRONG_PARAM_COUNT;

	if( (answer=get_answer_from_result(INTERNAL_FUNCTION_PARAM_PASSTHRU,result_id))==NULL )
	{
		wblogs("wb_result leave, FALSE\n");
		RETURN_FALSE;
	}
	WBLSM_START("wb_result",CDP_FROM_ANSWER_P(answer));

	if (Z_TYPE_PP(field)==IS_LONG) column=Z_LVAL_PP(field)-1;
	else {
		char *tmp_field;
#ifdef USE_WB_SCRIPT_ENCODING
		char *emalloc_field=NULL;
#endif
		/* Get column number from column name: */
		/* We simply traverse all the columns to find the right one. Hopefuly
		 * there are not hundreds of them. */

		convert_to_string_ex(field);
		if( !PREPARE_STRING_SCRIPT2SYSTEM(WBG(script_encoding),CONN_P_FROM_ANSWER_P(answer),&tmp_field,&emalloc_field,Z_STRVAL_PP(field)) )
		{
			wblogs("wb_result leave, ERROR: PREPARE_STRING_SCRIPT2SYSTEM() failed\n");
			WBLSM_END;
			RETURN_FALSE;
		}
		Upcase(tmp_field); // column name have to be uppercased
		for (column=0; column < answer->total_attrs; column++)
		{
			if( strcmp(answer->names[column],tmp_field)==0 ) break;
		}
		EFREE_EMALLOC_STRING(emalloc_field);
		if (column==answer->total_attrs) { /* failed to find */
			zend_error(E_WARNING, "602SQL: Column \"%s\" not present in the table",Z_STRVAL_PP(field));
			wblogs("wb_result leave, FALSE\n");
			WBLSM_END;
			RETURN_FALSE;
		}
	}
	if( !db_item_to_php(
#ifdef USE_WB_SCRIPT_ENCODING
				WBG(script_encoding),
#endif
				answer, answer->current_record, column, return_value, 0 TSRMLS_CC) )
	{
		wblogs("wb_result leave, FALSE, error in db_item_to_php\n");
		WBLSM_END;
		RETURN_FALSE;
	}

	/* there was performed following conversion of result in previous versions:
	 *	 convert_to_string_ex(&return_value);
	 * it was nonsense :( we can't convert result at least in case that result is
	 * LOB resource however in all other cases this only hides actual type of
	 * result (and is time and memory consuming too) */

	wblogs("wb_result leave\n");
	WBLSM_END;
}
/*}}}*/
/*{{{ wb_result_all
The most easy-to-use function on a result is display_result_all. It prints out
all rows in a HTML table format; resource identifier is in the first parameter, an
optional second parameter is used for formatting. Returns FALSE on error, number of
rows otherwise. */
ZEND_FUNCTION(wb_result_all)
{
	WBLSM_DECLARE
	zval **pv_result_id, **pv_format;
	trecnum row;
	int column;
	zval cell_value;
	int numArgs;
	struct sql_answer *answer;

	wblogs("wb_result_all enter\n");

	numArgs = ZEND_NUM_ARGS();
	switch (numArgs) {
	case 1:
		if (zend_get_parameters_ex(1, &pv_result_id) == FAILURE) WRONG_PARAM_COUNT;
		break;
	case 2:
		if (zend_get_parameters_ex(2, &pv_result_id, &pv_format) == FAILURE) WRONG_PARAM_COUNT;
		break;
	default:
		WRONG_PARAM_COUNT;
	}

	if( (answer=get_answer_from_result(INTERNAL_FUNCTION_PARAM_PASSTHRU,pv_result_id))==NULL )
	{
		wblogs("wb_result_all leave, FALSE\n");
		RETURN_FALSE;
	}
	WBLSM_START("wb_result_all",CDP_FROM_ANSWER_P(answer));

	if (numArgs==2) {
		convert_to_string_ex(pv_format);
		php_printf("\n<table %s >\n<tr>",Z_STRVAL_PP(pv_format));
	} else ZEND_PUTS("\n<table>\n<tr>");
		/* Print out row of column names: */
	for (column=0; column<answer->attrs; column++)
		zend_printf("<th>%s</th>", answer->names[column]);
		ZEND_PUTS("</tr>");
	for (row=0; ; row++) {
		wblog("wb_result_all row %d\n",row);
		ZEND_PUTS("\n<tr>");
		for (column=0; column<answer->attrs; column++)
		{ /* Print value of single attribute: */
			/* Note that long types passed thru are ignored here. */
			zval *cell_ptr=&cell_value;
			wblog("wb_result_all row %d col %d\n",row,column);
			if (!db_item_to_php(
#ifdef USE_WB_SCRIPT_ENCODING
						WBG(script_encoding),
#endif
						answer, row, column, cell_ptr, 1 TSRMLS_CC)) goto cont;
			switch( answer->types[column] )
			{
				/* Values of DATE and TIMESTAMP types are read as Unix timestamp
				 * we will convert them to human-readable notation.
				 * However we have to take care about NULL value. */
				case ATT_DATE:
				case ATT_TIMESTAMP:
					if( Z_TYPE_P(cell_ptr)==IS_NULL ) { convert_to_string_ex(&cell_ptr); }
					else
					{
						time_t t;
						char buf[200];
						convert_to_long_ex(&cell_ptr);
						t=(time_t)(Z_LVAL_P(cell_ptr));
						if( t==(time_t)-1 ) sprintf(buf,"[invalid %s]",(answer->types[column]==ATT_DATE)?"DATE":"TIMESTAMP");
						else
						{
							struct tm tmval;
							php_localtime_r(&t,&tmval);
  						*buf='\0';
							if( answer->types[column]==ATT_DATE ) strftime(buf,200,"%a %b %d %Y",&tmval);
							else strftime(buf,200,"%a %b %d %Y %H:%M:%S",&tmval);
						}
						zval_dtor(cell_ptr);
						ZVAL_STRING(cell_ptr,buf,1);
					}
					break;
				case ATT_BOOLEAN:
					// TRUE display as "TRUE", FALSE as "FALSE" and NULL as ""
					if( Z_TYPE_P(cell_ptr)==IS_NULL ) { convert_to_string_ex(&cell_ptr); }
					else
					{
						int tmp;
						convert_to_boolean_ex(&cell_ptr);
						tmp=Z_BVAL_P(cell_ptr);
						zval_dtor(cell_ptr);
						if( tmp ) { ZVAL_STRING(cell_ptr,"TRUE",1); }
						else { ZVAL_STRING(cell_ptr,"FALSE",1); }
					}
					break;
				default:
					convert_to_string_ex(&cell_ptr);
			}
			wblogs("wb_result_all before zend_printf()\n");
			zend_printf("<td>%s</td>", Z_STRVAL_P(cell_ptr));
			wblogs("wb_result_all before zval_dtor(cell_ptr)\n");
			zval_dtor(cell_ptr);
			wblog("wb_result_all row %d col %d finished\n",row,column);
		}
		ZEND_PUTS("</tr>");
	 }
cont:
	ZEND_PUTS("</table>");
	wblogs("wb_result_all leave\n");
	WBLSM_END;
	RETURN_LONG(row);
}
/*}}}*/
/*{{{ wb_longreadlen
The binmode and longreadlen are set by wb_binmode and wb_longreadlen
functions. They accept two parameters, result_id and new value of the variable.
If the result identifier has special value of 0, global variables (used as default) are
changed. */
ZEND_FUNCTION(wb_longreadlen)
{
	WBLSM_DECLARE
	zval **result_id, **llen;
	struct sql_answer* answer;

	wblogs("wb_longreadlen enter\n");

	if (ZEND_NUM_ARGS()!=2 || zend_get_parameters_ex(2, &result_id, &llen)!=SUCCESS)
		WRONG_PARAM_COUNT;

	convert_to_long_ex(llen);
	if ((*result_id)->value.lval==0) {
		WBG(longreadlen)=(*llen)->value.lval;
		wblogs("wb_longreadlen leave, TRUE (global parameter changed)\n");
		RETURN_TRUE;
	}

	if( (answer=get_answer_from_result(INTERNAL_FUNCTION_PARAM_PASSTHRU,result_id))==NULL )
	{
		wblogs("wb_longreadlen leave, FALSE\n");
		RETURN_FALSE;
	}
	WBLSM_START("wb_longreadlen",CDP_FROM_ANSWER_P(answer));
	answer->longreadlen=(*llen)->value.lval;
	wblogs("wb_longreadlen leave, TRUE\n");
	WBLSM_END;
	RETURN_TRUE;
}
/*}}}*/
/*{{{ wb_binmode */
ZEND_FUNCTION(wb_binmode)
{
	WBLSM_DECLARE
	zval **result_id, **mode;
	struct sql_answer* answer;

	wblogs("wb_binmode enter\n");

	if (ZEND_NUM_ARGS()!=2 || zend_get_parameters_ex(2, &result_id, &mode)!=SUCCESS)
		WRONG_PARAM_COUNT;

	convert_to_long_ex(mode);
	if (Z_LVAL_PP(result_id)==0) {
		WBG(binmode)=(binmode_t)((*mode)->value.lval);
		wblogs("wb_binmode leave, TRUE (global parameter changed)\n");
		RETURN_TRUE;
	}

	if( (answer=get_answer_from_result(INTERNAL_FUNCTION_PARAM_PASSTHRU,result_id))==NULL )
	{
		wblogs("wb_binmode leave, FALSE\n");
		RETURN_FALSE;
	}
	WBLSM_START("wb_binmode",CDP_FROM_ANSWER_P(answer));
	wblog("old answer->binmode=%d, new is %d\n",answer->binmode,(binmode_t)Z_LVAL_PP(mode));
	answer->binmode=(binmode_t)(Z_LVAL_PP(mode));
	wblogs("wb_binmode leave, TRUE\n");
	WBLSM_END;
	RETURN_TRUE;
}
/*}}}*/
/*{{{ wb_import_file */
/* Populating tables with data.
Database tables are useless without there data. While it is possible to enter the data
with a sequence of SQL commands, it may be more convenient (both in terms of time an
ease of use) to create a file with the data for the table and make the database system
deal with the file.

602SQL provides an API function Move_data that does (among other things) exactly
this. Details of the data transfer are in the transfer description. The function
declaration is not in wbkernel.h, so we explicitly declare it (it is in
wbprezen.h, which is assumed to be used in C++ programs; thus, we do not
include it.). Most of the parameters override
default settings in the transfer description. The data may be in CSV or fixed-width
column formats.

Note that the function, unlike most of the other 602SQL API functions, returns FALSE
on failure. */
DllPrezen BOOL WINAPI Move_data(cdp_t cdp, tobjnum move_descr_obj,
	const char * inpname, tobjnum inpobj, const char * outname,
	int inpformat, int outformat, int inpcode, int outcode, BOOL silent);

/* ZEND function: wb_import_file
The PHP interface to the Move_data is called wb_import_file. It has
three parameters: opened connection, file with data, and transfer description
name; the name must be converted to an object of CATEG_PGMSRC before use. */
ZEND_FUNCTION(wb_import_file)
{
	WBLSM_DECLARE
	zval **connhandle, **filename, **transfer;
	tobjnum tfnum; /* object number of transfer description */
	cdp_t cdp;
	wb_conn* conn;
	char *tmp_transfer;
#ifdef USE_WB_SCRIPT_ENCODING
	char *emalloc_transfer=NULL;
#endif

	wblogs("wb_import_file enter\n");

	if (ZEND_NUM_ARGS()!=3 ||
		zend_get_parameters_ex(3, &connhandle, &filename, &transfer)!=SUCCESS)
			WRONG_PARAM_COUNT;

	conn=get_conn_from_resource(INTERNAL_FUNCTION_PARAM_PASSTHRU,connhandle);
	if (!conn) RETURN_FALSE;
	cdp=CDP(conn);
	WBLSM_START("wb_import_file",cdp);
	convert_to_string_ex(transfer);
	if( !PREPARE_STRING_SCRIPT2SYSTEM(WBG(script_encoding),conn,&tmp_transfer,&emalloc_transfer,Z_STRVAL_PP(transfer)) )
	{
		wblogs("wb_import_file leave, ERROR: estrdup_with_encoding() failed\n");
		WBLSM_END;
		RETURN_FALSE;
	}
 
	if (cd_Find_object(cdp, tmp_transfer, CATEG_PGMSRC, &tfnum)) {
		CONN_REPORT_ERR(conn,"Acquiring transfer description");
		EFREE_EMALLOC_STRING(emalloc_transfer);
		WBLSM_END;
		RETURN_FALSE;
	}
	EFREE_EMALLOC_STRING(emalloc_transfer);
	convert_to_string_ex(filename);
	if (!Move_data(cdp, tfnum, Z_STRVAL_PP(filename), -1, "", -1, -1, -1, 0, TRUE)) {
		CONN_REPORT_ERR(conn,"Move data:");
		WBLSM_END;
		RETURN_FALSE;
	}
	wblogs("wb_import_file leave\n");
	WBLSM_END;
	RETURN_TRUE;
}
/*}}}*/
#if WBVERS>=800
/* wb_client_encoding {{{*/
ZEND_FUNCTION(wb_client_encoding)
{
	WBLSM_DECLARE
	zval **connhandle;
	wb_conn *conn;

	wblogs("wb_client_encoding enter\n");

	if (1!=ZEND_NUM_ARGS() ||
			zend_get_parameters_ex(1, &connhandle) != SUCCESS)
		WRONG_PARAM_COUNT;

	conn=get_conn_from_resource(INTERNAL_FUNCTION_PARAM_PASSTHRU,connhandle);
	if (!conn) { wblogs("wb_client_encoding leave, return FALSE\n"); RETURN_FALSE; /* error in get_conn_from_resource */ }
	WBLSM_START("wb_client_encoding",CDP(conn));
	wblog("wb_client_encoding leave, result is %d\n",ENCODING(conn));
	WBLSM_END;
	RETURN_LONG(ENCODING(conn));
}
/*}}}*/
/* wb_set_client_encoding {{{*/
ZEND_FUNCTION(wb_set_client_encoding)
{
	WBLSM_DECLARE
	zval **connhandle;
	zval **enc=NULL;
	wb_conn *conn;

	wblogs("wb_set_client_encoding enter\n");

	if (2!=ZEND_NUM_ARGS() ||
			zend_get_parameters_ex(2, &connhandle, &enc) != SUCCESS)
		WRONG_PARAM_COUNT;

	conn=get_conn_from_resource(INTERNAL_FUNCTION_PARAM_PASSTHRU,connhandle);
	if (!conn) { wblogs("wb_set_client_encoding leave, return FALSE\n"); RETURN_FALSE; /* error in get_conn_from_resource */ }
	WBLSM_START("wb_set_client_encoding",CDP(conn));
	convert_to_long_ex(enc);
	if( !IS_VALID_ENCODING(Z_LVAL_PP(enc)) )
	{
		zend_error(E_WARNING,"602SQL: %ld isn't valid host vars encoding number in wb_set_client_encoding function call",Z_LVAL_PP(enc));
		wblog("wb_set_client_encoding leave, return FALSE, %ld isn't valid host vars encoding number\n",Z_LVAL_PP(enc));
		WBLSM_END;
		RETURN_FALSE;
	}
	if( !IS_IMPLEMENTED_ENCODING(Z_LVAL_PP(enc)) )
	{
		zend_error(E_WARNING,"602SQL: host vars encoding number %ld isn't supported yet",Z_LVAL_PP(enc));
		wblog("wb_set_client_encoding leave, return FALSE, unsupported encoding\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	ENCODING(conn)=Z_LVAL_PP(enc);
	wblog("wb_set_client_encoding leave, result is TRUE\n");
	WBLSM_END;
	RETURN_TRUE;
}
/*}}}*/
#endif
#if WBVERS>=950
/* wb_data_transport {{{*/
/* internal_odbc_connect {{{
 * opens ODBC connection to DSN which is described by dsnarr PHP array
 * returns pointer to connection's client struct on success or NULL on error
 * dsnarr format:
 *   "dsn" => dsn-value,
 *   "uid" => uid-value,
 *   "pwd" => pwd-value
 * "uid" and "pwd" items are optional */
static t_pconnection internal_odbc_connect(zval **dsnarr,const char *param_name)
{
	zval **tmp;
	const char *dsn,*uid,*pwd;
	t_pconnection result;

	convert_to_array_ex(dsnarr);
	if( zend_hash_find(Z_ARRVAL_PP(dsnarr),"dsn",4,(void**)&tmp)!=SUCCESS )
	{
		zend_error(E_WARNING,"602SQL: parameter array \"%s\" doesn't have \"dsn\" array item with ODBC DSN name.",param_name);
		wblogs("internal_odbc_connect leave, ERROR: \"dsn\" item is missing");
		return NULL;
	}
	convert_to_string_ex(tmp);
	dsn=Z_STRVAL_PP(tmp);
	if( zend_hash_find(Z_ARRVAL_PP(dsnarr),"uid",4,(void**)&tmp)==SUCCESS )
	{
		convert_to_string_ex(tmp);
		uid=Z_STRVAL_PP(tmp);
	}
	else uid=NULL;
	if( zend_hash_find(Z_ARRVAL_PP(dsnarr),"pwd",4,(void**)&tmp)==SUCCESS )
	{
		convert_to_string_ex(tmp);
		pwd=Z_STRVAL_PP(tmp);
	}
	else pwd=NULL;
	result=ODBC_connect(dsn,uid,pwd,NULL);
	if( result==NULL )
	{
		zend_error(E_WARNING,"602SQL: Can't connect to ODBC data source which DSN is \"%s\".",dsn);
	}
	return result;
}
/*}}}*/
ZEND_FUNCTION(wb_data_transport)
{
	WBLSM_DECLARE
	zval **par_conn,**par_move_descr_obj,**par_src_conn,**par_trg_conn,
	  **par_src_schema,**par_trg_schema,**par_src_name,**par_trg_name,
		**par_src_format,**par_trg_format,**par_src_code,**par_trg_code,
		**par_flags;
	wb_conn *conn;
	cdp_t conn_cdp;
	tobjnum move_descr_obj;
	xcdp_t src_xcdp,trg_xcdp;
	int src_xcdp_is_odbc=0,trg_xcdp_is_odbc=0; /* 0 = given connection is 602SQL connection resource
	                                            * 1 = given connection is ODBC connection described by array with 'dsn', 'uid' and 'pwd' items
																							* 2 = given connection is ODBC connection resource */
	const char *src_schema,*trg_schema,*src_name,*trg_name;
	int src_format,trg_format,src_code,trg_code;
	unsigned flags;
	void *ptr;
	int type;

	wblogs("wb_data_transport enter\n");

	if( ZEND_NUM_ARGS()!=13
		|| zend_get_parameters_ex(13,&par_conn,&par_move_descr_obj,&par_src_conn,&par_trg_conn,
			&par_src_schema,&par_trg_schema,&par_src_name,&par_trg_name,
			&par_src_format,&par_trg_format,&par_src_code,&par_trg_code,
			&par_flags)!=SUCCESS )
	{
		WRONG_PARAM_COUNT;
	}

	conn=get_conn_from_resource(INTERNAL_FUNCTION_PARAM_PASSTHRU,par_conn);
	if( !conn ) { wblogs("wb_data_transport leave, return FALSE\n"); RETURN_FALSE; }
	conn_cdp=CDP(conn);
	WBLSM_START("wb_data_transport",conn_cdp);

	convert_to_string_ex(par_move_descr_obj);
	if( Z_STRLEN_PP(par_move_descr_obj)>0 )
	{
		// get objnum of move description object
		if( cd_Find_object(conn_cdp,Z_STRVAL_PP(par_move_descr_obj),CATEG_PGMSRC,&move_descr_obj) )
		{
			CONN_REPORT_ERR(conn,"wb_data_transport (can't find desired move description database object)");
			wblog("wb_data_transport leave, ERROR: object \"%s\" not found in database.\n",Z_STRVAL_PP(par_move_descr_obj));
			WBLSM_END;
			RETURN_FALSE;
		}
	}
	else
	{
		move_descr_obj=NOOBJECT;
	}

	convert_to_string_ex(par_src_schema);	src_schema=( Z_STRLEN_PP(par_src_schema)>0 )?Z_STRVAL_PP(par_src_schema):NULL;
	convert_to_string_ex(par_trg_schema);	trg_schema=( Z_STRLEN_PP(par_trg_schema)>0 )?Z_STRVAL_PP(par_trg_schema):NULL;
	convert_to_string_ex(par_src_name);	src_name=( Z_STRLEN_PP(par_src_name)>0 )?Z_STRVAL_PP(par_src_name):NULL;
	convert_to_string_ex(par_trg_name);	trg_name=( Z_STRLEN_PP(par_trg_name)>0 )?Z_STRVAL_PP(par_trg_name):NULL;
#define CONVERT_LONG_PARAM(par_name,var_name) \
	if( Z_TYPE_PP(par_name)==IS_NULL ) var_name=-1; \
	else { convert_to_long_ex(par_name); var_name=Z_LVAL_PP(par_name); }
	// when par_XXX is NULL then we need to set var_XXX=-1
	CONVERT_LONG_PARAM(par_src_format,src_format);
	CONVERT_LONG_PARAM(par_trg_format,trg_format);
	CONVERT_LONG_PARAM(par_src_code,src_code);
	CONVERT_LONG_PARAM(par_trg_code,trg_code);
	convert_to_long_ex(par_flags); flags=(unsigned)Z_LVAL_PP(par_flags); // when Z_TYPE_PP(par_flags)==IS_NULL then we want to set flags=0

	switch( Z_TYPE_PP(par_src_conn) )
	{
		case IS_NULL:
			src_xcdp=NULL;
			break;
		case IS_RESOURCE:
			ptr=zend_list_find(Z_RESVAL_PP(par_src_conn), &type);
			if( ptr==NULL )
			{
				zend_error(E_WARNING,"602SQL: Resource not found.");
				wblogs("wb_data_transport leave, FALSE: zend_list_find() returned NULL for src_conn\n");
				WBLSM_END;
				RETURN_FALSE;
			}
			if( type==wbres_conn )
			{ 
				src_xcdp=CDP((struct wb_conn*)ptr);
			}
#if WBVERS>=1000
			else if( type==wbres_odbc )
			{
				src_xcdp=T_PCONNECTION_FROM_ODBC_P((struct odbc_t*)ptr);
				src_xcdp_is_odbc=2;
			}
#endif
			else
			{
				zend_error(E_WARNING,"602SQL: Unsupported resource passed as src_conn parameter.");
				wblogs("wb_data_transport leave, FALSE: invalid resource in src_conn\n");
				WBLSM_END;
				RETURN_FALSE;
			}
			break;
		case IS_ARRAY:
			if( (src_xcdp=internal_odbc_connect(par_src_conn,"src_conn"))==NULL )
			{
				wblogs("wb_data_transport leave, FALSE: internal_odbc_connect(par_src_conn) failed\n");
				WBLSM_END;
				RETURN_FALSE;
			}
			src_xcdp_is_odbc=1;
			break;
		default:
			php_error(E_WARNING,"602SQL: Invalid value of src_conn parameter of wb_data_transport function.");
			wblogs("wb_data_transport leave, ERROR: invalid src_conn param.\n");
			WBLSM_END;
			RETURN_FALSE;
	};
	// if src_xcdp_is_odbc==1 then we have to call ODBC_disconnect(src_xcdp) before we leave this fuction
	switch( Z_TYPE_PP(par_trg_conn) )
	{
		case IS_NULL:
			trg_xcdp=NULL;
			break;
		case IS_RESOURCE:
			ptr=zend_list_find(Z_RESVAL_PP(par_trg_conn), &type);
			if( ptr==NULL )
			{
				zend_error(E_WARNING,"602SQL: Resource not found.");
				wblogs("wb_data_transport leave, FALSE: zend_list_find() returned NULL for trg_conn\n");
  			if( src_xcdp_is_odbc==1 ) { ODBC_disconnect(src_xcdp); }
				WBLSM_END;
				RETURN_FALSE;
			}
			if( type==wbres_conn )
			{ 
				trg_xcdp=CDP((struct wb_conn*)ptr);
			}
#if WBVERS>=1000
			else if( type==wbres_odbc )
			{
				trg_xcdp=T_PCONNECTION_FROM_ODBC_P((struct odbc_t*)ptr);
				trg_xcdp_is_odbc=2;
			}
#endif
			else
			{
				zend_error(E_WARNING,"602SQL: Unsupported resource passed as trg_conn parameter.");
				wblogs("wb_data_transport leave, FALSE: invalid resource in trg_conn\n");
				if( src_xcdp_is_odbc==1 ) { ODBC_disconnect(src_xcdp); }
				WBLSM_END;
				RETURN_FALSE;
			}
			break;
		case IS_ARRAY:
			if( (trg_xcdp=internal_odbc_connect(par_trg_conn,"trg_conn"))==NULL )
			{
				wblogs("wb_data_transport leave, FALSE: internal_odbc_connect(par_trg_conn) failed\n");
  			if( src_xcdp_is_odbc==1 ) { ODBC_disconnect(src_xcdp); }
				WBLSM_END;
				RETURN_FALSE;
			}
			trg_xcdp_is_odbc=1;
			break;
		default:
			php_error(E_WARNING,"602SQL: Invalid value of trg_conn parameter of wb_data_transport function.");
			wblogs("wb_data_transport leave, ERROR: invalid trg_conn param.");
			if( src_xcdp_is_odbc==1 ) { ODBC_disconnect(src_xcdp); }
			WBLSM_END;
			RETURN_FALSE;
	};
	// if trg_xcdp_is_odbc==1 then we have to call ODBC_disconnect(trg_xcdp) before we leave this fuction
	
	wblog("calling cd_Data_transport() with conn_cdp=0x%p, move_descr_obj=%d, src_xcdp=0x%p, trg_xcdp=0x%p, src_schema=\"%s\", trg_schema=\"%s\", src_name=\"%s\", trg_name=\"%s\", src_format=%d, trg_format=%d, src_code=%d, trg_code=%d, flags=%u\n",
			conn_cdp,move_descr_obj,src_xcdp,trg_xcdp,src_schema,trg_schema,src_name,trg_name,src_format,trg_format,src_code,trg_code,flags);
	if( cd_Data_transport(conn_cdp,move_descr_obj,src_xcdp,trg_xcdp,src_schema,trg_schema,src_name,trg_name,
				src_format,trg_format,src_code,trg_code,flags) )
	{
		if( src_xcdp_is_odbc==1 ) { ODBC_disconnect(src_xcdp); }
		if( trg_xcdp_is_odbc==1 ) { ODBC_disconnect(trg_xcdp); }
		wblogs("wb_data_transport leave, OK\n");
		WBLSM_END;
		RETURN_TRUE;
	}
	else
	{ // error
		int err=(conn_cdp!=NULL)?cd_Sz_error(conn_cdp):NO_ERROR;
		if( err==NO_ERROR )
		{
			php_error(E_WARNING,"602SQL: Invalid values of parameters of wb_data_transport function.");
			wblogs("wb_data_transport leave, ERROR: cd_Data_transport() failed and cd_Sz_error()==NO_ERROR\n");
		}
		else
		{
			report_err_from_number(conn,err,"wb_data_transport" TSRMLS_CC);
			wblog("wb_data_transport leave, ERROR: cd_Data_transport() failed, cd_Sz_error()=%d\n",err);
		} 
		if( src_xcdp_is_odbc==1 ) { ODBC_disconnect(src_xcdp); }
		if( trg_xcdp_is_odbc==1 ) { ODBC_disconnect(trg_xcdp); }
		WBLSM_END;
		RETURN_FALSE;
	}
} 
/*}}}*/
#endif
#ifdef ENABLE_XML_RESPONSE
/* wb_get_xml_response {{{*/
ZEND_FUNCTION(wb_get_xml_response)
{
	WBLSM_DECLARE
	zval **connhandle,**http_method,**hostname,**url,**body,**bodylength,**username,**password;
	wb_conn *conn;
	int http_method_code,http_status;
	char *response;
	const char *content_type;
	cdp_t cdp;
	char *options[HTTP_REQOPT_COUNT],realm_buffer[1024];
#if WBVERS>=1000
	int resplen;
#endif

	wblogs("wb_get_xml_response enter\n");

	if (8!=ZEND_NUM_ARGS() || zend_get_parameters_ex(8,&connhandle,&http_method,&hostname,&url,&body,&bodylength,&username,&password)!=SUCCESS) WRONG_PARAM_COUNT;
	convert_to_string_ex(http_method);
	convert_to_string_ex(hostname);
	convert_to_string_ex(url);
	convert_to_string_ex(body);
	convert_to_long_ex(bodylength);
	if( Z_TYPE_PP(username)!=IS_NULL ) convert_to_string_ex(username);
	if( Z_TYPE_PP(password)!=IS_NULL ) convert_to_string_ex(password);

	conn=get_conn_from_resource(INTERNAL_FUNCTION_PARAM_PASSTHRU,connhandle);
	if (!conn) { wblogs("wb_get_xml_response leave, error in get_conn_from_resource(), return FALSE\n"); RETURN_FALSE; /* error in get_conn_from_resource */ }
	cdp=CDP(conn);
	WBLSM_START("wb_get_xml_response",cdp);

	if( stricmp(Z_STRVAL_PP(http_method),"GET")==0 ) http_method_code=HTTP_METHOD_GET;
	else if( stricmp(Z_STRVAL_PP(http_method),"POST")==0 ) http_method_code=HTTP_METHOD_POST;
	else
	{
		zend_error(E_WARNING,"602SQL: invalid/unsupported HTTP method \"%s\" in wb_get_xml_response() call",Z_STRVAL_PP(http_method));
		wblog("wb_get_xml_response leave, error: invalid HTTP method \"%s\", return FALSE\n",Z_STRVAL_PP(http_method));
		WBLSM_END;
		RETURN_FALSE;
	}
	wblog("wb_get_xml_response, before get_xml_response, body=%p\n",body);
	wblog("bodylength=%d (value of the parameter), url length=%d\n",Z_LVAL_PP(bodylength),Z_STRLEN_PP(url));
	wblog("body=\"%s\", url=\"%s\"\n",Z_STRVAL_PP(body),Z_STRVAL_PP(url));
	response=NULL; content_type=NULL;
	memset(options,0,sizeof(options));
	options[HTTP_REQOPT_HOST]=Z_STRVAL_PP(hostname);
	options[HTTP_REQOPT_PATH]=Z_STRVAL_PP(url);
	options[HTTP_REQOPT_USERNAME]=(Z_TYPE_PP(username)==IS_NULL)?NULL:Z_STRVAL_PP(username);
	options[HTTP_REQOPT_PASSWORD]=(Z_TYPE_PP(password)==IS_NULL)?NULL:Z_STRVAL_PP(password);
	*realm_buffer='\0';
#if WBVERS>=1000
	http_status=get_xml_response(cdp,http_method_code,options,Z_STRVAL_PP(body),Z_LVAL_PP(bodylength),&response,&content_type,realm_buffer,&resplen);
#else
	http_status=get_xml_response(cdp,http_method_code,options,Z_STRVAL_PP(body),Z_LVAL_PP(bodylength),&response,&content_type,realm_buffer);
#endif
	wblog("wb_get_xml_response, after get_xml_response\n  http_status=%d\n  realm_buffer=\"%s\"\n  content_type=\"%s\"\n  response=\"%s\"\n",http_status,realm_buffer,content_type,response);
	if( array_init(return_value)!=SUCCESS )
	{
		zend_error(E_WARNING,"602SQL: can't initialize output array of wb_get_xml_response() function");
		wblogs("wb_get_xml_response leave, array_init failed\n");
		corefree(response);
		WBLSM_END;
		RETURN_FALSE;
	}
	{ // remove "Content-type:" from the start of content_type and trailing whitespaces
		const char *ct_start;
		uint ct_length; // uint is type of length parameter of PHP API add_assoc_stringl() function
		ct_start=content_type;
		while( isspace(*ct_start) ) ct_start++;
		if( strnicmp(ct_start,"content-type:",13)==0 )
		{
			ct_start+=13;
			while( isspace(*ct_start) ) ct_start++;
		}
		ct_length=strlen(ct_start);
		while( ct_length>0 && isspace(ct_start[ct_length-1]) ) ct_length--;
		add_assoc_stringl(return_value,"content-type",(char*)ct_start,ct_length,1);
	}
	if( response!=NULL )
	{
		add_assoc_string(return_value,"response",response,1);
		corefree(response);
#if WBVERS>=1000
		add_assoc_long(return_value,"response_size",resplen);
#else
		add_assoc_long(return_value,"response_size",strlen(response));
#endif
	}
	else add_assoc_string(return_value,"response","",1);
	add_assoc_string(return_value,"realm",realm_buffer,1);
	add_assoc_long(return_value,"status",http_status);
	wblogs("wb_get_xml_response leave\n");
	WBLSM_END;
}
/*}}}*/
#endif
#ifdef USE_WB_SCRIPT_ENCODING
/* wb_script_encoding {{{*/
ZEND_FUNCTION(wb_script_encoding)
{
	wblogs("wb_script_encoding enter\n");
	if (0!=ZEND_NUM_ARGS() ) WRONG_PARAM_COUNT;
	wblog("wb_client_encoding leave, result is %d\n",WBG(script_encoding));
	RETURN_LONG(WBG(script_encoding));
}
/*}}}*/
/* wb_set_script_encoding {{{*/
ZEND_FUNCTION(wb_set_script_encoding)
{
	zval **connhandle;
	zval **enc;

	wblogs("wb_set_script_encoding enter\n");

	switch( ZEND_NUM_ARGS() )
	{
		case 1:
			if( zend_get_parameters_ex(1, &enc) !=SUCCESS ) WRONG_PARAM_COUNT;
			connhandle=NULL;
			break;
#if WBVERS>=800
		case 2:
			if( zend_get_parameters_ex(2, &enc, &connhandle) != SUCCESS ) WRONG_PARAM_COUNT;
			break;
#endif
		default:
			WRONG_PARAM_COUNT;
	}

	convert_to_long_ex(enc);
	if( !IS_VALID_ENCODING(Z_LVAL_PP(enc)) )
	{
		zend_error(E_WARNING,"602SQL: %ld isn't valid script encoding number in wb_set_script_encoding function call",Z_LVAL_PP(enc));
		wblog("wb_set_script_encoding leave, return FALSE, %ld isn't valid script encoding number\n",Z_LVAL_PP(enc));
		RETURN_FALSE;
	}
	if( !IS_IMPLEMENTED_ENCODING(Z_LVAL_PP(enc)) )
	{
		zend_error(E_WARNING,"602SQL: script encoding number %ld isn't supported yet",Z_LVAL_PP(enc));
		wblog("wb_set_script_encoding leave, return FALSE, unsupported encoding\n");
		RETURN_FALSE;
	}
#if WBVERS>=800
	if( connhandle!=NULL )
	{
		wb_conn *conn=get_conn_from_resource(INTERNAL_FUNCTION_PARAM_PASSTHRU,connhandle);
		if (!conn) { wblogs("wb_set_script_encoding leave, return FALSE\n"); RETURN_FALSE; /* error in get_conn_from_resource */ }
		ENCODING(conn)=Z_LVAL_PP(enc);
	}
#endif
	WBG(script_encoding)=Z_LVAL_PP(enc);
	wblog("wb_set_client_encoding leave, result is TRUE\n");
	RETURN_TRUE;
}
/*}}}*/
#endif
#if WBVERS>=1000
/* wb_get_xml_form {{{*/
ZEND_FUNCTION(wb_get_xml_form)
{
	WBLSM_DECLARE
	zval **conn_id,**xml_form_ref,**data;
	cdp_t cdp;
	struct wb_conn *conn;
	char *result,*error_message;
	int result_size;

	switch( ZEND_NUM_ARGS() )
	{
		case 2:
			if( zend_get_parameters_ex(2,&conn_id,&xml_form_ref)!=SUCCESS ) WRONG_PARAM_COUNT;
			data=NULL;
			break;
		case 3:
			if( zend_get_parameters_ex(3,&conn_id,&xml_form_ref,&data)!=SUCCESS ) WRONG_PARAM_COUNT;
			break;
		default:
			WRONG_PARAM_COUNT;
	}

	conn=get_conn_from_resource(INTERNAL_FUNCTION_PARAM_PASSTHRU,conn_id);
	if( conn==NULL ) RETURN_FALSE;
	cdp=CDP(conn);
	if (!cdp) RETURN_FALSE;
	WBLSM_START("wb_get_xml_form",cdp);
	convert_to_string_ex(xml_form_ref);
	if( data!=NULL ) convert_to_string_ex(data);
	if( (result=Merge_XML_formM(cdp,Z_STRVAL_PP(xml_form_ref),Z_STRLEN_PP(xml_form_ref),(data==NULL)?NULL:Z_STRVAL_PP(data),&result_size,&error_message))==NULL )
	{
		CONN_REPORT_ERR(conn,"wb_get_xml_form while merging XML form and XML data");
		if( error_message!=NULL )
		{
			zend_error(E_WARNING,"602SQL: wb_get_xml_form error: %s",error_message);
			corefree(error_message);
		}
		WBLSM_END;
		RETURN_FALSE;
	}
	RETVAL_STRINGL(result,result_size,1/*duplicate*/);
	corefree(result);
	WBLSM_END;
	// and return
}
/*}}}*/
/* wb_match_form_and_dad {{{
 * int wb_match_form_and_dad(resource connection,string xml_form_source,string dad_ref)
 * returns one of MFD_xxx constants:
 *   MFD_AGR - data modell of the XML form matches DAD
 *   MFD_DISAGR - doesn't match
 *   MFD_FORMREADFAIL - there was an error during XML form source reading
 *   MFD_FORMPARSEFAIL - there was an error during XML form source parsing
 *   MFD_DADREADFAIL - there was an error during DAD source reading
 *   MFD_DADPARSEFAIL - there was an error during DAD source parsing
 *   MFD_OTHER - there was another error, i.e. out of memory */
ZEND_FUNCTION(wb_match_form_and_dad)
{
	WBLSM_DECLARE
	zval **conn_zval,**form_source_zval,**dad_ref_zval;
	struct wb_conn *conn;
	cdp_t cdp;
	int result;

	if( ZEND_NUM_ARGS()!=3 || zend_get_parameters_ex(3,&conn_zval,&form_source_zval,&dad_ref_zval)!=SUCCESS ) WRONG_PARAM_COUNT;
	conn=get_conn_from_resource(INTERNAL_FUNCTION_PARAM_PASSTHRU,conn_zval);
	if( conn==NULL ) RETURN_FALSE;
	cdp=CDP(conn);
	if (!cdp) RETURN_FALSE;
	WBLSM_START("wb_match_form_and_dad",cdp);
	convert_to_string_ex(form_source_zval);
	convert_to_string_ex(dad_ref_zval);
	if( *(DWORD *)Z_STRVAL_PP(form_source_zval) == ('P' + ('K' << 8) + (3 << 16) + (4 << 24)) )
	{
		result=MatchFormAndDADM(cdp,Z_STRVAL_PP(form_source_zval),Z_STRLEN_PP(form_source_zval),Z_STRVAL_PP(dad_ref_zval));
	}
	else
	{
		result=MatchFormAndDAD(cdp,Z_STRVAL_PP(form_source_zval),Z_STRVAL_PP(dad_ref_zval));
	}
	WBLSM_END;
	RETURN_LONG(result);
}
/*}}}*/
#endif
#if WBVERS>=1000
/* wb_set_client_error_callback {{{*/
void WINAPI php_602sql_client_error_callback(cdp_t cdp_param,const char *text) /*{{{*/
{
	int i,nument,type;
	void *ptr;
	TSRMLS_FETCH();
	wblog("php_602sql_client_error_callback: searching for connection resource, cdp=0x%p\n",cdp_param);
	nument = zend_hash_next_free_element(&EG(regular_list));
	for(i = 1; i < nument; i++)
	{
		ptr = zend_list_find(i, &type);
		if( ptr!=NULL && (type==wbres_conn||type==wbres_pconn) )
		{
			if( CDP((struct wb_conn *)ptr)==cdp_param )
			{
				struct wb_conn *conn=(struct wb_conn *)ptr;
				wblog("php_602sql_client_error_callback: found resource: struct wb_conn=0x%p, cdp=0x%p\n",ptr,CDP(conn));
				if( conn->client_error_level!=0 )
				{
					wblogs("php_602sql_client_error_callback: calling zend_error\n");
					zend_error(conn->client_error_level,text);
				}
				if( conn->client_error_callback!=NULL )
				{
					zval retval,function_name;
					zval *text_zval;
					MAKE_STD_ZVAL(text_zval);
					ZVAL_STRING(text_zval,(char*)text,1);
					ZVAL_STRING(&function_name,conn->client_error_callback,1);
					wblog("php_602sql_client_error_callback: calling function %s\n",conn->client_error_callback);
					if( call_user_function(EG(function_table),NULL,&function_name,&retval,1,&text_zval TSRMLS_CC)!=SUCCESS )
					{
						wblogs("php_602sql_client_error_callback: ERROR, call_user_function() failed\n");
					}
					zval_dtor(&retval);
					zval_dtor(text_zval);
					FREE_ZVAL(text_zval);
				}
				break;
			}
		}
	}
} /*}}}*/
ZEND_FUNCTION(wb_set_client_error_callback) /*{{{*/
{
	WBLSM_DECLARE
	zval **conn_zval,**callback_zval;
	wb_conn *conn;

	wblogs("wb_set_client_error_callback enter\n");

	if (2!=ZEND_NUM_ARGS() || zend_get_parameters_ex(2, &conn_zval, &callback_zval) != SUCCESS)	WRONG_PARAM_COUNT;

	conn=get_conn_from_resource(INTERNAL_FUNCTION_PARAM_PASSTHRU,conn_zval);
	if (!conn) { wblogs("wb_set_client_error_callback leave, FALSE, error in get_conn_from_resource()\n"); RETURN_FALSE; }
	WBLSM_START("wb_set_client_error_callback",CDP(conn));
	switch( Z_TYPE_PP(callback_zval) )
	{
		case IS_NULL:
			set_client_error_callback2(CDP(conn),NULL);
			conn->client_error_level=0;
			if( conn->client_error_callback!=NULL ) { efree(conn->client_error_callback); conn->client_error_callback=NULL; }
			wblogs("wb_set_client_error_callback leave, TRUE, removed both callbacks\n");
			WBLSM_END;
			RETURN_TRUE;
		case IS_LONG:
			conn->client_error_level=Z_LVAL_PP(callback_zval);
			set_client_error_callback2(CDP(conn),(conn->client_error_level!=0||conn->client_error_callback!=NULL)?php_602sql_client_error_callback:NULL);
			wblog("wb_set_client_error_callback leave, TRUE, set client_error_level callback to %d\n",Z_LVAL_PP(callback_zval));
			WBLSM_END;
			RETURN_TRUE;
		case IS_STRING:
			if( conn->client_error_callback!=NULL ) { efree(conn->client_error_callback); conn->client_error_callback=NULL; }
			if( Z_STRLEN_PP(callback_zval)!=0 )
			{
				conn->client_error_callback=estrdup(Z_STRVAL_PP(callback_zval));
			}
			set_client_error_callback2(CDP(conn),(conn->client_error_level!=0||conn->client_error_callback!=NULL)?php_602sql_client_error_callback:NULL);
			wblog("wb_set_client_error_callback leave, TRUE, set client_error_callback to \"%s\"\n",Z_STRVAL_PP(callback_zval));
			WBLSM_END;
			RETURN_TRUE;
		default:
			wblogs("wb_set_client_error_callback leave, FALSE, invalid value of callback\n");
			zend_error(E_WARNING,"602SQL: Invalid value of callback parameter of wb_set_client_error_callback function.");
			WBLSM_END;
			RETURN_FALSE;
	}
} /*}}}*/
/*}}}*/
/* wb_odbc_connect {{{
 * mixed wb_odbc_connect(string dsn [, string uid [, string pwd] ] )*/
ZEND_FUNCTION(wb_odbc_connect)
{
	zval **dsn_zval,**uid_zval,**pwd_zval;
	struct odbc_t *odbc;
	int id;
	
	wblogs("wb_odbc_connect enter\n");
	switch( ZEND_NUM_ARGS() )
	{
		case 1:
			if( zend_get_parameters_ex(1,&dsn_zval)!=SUCCESS ) WRONG_PARAM_COUNT;
			uid_zval=pwd_zval=NULL;
			break;
		case 2:
			if( zend_get_parameters_ex(2,&dsn_zval,&uid_zval)!=SUCCESS ) WRONG_PARAM_COUNT;
			pwd_zval=NULL;
			break;
		case 3:
			if( zend_get_parameters_ex(3,&dsn_zval,&uid_zval,&pwd_zval)!=SUCCESS ) WRONG_PARAM_COUNT;
			break;
		default:
			WRONG_PARAM_COUNT;
	}
	convert_to_string_ex(dsn_zval);
	if( uid_zval!=NULL ) convert_to_string_ex(uid_zval);
	if( pwd_zval!=NULL ) convert_to_string_ex(pwd_zval);
	wblogs("wb_odbc_connect leave\n");
	odbc=(struct odbc_t*)emalloc(sizeof(struct odbc_t));
	if( odbc==NULL )
	{
		report_out_of_memory_error("Can't allocate internal structure.");
		wblogs("wb_odbc_connect leave, error: out of memory\n");
		RETURN_FALSE;
	}
	// initialize data fields of odbc_t struct
	T_PCONNECTION_FROM_ODBC_P(odbc)=NULL;
	// connect to ODBC data source
	if( (T_PCONNECTION_FROM_ODBC_P(odbc)=ODBC_connect(Z_STRVAL_PP(dsn_zval),(uid_zval==NULL)?NULL:Z_STRVAL_PP(uid_zval),(pwd_zval==NULL)?NULL:Z_STRVAL_PP(pwd_zval),NULL))==NULL )
	{
		zend_error(E_WARNING,"602SQL: Failed to connect to ODBC DSN %s.",Z_STRVAL_PP(dsn_zval));
		wblog("wb_odbc_connect leave, ERROR: ODBC_connect() failed for DSN %s.",Z_STRVAL_PP(dsn_zval));
		efree(odbc);
		RETURN_FALSE;
	}
	/* create and register resource */
	id=zend_list_insert(odbc,wbres_odbc
#if PHP_VERSION_ID>50400
				TSRMLS_CC
#endif
		);
	wblog("wb_odbc_connect leave, result=%d (odbc resource)\n",id);
	/* return resource */
	RETURN_RESOURCE(id);
} 
/*}}}*/
/* wb_odbc_disconnect - is an alias for wb_close */
#endif
/* wb_escape_string {{{
 * string wb_escape_string(resource connection,string value)
 * Returns escaped form of given string value or FALSE on error.
 * Converts value from script encoding to database system encoding.
 * Then doubles all apostrophes and replaces unprintable characters
 * with '#DECIMALASCIICODE' construction.
 * Example of usage:
 *   $cursor=wb_exec($connection,'SELECT * FROM table WHERE column=\''.wb_escape_string($connection,$_REQUEST['userinput']).'\';');
 * Notice outer apostrophes of string constant have to be placed around result of wb_escape_string() function. */
ZEND_FUNCTION(wb_escape_string)
{
	WBLSM_DECLARE
	zval **string_zval,**conn_zval;
	wb_conn *conn;
	char *tmp_string,*ptr,*bufptr;
#ifdef USE_WB_SCRIPT_ENCODING
	char *emalloc_string=NULL;
#endif
	int string_len_diff,last_char_is_sharp_escaped;

	wblogs("wb_escape_string enter\n");

	if (2!=ZEND_NUM_ARGS() || zend_get_parameters_ex(2,&conn_zval,&string_zval)!=SUCCESS) WRONG_PARAM_COUNT;
	convert_to_string_ex(string_zval);

	conn=get_conn_from_resource(INTERNAL_FUNCTION_PARAM_PASSTHRU,conn_zval);
	if (!conn) { wblogs("wb_escape_string leave, return FALSE\n"); RETURN_FALSE; /* error in get_conn_from_resource */ }
	WBLSM_START("wb_escape_string",CDP(conn));

	/* get string */
	if( !PREPARE_STRING_SCRIPT2SYSTEM(WBG(script_encoding),conn,&tmp_string,&emalloc_string,Z_STRVAL_PP(string_zval)) )
	{
		wblogs("wb_escape_string leave, ERROR, PREPARE_STRING_SCRIPT2SYSTEM() failed\n");
		WBLSM_END;
		RETURN_FALSE;
	}

	/* count difference between string length and escaped string length */
	string_len_diff=0; last_char_is_sharp_escaped=0;
	for( ptr=tmp_string;*ptr!='\0';ptr++ )
	{
		if( *ptr=='\'' ) string_len_diff++;
		else if( *ptr<32 )
		{
			if( last_char_is_sharp_escaped ) string_len_diff+=3;
			else { string_len_diff+=5; last_char_is_sharp_escaped=1; }
		}
		if( last_char_is_sharp_escaped && *ptr>=32 ) last_char_is_sharp_escaped=0;
	}

	/* compose return value */
	if( string_len_diff==0 )
	{
		RETVAL_STRING(tmp_string,1);
	}
	else
	{
		char *buf=emalloc(strlen(tmp_string)+string_len_diff+1);
		if( buf==NULL )
		{
			report_out_of_memory_error("Can't allocate temporary buffer.");
			wblogs("wb_escape_string leave, false, ERROR, out of memory\n");
			WBLSM_END;
			RETURN_FALSE;
		}
		last_char_is_sharp_escaped=0; bufptr=buf;
		for( ptr=tmp_string;*ptr!='\0';ptr++ )
		{
			if( last_char_is_sharp_escaped && *ptr>=32 )
			{
				*bufptr='\'';
				bufptr++;
				last_char_is_sharp_escaped=0;
			}
			if( *ptr=='\'' ) { *bufptr='\''; bufptr++; *bufptr='\''; bufptr++; }
			else if( *ptr<32 )
			{
				if( !last_char_is_sharp_escaped )
				{
					*bufptr='\''; bufptr++;
				}
				sprintf(bufptr,"#%d",(int)*ptr);
				bufptr+=((*ptr<10)?2:3);
				last_char_is_sharp_escaped=1;
			}
			else { *bufptr=*ptr; bufptr++; }
		}
		if( last_char_is_sharp_escaped ) { *bufptr='\''; }
		RETVAL_STRING(buf,0);
	}

	/* clean up temporary variables */
  EFREE_EMALLOC_STRING(emalloc_string);
	wblog("wb_escape_string leave, result is \"%s\"\n",Z_STRVAL_P(return_value));
	WBLSM_END;
}
/*}}}*/
#if WBVERS>=1000
/* wb_zfo_to_fo {{{
 * mixed wb_zfo_to_fo(string zfo_source)
 * returns false on error or FO source on success */
ZEND_FUNCTION(wb_zfo_to_fo)
{
	zval **zfo_zval;
	INT_PTR handle,find_handle,entry_handle;
	QWORD size;
	char *buffer;
	int errnum;

	wblogs("wb_zfo_to_fo enter\n");
	if( 1!=ZEND_NUM_ARGS() || zend_get_parameters_ex(1,&zfo_zval)!=SUCCESS ) WRONG_PARAM_COUNT;
	convert_to_string_ex(zfo_zval);
	if( (errnum=UnZipOpenM(Z_STRVAL_PP(zfo_zval),Z_STRLEN_PP(zfo_zval),&handle))!=Z_OK )
	{ 
		wblog("wb_zfo_to_fo leave: ERROR, UnZipOpenM() failed (%d)\n",errnum);
		zend_error(E_WARNING,"602SQL: Failed to convert XML-ZFO to XML-FO. (UnZipOpenM()=%d)",errnum);
		RETURN_FALSE;
	}
	if( (errnum=UnZipFindFirst(handle,L"\\",1,&find_handle,&entry_handle))!=Z_OK )
	{ 
		wblog("wb_zfo_to_fo leave: ERROR, UnZipFindFirst() failed (%d)\n",errnum);
		zend_error(E_WARNING,"602SQL: Failed to convert XML-ZFO to XML-FO. (UnZipFindFirst()=%d)",errnum);
		RETURN_FALSE;
	}
	if( (errnum=UnZipGetFileSize(handle,entry_handle,&size))!=Z_OK )
	{ 
		wblog("wb_zfo_to_fo leave: ERROR, UnZipGetFileSize() failed (%d)\n",errnum);
		zend_error(E_WARNING,"602SQL: Failed to convert XML-ZFO to XML-FO. (UnZipGetFileSize()=%d)",errnum);
		RETURN_FALSE;
	}
	buffer=(char*)emalloc((int)size+1);
	if( buffer==NULL )
	{
		report_out_of_memory_error("Can not allocate buffer for XML-FO code.");
		wblogs("wb_zfo_to_fo leave: ERROR, emalloc() failed\n");
		RETURN_FALSE;
	}
	if( (errnum=UnZipFileM(handle,entry_handle,buffer,(int)size))!=Z_OK )
	{ 
		wblog("wb_zfo_to_fo leave: ERROR, UnZipFileM() failed (%d)\n",errnum);
		zend_error(E_WARNING,"602SQL: Failed to convert XML-ZFO to XML-FO. (UnZipFileM()=%d)",errnum);
		RETURN_FALSE;
	}
	UnZipClose(handle);
	RETURN_STRINGL(buffer,(int)size,0);
}
/*}}}*/
/* wb_fo_to_zfo {{{
 * mixed wb_fo_to_zfo(string fo_source)
 * return false on error or ZFO source on success */
ZEND_FUNCTION(wb_fo_to_zfo)
{
	zval **fo_zval;
	INT_PTR handle;
	QWORD size;
	char *buffer;
	int errnum;
	wchar_t *null_wchar_t=NULL;

	wblogs("wb_fo_to_zfo enter\n");
	if( 1!=ZEND_NUM_ARGS() || zend_get_parameters_ex(1,&fo_zval)!=SUCCESS ) WRONG_PARAM_COUNT;
	convert_to_string_ex(fo_zval);
	if( (errnum=ZipCreate(null_wchar_t,null_wchar_t,&handle))!=Z_OK )
	{
		wblog("wb_fo_to_zfo leave: ERROR, ZipCreate() failed (%d)\n",errnum);
		zend_error(E_WARNING,"602SQL: Failed to convert XML-FO to XML-ZFO. (ZipCreate()=%d)",errnum);
		RETURN_FALSE;
	}
	if( (errnum=ZipAddM(handle,Z_STRVAL_PP(fo_zval),Z_STRLEN_PP(fo_zval),L"form.fo",1))!=Z_OK )
	{
		ZipCloseM(handle,null_wchar_t,0);
		wblog("wb_fo_to_zfo leave: ERROR, ZipAddM() failed (%d)\n",errnum);
		zend_error(E_WARNING,"602SQL: Failed to convert XML-FO to XML-ZFO. (ZipAddM()=%d)",errnum);
		RETURN_FALSE;
	}
	if( (errnum=ZipGetSize(handle,&size))!=Z_OK )
	{
		ZipCloseM(handle,NULL,0);
		wblog("wb_fo_to_zfo leave: ERROR, ZipGetSize() failed (%d)\n",errnum);
		zend_error(E_WARNING,"602SQL: Failed to convert XML-FO to XML-ZFO. (ZipGetSize()=%d)",errnum);
		RETURN_FALSE;
	}
	buffer=(char*)emalloc((int)size+1);
	if( buffer==NULL )
	{
		report_out_of_memory_error("Can not allocate buffer for XML-ZFO code.");
		wblogs("wb_fo_to_zfo leave: ERROR, emalloc() failed\n");
		RETURN_FALSE;
	}
	if( (errnum=ZipCloseM(handle,buffer,(int)size))!=Z_OK )
	{
		wblog("wb_fo_to_zfo leave: ERROR, ZipCloseM() failed (%d)\n",errnum);
		zend_error(E_WARNING,"602SQL: Failed to convert XML-FO to XML-ZFO. (ZipCloseM()=%d)",errnum);
		RETURN_FALSE;
	}
	buffer[size]='\0';
	RETVAL_STRINGL(buffer,(int)size,0);
}
/*}}}*/
#endif
/*}}}*/
#if WBVERS<900
/*{{{ language selection and metastring translation functions */
/* wb_select_language {{{
 * bool wb_select_language(resource connection,int language_id)
 * bool wb_select_language(resource connection,string language_name)
 * returns TRUE on success, FALSE on error */
ZEND_FUNCTION(wb_select_language)
{ 
	zval **connhandle, **lang;
	wb_conn *conn;
	cdp_t cdp;
	int lang_id;

	wblogs("wb_select_language enter\n");

	if (ZEND_NUM_ARGS()!=2|| zend_get_parameters_ex(2, &connhandle, &lang)!=SUCCESS)
			WRONG_PARAM_COUNT;

	conn=get_conn_from_resource(INTERNAL_FUNCTION_PARAM_PASSTHRU,connhandle);
	cdp=CDP(conn);
	if (!conn) RETURN_FALSE; /* error in get_conn_from_resource */

	if (Z_TYPE_PP(lang)==IS_LONG) lang_id=Z_LVAL_PP(lang);
	else {
		tobjnum multiling_table;
		tattrib attnum;
		uns8 atttype,attmulti;
		uns16 attspecif;
		/* Get language ID from language name: */
		convert_to_string_ex(lang);
		if( cd_Find_object(cdp,"_multiling",CATEG_TABLE,&multiling_table) )
		{
			CONN_REPORT_ERR(conn,"wb_select_language (_multiling table doesn't exist in 602SQL application)");
			wblogs("wb_select_language leave, cd_Find_object failed\n");
			RETURN_FALSE;
		}
		if( !cd_Attribute_info(cdp,multiling_table,Z_STRVAL_PP(lang),&attnum,&atttype,&attmulti,&attspecif) )
		{
			CONN_REPORT_ERR(conn,"wb_select_language (can't read column info from _multiling table)");
			wblogs("wb_select_language leave, cd_Attribute_info failed\n");
			RETURN_FALSE;
		}
		lang_id=attnum-2; // first column is ID (with attrnum=1), first language is stored in second column of _multiling table
	}

	if( !Select_language(cdp,lang_id) )
	{
		CONN_REPORT_ERR(conn,"wb_select_language (can't select desired language)");
		wblogs("wb_select_language leave, Select_language failed\n");
		RETURN_FALSE;
	}
	/* Save selected language ID to conn structure */
	conn->selected_language_id=lang_id;
	wblogs("wb_select_language leave\n");
	RETURN_TRUE;
}
/*}}}*/
/* wb_get_selected_language_id {{{
 * int wb_get_selected_language_id(resource connection)
 * returns language ID on success (positive or zero), or -1 on error */
ZEND_FUNCTION(wb_get_selected_language_id)
{
	zval **connhandle;
	wb_conn *conn;
	cdp_t cdp;

	wblogs("wb_get_selected_language_id enter\n");

	if (ZEND_NUM_ARGS()!=1|| zend_get_parameters_ex(1, &connhandle)!=SUCCESS)
			WRONG_PARAM_COUNT;

	conn=get_conn_from_resource(INTERNAL_FUNCTION_PARAM_PASSTHRU,connhandle);
	cdp=CDP(conn);
	if (!conn) RETURN_FALSE; /* error in get_conn_from_resource */

	RETURN_LONG(conn->selected_language_id);
} 
/*}}}*/
/* wb_get_selected_language_name {{{*/
static int get_language_name(wb_conn *conn,int lang_id,char *lang_name TSRMLS_DC)
{
	uns32 results[10];
	struct t_clivar cv[2];

	/* Get language name from language ID: */
	*results=0;
	*lang_name='\0';
	strcpy(cv[0].name,"COLNAME");
	cv[0].mode=MODE_OUT;
	cv[0].wbtype=ATT_STRING;
#if WBVERS>=800
	cv[0].specif=0; // initialize
	T_SPECIF_CAST(cv[0].specif).length=NAMELEN;
#endif
	cv[0].buf=lang_name;
	cv[0].buflen=NAMELEN;
	cv[0].actlen=0;
	strcpy(cv[1].name,"LANGID");
	cv[1].mode=MODE_IN;
	cv[1].wbtype=ATT_INT32;
#if WBVERS>=800
	T_SPECIF_CAST(cv[1].specif).scale=0;
	T_SPECIF_CAST(cv[1].specif).precision=14;
#endif
	cv[1].buf=&lang_id;
	cv[1].buflen=cv[1].actlen=sizeof(lang_id);
	if( cd_SQL_host_execute(CDP(conn),"SELECT column_name INTO :>COLNAME FROM _iv_table_columns WHERE table_name=\"_multiling\" AND schema_name=current_application"
				" AND ordinal_position=:<LANGID+2",results,cv,2) )
	{
		CONN_REPORT_ERR(conn,"get_language_name (can't read language name)");
		return 0;
	}
	return 1;
}
ZEND_FUNCTION(wb_get_selected_language_name)
{
	zval **connhandle;
	wb_conn *conn;
	cdp_t cdp;
	tname lang_name;

	wblogs("wb_get_selected_language_name enter\n");

	if (ZEND_NUM_ARGS()!=1|| zend_get_parameters_ex(1, &connhandle)!=SUCCESS)
			WRONG_PARAM_COUNT;

	conn=get_conn_from_resource(INTERNAL_FUNCTION_PARAM_PASSTHRU,connhandle);
	cdp=CDP(conn);
	if (!conn) RETURN_FALSE; /* error in get_conn_from_resource */
	
	if( conn->selected_language_id==-1 )
	{
		wblogs("wb_get_selected_language_name leave, no language selected\n");
		RETURN_EMPTY_STRING();
	}
	if( !get_language_name(conn,conn->selected_language_id,lang_name TSRMLS_CC) )
	{
		RETURN_EMPTY_STRING();
	}

	wblogs("wb_get_selected_language_name leave\n");
	RETURN_STRING(lang_name,1/*duplicate*/);
}
/*}}}*/
/* wb_get_languages {{{
 * array wb_get_languages(resource connection)
 * returns NULL if none language is defined in this 602SQL application or on error
 * returns array indexed by language IDs and each item contains name of language (string) */
ZEND_FUNCTION(wb_get_languages)
{
	zval **connhandle;
	wb_conn *conn;
	cdp_t cdp;
	uns32 results[10];
	struct t_clivar cv;
	sig32 colcount,i;
	tobjnum multiling_table;

	wblogs("wb_get_languages enter\n");

	if (ZEND_NUM_ARGS()!=1|| zend_get_parameters_ex(1, &connhandle)!=SUCCESS)
			WRONG_PARAM_COUNT;

	conn=get_conn_from_resource(INTERNAL_FUNCTION_PARAM_PASSTHRU,connhandle);
	cdp=CDP(conn);
	if (!conn) RETURN_NULL(); /* error in get_conn_from_resource */

	/* test if _multiling table exists */
	if( cd_Find_object(cdp,"_multiling",CATEG_TABLE,&multiling_table) )
	{
		CONN_REPORT_ERR(conn,"wb_get_languages (_multiling table doesn't exist in 602SQL application)");
		wblogs("wb_get_languages leave, cd_Find_object failed\n");
		RETURN_NULL();
	}
	/* get number of columns in _multiling table */
	*results=0;
	strcpy(cv.name,"COLCOUNT");
	cv.mode=MODE_OUT;
	cv.wbtype=ATT_INT32;
#if WBVERS>=800
	T_SPECIF_CAST(cv.specif).scale=0;
	T_SPECIF_CAST(cv.specif).precision=14;
#endif
	cv.buf=&colcount;
	cv.buflen=cv.actlen=sizeof(colcount);
	if( cd_SQL_host_execute(cdp,"SELECT Count(*) INTO :>COLCOUNT FROM _iv_table_columns WHERE table_name=\"_multiling\" AND schema_name=current_application",
				results,&cv,1) )
	{
		CONN_REPORT_ERR(conn,"wb_get_languages (can't read number of languages)");
		wblogs("wb_get_languages leave, cd_SQL_host_execute() failed\n");
		RETURN_NULL();
	}
	/* get names of all languages */
	if( array_init(return_value)!=SUCCESS )
	{
		php_error(E_WARNING,"602SQL: can't initialize array for language names");
		wblogs("wb_get_languages leave, array_init failed\n");
		RETURN_NULL();
	}
	for( i=0;i<colcount-1;i++ )
	{
		tname lang_name;
		if( !get_language_name(conn,i,lang_name TSRMLS_CC) )
		{
			RETURN_NULL();
		}
		add_index_string(return_value,i,lang_name,1);
	}
} 
/*}}}*/
/* wb_metastring {{{
 * string wb_metastring(resource connection,string metastring) */
ZEND_FUNCTION(wb_metastring)
{
#define MS_BUF_LEN 1000
	zval **connhandle, **metastring;
	wb_conn *conn;
	cdp_t cdp;
	char ms_buf[MS_BUF_LEN+1];

	wblogs("wb_metastring enter\n");

	if (ZEND_NUM_ARGS()!=2|| zend_get_parameters_ex(2, &connhandle, &metastring)!=SUCCESS)
			WRONG_PARAM_COUNT;

	conn=get_conn_from_resource(INTERNAL_FUNCTION_PARAM_PASSTHRU,connhandle);
	cdp=CDP(conn);
	if (!conn) RETURN_FALSE; /* error in get_conn_from_resource */

	convert_to_string_ex(metastring);
	*ms_buf='@';
	strncpy(ms_buf+1,Z_STRVAL_PP(metastring),MS_BUF_LEN-1);
	ms_buf[MS_BUF_LEN]='\0';
	if( !Translate_to_language(cdp,ms_buf,MS_BUF_LEN) )
	{
		zend_error(E_WARNING,"602SQL: can't translate metastring \"%s\"",Z_STRVAL_PP(metastring));
		wblogs("wb_metastring leave, Translate_to_language failed\n");
		RETURN_EMPTY_STRING();
	}
#if WBVERS>=800
	wblog("strlen(ms_buf)=%d\n",strlen(ms_buf));
	if( strlen(ms_buf)>=MS_BUF_LEN-1 )
	{
		char *translated;
		*ms_buf='@';
		strncpy(ms_buf+1,Z_STRVAL_PP(metastring),MS_BUF_LEN-1);
		ms_buf[MS_BUF_LEN]='\0';
		if( !Translate_and_alloc(cdp,ms_buf,&translated) )
		{
			zend_error(E_WARNING,"602SQL: can't translate metastring \"%s\"",Z_STRVAL_PP(metastring));
			wblogs("wb_metastring leave, Translate_and_alloc failed\n");
			RETURN_EMPTY_STRING();
		}
		RETVAL_STRING(translated,1/*duplicate*/);
		corefree(translated);
		wblogs("wb_metastring leave\n");
		return;
	}
#endif
	wblogs("wb_metastring leave\n");
	RETURN_STRING(ms_buf,1/*duplicate*/);
}

/*}}}*/
/*}}}*/
#endif
/*{{{ LOB resources manipulation functions */
/*{{{ wb_set_lob_mode
 * boolean wb_set_lob_mode(resource cursor,integer lob_mode)
 * sets cursor's LOB manipulation mode to lob_mode
 * returns TRUE on success, FALSE on error */
ZEND_FUNCTION(wb_set_lob_mode)
{
	WBLSM_DECLARE
	zval **result_id, **lob_mode;
	struct sql_answer *answer;

	wblogs("wb_set_lob_mode enter\n");

	if( ZEND_NUM_ARGS()!=2||zend_get_parameters_ex(2, &result_id, &lob_mode)!=SUCCESS )
			WRONG_PARAM_COUNT;

	if( (answer=get_answer_from_result(INTERNAL_FUNCTION_PARAM_PASSTHRU,result_id))==NULL )
	{
		wblogs("wb_set_lob_mode leave, FALSE\n");
		RETURN_FALSE;
	}
	WBLSM_START("wb_set_lob_mode",CDP_FROM_ANSWER_P(answer));

	convert_to_long_ex(lob_mode);

	if( Z_LVAL_PP(lob_mode)!=WB_LOBMODE_DIRECT && Z_LVAL_PP(lob_mode)!=WB_LOBMODE_RESOURCE )
	{
		zend_error(E_WARNING,"602SQL: unknown LOB manipulation mode \"%ld\"",Z_LVAL_PP(lob_mode));
		wblogs("wb_set_lob_mode leave, FALSE, unknown LOB manipulation mode\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	answer->lob_mode=Z_LVAL_PP(lob_mode);

	wblogs("wb_set_lob_mode leave\n");
	WBLSM_END;
	RETURN_TRUE;
}
/*}}}*/
/*{{{ wb_lob_write
 * boolean wb_lob_write(resource lob,integer offset,integer size,string data)
 * writes first size characters from data to lob from offset position; if lob
 * is CLOB or NCLOB size can be 0 which means all content of data will be
 * written to lob
 * start of lob has offset 0
 * returns TRUE on success, FALSE on error
 * offset is
 *   in case of CLOB or BLOB: number of 8-bit chars from start of lob
 *             NCLOB:         number of 16-bit chars from start of lob
 * size is
 *   in case of client_encoding==WB_ENC_ASCII: number of 8-bit chars
 *              client_encoding==WB_ENC_UTF8:  number of UTF-8 chars (variable length)
 * Allowed combinations of client_encoding and LOB type are:
 *   WB_ENC_ASCII and ( CLOB or BLOB )
 *   WB_ENC_UTF8  and NCLOB 
 * In all other cases the function returns FALSE. */
ZEND_FUNCTION(wb_lob_write)
{
	WBLSM_DECLARE
	zval **res_id,**par_offset,**par_size,**par_data;
	struct lob_info_t *lob;
	cdp_t cdp;
	uns32 size,offset;
	void *buf=NULL;

	wblogs("wb_lob_write enter\n");

	if( ZEND_NUM_ARGS()!=4||zend_get_parameters_ex(4, &res_id, &par_offset, &par_size, &par_data)!=SUCCESS )
			WRONG_PARAM_COUNT;

	if( (lob=get_lob_from_resource(INTERNAL_FUNCTION_PARAM_PASSTHRU,res_id))==NULL )
	{
		wblogs("wb_lob_write leave, FALSE\n");
		RETURN_FALSE;
	}
	WBLSM_START("wb_lob_write",CDP_FROM_LOB_P(lob));

	convert_to_long_ex(par_offset);
	convert_to_long_ex(par_size);
	convert_to_string_ex(par_data);

	offset=Z_LVAL_PP(par_offset);
	size=Z_LVAL_PP(par_size);
	wblog("raw values: offset=%d, size=%d\n",offset,size);
#if WBVERS>=800
	if( IS_LOB_NCLOB(lob) )
	{
		int tmp_size;
		offset*=2; // convert from UCS-2 character length to byte length
		switch( ENCODING(lob->answer->conn) ) {
			case WB_ENC_ASCII:
				buf=strASCIItoUCS2(Z_STRVAL_PP(par_data),size,&tmp_size);
				if( buf==NULL ) {
					wblogs("wb_lob_write leave: ERROR, strASCIItoUCS2() failed\n");
					WBLSM_END;
					RETURN_FALSE;
				}
				break;
			case WB_ENC_WIN1250:
				buf=(char*)strCP1250toUCS2(Z_STRVAL_PP(par_data),size,&tmp_size);
				if( buf==NULL ) {
					wblogs("wb_lob_write leave: ERROR, strCP1250toUCS2() failed\n");
					WBLSM_END;
					RETURN_FALSE;
				}
				break;
			case WB_ENC_ISO_8859_2:
				buf=(char*)strISO8859_2toUCS2(Z_STRVAL_PP(par_data),size,&tmp_size);
				if( buf==NULL ) {
					wblogs("wb_lob_write leave: ERROR, strISO8859_2toUCS2() failed\n");
					WBLSM_END;
					RETURN_FALSE;
				}
				break;
			case WB_ENC_UTF8:
				buf=strUTF8toUCS2(Z_STRVAL_PP(par_data),size,&tmp_size);
				if( buf==NULL ) {
					wblogs("wb_lob_write leave: ERROR, strUTF8toUCS2() failed\n");
					WBLSM_END;
					RETURN_FALSE;
				}
				break;
			default:
				php_error(E_WARNING,"602SQL error: unsupported encoding no. %d for writing value to NCLOB column",ENCODING(lob->answer->conn));
				wblog("wb_lob_write leave: false, ERROR unsupported encoding no. %d for writing value to NCLOB column\n",ENCODING(lob->answer->conn));
				WBLSM_END;
				RETURN_FALSE;
		}
		size=(uns32)tmp_size;
		// buf is emalloc-ed buffer
	}
	else // CLOB & BLOB
	{
		if( IS_LOB_CLOB(lob) && ENCODING(lob->answer->conn)==WB_ENC_UTF8 )
		{ 
			wblogs("wb_lob_write leave, FALSE - can't write UTF-8 string to non-NCLOB column\n");
			zend_error(E_WARNING,"602SQL: can't write UTF-8 string to a database column which type isn't NCLOB");
			WBLSM_END;
			RETURN_FALSE;
		}
#endif
		if( size==0 ) size=Z_STRLEN_PP(par_data);
		buf=Z_STRVAL_PP(par_data);
#if WBVERS>=800
	}
#endif
	cdp=CDP_FROM_LOB_P(lob);

	wblog("calling cd_Write_var with parameters: recnum=%d, attrib=%d, offset=%d, size=%d\n",lob->recnum,lob->attrib,offset,size);
	if( size>=10 )
	{
		wblogmem("first 10 chars of buf: ",buf,10);
	}
	if( cd_Write_var(cdp,lob->answer->cursor,lob->recnum,lob->attrib,NOINDEX,offset,size/*in bytes*/,buf) )
	{
		CONN_REPORT_ERR(CONN_P_FROM_ANSWER_P(ANSWER_P_FROM_LOB_P(lob)),"Write_var");
		wblogs("wb_lob_write leave, FALSE, cd_Write_var failed\n");
#if WBVERS>=800
		if( IS_LOB_NCLOB(lob) ) efree(buf);
#endif
		WBLSM_END;
		RETURN_FALSE;
	}

#if WBVERS>=800
  if( IS_LOB_NCLOB(lob) ) efree(buf);
#endif
	wblogs("wb_lob_write leave, TRUE\n");
	WBLSM_END;
	RETURN_TRUE;
}
/*}}}*/
/*{{{ wb_get_lob_mode
 * integer wb_get_lob_mode(resource cursor)
 * returns cursor's LOB manipulation mode or FALSE on error */
ZEND_FUNCTION(wb_get_lob_mode)
{
	WBLSM_DECLARE
	zval **result_id;
	struct sql_answer *answer;
	int result;

	wblogs("wb_get_lob_mode enter\n");
	if (ZEND_NUM_ARGS()!=1||zend_get_parameters_ex(1, &result_id)!=SUCCESS)
		WRONG_PARAM_COUNT;
	if( (answer=get_answer_from_result(INTERNAL_FUNCTION_PARAM_PASSTHRU,result_id))==NULL )
	{
		wblogs("wb_get_lob_mode leave, FALSE\n");
		RETURN_FALSE;
	}
	WBLSM_START("wb_get_lob_mode",CDP_FROM_ANSWER_P(answer));
	result=((answer)?answer->lob_mode:0);
	wblogs("wb_get_lob_mode leave\n");
	WBLSM_END;
	RETURN_LONG(result);
}
/*}}}*/
/*{{{ wb_lob_length
 * integer wb_lob_length(resource lob)
 * returns size of lob in characters or FALSE on error
 * characters are:
 *   in case of CLOB	8-bit chars (ASCII)
 *             NCLOB	16-bit chars (UCS-2)
 *              BLOB	8-bit (binary) chars */
ZEND_FUNCTION(wb_lob_length)
{
	WBLSM_DECLARE
	uns32 lobsize;
	cdp_t cdp;
	zval **res_id;
	struct lob_info_t *lob;

	wblogs("wb_lob_length enter\n");

	if( ZEND_NUM_ARGS()!=1||zend_get_parameters_ex(1, &res_id)!=SUCCESS )
			WRONG_PARAM_COUNT;

	if( (lob=get_lob_from_resource(INTERNAL_FUNCTION_PARAM_PASSTHRU,res_id))==NULL )
	{
		wblogs("wb_lob_length leave, FALSE\n");
		RETURN_FALSE;
	}

	cdp=CDP_FROM_LOB_P(lob);
	WBLSM_START("wb_lob_length",cdp);
	if( cd_Read_len(cdp,lob->answer->cursor,lob->recnum,lob->attrib,NOINDEX,&lobsize) )
	{
		CONN_REPORT_ERR(CONN_P_FROM_ANSWER_P(ANSWER_P_FROM_LOB_P(lob)),"Can't read size of LOB value");
		wblogs("wb_lob_length leave, FALSE, error in cd_Read_len()\n");
		WBLSM_END;
		RETURN_FALSE;
	}
#if WBVERS>=800
	if( IS_LOB_NCLOB(lob) )
	{
		wblogs("wb_lob_length: NCLOB size recomputed\n");
		lobsize/=2; /* UCS-2 chars are 2 bytes long */
	}
#endif

	wblog("wb_lob_length leave, result is %lu\n",lobsize);
	WBLSM_END;
	RETURN_LONG((int)lobsize);
}
/*}}}*/
/*{{{ wb_lob_read
 * string wb_lob_read(resource lob,integer offset,integer size)
 * returns lob's value segment starting at offset character size characters long or FALSE on error
 * first character of lob's value has offset 0, size have to be positive number
 * characters are:
 *   in case of CLOB	8-bit chars (ASCII)
 *             NCLOB	16-bit chars (UCS-2)
 *              BLOB	8-bit (binary) chars */
ZEND_FUNCTION(wb_lob_read)
{ 
	WBLSM_DECLARE
	zval **res_id,**par_offset,**par_size;
	struct lob_info_t *lob;
	cdp_t cdp;
	uns32 readed,size,offset,readed_bytes;
	char *buf;
	int convert_binary_to_hexa;

	wblogs("wb_lob_read enter\n");
 
	if( ZEND_NUM_ARGS()!=3||zend_get_parameters_ex(3, &res_id, &par_offset, &par_size)!=SUCCESS )
	{
		WRONG_PARAM_COUNT;
	}
 
	convert_to_long_ex(par_offset);
	offset=Z_LVAL_PP(par_offset);
	convert_to_long_ex(par_size);
	size=Z_LVAL_PP(par_size);
	wblog("wb_lob_read: offset=%d, size=%d (type=%d)\n",offset,size,Z_TYPE_PP(par_size));

	if( (lob=get_lob_from_resource(INTERNAL_FUNCTION_PARAM_PASSTHRU,res_id))==NULL )
	{
		wblogs("wb_lob_read leave, FALSE\n");
		RETURN_FALSE;
	}
	WBLSM_START("wb_lob_read",CDP_FROM_LOB_P(lob));
 
	if( Z_LVAL_PP(par_offset)<0 )
	{
		zend_error(E_WARNING,"602SQL: parameter 'offset' of function wb_lob_read can't be negative");
		wblogs("wb_lob_read leave, FALSE, negative offset\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	if( Z_LVAL_PP(par_size)<=0 )
	{
		zend_error(E_WARNING,"602SQL: parameter 'size' of function wb_lob_read have to be positive");
		wblog("wb_lob_read leave, FALSE, non-positive size, size is %d\n",size);
		WBLSM_END;
		RETURN_FALSE;
	}

#if WBVERS>=800
	if( IS_LOB_NCLOB(lob) )
	{
		offset*=2;
		size*=2;
		wblog("wb_lob_read: IS_LOB_NCLOB - offset=%d, size=%d\n",offset,size);
	}
#endif
	cdp=CDP_FROM_LOB_P(lob);

  convert_binary_to_hexa=lob->answer->binmode==CONVERT && lob->answer->types[lob->attrib-1/*types array items start from 0 while attributes in cursor start from 1*/]!=ATT_TEXT
#if WBVERS>=1100
		&& lob->answer->types[lob->attrib-1/*types array items start from 0 while attributes in cursor start from 1*/]!=ATT_EXTCLOB
#endif
		;
	if( !convert_binary_to_hexa )	buf=(char*)emalloc(size+2);
	else buf=(char*)emalloc(size*2+2); // we need double space to convert binary value to hexadecimal notation
	if( buf==NULL )
	{
		zend_error(E_WARNING,"602SQL: Out of memory");
		wblogs("wb_lob_read leave, FALSE\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	wblog("lob->attrib=%d\n",lob->attrib);
	if( !db_read_lob_value(CONN_P_FROM_ANSWER_P(ANSWER_P_FROM_LOB_P(lob)),lob->answer->cursor,lob->recnum,lob->attrib,offset,&size,&readed,&readed_bytes,&buf,convert_binary_to_hexa,lob->answer->types[lob->attrib-1]
#if WBVERS>=800
				,lob->answer->specif[lob->attrib-1]
#endif
#ifdef USE_WB_SCRIPT_ENCODING
				,WBG(script_encoding)
#endif
				TSRMLS_CC) )
	{
		efree(buf);
		wblogs("wb_lob_read leave, FALSE, error in db_read_lob_value()\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	if( readed==0 )
	{
		efree(buf);
		wblogs("wb_lob_read leave, FALSE, nothing readed\n");
		WBLSM_END;
		RETURN_FALSE;
	}

	wblog("wb_lob_read leave, OK, readed=%d, size=%d, result is \"%s\"\n",readed,size,buf);
	WBLSM_END;
	RETURN_STRINGL(buf,readed,FALSE);
} 
/*}}}*/
/*{{{ wb_lob_truncate
 * boolean wb_lob_truncate(resource lob,integer size)
 * truncates lob value to first size characters
 * size have to be between 0 and current lob length
 * returns TRUE on success, FALSE on error
 * characters are:
 *   in case of CLOB	8-bit chars (ASCII)
 *             NCLOB	16-bit chars (UCS-2)
 *              BLOB	8-bit (binary) chars */
ZEND_FUNCTION(wb_lob_truncate)
{
	WBLSM_DECLARE
	zval **res_id,**par_size;
	struct lob_info_t *lob;
	uns32 size;
	cdp_t cdp;

	wblogs("wb_lob_truncate enter\n");

	if( ZEND_NUM_ARGS()!=2||zend_get_parameters_ex(2, &res_id, &par_size)!=SUCCESS )
			WRONG_PARAM_COUNT;

	if( (lob=get_lob_from_resource(INTERNAL_FUNCTION_PARAM_PASSTHRU,res_id))==NULL )
	{
		wblogs("wb_lob_truncate leave, FALSE\n");
		RETURN_FALSE;
	}
	WBLSM_START("wb_lob_truncate",CDP_FROM_LOB_P(lob));

	convert_to_long_ex(par_size);
	if( Z_LVAL_PP(par_size)<0 )
	{
		zend_error(E_WARNING,"602SQL: new size of LOB can't be negative");
		wblogs("wb_lob_truncate leave, FALSE, negative size\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	size=Z_LVAL_PP(par_size);
#if WBVERS>=800
	if( IS_LOB_NCLOB(lob) )
	{
		wblogs("wb_lob_truncate: NCLOB size recomputed\n");
		size*=2; /* UCS-2 chars are 2 bytes long */
	}
#endif

	cdp=CDP_FROM_LOB_P(lob);
	if( cd_Write_len(cdp,lob->answer->cursor,lob->recnum,lob->attrib,NOINDEX,size) )
	{
		CONN_REPORT_ERR(CONN_P_FROM_ANSWER_P(ANSWER_P_FROM_LOB_P(lob)),"Can't truncate LOB value");
		wblogs("wb_lob_truncate leave, FALSE, error in cd_Write_len()\n");
		WBLSM_END;
		RETURN_FALSE;
	}

	wblogs("wb_lob_truncate leave, TRUE\n");
	WBLSM_END;
	RETURN_TRUE;
}
/*}}}*/
/*}}}*/
#ifdef USE_PSQL
/*{{{ prepared SQL statement functions */
/*{{{ support functions */
/*{{{ internal_psql_phpvalue2wbinparam() 
 * converts PHP variable's value to data
 * which can be sent to SQL server as PSQL statement parameter value
 * returns 0 on error and reports error to PHP core
 *      or 1 on success */
static int internal_psql_phpvalue2wbinparam(
		struct psql_t *psql, // prepared PSQL statement
		struct psql_param_info_t *ppi, // param whose value is being sent to SQL server
		zval **phpvar, // PHP variable which contains value of parameter
		uns32 phpval_offset, // optional offset of value to send in PHP variable's value or 0
		uns32 phpval_size, // optional size of value to send or 0
		void **pvalue, // outparam: pointer to value which has to be sent to SQL server
		uns32 *pvalue_offset, // outparam: offset of value to send in pvalue buffer
		uns32 *pvalue_size, // outparam: size of value to send
		int *is_emalloced // outparam: if is 0 then pvalue points to PHP variable's value - i.e. caller mustn't efree pvalue buffer
		                  //           if is 1 then pvalue is emalloc-ed buffer and caller have to efree it
		)
{
	return internal_phpvalue2wbvalue(
#if WBVERS>=800
			ENCODING(CONN_P_FROM_PSQL_P(psql)),
#endif
			ppi->type_id,ppi->specif,phpvar,phpval_offset,phpval_size,pvalue,pvalue_offset,pvalue_size,is_emalloced);
}
/*}}}*/
/*{{{ internal_psql_send_param */
static int internal_psql_send_param(struct psql_t *psql,sig16 param_idx,zval **buf_zval,uns32 buf_offset,uns32 buf_size TSRMLS_DC)
{
	void *pvalue;
	uns32 pvalue_offset,pvalue_size;
	int is_pvalue_emalloced;
	wblog("internal_psql_send_param enter, psql=%p, param_idx=%d, buf_zval=%p, buf_offset=%u, buf_size=%u\n",psql,param_idx,buf_zval,buf_offset,buf_size);
	if( !internal_psql_phpvalue2wbinparam(psql,
				PSQL_INPARAM(psql,param_idx),
				buf_zval,buf_offset,buf_size,
				&pvalue,&pvalue_offset,&pvalue_size,&is_pvalue_emalloced) )
	{
		wblogs("internal_psql_send_param leave, 0, internal_psql_send_param failed\n");
		return 0;
	}
	wblog("  pvalue=%p pvalue_size=%u pvalue_offset=%u is_pvalue_emalloced=%d\n",pvalue,pvalue_size,pvalue_offset,is_pvalue_emalloced);
	wblog("  pvalue as int = %d\n",*(int*)&pvalue);
	if( cd_Send_parameter(CDP_FROM_PSQL_P(psql),PSQL_HANDLE(psql),param_idx,pvalue_size,pvalue_offset,pvalue) )
	{
		CONN_REPORT_ERR(CONN_P_FROM_PSQL_P(psql),"cd_Send_parameter");
		wblogs("internal_psql_send_param leave, error in cd_Send_parameter\n");
		if( is_pvalue_emalloced ) efree(pvalue);
		return 0;
	}
	if( is_pvalue_emalloced ) efree(pvalue);
	wblogs("internal_psql_send_param leave, 1\n");
	return 1;
}
/*}}}*/
/*{{{ internal_psql_get_outparam_ptr()
 * */
static char *internal_psql_get_outparam_ptr(struct psql_t *psql,sig16 paridx)
{
	char *outptr;
	sig16 parcounter;
	wblog("internal_psql_get_outparam_ptr enter, psql=%p, paridx=%d\n",psql,paridx);
	outptr=psql->outparam_info;
	wblog("outparam_info=%p\n",psql->outparam_info);
	parcounter=0;
	while( *outptr!=SEND_PAR_DELIMITER && parcounter<paridx )
	{
		uns32 len=*((uns32*)(outptr+=sizeof(sig16)));
		outptr+=sizeof(uns32);
		if( len<=OUTPARAM_SIZE_LIMIT ) outptr+=len;
		parcounter++;
	}
	if( *outptr==SEND_PAR_DELIMITER )
	{
		zend_error(E_WARNING,"602SQL: OUT parameter with index %d doesn't exist.",paridx);
		wblog("internal_psql_get_outparam_ptr leave, false, ERROR, invalid index %d\n",paridx);
		return NULL;
	}
	// outptr points to start of paridx-th OUT parameter data in psql->outparam_info struct
	wblog("internal_psql_get_outparam_ptr leave: result is %p\n",outptr);
	return outptr;
}
/*}}}*/
/*{{{ internal_psql_wboutparam2phpvalue() */
/* Reads value of OUT param into PHP value.
 * Parameters:
 *   psql is prepared statement
 *   paridx is index of OUT param in psql->outparam_info which value has to be read
 *   phpval is PHP value structure into which the value of OUT parameter has to be read
 *   offset and size determine which part of OUT parameter value has to be read
 *     i.e. for offset==0 && size==0 the whole value has to be read
 * Function returns 1 on success, or 0 on error.*/
static int internal_psql_wboutparam2phpvalue(
#ifdef USE_WB_SCRIPT_ENCODING
		wb_encoding script_encoding,
#endif
		struct psql_t *psql,sig16 paridx,zval *phpval,uns32 offset,uns32 size TSRMLS_DC)
{
	char *outptr;
	sig16 outparidx;
	uns32 parlen;
	wblog("internal_psql_wboutparam2phpvalue enter, psql=%p, paridx=%d, phpval=%p, offset=%u, size=%u\n",psql,paridx,phpval,offset,size);
	outptr=internal_psql_get_outparam_ptr(psql,paridx);
	if( outptr==NULL ) {
		wblogs("internal_psql_wboutparam2phpvalue leave, ERROR, internal_psql_get_outparam_ptr failed\n");
		return 0;
	}
	// outptr points to start of paridx-th OUT parameter data in psql->outparam_info struct
	// set outparidx to index of OUT parameter in psql->param_info
	outparidx=*((sig16*)outptr);
	outptr+=sizeof(sig16); // now outptr points to length of OUT param
	parlen=*((uns32*)outptr);
	wblog("parlen=%lu, param type id=%d, param specif=0x%x, param parmode=%d\n",parlen,PSQL_INPARAM(psql,outparidx)->type_id,PSQL_INPARAM(psql,outparidx)->specif,PSQL_INPARAM(psql,outparidx)->parmode);
	outptr+=sizeof(uns32); // now outptr points do OUT param value (if value size is <= OUTPARAM_SIZE_LIMIT)
	// check validity of offset and size parameters
	if( parlen<=offset && offset>0 )
	{
		zend_error(E_WARNING,"602SQL: invalid 'offset' parameter, OUT parameter size is %u.",parlen);
		wblogs("internal_psql_wboutparam2phpvalue leave, false, ERROR, invalid offset\n");
		return 0;
	}
	if( parlen<offset+size )
	{
		zend_error(E_WARNING,"602SQL: invalid 'size' parameter, OUT parameter size is %u.",parlen);
		wblogs("internal_psql_wboutparam2phpvalue leave, false, ERROR, invalid size\n");
		return 0;
	}
	// if size isn't assigned set it
	if( size==0 )
	{
#if WBVERS>=800
		if( (PSQL_INPARAM(psql,outparidx)->type_id==ATT_STRING||PSQL_INPARAM(psql,outparidx)->type_id==ATT_TEXT
#if WBVERS>=1100
					||PSQL_INPARAM(psql,outparidx)->type_id==ATT_EXTCLOB
#endif
				)&&PSQL_INPARAM(psql,outparidx)->specif.wide_char
		)
		{ // in case of UNICODE string or CLOB set size in UCS-2 characters which are 2 bytes wide
			// parlen is even and offset is index of first UCS_2 character which we have to copy
			size=parlen/2-offset;
		}
		else
		{
#endif
		  size=parlen-offset;
#if WBVERS>=800
		}
#endif
	}
	// get param value and convert it to PHP value
	if( parlen<=OUTPARAM_SIZE_LIMIT )
	{ // value of OUT param is in psql->outparam_info struct
		wblogs("parlen<=OUTPARAM_SIZE_LIMIT - value of param is in psql->outparam_info struct\n");
		// if parlen is zero then given output parameter value is NULL
		if( parlen==0 )
		{ // set it to NULL
			ZVAL_NULL(phpval);
			wblogs("parlen is 0 => output parameter value is NULL\n");
		}
		else
		{
			switch( PSQL_INPARAM(psql,outparidx)->type_id )
			{
				CASE_smallwbvalue2phpvalue
#if WBVERS<900
					if( !smallwbvalue2phpvalue(PSQL_INPARAM(psql,outparidx)->type_id,outptr,phpval) )
#else
					if( !smallwbvalue2phpvalue(PSQL_INPARAM(psql,outparidx)->type_id,PSQL_INPARAM(psql,outparidx)->specif,outptr,phpval) )
#endif
					{
						wblogs("internal_psql_wboutparam2phpvalue leave, false, ERROR, smallwbvalue2phpvalue failed\n");
						return 0;
					}
					break;
				CASE_wbstring2phpvalue
				case ATT_TEXT:
#if WBVERS>=1100
				case ATT_EXTCLOB:
#endif
					{
						char *string;
#if WBVERS>=800
						if( PSQL_INPARAM(psql,outparidx)->specif.wide_char )
						{
							// parlen is size of UCS-2 string in bytes => it's even one
							// offset is index of first UCS-2 characted which we have to copy
							// size is length of UCS-2 substring
							offset*=2;
							size*=2;
							wblog("param is wide string - offset and size multiplied by 2: offset=%d, size=%d\n",offset,size);
						}
#endif
						string=(char*)emalloc(size+2);
						/* We have to copy value of string to temporary buffer
						 * because its value isn't followed by '\0' in psql->outparam_info.
						 * In advance we will copy only desired portion of string (size chars from offset). */
						if( string==NULL ) {
							report_out_of_memory_error("Can't allocate temporary buffer.");
							wblogs("internal_psql_wboutparam2phpvalue leave, false, ERROR, out of memory\n");
							return 0;
						}
						wblog("before memcpy: string is %d bytes long (this is size+2)\n",size+2);
						memcpy(string,outptr+offset,size);
						string[size]='\0';
						string[size+1]='\0';
						wblogs("string: =========\n"); wblogs(string); wblogs("\nend of string ==========\n");
#if WBVERS>=800
						if( !wbstring2phpvalue(
#ifdef USE_WB_SCRIPT_ENCODING
									script_encoding,
#endif
									PSQL_INPARAM(psql,outparidx)->type_id,PSQL_INPARAM(psql,outparidx)->specif,string,phpval) )
#else
						if( !wbstring2phpvalue(PSQL_INPARAM(psql,outparidx)->type_id,string,phpval) )
#endif
						{
							wblogs("internal_psql_wboutparam2phpvalue leave, false, ERROR, wbstring2phpvalue failed\n");
							efree(string);
							return 0;
						}
						efree(string);
					}
					break;
				case ATT_NOSPEC:
#if WBVERS>=1100
				case ATT_EXTBLOB:
#endif
				case ATT_RASTER:
				case ATT_BINARY:
					{
						// convert binary value to hexadecimal notation
						char *buf;
						uns32 bufsize;
						buf=(char*)emalloc(size*2+1);
						if( buf==NULL ) {
							report_out_of_memory_error("Can't allocate buffer.");
							wblogs("internal_psql_wboutparam2phpvalue leave, false, ERROR, out of memory\n");
							return 0;
						}
						memcpy(buf,outptr+offset,size);
						bufsize=(uns32)size;
						convert_binary_to_hexa((unsigned char*)buf,&bufsize);
						ZVAL_STRING(phpval,buf,FALSE/*don't duplicate buffer, use buf*/);
					}
					break;
				default:
					zend_error(E_WARNING,"602SQL error: unsupported type of OUT parameter, type ID is %d.",PSQL_INPARAM(psql,outparidx)->type_id);
					wblogs("internal_psql_wboutparam2phpvalue leave, false, ERROR, unsupported type of param\n");
					return 0;
			}
		}
	}
	else
	{ // value of OUT param have to be read by cd_Pull_parameter()
		char *buf;
		size_t bufsize;
		wblog("parlen=%d, OUTPARAM_SIZE_LIMIT=%d\n",parlen,OUTPARAM_SIZE_LIMIT);
		/* allocate buffer
		 * size of buffer will be 'size' when parameter's type is non-Unicode (i.e. its specif.wide_char==0) ATT_[CS[I]]STRING or ATT_TEXT or ATT_EXTCLOB
		 * and '2*size' when type is binary string or Unicode ATT_STRING or ATT_TEXT or ATT_EXTCLOB
		 *   in case of binary string buffer have to be two times longer because of convert_binary_to_hexa() function
		 *   which is used to convert binary value to its hexadecimal human-readable form
		 *   (this function makes in-place conversion, i.e. it needs twice longer buffer) */
		if( 
#if WBVERS>=800
				(
#endif
				  PSQL_INPARAM(psql,outparidx)->type_id==ATT_STRING || PSQL_INPARAM(psql,outparidx)->type_id==ATT_TEXT
#if WBVERS<800
				|| PSQL_INPARAM(psql,outparidx)->type_id==ATT_CSSTRING || PSQL_INPARAM(psql,outparidx)->type_id==ATT_CSISTRING
#endif
#if WBVERS>=1100
				|| PSQL_INPARAM(psql,outparidx)->type_id==ATT_EXTCLOB
#endif
#if WBVERS>=800
				) && !(PSQL_INPARAM(psql,outparidx)->specif.wide_char)
#endif
			)
		{
			bufsize=size+1;
			wblog("string isn't wide_char, bufsize=%d (size+1)\n",bufsize);
		}
		else
		{
#if WBVERS>=800
			if( PSQL_INPARAM(psql,outparidx)->specif.wide_char )
			{
				// parlen is size of UCS-2 string in bytes => it's even one
				// offset is index of first UCS-2 characted which we have to copy
				// size is length of UCS-2 substring
				offset*=2;
				size*=2;
			}
#endif
			bufsize=2*size+2;
			wblog("string is wide_char, bufsize=%d (2*size+2)\n",bufsize);
		}
		buf=(char*)emalloc(bufsize);
		if( buf==NULL ) {
			report_out_of_memory_error("Can't allocate buffer.");
			wblogs("internal_psql_wboutparam2phpvalue leave, false, ERROR, out of memory\n");
			return 0;
		}
		wblog("before cd_Pull_parameter(): offset=%ld, size=%ld\n",offset,size);
		// read value
		{
			int aux_offset;
			aux_offset=offset;
			do
			{
				int aux_size;
				aux_size=size-aux_offset;
				if( aux_size>0xe000 ) aux_size=0xe000; // taken from ODBC driver - odbc/execute.cpp, execute_stmt() function
				if( cd_Pull_parameter(CDP_FROM_PSQL_P(psql),PSQL_HANDLE(psql),outparidx,aux_offset,aux_size,buf+(aux_offset-offset)) ) {
					CONN_REPORT_ERR(CONN_P_FROM_PSQL_P(psql),"cd_Pull_parameter");
					wblogs("internal_psql_wboutparam2phpvalue leave, false, ERROR, cd_Pull_parameter failed\n");
					efree(buf);
					return 0;
				}
				aux_offset+=aux_size;
			} while( aux_offset<(int)size );
		}
		wblogs("after cd_Pull_parameter()\nbuf: =======\n"); wblogs(buf); wblogs("end of buf ========\n");
		wblog("strlen(buf)=%ld\n",strlen(buf));
		switch( PSQL_INPARAM(psql,outparidx)->type_id ) {
			CASE_wbstring2phpvalue
			case ATT_TEXT:
#if WBVERS>=1100
			case ATT_EXTCLOB:
#endif
				buf[size]='\0';
#if WBVERS>=800
				if( PSQL_INPARAM(psql,outparidx)->specif.wide_char ) buf[size+1]='\0';
#endif
#if WBVERS>=800
				if( !wbstring2phpvalue(
#ifdef USE_WB_SCRIPT_ENCODING
							script_encoding,
#endif
							PSQL_INPARAM(psql,outparidx)->type_id,PSQL_INPARAM(psql,outparidx)->specif,buf,phpval) )
#else
				if( !wbstring2phpvalue(PSQL_INPARAM(psql,outparidx)->type_id,buf,phpval) )
#endif
				{
					wblogs("internal_psql_wboutparam2phpvalue leave, false, ERROR, wbstring2phpvalue failed\n");
					efree(buf);
					return 0;
				}
				efree(buf);
			  break;
			case ATT_NOSPEC:
#if WBVERS>=1100
			case ATT_EXTBLOB:
#endif
			case ATT_RASTER:
			case ATT_BINARY:
				{
					uns32 bufsize=(uns32)size;
					// convert binary value to hexadecimal notation
					convert_binary_to_hexa((unsigned char*)buf,&bufsize);
					ZVAL_STRING(phpval,buf,FALSE/*don't duplicate buffer, use buf*/);
				}
				break;
			default:
				zend_error(E_WARNING,"602SQL error: unsupported type of OUT parameter, type ID is %d.",PSQL_INPARAM(psql,outparidx)->type_id);
				wblogs("internal_psql_wboutparam2phpvalue leave, false, ERROR, unsupported type of param\n");
				efree(buf);
				return 0;
		}
	} 
	wblogs("internal_psql_wboutparam2phpvalue leave, true\n");
	return 1;
}
/*}}}*/
/*{{{ internal_psql_out_param_length()
 * */
static int internal_psql_out_param_length(struct psql_t *psql,sig16 paridx,uns32 *parlength)
{
	char *outptr;
	wblog("internal_psql_out_param_length enter, psql=%p, paridx=%d\n",psql,paridx);
	outptr=internal_psql_get_outparam_ptr(psql,paridx);
	if( outptr==NULL ) {
		wblogs("internal_psql_out_param_length leave, ERROR, internal_psql_get_outparam_ptr failed\n");
		return 0;
	}
	*parlength=*((uns32*)(outptr+sizeof(sig16)));
	wblog("internal_psql_out_param_length leave, OK, param length is %u\n",*parlength);
	return 1;
}
/*}}}*/
/*}}}*/
/*{{{ wb_psql_prepare */
ZEND_FUNCTION(wb_psql_prepare)
{
	WBLSM_DECLARE
	zval **connhandle,**statement;
	wb_conn *conn;
	struct psql_t *psql;
	int id;
	char *tmp_statement;
#ifdef USE_WB_SCRIPT_ENCODING
	char *emalloc_statement=NULL;
#endif
	wblogs("wb_psql_prepare enter\n");
	if( ZEND_NUM_ARGS()!=2 || zend_get_parameters_ex(2,&connhandle,&statement)!=SUCCESS ) WRONG_PARAM_COUNT;
	conn=get_conn_from_resource(INTERNAL_FUNCTION_PARAM_PASSTHRU,connhandle);
	if (!conn) RETURN_FALSE; /* error in get_conn_from_resource */
	WBLSM_START("wb_psql_prepare",CDP(conn));
	convert_to_string_ex(statement);
	psql=(struct psql_t*)emalloc(sizeof(struct psql_t));
	if( psql==NULL )
	{
		report_out_of_memory_error("Can't allocate internal structure.");
		wblogs("wb_psql_prepare leave, error: out of memory\n");
		WBLSM_END;
		RETURN_FALSE;
	}
#ifdef USE_STRUCT_NAME
	strcpy(psql->struct_name,"psql_t");
#endif
	/* initialize psql_t structure */
	psql->conn=conn;
	psql->status=PSQL_STATUS_CLOSED;
	psql->psql_handle=0;
	psql->param_info=NULL;
	psql->outparam_info=NULL;
	psql->outparam_count=0;
#ifdef LOG_SERVER_MEMORY
	psql->sql_statement=(char*)emalloc(Z_STRLEN_PP(statement)+1);
	if( psql->sql_statement!=NULL ) strcpy(psql->sql_statement,Z_STRVAL_PP(statement));
#endif
	/* eventually convert statement source */
	if( !PREPARE_STRING_SCRIPT2SYSTEM(WBG(script_encoding),conn,&tmp_statement,&emalloc_statement,Z_STRVAL_PP(statement)) )
	{
		wblogs("wb_psql_prepare leave, error in PREPARE_STRING_SCRIPT2SYSTEM()\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	/* prepare statement */
	if( cd_SQL_prepare(CDP(conn),tmp_statement,&psql->psql_handle) )
	{
		CONN_REPORT_ERR(conn,"SQL_prepare");
		wblogs("wb_psql_prepare leave, error in cd_SQL_prepare\n");
		efree(psql);
		EFREE_EMALLOC_STRING(emalloc_statement);
		WBLSM_END;
		RETURN_FALSE;
	}
	EFREE_EMALLOC_STRING(emalloc_statement);
	/* get description of all parameters of statement */
	if( cd_SQL_get_param_info(CDP(conn),PSQL_HANDLE(psql),(void**)&psql->param_info) )
	{
		CONN_REPORT_ERR(conn,"SQL_get_param_info");
		wblogs("wb_psql_prepare leave, error in cd_SQL_get_param_info\n");
		efree(psql);
		WBLSM_END;
		RETURN_FALSE;
	}
	/* psql is prepared now */
	psql->status=PSQL_STATUS_PREPARED;
	/* create and register resource */
	id=zend_list_insert(psql,wbres_psql
#if PHP_VERSION_ID>50400
				TSRMLS_CC
#endif
			);
	conn->usage++;
	wblog("wb_psql_prepare leave, result=%d (psql resource)\n",id);
	WBLSM_END;
	/* return resource */
	RETURN_RESOURCE(id);
}
/*}}}*/
/*{{{ wb_psql_execute */
ZEND_FUNCTION(wb_psql_execute)
{
	WBLSM_DECLARE
	zval ***args;
	struct psql_t *psql;
	int argv;
	uns32 results[MAX_RETURN_FIELDS]; /* results of SQL as returned by server */
	int results_count;

	wblogs("wb_psql_execute enter\n");
	if( (argv=ZEND_NUM_ARGS())<1 )
	{
		WRONG_PARAM_COUNT;
	}
	args=(zval***)emalloc(argv*sizeof(zval**));
	if( args==NULL )
	{
		php_error(E_WARNING,"602SQL: Out of memory.");
		wblogs("wb_psql_execute leave, error: out of memory\n");
		RETURN_FALSE;
	}
	if( zend_get_parameters_array_ex(argv,args)==FAILURE )
	{
		wblogs("wb_psql_execute leave, error in zend_get_parameters_array_ex\n");
		efree(args);
		RETURN_FALSE;
	}
	if( (psql=get_psql_from_resource(INTERNAL_FUNCTION_PARAM_PASSTHRU,args[0]))==NULL )
	{
		efree(args);
		wblogs("wb_psql_execute leave, false\n");
		RETURN_FALSE;
	}
#ifdef LOG_SERVER_MEMORY
	WBLSM_START_EX("wb_psql_execute",CDP_FROM_PSQL_P(psql),psql->sql_statement);
#endif
	if( psql->status==PSQL_STATUS_SENDING_PARAMS && argv>1 )
	{
		zend_error(E_WARNING,"602SQL: can't send parameters through wb_psql_execute because at least one parameter was sent by wb_psql_send_param");
		wblogs("wb_psql_execute leave, false - params sending methods mismatched\n");
		efree(args);
		WBLSM_END;
		RETURN_FALSE;
	}
	if( psql->status==PSQL_STATUS_CLOSED )
	{
		zend_error(E_WARNING,"602SQL: can't execute prepared statement because it's already closed");
		wblogs("wb_psql_execute leave, false - statement is closed\n");
		efree(args);
		WBLSM_END;
		RETURN_FALSE;
	}
	if( !(   psql->status==PSQL_STATUS_PREPARED // statement is prepared and none parameter was sent yet
			  ||(psql->status==PSQL_STATUS_SENDING_PARAMS&&argv==1) // statement is prepared, some params were sent, however none params are to be sent now
				||(psql->status==PSQL_STATUS_EXECUTED) // statement was executed and is being executed again
			 ) )
	{
		zend_error(E_WARNING,"602SQL: invalid status of prepared statement (status=%d, number of arguments=%d)",(int)psql->status,argv);
		wblog("wb_psql_execute leave, error - invalid status of prepared statement (status=%d, number of arguments=%d)\n",(int)psql->status,argv);
		efree(args);
		WBLSM_END;
		RETURN_FALSE;
	}
	if( argv>1 )
	{
		int i;
		/* send values of IN and INOUT parameters
		 * This is slightly inefficient appoach. We can send all parameter values together by cd_Send_params() function.
		 * We will rectify this in some future release. */
		for( i=1;i<argv;i++ )
		{
			if( PSQL_INPARAM(psql,(sig16)(i-1))->parmode!=MODE_OUT )
				if( !internal_psql_send_param(psql,(sig16)(i-1)/*parnum - param index starts at 0*/,args[i]/*value*/,0,0/*all data*/ TSRMLS_CC) )
				{
					wblogs("wb_psql_execute leave, false - internal_psql_send_param failed\n");
					efree(args);
					WBLSM_END;
					RETURN_FALSE;
				}
		}
	}
	results[0]=0;
	if( cd_SQL_exec_prepared(CDP_FROM_PSQL_P(psql),PSQL_HANDLE(psql),results,&results_count) )
	{
		CONN_REPORT_ERR(CONN_P_FROM_PSQL_P(psql),"SQL_exec_prepared");
		wblogs("wb_psql_execute leave, error in cd_SQL_exec_prepared\n");
  	efree(args);
		WBLSM_END;
		RETURN_FALSE;
	}
	wblogs("  after cd_SQL_exec_prepared\n");
	if( cd_Pull_params(CDP_FROM_PSQL_P(psql),PSQL_HANDLE(psql),SEND_PAR_PULL,(void**)&psql->outparam_info) )
	{
		CONN_REPORT_ERR(CONN_P_FROM_PSQL_P(psql),"Pull_params");
		wblogs("wb_psql_execute leave, error in cd_Pull_params\n");
  	efree(args);
		WBLSM_END;
		RETURN_FALSE;
	}
	psql->outparam_count=internal_psql_out_param_count(psql);
	wblog("wb_psql_execute: outparam_count=%d\n",psql->outparam_count);
	psql->status=PSQL_STATUS_EXECUTED;

#ifdef _DEBUG
	// in case of debugging we can log OUT params values
	if( psql->outparam_count>0 )
	{
		char *parptr=psql->outparam_info;
		sig16 paridx=0;
		while( *(sig16*)parptr!=SEND_PAR_DELIMITER ) {
			uns32 length=*(uns32*)(parptr+sizeof(sig16));
			wblog("%d. OUT param: data size %lu\n",paridx,length);
			parptr+=sizeof(sig16)+sizeof(uns32);
			if( length<=OUTPARAM_SIZE_LIMIT )
			{
				// data jsou v bufferu parptr
				wblogmem("data: ",parptr,length);
				parptr+=length;
			}
			else
			{
				// data musi byt prectena pomoci cd_Pull_parameter()
			}
			paridx++;
		}
	} 
#endif

	if( argv>1 && psql->outparam_count>0 )
	{ /* read OUT parameters */
		sig16 i;
		for( i=0;i<psql->outparam_count;i++ )
		{
			char *outptr=internal_psql_get_outparam_ptr(psql,i);
			if( outptr==NULL ) {
				wblogs("wb_psql_execute leave, false, ERROR, internal_psql_get_outparam_ptr failed\n");
				efree(args);
				WBLSM_END;
				RETURN_FALSE;
			}
			// outptr points to index of i-th OUT param in psql->param_info
			if( !internal_psql_wboutparam2phpvalue(
#ifdef USE_WB_SCRIPT_ENCODING
						WBG(script_encoding),
#endif
						psql,i,*(args[*((sig16*)outptr)+1]),0,0/*read whole value*/ TSRMLS_CC) ) {
				wblogs("wb_psql_execute leave, false, ERROR, internal_psql_wboutparam2phpvalue failed\n");
				efree(args);
				WBLSM_END;
				RETURN_FALSE;
			}
		}
	}
	efree(args);
	if( IS_CURSOR(results[0]) )
	{
		struct sql_answer* answer;
		int id;
		if( !create_cursor_resource(CONN_P_FROM_PSQL_P(psql),results[0],&answer TSRMLS_CC) ) {
			WBLSM_END;
			RETURN_FALSE;
		}
		id=zend_list_insert(answer, wbres_cursor
#if PHP_VERSION_ID>50400
				TSRMLS_CC
#endif
				);
		if( PSQL_INPARAM_COUNT(psql)>0 )
		{
			/* When prepared SQL statement has parameters and result of statement is cursor
			 * we need to fully evaluate cursor to allow to execute this statement with
			 * another set of parameter values or to close prepared statement
			 * immediately after end of wb_psql_execute().
			 * To accomplish this we use cd_Rec_cnt() which can be time consuming operation
			 * when applied to cursors with huge number of records. */
			trecnum cnt;
			if( !cd_Rec_cnt(CDP_FROM_PSQL_P(psql),(tcursnum)(results[0]),&cnt) ) answer->rec_cnt=cnt;
		}
		wblog("wb_psql_execute leave, result=%ld (cursor)\n",id);
		WBLSM_END;
		RETURN_RESOURCE(id);
	}
	else
	{
		wblog("wb_psql_execute leave, result=%u\n",results[0]);
		WBLSM_END;
		RETURN_LONG((results[0]==0)?-1:(int)results[0]);
	}
} 
/*}}}*/
/*{{{ wb_psql_param_count */
ZEND_FUNCTION(wb_psql_param_count)
{
	WBLSM_DECLARE
	zval **psql_zval;
	struct psql_t *psql;
	wblogs("wb_psql_param_count enter\n");
	if( ZEND_NUM_ARGS()!=1 || zend_get_parameters_ex(1,&psql_zval)!=SUCCESS )
	{
		WRONG_PARAM_COUNT;
	}
	if( (psql=get_psql_from_resource(INTERNAL_FUNCTION_PARAM_PASSTHRU,psql_zval))==NULL )
	{
		wblogs("wb_psql_param_count leave, false\n");
		RETURN_FALSE;
	}
	WBLSM_START("wb_psql_param_count",CDP_FROM_PSQL_P(psql));
	if( psql->status==PSQL_STATUS_CLOSED )
	{
		zend_error(E_WARNING,"602SQL: prepared statement is already closed");
		wblogs("wb_psql_param_count leave, false - statement is closed\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	wblog("wb_psql_param_count leave, result is %d\n",PSQL_INPARAM_COUNT(psql));
	WBLSM_END;
	RETURN_LONG(PSQL_INPARAM_COUNT(psql));
}
/*}}}*/
/*{{{ wb_psql_param_info */
ZEND_FUNCTION(wb_psql_param_info)
{
	WBLSM_DECLARE
	zval **psql_zval,**idx_zval;
	struct psql_t *psql;
	struct psql_param_info_t *pi;
	wblogs("wb_psql_param_info enter\n");
	if( ZEND_NUM_ARGS()!=2 || zend_get_parameters_ex(2,&psql_zval,&idx_zval)!=SUCCESS )
	{
		WRONG_PARAM_COUNT;
	}
	if( (psql=get_psql_from_resource(INTERNAL_FUNCTION_PARAM_PASSTHRU,psql_zval))==NULL )
	{
		wblogs("wb_psql_param_info leave, false\n");
		RETURN_FALSE;
	}
	WBLSM_START("wb_psql_param_info",CDP_FROM_PSQL_P(psql));
	if( psql->status==PSQL_STATUS_CLOSED )
	{
		zend_error(E_WARNING,"602SQL: prepared statement is already closed");
		wblogs("wb_psql_param_info leave, false - statement is closed\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	convert_to_long_ex(idx_zval);
	if( Z_LVAL_PP(idx_zval)<0 || Z_LVAL_PP(idx_zval)>=PSQL_INPARAM_COUNT(psql) )
	{
		zend_error(E_WARNING,"602SQL: parameter with index %ld doesn't exist",Z_LVAL_PP(idx_zval));
		wblog("wb_psql_param_info leave, false - invalid index %ld\n",Z_LVAL_PP(idx_zval));
		WBLSM_END;
		RETURN_FALSE;
	}
	pi=PSQL_INPARAM(psql,Z_LVAL_PP(idx_zval));
	array_init(return_value);
	add_assoc_long(return_value,"type",pi->type_id);
	add_assoc_long(return_value,"specif",*((int*)&(pi->specif)));
	add_assoc_long(return_value,"mode",pi->parmode);
	add_assoc_long(return_value,"position",pi->position);
	wblogs("wb_psql_param_info leave, OK\n");
	WBLSM_END;
	// return_value is already set so there's no need to RETURN;
} 
/*}}}*/
/*{{{ wb_psql_drop_params */
ZEND_FUNCTION(wb_psql_drop_params)
{
	WBLSM_DECLARE
	zval **psql_zval;
	struct psql_t *psql;
	wblogs("wb_psql_drop_params enter\n");
	if( ZEND_NUM_ARGS()!=1 || zend_get_parameters_ex(1,&psql_zval)!=SUCCESS )
	{
		WRONG_PARAM_COUNT;
	}
	if( (psql=get_psql_from_resource(INTERNAL_FUNCTION_PARAM_PASSTHRU,psql_zval))==NULL )
	{
		wblogs("wb_psql_drop_params leave, false\n");
		RETURN_FALSE;
	}
	WBLSM_START("wb_psql_drop_params",CDP_FROM_PSQL_P(psql));
	if( psql->status==PSQL_STATUS_CLOSED )
	{
		zend_error(E_WARNING,"602SQL: prepared statement is already closed");
		wblogs("wb_psql_drop_params leave, false - statement is closed\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	wblog("psql->status=%d\n",psql->status);
	if( psql->status!=PSQL_STATUS_SENDING_PARAMS )
	{
		zend_error(E_WARNING,"602SQL: can't drop parameters because none was sent to SQL server");
		wblogs("wb_psql_drop_params leave, false - none parameter sent\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	{
    sig16 oper = SEND_PAR_DROP;
    if( cd_Send_params(CDP_FROM_PSQL_P(psql),PSQL_HANDLE(psql),sizeof(sig16),&oper) )
		{
			CONN_REPORT_ERR(CONN_P_FROM_PSQL_P(psql),"Send_params(SEND_PAR_DROP)");
			wblogs("wb_psql_drop_params leave, false - error in cd_Send_params()\n");
			WBLSM_END;
			RETURN_FALSE;
		}
		psql->status=PSQL_STATUS_PREPARED;
	}
	wblogs("wb_psql_drop_params leave, true\n");
	WBLSM_END;
	RETURN_TRUE;	
}
/*}}}*/
/*{{{ wb_psql_send_param */
ZEND_FUNCTION(wb_psql_send_param)
{
	WBLSM_DECLARE
	zval **psql_zval,**parindex_zval,**buf_zval,**offset_zval,**size_zval;
	struct psql_t *psql;
	wblogs("wb_psql_send_param enter\n");
	switch( ZEND_NUM_ARGS() )
	{
		case 5:
			if( zend_get_parameters_ex(5,&psql_zval,&parindex_zval,&buf_zval,&offset_zval,&size_zval)!=SUCCESS )
			{ 
				WRONG_PARAM_COUNT;
			}
			break;
		case 3:
			if( zend_get_parameters_ex(3,&psql_zval,&parindex_zval,&buf_zval)!=SUCCESS )
			{ 
				WRONG_PARAM_COUNT;
			}
			offset_zval=size_zval=NULL;
			break;
		default:
			WRONG_PARAM_COUNT;
	}
	if( (psql=get_psql_from_resource(INTERNAL_FUNCTION_PARAM_PASSTHRU,psql_zval))==NULL )
	{
		wblogs("wb_psql_send_param leave, false\n");
		RETURN_FALSE;
	}
	WBLSM_START("wb_psql_send_param",CDP_FROM_PSQL_P(psql));
	if( psql->status==PSQL_STATUS_CLOSED )
	{
		zend_error(E_WARNING,"602SQL: prepared statement is already closed");
		wblogs("wb_psql_send_param leave, false - statement is closed\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	convert_to_long_ex(parindex_zval);
	if( offset_zval!=NULL ) convert_to_long_ex(offset_zval);
	if( size_zval!=NULL ) convert_to_long_ex(size_zval);
	if( Z_LVAL_PP(parindex_zval)<0 || Z_LVAL_PP(parindex_zval)>=PSQL_INPARAM_COUNT(psql) )
	{
		zend_error(E_WARNING,"602SQL: parameter with index %ld doesn't exist",Z_LVAL_PP(parindex_zval));
		wblog("wb_psql_send_param leave, false - invalid index %ld\n",Z_LVAL_PP(parindex_zval));
		WBLSM_END;
		RETURN_FALSE;
	}
	if( !internal_psql_send_param(psql,(sig16)Z_LVAL_PP(parindex_zval),buf_zval,((offset_zval==NULL)?0:Z_LVAL_PP(offset_zval)),((size_zval==NULL)?0:Z_LVAL_PP(size_zval)) TSRMLS_CC) )
	{
		WBLSM_END;
		RETURN_FALSE;
	}
	psql->status=PSQL_STATUS_SENDING_PARAMS;
	wblogs("wb_psql_send_param leave, TRUE\n");
	WBLSM_END;
	RETURN_TRUE;
}
/*}}}*/
/*{{{ wb_psql_out_param_count */
ZEND_FUNCTION(wb_psql_out_param_count)
{
	WBLSM_DECLARE
	zval **psql_zval;
	struct psql_t *psql;
	wblogs("wb_psql_out_param_count enter\n");
	if( ZEND_NUM_ARGS()!=1 || zend_get_parameters_ex(1,&psql_zval)!=SUCCESS )
	{
		WRONG_PARAM_COUNT;
	}
	if( (psql=get_psql_from_resource(INTERNAL_FUNCTION_PARAM_PASSTHRU,psql_zval))==NULL )
	{
		wblogs("wb_psql_out_param_count leave, false\n");
		RETURN_FALSE;
	}
	WBLSM_START("wb_psql_out_param_count",CDP_FROM_PSQL_P(psql));
	switch( psql->status ) {
		case PSQL_STATUS_CLOSED:
			zend_error(E_WARNING,"602SQL: prepared statement is already closed");
			wblogs("wb_psql_out_param_count leave, false - statement is closed\n");
			WBLSM_END;
			RETURN_FALSE;
		case PSQL_STATUS_EXECUTED:
			break;
		default:
			zend_error(E_WARNING,"602SQL: prepared statement isn't executed yet");
			wblogs("wb_psql_out_param_count leave, false - statement isn't executed\n");
			WBLSM_END;
			RETURN_FALSE;
	}
	wblog("wb_psql_out_param_count leave, result is %d\n",psql->outparam_count);
	WBLSM_END;
	RETURN_LONG(psql->outparam_count);
}
/*}}}*/
/*{{{ wb_psql_get_out_param */
ZEND_FUNCTION(wb_psql_get_out_param)
{
	WBLSM_DECLARE
	zval **psql_zval,**paridx_zval,**offset_zval,**size_zval;
	struct psql_t *psql;
	wblogs("wb_psql_get_out_param enter\n");
	switch( ZEND_NUM_ARGS() ) {
		case 2:
		  if( zend_get_parameters_ex(2,&psql_zval,&paridx_zval)!=SUCCESS ) { WRONG_PARAM_COUNT; }
			convert_to_long_ex(paridx_zval);
			offset_zval=size_zval=NULL;
		  break;
		case 4:
			if( zend_get_parameters_ex(4,&psql_zval,&paridx_zval,&offset_zval,&size_zval)!=SUCCESS ) { WRONG_PARAM_COUNT; }
			convert_to_long_ex(paridx_zval);
			convert_to_long_ex(offset_zval);
			convert_to_long_ex(size_zval);
			break;
		default:
		  WRONG_PARAM_COUNT;
	}
	if( (psql=get_psql_from_resource(INTERNAL_FUNCTION_PARAM_PASSTHRU,psql_zval))==NULL )
	{
		wblogs("wb_psql_get_out_param leave, false\n");
		RETURN_FALSE;
	}
	WBLSM_START("wb_psql_get_out_param",CDP_FROM_PSQL_P(psql));
	switch( psql->status ) {
		case PSQL_STATUS_CLOSED:
			zend_error(E_WARNING,"602SQL: prepared statement is already closed");
			wblogs("wb_psql_get_out_param leave, false - statement is closed\n");
			WBLSM_END;
			RETURN_FALSE;
		case PSQL_STATUS_EXECUTED:
			break;
		default:
			zend_error(E_WARNING,"602SQL: prepared statement isn't executed yet");
			wblogs("wb_psql_get_out_param leave, false - statement isn't executed\n");
			WBLSM_END;
			RETURN_FALSE;
	}
	if( Z_LVAL_PP(paridx_zval)<0 || Z_LVAL_PP(paridx_zval)>=psql->outparam_count )
	{
		zend_error(E_WARNING,"602SQL: OUT parameter with index %ld doesn't exist",Z_LVAL_PP(paridx_zval));
		wblog("wb_psql_get_out_param leave, false, ERROR invalid index %ld\n",Z_LVAL_PP(paridx_zval));
		WBLSM_END;
		RETURN_FALSE;
	}
	if( !internal_psql_wboutparam2phpvalue(
#ifdef USE_WB_SCRIPT_ENCODING
				WBG(script_encoding),
#endif
				psql,(sig16)Z_LVAL_PP(paridx_zval),return_value,(offset_zval==NULL)?0:Z_LVAL_PP(offset_zval),(size_zval==NULL)?0:Z_LVAL_PP(size_zval) TSRMLS_CC) )
	{
		wblogs("wb_psql_get_out_param leave, false, ERROR in internal_psql_wboutparam2phpvalue\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	WBLSM_END;
	// return_value is already set so there's no need to RETURN;
}
/*}}}*/
/*{{{ wb_psql_out_param_length
 * */
ZEND_FUNCTION(wb_psql_out_param_length)
{
	WBLSM_DECLARE
	zval **psql_zval,**paridx_zval;
	struct psql_t *psql;
	wblogs("wb_psql_out_param_length enter\n");
	if( ZEND_NUM_ARGS()!=2 || zend_get_parameters_ex(2,&psql_zval,&paridx_zval)!=SUCCESS )
	{
		WRONG_PARAM_COUNT;
	}
	if( (psql=get_psql_from_resource(INTERNAL_FUNCTION_PARAM_PASSTHRU,psql_zval))==NULL )
	{
		wblogs("wb_psql_out_param_length leave, false\n");
		RETURN_FALSE;
	}
	WBLSM_START("wb_psql_out_param_length",CDP_FROM_PSQL_P(psql));
	convert_to_long_ex(paridx_zval);
	// check PSQL status
	switch( psql->status ) {
		case PSQL_STATUS_CLOSED:
			zend_error(E_WARNING,"602SQL: prepared statement is already closed");
			wblogs("wb_psql_out_param_length leave, false - statement is closed\n");
			WBLSM_END;
			RETURN_FALSE;
		case PSQL_STATUS_EXECUTED:
			break;
		default:
			zend_error(E_WARNING,"602SQL: prepared statement isn't executed yet");
			wblogs("wb_psql_out_param_length leave, false - statement isn't executed\n");
			WBLSM_END;
			RETURN_FALSE;
	}
	// check param index
	if( Z_LVAL_PP(paridx_zval)<0 || Z_LVAL_PP(paridx_zval)>=psql->outparam_count )
	{
		zend_error(E_WARNING,"602SQL: OUT parameter with index %ld doesn't exist",Z_LVAL_PP(paridx_zval));
		wblog("wb_psql_out_param_length leave, false, ERROR invalid index %ld\n",Z_LVAL_PP(paridx_zval));
		WBLSM_END;
		RETURN_FALSE;
	}
	// do the job
	{
		uns32 tmp;
		if( !internal_psql_out_param_length(psql,(sig16)Z_LVAL_PP(paridx_zval),&tmp) ) {
			wblogs("wb_psql_out_param_length leave, false, ERROR internal_psql_out_param_length failed\n");
			WBLSM_END;
			RETURN_FALSE;
		}
	  wblog("wb_psql_out_param_length leave, result is %u\n",tmp);
		WBLSM_END;
	  RETURN_LONG(tmp);
	}
}
/*}}}*/
/*}}}*/
#endif
#ifdef ENABLE_XML_IMPORT_EXPORT
/*{{{ import/export to/from XML functions */
/*{{{ support functions */
/*{{{ internal_xml_get_params
 * parses common parameters of XML import/export functions and returns 0 on error or 1 on success
 * sets
 *   ppconn to point to connection struct
 *   pcdp to point to 602SQL client struct which has to be used in a 602SQL API function call
 *   ppanswer to point to cursor struct or sets ppanswer to NULL  when none cursor was passed
 *   dad_ref to point to value of "dad_ref" parameter
 *   fname_or_buffer to point to value of "fname" parameter
 *   hostvars to point to value of "hostvars" parameter or sets hostvars to NULL when none hostvars were passed
 * if ppanswer is NULL then function doesn't accept "cursor" parameter
 * if fname_or_buffer is NULL then function doesn't accept "fname" or "buffer" parameter */
static int internal_xml_get_params(wb_conn **ppconn,cdp_t *pcdp,struct sql_answer **ppanswer,zval ***dad_ref,zval ***fname_or_buffer,zval ***hostvars, INTERNAL_FUNCTION_PARAMETERS)
{
	zval **connhandle,**cursor;

	wblog("internal_xml_get_params enter: ZEND_NUM_ARGS()=%d, ppanswer=%p\n",ZEND_NUM_ARGS(),ppanswer);
	if( ppanswer==NULL )
	{ // none cursor => wb_import_from_xml[_buffer]
		//   => fname_or_buffer!=NULL
		cursor=NULL;
		switch( ZEND_NUM_ARGS() )
		{
			case 4: // hostvars are present
				if( fname_or_buffer==NULL || hostvars==NULL || zend_get_parameters_ex(4,&connhandle,dad_ref,fname_or_buffer,hostvars)!=SUCCESS )
				{
					wblogs("internal_xml_get_params leave: ERROR, wrong param count\n");
					WRONG_PARAM_COUNT_WITH_RETVAL(0);
				}
				break;
			case 3: // without hostvars parameter
				if( hostvars!=NULL ) *hostvars=NULL;
				if( fname_or_buffer==NULL || zend_get_parameters_ex(3,&connhandle,dad_ref,fname_or_buffer)!=SUCCESS )
				{
					wblogs("internal_xml_get_params leave: ERROR, wrong param count\n");
					WRONG_PARAM_COUNT_WITH_RETVAL(0);
				}
				break;
			default:
				wblogs("internal_xml_get_params leave: ERROR, wrong param count\n");
				WRONG_PARAM_COUNT_WITH_RETVAL(0);
		}
	}
	else
	{ // cursor have to be present too - wb_export_to_xml[_buffer]
		switch( ZEND_NUM_ARGS() )
		{
			case 5: // hostvars, answer and fname_or_buffer => wb_export_to_xml
				if( ppanswer==NULL || fname_or_buffer==NULL || hostvars==NULL || zend_get_parameters_ex(5,&connhandle,dad_ref,fname_or_buffer,&cursor,hostvars)!=SUCCESS )
				{
					wblogs("internal_xml_get_params leave: ERROR, wrong param count\n");
					WRONG_PARAM_COUNT_WITH_RETVAL(0);
				}
				break;
			case 4:
				if( fname_or_buffer==NULL )
				{ // => wb_export_to_xml_buffer
					if( zend_get_parameters_ex(4,&connhandle,dad_ref,&cursor,hostvars)!=SUCCESS )
					{
						wblogs("internal_xml_get_params leave: ERROR, wrong param count\n");
						WRONG_PARAM_COUNT_WITH_RETVAL(0);
					}
				}
				else
				{ // => wb_export_to_xml
					*hostvars=NULL;
					if( fname_or_buffer==NULL || zend_get_parameters_ex(4,&connhandle,dad_ref,fname_or_buffer,&cursor)!=SUCCESS )
					{
						wblogs("internal_xml_get_params leave: ERROR, wrong param count\n");
						WRONG_PARAM_COUNT_WITH_RETVAL(0);
					}
				}
				break;
			case 3: // => wb_export_to_xml_buffer & fname_or_buffer have to be NULL
				if( fname_or_buffer!=NULL )
				{
					wblogs("internal_xml_get_params leave: ERROR, fname_or_buffer is NULL while just 3 params passed into\n");
					WRONG_PARAM_COUNT_WITH_RETVAL(0);
				}
				if( zend_get_parameters_ex(3,&connhandle,dad_ref,&cursor)!=SUCCESS )
				{
					wblogs("internal_xml_get_params leave: ERROR, wrong param count\n");
					WRONG_PARAM_COUNT_WITH_RETVAL(0);
				}
				if( hostvars!=NULL ) *hostvars=NULL;
				break;
			default:
				wblogs("internal_xml_get_params leave: ERROR, wrong param count\n");
				WRONG_PARAM_COUNT_WITH_RETVAL(0);
		}
	}
	// get connection from resource
	*ppconn=get_conn_from_resource(INTERNAL_FUNCTION_PARAM_PASSTHRU,connhandle);
	if (!*ppconn)
	{
		wblogs("internal_xml_get_params leave: ERROR in get_conn_from_resource()\n");
		return 0;
	}
	// get cursor from resource
	if( cursor!=NULL )
	{	// if cursor parameter is resource
		if( Z_TYPE_PP(cursor)==IS_RESOURCE )
		{ // get struct answer from it
			if( (*ppanswer=get_answer_from_result(INTERNAL_FUNCTION_PARAM_PASSTHRU,cursor))==NULL )
			{
				wblogs("internal_xml_get_params leave: ERROR in get_answer_from_result()\n");
				return 0;
			}
		}
		else
		{ // else forget cursor, i.e. cursor parameter is NULL
			*ppanswer=NULL;
			cursor=NULL;
		}
	}
	// convert parameter values to expected PHP types
	convert_to_string_ex(*dad_ref);
	if( fname_or_buffer!=NULL ) convert_to_string_ex(*fname_or_buffer);
	// get 602SQL client struct
	if( cursor==NULL ) *pcdp=CDP(*ppconn); // use connection's one
	else *pcdp=CDP_FROM_ANSWER_P(*ppanswer); // use cursor's one
	// ensure that hostvars parameter is either PHP null or array
	if( hostvars!=NULL && *hostvars!=NULL && Z_TYPE_PP(*hostvars)!=IS_NULL && Z_TYPE_PP(*hostvars)!=IS_ARRAY )
	{
		zend_error(E_WARNING,"602SQL: XML import/export function expects an array or a NULL value as \"hostvars\" parameter");
		wblogs("internal_xml_get_params leave: ERROR, hostvars isn't NULL nor array\n");
		return 0;
	}
	wblogs("internal_xml_get_params leave: OK\n");
	return 1;
} 
/*}}}*/
/*{{{ internal_xml_get_hostvars
 * prepares t_clivar array of parameters from hostvars PHP array
 * returns 1 on success or 0 on error
 *   on success sets: ppcv to point to emalloc-ed array of t_clivar
 *                    pcvcount to point to number of t_clivar elements in ppcv array
 *                    ppemalloced_buffers to point to array of emalloc-ed buffers
 *                      with values of parameters that will be sent to 602SQL server
 *   caller have to efree ppcv if it isn't NULL
 *   and have to efree every non-NULL item of ppemalloced_buffers array and array itself
 *     if ppemalloced_buffers isn't NULL (or use internal_efree_emalloced_buffers() ) */
static int internal_xml_get_hostvars(struct wb_conn *pconn,zval **hostvars,struct t_clivar **ppcv,int *pcvcount,char ***ppemalloced_buffers,int *pemalloced_buffers_size)
{
	wblogs("internal_xml_get_hostvars enter\n");
	if( hostvars==NULL || Z_TYPE_PP(hostvars)==IS_NULL )
	{
		*ppcv=NULL;
		*pcvcount=0;
		*ppemalloced_buffers=NULL;
		*pemalloced_buffers_size=0;
		wblogs("internal_xml_get_hostvars leave: OK, hostvars is NULL\n");
		return 1;
	}
	if( Z_TYPE_PP(hostvars)!=IS_ARRAY )
	{ 
		zend_error(E_WARNING,"602SQL: expecting array or NULL as \"hostvars\" parameter of XML import/export function");
		wblogs("internal_xml_get_hostvars leave: ERROR, hostvars isn't array\n");
		return 0;
	}
	// find out number of elements in hostvars array
	*pemalloced_buffers_size=zend_hash_num_elements(Z_ARRVAL_PP(hostvars));
	wblog("number of hostvars: %d\n",*pemalloced_buffers_size);
	/* We will alloc emalloced_buffers array to be able to hold buffer for every parameter (element in hostvars array) */
	*ppemalloced_buffers=(char**)emalloc(*pemalloced_buffers_size*sizeof(char*));
	if( *ppemalloced_buffers==NULL )
	{
		*pemalloced_buffers_size=0;
		report_out_of_memory_error("Can't allocate internal structure.");
		wblogs("internal_xml_get_hostvars leave: ERROR, failed allocation of ppemalloced_buffers\n");
		return 0;
	}
	// zeroize array of pointers
	memset(*ppemalloced_buffers,0,*pemalloced_buffers_size*sizeof(char*));
	// allocate ppcv array of t_clivar
	*ppcv=(struct t_clivar*)emalloc(*pemalloced_buffers_size*sizeof(struct t_clivar));
	if( *ppcv==NULL )
	{
		*pemalloced_buffers_size=0;
		efree(*ppemalloced_buffers);
		*ppemalloced_buffers=NULL;
		report_out_of_memory_error("Can't allocate internal structure.");
		wblogs("internal_xml_get_hostvars leave: ERROR, failed allocation of ppcv\n");
		return 0;
	}
	*pcvcount=*pemalloced_buffers_size;
	// zeroize array
	memset(*ppcv,0,*pcvcount*sizeof(struct t_clivar));
#define internal_xml_get_hostvars_EFREE_ON_ERROR \
	  { \
			int i; \
			for( i=0;i<*pemalloced_buffers_size;i++ ) \
				if( (*ppemalloced_buffers)[i]!=NULL ) efree((*ppemalloced_buffers)[i]); \
			efree(*ppemalloced_buffers); *ppemalloced_buffers=NULL; *pemalloced_buffers_size=0; \
			efree(*ppcv); *ppcv=NULL; *pcvcount=0; \
		}
	/* Convert elements of hostvars to ppcv.
	 * When we need to convert n-th parameter's PHP value to 602SQL kind of value
	 * we emalloc-ate buffer (*ppemalloced_buffers)[index_of_n_th_parameter]
	 * and write 602SQL value there. t_clivar::buf of corresponding element of ppcv
	 * points to this value. Emalloc-ed buffer will be efree-d by caller. */
	{
		int i;
		// set internal pointer of hostvars array to first element
		zend_hash_internal_pointer_reset(Z_ARRVAL_PP(hostvars));
		for( i=0;i<*pemalloced_buffers_size;i++ )
		{
			zval **elem, // element of hostvars
			  **param_value; // value of parameter
			char *string_key=NULL; // key value of elem - parameter name
			ulong dummy_ulong; // dummy
			zval **tmp;
			int desired_buflen=0; // size of buffer when "length" element is present - only for variable-size types
			// get element key value - parameter name
			if( zend_hash_get_current_key(Z_ARRVAL_PP(hostvars),&string_key,&dummy_ulong,1)!=HASH_KEY_IS_STRING )
			{
				zend_error(E_WARNING,"602SQL: %d-th parameter of DAD hasn't string type key.",i+1);
				wblog("internal_xml_get_hostvars leave: ERROR, %d-th parameter of DAD hasn't string type key\n",i+1);
				internal_xml_get_hostvars_EFREE_ON_ERROR;
				return 0;
			}
			// -> have to efree() string_key
			// get element value
			if( zend_hash_get_current_data(Z_ARRVAL_PP(hostvars),(void**)&elem)!=SUCCESS )
			{
				zend_error(E_WARNING,"602SQL: Can't read \"%s\" parameter of DAD of XML import/export function.",string_key);
				wblog("internal_xml_get_hostvars leave: ERROR, can't read \"%s\" parameter of DAD of XML import/export function\n",string_key);
				internal_xml_get_hostvars_EFREE_ON_ERROR;
				efree(string_key);
				return 0;
			}
			// test if element value is array
			if( Z_TYPE_PP(elem)!=IS_ARRAY )
			{
				zend_error(E_WARNING,"602SQL: Value of \"%s\" parameter of DAD of XML import/export function isn't array.",string_key);
				wblog("internal_xml_get_hostvars leave: ERROR, value of \"%s\" parameter of DAD of XML import/export function isn't array\n",string_key);
				internal_xml_get_hostvars_EFREE_ON_ERROR;
				efree(string_key);
				return 0;
			}
			// get type of parameter
			if( zend_hash_find(Z_ARRVAL_PP(elem),"type",5,(void**)&tmp)!=SUCCESS )
			{
				zend_error(E_WARNING,"602SQL: \"type\" value of \"%s\" parameter of DAD is missing.",string_key);
				wblog("internal_xml_get_hostvars leave: ERROR, \"type\" value of \"%s\" parameter of DAD is missing\n",string_key);
				internal_xml_get_hostvars_EFREE_ON_ERROR;
				efree(string_key);
				return 0;
			}
			convert_to_long_ex(tmp);
			(*ppcv)[i].wbtype=Z_LVAL_PP(tmp);
			// get value of parameter
			if( zend_hash_find(Z_ARRVAL_PP(elem),"value",6,(void**)&param_value)!=SUCCESS )
			{
				zend_error(E_WARNING,"602SQL: \"value\" value of \"%s\" parameter of DAD is missing.",string_key);
				wblog("internal_xml_get_hostvars leave: ERROR, \"value\" value of \"%s\" parameter of DAD is missing\n",string_key);
				internal_xml_get_hostvars_EFREE_ON_ERROR;
				efree(string_key);
				return 0;
			}
			// get additional type-specific information about parameter
			// and update parameter's specif structure
			{
				int character_size=1; // default is 8-bit chars or binary chars 
				switch( (*ppcv)[i].wbtype )
				{
					case ATT_STRING:
					case ATT_TEXT:
#if WBVERS>=1100
					case ATT_EXTCLOB:
#endif
						// get wide_char - optional
						if( zend_hash_find(Z_ARRVAL_PP(elem),"wide_char",10,(void**)&tmp)==SUCCESS )
						{
							convert_to_boolean_ex(tmp);
							// update wide_char elem of specif struct
							if( Z_BVAL_PP(tmp) )
							{
								character_size=2; // 16-bit chars
								T_SPECIF_CAST((*ppcv)[i].specif).wide_char=1;
							}
						}
						/* no break! */
					case ATT_NOSPEC:
#if WBVERS>=1100
					case ATT_EXTBLOB:
#endif
					case ATT_RASTER:
						// get length - optional
						if( zend_hash_find(Z_ARRVAL_PP(elem),"length",7,(void**)&tmp)==SUCCESS )
						{
							convert_to_long_ex(tmp);
							// note down desired size of buffer in bytes
							desired_buflen=Z_LVAL_PP(tmp)*character_size;
							wblog("character_size=%d, desired_buflen=%d\n",character_size,desired_buflen);
						}
						break;
#if WBVERS>=900
					case ATT_INT8:
					case ATT_INT16:
					case ATT_INT32:
					case ATT_INT64:
						// get scale - optional, for 602SQL 9.0 or newer only
						if( zend_hash_find(Z_ARRVAL_PP(elem),"scale",6,(void**)&tmp)==SUCCESS )
						{
							convert_to_long_ex(tmp);
							// update scale
							T_SPECIF_CAST((*ppcv)[i].specif).scale=(sig8)Z_LVAL_PP(tmp);
						}
						break;
#endif
				}
			}
			/* Convert PHP value to 602SQL value. */
			{
				void *pvalue;
				uns32 pvalue_offset,pvalue_size;
				int is_emalloced;
			  if( !internal_phpvalue2wbvalue(
#if WBVERS>=800
							ENCODING(pconn),
#endif
							(uns8)((*ppcv)[i].wbtype),T_SPECIF_CAST((*ppcv)[i].specif),param_value,0,0,&pvalue,&pvalue_offset,&pvalue_size,&is_emalloced) )
				{
					zend_error(E_WARNING,"602SQL: previous error occured during conversion of value of \"%s\" parameter of DAD to 602SQL format.",string_key);
					wblog("internal_xml_get_hostvars leave: ERROR, in internal_phpvalue2wbvalue() while converting value of \"%s\" parameter\n",string_key);
					internal_xml_get_hostvars_EFREE_ON_ERROR;
					efree(string_key);
					return 0;
				}
				wblog("pvalue_offset=%lu, pvalue_size=%lu\n",pvalue_offset,pvalue_size);
				wblogmem("pvalue",pvalue,pvalue_size);
				/* if "length" was specified for variable-length types realloc buffer */
				if( ( (*ppcv)[i].wbtype==ATT_STRING || (*ppcv)[i].wbtype==ATT_TEXT
#if WBVERS>=1100
							||(*ppcv)[i].wbtype==ATT_EXTCLOB || (*ppcv)[i].wbtype==ATT_EXTBLOB
#endif
						 	|| (*ppcv)[i].wbtype==ATT_NOSPEC || (*ppcv)[i].wbtype==ATT_RASTER ) && desired_buflen!=0 
				)
				{
					char *tmp;
					// reallocate or allocate buffer
					if( is_emalloced ) tmp=(char*)erealloc(pvalue,desired_buflen+2);
					else tmp=(char*)emalloc(desired_buflen+2);
					if( tmp==NULL )
					{
						report_out_of_memory_error("Can't allocate buffer for DAD parameter value.");
						wblog("internal_xml_get_hostvars leave: ERROR, emalloc() or erealloc() failed while expanding buffer for value of \"%s\" parameter\n",string_key);
						if( is_emalloced ) efree(pvalue);
						internal_xml_get_hostvars_EFREE_ON_ERROR;
						efree(string_key);
						return 0;
					}
					if( !is_emalloced )
					{
						is_emalloced=1;
						memcpy(tmp,pvalue,pvalue_size);
						tmp[pvalue_size]=tmp[pvalue_size+1]='\0';
					}
					// set elements of t_clivar struct
					(*ppcv)[i].buf=(pvalue=tmp);
					(*ppcv)[i].buflen=desired_buflen+2; // buflen have to include space for ending '\0' char too
					(*ppcv)[i].actlen=pvalue_size;
				}
				else
				{ // simply set corresponding elements of t_clivar struct of this parameter
					(*ppcv)[i].buf=pvalue;
					(*ppcv)[i].actlen=(int)pvalue_size;
					if( (*ppcv)[i].wbtype==ATT_STRING || (*ppcv)[i].wbtype==ATT_TEXT
#if WBVERS>=1100
							|| (*ppcv)[i].wbtype==ATT_EXTCLOB
#endif
					)
					{
						(*ppcv)[i].buflen=(*ppcv)[i].actlen+( ( T_SPECIF_CAST((*ppcv)[i].specif).wide_char )?2:1 ); // buflen have to include space for ending '\0' char too
					}
					else (*ppcv)[i].buflen=(*ppcv)[i].actlen;
				} 
				// finally set specif.length value for ATT_STRING
				if( (*ppcv)[i].wbtype==ATT_STRING ) T_SPECIF_CAST((*ppcv)[i].specif).length=(uns16)((*ppcv)[i].buflen-( ( T_SPECIF_CAST((*ppcv)[i].specif).wide_char )?2:1 ));
				// if buffer is emalloc-ed then note it down
				if( is_emalloced )
				{
					(*ppemalloced_buffers)[i]=(char*)pvalue;
				}
			} 
			// set parameter name
			Upcase(string_key);
			strncpy((*ppcv)[i].name,string_key,NAMELEN);
			(*ppcv)[i].name[NAMELEN]='\0';
			// and finally set MODE_IN - even if this is INOUT parameter
			(*ppcv)[i].mode=MODE_IN;
			// efree string_key
			efree(string_key);
			// move to next element of hostvars
			zend_hash_move_forward(Z_ARRVAL_PP(hostvars));
		} 
	}
	wblogs("internal_xml_get_hostvars leave: ok\n");
	return 1;
}
/*}}}*/
/*{{{ internal_xml_pull_outhostvars
 * reads values of OUT hostvars from t_clivar array and writes its PHP equivalent
 * to hostvars PHP array
 * returns 1 on success or 0 on error */
static int internal_xml_pull_outhostvars(
#ifdef USE_WB_SCRIPT_ENCODING
		wb_encoding script_encoding,
#endif
		struct wb_conn *pconn,zval **hostvars,struct t_clivar *pcv,int cvcount,char **pemalloced_buffers,int emalloced_buffers_size)
{
	int hostvars_size;
	if( hostvars==NULL || Z_TYPE_PP(hostvars)==IS_NULL )
	{ // none hostvars
		return 1;
	}
	if( Z_TYPE_PP(hostvars)!=IS_ARRAY )
	{ 
		zend_error(E_WARNING,"602SQL: expecting array or NULL as \"hostvars\" parameter of XML import/export function");
		return 0;
	}
	// find out number of elements in hostvars array
	hostvars_size=zend_hash_num_elements(Z_ARRVAL_PP(hostvars)); 
#define internal_xml_pull_outhostvars_EFREE_ON_ERROR \
	  { \
		}
	/* Search through hostvars elements and for element which have "out"=TRUE
	 * convert its 602SQL value to PHP value. */
	{
		int i;
		// set internal pointer of hostvars array to first element
		zend_hash_internal_pointer_reset(Z_ARRVAL_PP(hostvars));
		for( i=0;i<hostvars_size;i++ )
		{
			zval **elem, // element of hostvars
			  **param_value; // value of parameter
			char *string_key=NULL; // key value of elem - parameter name
			ulong dummy_ulong; // dummy
			zval **tmp;
			/* if parameter value isn't in pemalloced_buffers, i.e. buffer with value of parameter
			 * is emalloc-ed, i.e. pointer to parameter value does point into zval PHP value struct,
			 * we can skip this this parameter because output value of parameter is already
			 * written in zval PHP struct */
			if( pemalloced_buffers[i]==NULL ) continue;
			// get element key value - parameter name
			if( zend_hash_get_current_key(Z_ARRVAL_PP(hostvars),&string_key,&dummy_ulong,1)!=HASH_KEY_IS_STRING )
			{
				zend_error(E_WARNING,"602SQL: %d-th parameter of DAD hasn't string type key.",i+1);
				internal_xml_pull_outhostvars_EFREE_ON_ERROR;
				return 0;
			}
			// -> have to efree() string_key
			// get element value
			if( zend_hash_get_current_data(Z_ARRVAL_PP(hostvars),(void**)&elem)!=SUCCESS )
			{
				zend_error(E_WARNING,"602SQL: Can't read \"%s\" parameter of DAD of XML import/export function.",string_key);
				internal_xml_pull_outhostvars_EFREE_ON_ERROR;
				efree(string_key);
				return 0;
			}
			// test if element value is array
			if( Z_TYPE_PP(elem)!=IS_ARRAY )
			{
				zend_error(E_WARNING,"602SQL: Value of \"%s\" parameter of DAD of XML import/export function isn't array.",string_key);
				internal_xml_pull_outhostvars_EFREE_ON_ERROR;
				efree(string_key);
				return 0;
			}
			// is this OUT parameter?
			if( zend_hash_find(Z_ARRVAL_PP(elem),"out",4,(void**)&tmp)==SUCCESS )
			{
				convert_to_boolean_ex(tmp);
				if( Z_BVAL_PP(tmp) )
				{
					/* yes, this is OUT parameter
					 *   type of parameter: pcv[i].wbtype
					 *   specif: pcv[i].specif
					 *   value of parameter: pcv[i].buf */
					// get PHP zval for storing value of parameter
					if( zend_hash_find(Z_ARRVAL_PP(elem),"value",6,(void**)&param_value)!=SUCCESS )
					{
						zend_error(E_WARNING,"602SQL: \"value\" value of \"%s\" parameter of DAD is missing.",string_key);
						internal_xml_pull_outhostvars_EFREE_ON_ERROR;
						efree(string_key);
						return 0;
					}
					// convert 602SQL value of parameter into param_value PHP value
					switch( pcv[i].wbtype )
					{
						CASE_smallwbvalue2phpvalue
#if WBVERS<900
							if( !smallwbvalue2phpvalue((uns8)pcv[i].wbtype,pcv[i].buf,*param_value) )
#else
							if( !smallwbvalue2phpvalue((uns8)pcv[i].wbtype,T_SPECIF_CAST(pcv[i].specif),pcv[i].buf,*param_value) )
#endif
							{
								internal_xml_pull_outhostvars_EFREE_ON_ERROR;
								efree(string_key);
								return 0;
							}
							break;
						CASE_wbstring2phpvalue
						case ATT_TEXT:
#if WBVERS>=1100
						case ATT_EXTCLOB:
#endif
							{
#if WBVERS>=800
								if( !wbstring2phpvalue(
#ifdef USE_WB_SCRIPT_ENCODING
											script_encoding,
#endif
											(uns8)pcv[i].wbtype,T_SPECIF_CAST(pcv[i].specif),pcv[i].buf,*param_value) )
#else
								if( !wbstring2phpvalue((uns8)pcv[i].wbtype,pcv[i].buf,*param_value) )
#endif
								{
									internal_xml_pull_outhostvars_EFREE_ON_ERROR;
									efree(string_key);
									return 0;
								}
							}
							break;
						case ATT_NOSPEC:
#if WBVERS>=1100
						case ATT_EXTBLOB:
#endif
						case ATT_RASTER:
						case ATT_BINARY:
							{
								// convert binary value to hexadecimal notation
								char *buf;
								uns32 bufsize;
								buf=(char*)emalloc(pcv[i].actlen*2+1);
								if( buf==NULL ) {
									report_out_of_memory_error("Can't allocate buffer.");
									internal_xml_pull_outhostvars_EFREE_ON_ERROR;
									efree(string_key);
									return 0;
								}
								memcpy(buf,pcv[i].buf,pcv[i].actlen);
								bufsize=(uns32)pcv[i].actlen;
								convert_binary_to_hexa((unsigned char*)buf,&bufsize);
								ZVAL_STRING(*param_value,buf,FALSE/*don't duplicate buffer, use buf*/);
							}
							break;
						default:
							zend_error(E_WARNING,"602SQL error: unsupported type of OUT parameter \"%s\" of DAD, type ID is %d.",string_key,pcv[i].wbtype);
							internal_xml_pull_outhostvars_EFREE_ON_ERROR;
							efree(string_key);
							return 0;
					}
				}
			}
			// efree string_key
			efree(string_key);
			// move to next element of hostvars
			zend_hash_move_forward(Z_ARRVAL_PP(hostvars));
		} 
	}
	return 1;
}
/*}}}*/
/*{{{ internal_efree_emalloced_buffers */
static void internal_efree_emalloced_buffers(char **pemalloced_buffers,int emalloced_buffers_size,struct t_clivar *pcv)
{
	if( pemalloced_buffers!=NULL )
	{
		int i;
		for( i=0;i<emalloced_buffers_size;i++ )
			if( pemalloced_buffers[i]!=NULL ) efree(pemalloced_buffers[i]);
		efree(pemalloced_buffers);
	}
	if( pcv!=NULL )
	{
		efree(pcv);
	}
}
/*}}}*/
/*}}}*/
/*{{{ wb_export_to_xml */
ZEND_FUNCTION(wb_export_to_xml)
{
	WBLSM_DECLARE
	zval **dad_ref,**fname,**hostvars;
	wb_conn *conn;
	cdp_t cdp;
	struct sql_answer *answer;
	struct t_clivar *pcv;
	int cvcount;
	char **pemalloced_buffers;
	int emalloced_buffers_size;
	char *tmp_dad_ref;
#ifdef USE_WB_SCRIPT_ENCODING
	char *emalloc_dad_ref=NULL;
#endif

	wblogs("wb_export_to_xml enter\n");
	if( !internal_xml_get_params(&conn,&cdp,&answer,&dad_ref,&fname,&hostvars,INTERNAL_FUNCTION_PARAM_PASSTHRU) )
	{
		wblogs("wb_export_to_xml leave: ERROR, internal_xml_get_params() failed\n");
		RETURN_FALSE;
	}
	WBLSM_START("wb_export_to_xml",cdp);
	if( !internal_xml_get_hostvars(conn,hostvars,&pcv,&cvcount,&pemalloced_buffers,&emalloced_buffers_size) )
	{
		wblogs("wb_export_to_xml leave: ERROR, internal_xml_get_hostvars() failed\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	// -> pemalloced_buffers is possibly allocated!
	wblog("dad_ref=\"%s\", fname=\"%s\",answer=0x%x,cvcount=%d\n",Z_STRVAL_PP(dad_ref),Z_STRVAL_PP(fname),(answer==NULL)?-1:answer->cursor,cvcount);
	wblogt_clivar_array("cv",pcv,cvcount);
#ifdef USE_WB_SCRIPT_ENCODING
	if( *(Z_STRVAL_PP(dad_ref))=='*' )
	{
		if( !PREPARE_STRING_SCRIPT2SYSTEM(WBG(script_encoding),conn,&tmp_dad_ref,&emalloc_dad_ref,Z_STRVAL_PP(dad_ref)) )
		{
			internal_efree_emalloced_buffers(pemalloced_buffers,emalloced_buffers_size,pcv);
			wblogs("wb_export_to_xml leave, ERROR: PREPARE_STRING_SCRIPT2SYSTEM() failed\n");
			WBLSM_END;
			RETURN_FALSE;
		}
	}
	else
#endif
		tmp_dad_ref=Z_STRVAL_PP(dad_ref);
	// call 602SQL API function
  if( !Export_to_XML(cdp,tmp_dad_ref,Z_STRVAL_PP(fname),(answer==NULL)?-1:answer->cursor,pcv,cvcount) )
	{
		CONN_REPORT_ERR(conn,"Export_to_XML");
		internal_efree_emalloced_buffers(pemalloced_buffers,emalloced_buffers_size,pcv);
		EFREE_EMALLOC_STRING(emalloc_dad_ref);
		wblogs("wb_export_to_xml leave: ERROR in Export_to_XML()\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	EFREE_EMALLOC_STRING(emalloc_dad_ref);
	if( !internal_xml_pull_outhostvars(WBG(script_encoding),conn,hostvars,pcv,cvcount,pemalloced_buffers,emalloced_buffers_size) )
	{
		internal_efree_emalloced_buffers(pemalloced_buffers,emalloced_buffers_size,pcv);
		wblogs("wb_export_to_xml leave: ERROR, internal_xml_pull_outhostvars() failed\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	internal_efree_emalloced_buffers(pemalloced_buffers,emalloced_buffers_size,pcv);
	wblogs("wb_export_to_xml leave\n");
	WBLSM_END;
	RETURN_TRUE;
}
/*}}}*/
/*{{{ wb_export_to_xml_buffer */
ZEND_FUNCTION(wb_export_to_xml_buffer)
{
	WBLSM_DECLARE
	zval **dad_ref,**hostvars;
	wb_conn *conn;
	cdp_t cdp;
	struct t_clivar *pcv;
	int cvcount;
	struct sql_answer *answer;
	char **pemalloced_buffers;
	int emalloced_buffers_size;
	char *tmp_dad_ref;
#ifdef USE_WB_SCRIPT_ENCODING
	char *emalloc_dad_ref=NULL;
#endif

	wblogs("wb_export_to_xml_buffer enter\n");
	if( !internal_xml_get_params(&conn,&cdp,&answer,&dad_ref,NULL,&hostvars,INTERNAL_FUNCTION_PARAM_PASSTHRU) )
	{
		wblogs("wb_export_to_xml_buffer leave: ERROR, internal_xml_get_params() failed\n");
		RETURN_FALSE;
	}
	WBLSM_START("wb_export_to_xml_buffer",cdp);
	if( !internal_xml_get_hostvars(conn,hostvars,&pcv,&cvcount,&pemalloced_buffers,&emalloced_buffers_size) )
	{
		wblogs("wb_export_to_xml_buffer leave: ERROR, internal_xml_get_hostvars() failed\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	// -> pemalloced_buffers is possibly allocated!
	wblog("dad_ref=\"%s\", answer=0x%x,cvcount=%d\n",Z_STRVAL_PP(dad_ref),(answer==NULL)?-1:answer->cursor,cvcount);
	wblogt_clivar_array("cv",pcv,cvcount);
#ifdef USE_WB_SCRIPT_ENCODING
	if( *(Z_STRVAL_PP(dad_ref))=='*' )
	{
		if( !PREPARE_STRING_SCRIPT2SYSTEM(WBG(script_encoding),conn,&tmp_dad_ref,&emalloc_dad_ref,Z_STRVAL_PP(dad_ref)) )
		{
			internal_efree_emalloced_buffers(pemalloced_buffers,emalloced_buffers_size,pcv);
			wblogs("wb_export_to_xml_buffer leave, ERROR: PREPARE_STRING_SCRIPT2SYSTEM() failed\n");
			WBLSM_END;
			RETURN_FALSE;
		}
	}
	else
#endif
		tmp_dad_ref=Z_STRVAL_PP(dad_ref);
	// call 602SQL API function
	{
#define wb_export_to_xml_buffer_BUFSIZE 4096
		char buf[wb_export_to_xml_buffer_BUFSIZE+1];
		int xmlsize;
		if( !Export_to_XML_buffer(cdp,tmp_dad_ref,buf,wb_export_to_xml_buffer_BUFSIZE,&xmlsize,(answer==NULL)?-1:answer->cursor,pcv,cvcount) )
		{
			CONN_REPORT_ERR(conn,"Export_to_XML_buffer");
			EFREE_EMALLOC_STRING(emalloc_dad_ref);
  		internal_efree_emalloced_buffers(pemalloced_buffers,emalloced_buffers_size,pcv);
			wblogs("wb_export_to_xml_buffer leave: ERROR in Export_to_XML_buffer()\n");
			WBLSM_END;
			RETURN_FALSE;
		}
		buf[wb_export_to_xml_buffer_BUFSIZE]='\0';
		// test if whole XML document is in the buffer
		if( xmlsize>wb_export_to_xml_buffer_BUFSIZE )
		{
			int dynbufsize=(xmlsize+=2);
			char *dynbuf=(char*)emalloc(dynbufsize+1);
			if( dynbuf==NULL )
			{
				report_out_of_memory_error("Can't allocate internal structure.");
				EFREE_EMALLOC_STRING(emalloc_dad_ref);
    		internal_efree_emalloced_buffers(pemalloced_buffers,emalloced_buffers_size,pcv);
				wblogs("wb_export_to_xml_buffer leave, error: out of memory\n");
				WBLSM_END;
				RETURN_FALSE;
			}
			if( !Export_to_XML_buffer(cdp,tmp_dad_ref,dynbuf,xmlsize/* two chars longer than is necessary */,&xmlsize,(answer==NULL)?-1:answer->cursor,pcv,cvcount) )
			{
				CONN_REPORT_ERR(conn,"Export_to_XML_buffer");
				EFREE_EMALLOC_STRING(emalloc_dad_ref);
    		internal_efree_emalloced_buffers(pemalloced_buffers,emalloced_buffers_size,pcv);
				wblogs("wb_export_to_xml_buffer leave: ERROR in Export_to_XML_buffer()\n");
				efree(dynbuf);
				WBLSM_END;
				RETURN_FALSE;
			}
			EFREE_EMALLOC_STRING(emalloc_dad_ref);
			wblog("dynbufsize=%d, xmlsize=%d\n",dynbufsize,xmlsize);
			wblogmem("dynbuf (last 10 chars, without extra null chars):",dynbuf+dynbufsize-9,10);
			// append end-of-the-string char
			dynbuf[dynbufsize]='\0'; // at the end of buffer
			dynbuf[xmlsize]='\0'; // behind the value SQL server wrote into buffer
			wblogmem("dynbuf (last 10 chars, with extra null chars):",dynbuf+dynbufsize-9,10);
		  // set XML document as return value
			RETVAL_STRING(dynbuf,FALSE); // and emalloc-ed dynbuf buffer is owned by return value now
		}
		else
		{
			EFREE_EMALLOC_STRING(emalloc_dad_ref);
		  // set XML document as return value
		  RETVAL_STRING(buf,TRUE);
		}
	}
	if( !internal_xml_pull_outhostvars(WBG(script_encoding),conn,hostvars,pcv,cvcount,pemalloced_buffers,emalloced_buffers_size) )
	{
		internal_efree_emalloced_buffers(pemalloced_buffers,emalloced_buffers_size,pcv);
		wblogs("wb_export_to_xml_buffer leave: ERROR, internal_xml_pull_outhostvars() failed\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	internal_efree_emalloced_buffers(pemalloced_buffers,emalloced_buffers_size,pcv);
	wblogs("wb_export_to_xml_buffer leave\n");
	WBLSM_END;
}
/*}}}*/
/*{{{ wb_import_from_xml */
ZEND_FUNCTION(wb_import_from_xml)
{
	WBLSM_DECLARE
	zval **dad_ref,**fname,**hostvars;
	wb_conn *conn;
	cdp_t cdp;
	struct t_clivar *pcv;
	int cvcount;
	char **pemalloced_buffers;
	int emalloced_buffers_size;
	char *tmp_dad_ref;
#ifdef USE_WB_SCRIPT_ENCODING
	char *emalloc_dad_ref=NULL;
#endif

	wblogs("wb_import_from_xml enter\n");
	if( !internal_xml_get_params(&conn,&cdp,NULL,&dad_ref,&fname,&hostvars,INTERNAL_FUNCTION_PARAM_PASSTHRU) )
	{
		wblogs("wb_import_from_xml leave: ERROR, internal_xml_get_params() failed\n");
		RETURN_FALSE;
	}
	WBLSM_START("wb_import_from_xml",cdp);
	if( !internal_xml_get_hostvars(conn,hostvars,&pcv,&cvcount,&pemalloced_buffers,&emalloced_buffers_size) )
	{
		wblogs("wb_import_from_xml leave: ERROR, internal_xml_get_hostvars() failed\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	// -> pemalloced_buffers is possibly allocated!
	wblog("dad_ref=\"%s\", fname=\"%s\", cvcount=%d\n",Z_STRVAL_PP(dad_ref),Z_STRVAL_PP(fname),cvcount);
	wblogt_clivar_array("cv",pcv,cvcount);
#ifdef USE_WB_SCRIPT_ENCODING
	if( *(Z_STRVAL_PP(dad_ref))=='*' )
	{
		if( !PREPARE_STRING_SCRIPT2SYSTEM(WBG(script_encoding),conn,&tmp_dad_ref,&emalloc_dad_ref,Z_STRVAL_PP(dad_ref)) )
		{
			internal_efree_emalloced_buffers(pemalloced_buffers,emalloced_buffers_size,pcv);
			wblogs("wb_import_from_xml leave: ERROR in PREPARE_STRING_SCRIPT2SYSTEM()\n");
			WBLSM_END;
			RETURN_FALSE;
		}
	}
	else
#endif
		tmp_dad_ref=Z_STRVAL_PP(dad_ref);
	// call 602SQL API function
  if( !Import_from_XML(cdp,Z_STRVAL_PP(dad_ref),Z_STRVAL_PP(fname),pcv,cvcount) )
	{
		CONN_REPORT_ERR(conn,"Import_from_XML");
		EFREE_EMALLOC_STRING(emalloc_dad_ref);
		internal_efree_emalloced_buffers(pemalloced_buffers,emalloced_buffers_size,pcv);
		wblogs("wb_import_from_xml leave: ERROR in Import_from_XML()\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	EFREE_EMALLOC_STRING(emalloc_dad_ref);
	wblogt_clivar_array("cv - after Import_from_XML() call",pcv,cvcount);
	if( !internal_xml_pull_outhostvars(WBG(script_encoding),conn,hostvars,pcv,cvcount,pemalloced_buffers,emalloced_buffers_size) )
	{
		wblogs("wb_import_from_xml leave: ERROR, internal_xml_pull_outhostvars() failed\n");
  	internal_efree_emalloced_buffers(pemalloced_buffers,emalloced_buffers_size,pcv);
		WBLSM_END;
		RETURN_FALSE;
	}
	internal_efree_emalloced_buffers(pemalloced_buffers,emalloced_buffers_size,pcv);
	wblogs("wb_import_from_xml leave\n");
	WBLSM_END;
	RETURN_TRUE;
}
/*}}}*/
/*{{{ wb_import_from_xml_buffer */
ZEND_FUNCTION(wb_import_from_xml_buffer)
{
	WBLSM_DECLARE
	zval **dad_ref,**buffer,**hostvars;
	wb_conn *conn;
	cdp_t cdp;
	struct t_clivar *pcv;
	int cvcount;
	char **pemalloced_buffers;
	int emalloced_buffers_size;
	char *tmp_dad_ref;
#ifdef USE_WB_SCRIPT_ENCODING
	char *emalloc_dad_ref=NULL;
#endif

	wblogs("wb_import_from_xml_buffer enter\n");
	if( !internal_xml_get_params(&conn,&cdp,NULL,&dad_ref,&buffer,&hostvars,INTERNAL_FUNCTION_PARAM_PASSTHRU) )
	{
		wblogs("wb_import_from_xml_buffer leave: ERROR, internal_xml_get_params() failed\n");
		RETURN_FALSE;
	}
	WBLSM_START("wb_import_from_xml_buffer",cdp);
	if( !internal_xml_get_hostvars(conn,hostvars,&pcv,&cvcount,&pemalloced_buffers,&emalloced_buffers_size) )
	{
		wblogs("wb_import_from_xml_buffer leave: ERROR, internal_xml_get_hostvars() failed\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	// -> pemalloced_buffers is possibly allocated!
	wblog("dad_ref=\"%s\", buflen=%d, cvcount=%d\n",Z_STRVAL_PP(dad_ref),Z_STRLEN_PP(buffer),cvcount);
	wblogt_clivar_array("cv",pcv,cvcount);
#ifdef USE_WB_SCRIPT_ENCODING
	if( *(Z_STRVAL_PP(dad_ref))=='*' )
	{
		if( !PREPARE_STRING_SCRIPT2SYSTEM(WBG(script_encoding),conn,&tmp_dad_ref,&emalloc_dad_ref,Z_STRVAL_PP(dad_ref)) )
		{
			internal_efree_emalloced_buffers(pemalloced_buffers,emalloced_buffers_size,pcv);
			wblogs("wb_import_from_xml_buffer leave: ERROR in PREPARE_STRING_SCRIPT2SYSTEM()\n");
			WBLSM_END;
			RETURN_FALSE;
		}
	}
	else
#endif
		tmp_dad_ref=Z_STRVAL_PP(dad_ref);
	// call 602SQL API function
  if( !Import_from_XML_buffer(cdp,Z_STRVAL_PP(dad_ref),Z_STRVAL_PP(buffer),Z_STRLEN_PP(buffer),pcv,cvcount) )
	{
		CONN_REPORT_ERR(conn,"Import_from_XML");
		EFREE_EMALLOC_STRING(emalloc_dad_ref);
		internal_efree_emalloced_buffers(pemalloced_buffers,emalloced_buffers_size,pcv);
		wblogs("wb_import_from_xml_buffer leave: ERROR in Import_from_XML_buffer()\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	EFREE_EMALLOC_STRING(emalloc_dad_ref);
	if( !internal_xml_pull_outhostvars(
#ifdef USE_WB_SCRIPT_ENCODING
				WBG(script_encoding),
#endif
				conn,hostvars,pcv,cvcount,pemalloced_buffers,emalloced_buffers_size) )
	{
		wblogs("wb_import_from_xml_buffer leave: ERROR, internal_xml_pull_outhostvars() failed\n");
  	internal_efree_emalloced_buffers(pemalloced_buffers,emalloced_buffers_size,pcv);
		WBLSM_END;
		RETURN_FALSE;
	}
	internal_efree_emalloced_buffers(pemalloced_buffers,emalloced_buffers_size,pcv);
	wblogs("wb_import_from_xml_buffer leave\n");
	WBLSM_END;
	RETURN_TRUE;
}
/*}}}*/
/*}}}*/
#endif
#ifdef USE_XSD_PARSER
/*{{{ XSD parser functions */
/* support functions {{{*/
/* internal_xsdp_log_error {{{*/
static void internal_xsdp_log_error(HANDLE xsdp_handle,const char *error_message)
{
	int errcode;
	const char *errmsg;
	if( DSParserGetLastError(xsdp_handle,&errcode,&errmsg) )
	{
		errcode=0;
		errmsg="Unknown XSD parser error.";
	}
	if( error_message==NULL )
	{
		zend_error(E_WARNING,"602SQL: XSD parser error number %d: %s",errcode,errmsg);
		wblog("XSD parser error, errcode=%d, errmsg=%s\n",errcode,errmsg);
	}
	else
	{
		zend_error(E_WARNING,"602SQL: %s XSD parser error number %d: %s",error_message,errcode,errmsg);
		wblog("%s XSD parser error, errcode=%d, errmsg=%s\n",error_message,errcode,errmsg);
	}
} 
/*}}}*/
/* internal_xsdp_set_prop {{{
 * sets property_id XSD parser property to property_value_zval value
 * returns 1 on success or 0 on error (and logs error too)
 * tests validity of property_id only - all other tests (validity and status of XSD parser ...) have to be done by caller! */
static int internal_xsdp_set_prop(struct xsdp_t *xsdp,int property_id,zval **property_value_zval)
{
	wblog("internal_xsdp_set_prop enter, property_id=%d\n",property_id);
	switch( property_id )
	{
		case DSP_SCHEMANAME:
		case DSP_FOLDERNAME:
		case DSP_DADNAME:
		case DSP_TABLEPREFIX:
			convert_to_string_ex(property_value_zval);
			if( DSParserSetStrProp(xsdp->xsdp_handle,property_id,Z_STRVAL_PP(property_value_zval)) )
			{
				internal_xsdp_log_error(xsdp->xsdp_handle,"Can't set string property of the XSD parser.");
				wblogs("internal_xsdp_set_prop leave, ERROR\n");
				return 0;
			}
			wblog("internal_xsdp_set_prop - string property value \"%s\"\n",Z_STRVAL_PP(property_value_zval));
			break;
		case DSP_DEFSTRLENGTH:
		case DSP_UNICODE:
		case DSP_IDTYPE:
		case DSP_REFINT:
		case DSP_SPECROOTSEQ:
			convert_to_long_ex(property_value_zval);
			if( DSParserSetIntProp(xsdp->xsdp_handle,property_id,Z_LVAL_PP(property_value_zval)) )
			{
				internal_xsdp_log_error(xsdp->xsdp_handle,"Can't set integer property of the XSD parser.");
				wblogs("internal_xsdp_set_prop leave, ERROR\n");
				return 0;
			}
			wblog("internal_xsdp_set_prop - int property value %d\n",Z_LVAL_PP(property_value_zval));
			break;
		default:
			zend_error(E_WARNING,"602SQL: Invalid XSD parser property number %d. You are trying to set a value of nonexistent property.",property_id);
			wblog("internal_xsdp_set_prop leave, ERROR, invalid property number %d\n",property_id);
			return 0;
	}
	wblogs("internal_xsdp_set_prop leave, OK\n");
	return 1;
} 
/*}}}*/
/*}}}*/
/*{{{ wb_xsdp_open
 * resource wb_xsdp_open(resource connection [,array properties] )
 * opens and returns new XSD parser instance or FALSE on error
 * properties is array which items are indexed by WB_DSP_xxx property numbers and that contain value of particular property */
ZEND_FUNCTION(wb_xsdp_open)
{
	WBLSM_DECLARE
	zval **connhandle,**properties;
	wb_conn *conn;
	struct xsdp_t *xsdp;
	int id;

	wblogs("wb_xsdp_open enter\n");
	switch( ZEND_NUM_ARGS() )
	{
		case 1:
			if( zend_get_parameters_ex(1,&connhandle)!=SUCCESS ) WRONG_PARAM_COUNT;
			properties=NULL;
			break;
		case 2:
		  if( zend_get_parameters_ex(2,&connhandle,&properties)!=SUCCESS ) WRONG_PARAM_COUNT;
			break;
		default:
			WRONG_PARAM_COUNT;
	}
	conn=get_conn_from_resource(INTERNAL_FUNCTION_PARAM_PASSTHRU,connhandle);
	if (!conn)
	{
		wblogs("wb_xsdp_open leave, FALSE, error in get_conn_from_resource()\n");
		RETURN_FALSE; /* error in get_conn_from_resource */
	}
	WBLSM_START("wb_xsdp_open",CDP(conn));
	xsdp=(struct xsdp_t*)emalloc(sizeof(struct xsdp_t));
	if( xsdp==NULL )
	{
		report_out_of_memory_error("Can't allocate internal structure.");
		wblogs("wb_xsdp_open leave, error: out of memory\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	if( DSParserOpen(CDP(conn),&(xsdp->xsdp_handle)) )
	{
		CONN_REPORT_ERR(conn,"DSParserOpen");
		wblogs("wb_xsdp_open leave, FALSE, error in DSParserOpen()\n");
		efree(xsdp);
		WBLSM_END;
		RETURN_FALSE;
	}
	xsdp->conn=conn;
	xsdp->status=XSDP_STATUS_OPENED;
	// if properties aren't empty set them
	if( properties!=NULL )
	{
		int count,i;
		if( Z_TYPE_PP(properties)!=IS_ARRAY )
		{
			efree(xsdp);
			zend_error(E_WARNING,"602SQL: Expecting array as 'properties' parameter of wb_xsdp_open() function.");
			wblogs("wb_xsdp_open leave, FALSE, properties isn't array\n");
			WBLSM_END;
			RETURN_FALSE;
		}
		// find out number of elements in properties array
		count=zend_hash_num_elements(Z_ARRVAL_PP(properties));
		// set internal pointer of properties array to first element
		zend_hash_internal_pointer_reset(Z_ARRVAL_PP(properties));
		for( i=0;i<count;i++ )
		{
			zval **elem; // element of properties
			char *string_key=NULL; // string key value of elem
			ulong int_key; // int key valud of elem
			// get element key value
			if( zend_hash_get_current_key(Z_ARRVAL_PP(properties),&string_key,&int_key,1)==HASH_KEY_IS_STRING )
			{
				zend_error(E_WARNING,"602SQL: 'properties' parameter of wb_xsdp_open() function have to be integer-indexed array, however it contains \"%s\" string key item.",string_key);
				wblog("wb_xsdp_open leave: FALSE, 'properties' parameter contains \"%s\" string key item.\n",string_key);
				efree(xsdp);
				WBLSM_END;
				RETURN_FALSE;
			}
			// get element value
			if( zend_hash_get_current_data(Z_ARRVAL_PP(properties),(void**)&elem)!=SUCCESS )
			{
				zend_error(E_WARNING,"602SQL: Can't read 'properties' parameter array item value which index is %lu.",int_key);
				wblog("wb_xsdp_open leave, FALSE, can't read properties[%lu] item\n",int_key);
				if( string_key!=NULL ) efree(string_key);
				efree(xsdp);
				WBLSM_END;
				RETURN_FALSE;
			}
			if( !internal_xsdp_set_prop(xsdp,int_key/*property ID*/,elem) )
			{
				wblog("wb_xsdp_open leave, FALSE, internal_xsdp_set_prop() failed for properties[%lu] item\n",int_key);
				if( string_key!=NULL ) efree(string_key);
				efree(xsdp);
				WBLSM_END;
				RETURN_FALSE;
			}
			if( string_key!=NULL ) efree(string_key);
			// move to next element of properties
			zend_hash_move_forward(Z_ARRVAL_PP(properties));
		} 
	} 
	/* create and register resource */
	id=zend_list_insert(xsdp,wbres_xsdp
#if PHP_VERSION_ID>50400
				TSRMLS_CC
#endif
			);
	conn->usage++;
	wblog("wb_xsdp_open leave, result=%d (xsdp resource)\n",id);
	WBLSM_END;
	/* return resource */
	RETURN_RESOURCE(id);
}
/*}}}*/
/*{{{ wb_xsdp_set_prop
 * boolean wb_xsdp_set_prop(resource xsdp,int property_id,mixed property_value)
 * assigns new value to the property, returns TRUE on success or FALSE on error */
ZEND_FUNCTION(wb_xsdp_set_prop)
{
	WBLSM_DECLARE
	zval **xsdp_zval,**property_id_zval,**property_value_zval;
	struct xsdp_t *xsdp;

	wblogs("wb_xsdp_set_prop enter\n");
	if( ZEND_NUM_ARGS()!=3 || zend_get_parameters_ex(3,&xsdp_zval,&property_id_zval,&property_value_zval)!=SUCCESS ) WRONG_PARAM_COUNT;
	xsdp=get_xsdp_from_resource(INTERNAL_FUNCTION_PARAM_PASSTHRU,xsdp_zval);
	if( !xsdp )
	{
		wblogs("wb_xsdp_set_prop leave, FALSE, error in get_xsdp_from_resource()\n");
		RETURN_FALSE;
	}
	WBLSM_START("wb_xsdp_set_prop",CDP_FROM_XSDP_P(xsdp));
	if( xsdp->status==XSDP_STATUS_CLOSED )
	{
		zend_error(E_WARNING,"602SQL: Can't set property value of XSD parser, parser is already closed.");
		wblogs("wb_xsdp_set_prop leave, FALSE, parser is closed.\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	convert_to_long_ex(property_id_zval);
	if( internal_xsdp_set_prop(xsdp,Z_LVAL_PP(property_id_zval),property_value_zval) )
	{
		wblogs("wb_xsdp_set_prop leave, TRUE\n");
		WBLSM_END;
		RETURN_TRUE;
	}
	else
	{
		wblogs("wb_xsdp_set_prop leave, FALSE, internal_xsdp_set_prop() failed\n");
		WBLSM_END;
		RETURN_FALSE;
	}
}
/*}}}*/
/*{{{ wb_xsdp_get_prop
 * boolean wb_xsdp_get_prop(resource xsdp,int property_id,mixed &property_value)
 * assigns value of the property into property_value output parameter, returns TRUE on success or FALSE on error */
ZEND_FUNCTION(wb_xsdp_get_prop)
{
	WBLSM_DECLARE
	zval **xsdp_zval,**property_id_zval,**property_value_zval;
	struct xsdp_t *xsdp;

	wblogs("wb_xsdp_get_prop enter\n");
	if( ZEND_NUM_ARGS()!=3 || zend_get_parameters_ex(3,&xsdp_zval,&property_id_zval,&property_value_zval)!=SUCCESS ) WRONG_PARAM_COUNT;
	xsdp=get_xsdp_from_resource(INTERNAL_FUNCTION_PARAM_PASSTHRU,xsdp_zval);
	if( !xsdp )
	{
		wblogs("wb_xsdp_get_prop leave, FALSE, error in get_xsdp_from_resource()\n");
		RETURN_FALSE;
	}
	WBLSM_START("wb_xsdp_get_prop",CDP_FROM_XSDP_P(xsdp));
	if( xsdp->status==XSDP_STATUS_CLOSED )
	{
		zend_error(E_WARNING,"602SQL: Can't get property value of XSD parser, parser is already closed.");
		wblogs("wb_xsdp_get_prop leave, FALSE, parser is closed.\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	convert_to_long_ex(property_id_zval);
	switch( Z_LVAL_PP(property_id_zval) )
	{
    case DSP_ERRMSG:
    case DSP_ERRFILE:
		case DSP_XMLFORMSOURCE:
		case DSP_XSD:
		case DSP_ROOTTABLE:
		case DSP_ROOTTABLEKEY:
		case DSP_ROOTTABLESEQ:
		case DSP_FORMID:
		case DSP_FORMDATAROOTNAME:
		case DSP_FORMDATANSPREFIX:
		case DSP_FORMDATANSURI:
			{ 
				const char *value;
				value=NULL;
				wblog("string property no. %d\n",Z_LVAL_PP(property_id_zval));
				// string properties
				if( DSParserGetStrProp(xsdp->xsdp_handle,Z_LVAL_PP(property_id_zval),&value) )
				{
					internal_xsdp_log_error(xsdp->xsdp_handle,"Can't get string property of the XSD parser.");
					wblogs("wb_xsdp_get_prop leave, FALSE\n");
					WBLSM_END;
					RETURN_FALSE;
				}
				wblog("DSParserGetStrProp() returned value=%p\n",value);
				if( value==NULL )
				{
					ZVAL_NULL(*property_value_zval);
				}
				else
				{
					ZVAL_STRING(*property_value_zval,(char*)value,TRUE);
				}
			}	
			break;
    case DSP_ERRCODE:
    case DSP_ERRSOURCE:
    case DSP_ERRTYPEXML:
    case DSP_ERRLINE:
    case DSP_ERRCOL:
			{ 
				int value;
				// int properties
				if( DSParserGetIntProp(xsdp->xsdp_handle,Z_LVAL_PP(property_id_zval),&value) )
				{
					internal_xsdp_log_error(xsdp->xsdp_handle,"Can't get integer property of the XSD parser.");
					wblogs("wb_xsdp_get_prop leave, FALSE\n");
					WBLSM_END;
					RETURN_FALSE;
				}
				ZVAL_LONG(*property_value_zval,value);
			}	
      break;
		default:
			zend_error(E_WARNING,"602SQL: Invalid property number %ld in wb_xsdp_get_prop function.",Z_LVAL_PP(property_id_zval));
			wblog("wb_xsdp_get_prop leave, FALSE, invalid property number %ld\n",Z_LVAL_PP(property_id_zval));
			WBLSM_END;
			RETURN_FALSE;
	}
	wblogs("wb_xsdp_get_prop leave, TRUE\n");
	WBLSM_END;
	RETURN_TRUE;
}
/*}}}*/
/*{{{ wb_xsdp_parse
 * boolean wb_xsdp_parse(resource xsdp,string source,array &result)
 * parses source with XSD parser, returns TRUE on success or FALSE on error
 * returns list of database objects that will be created by XSD parser in result parameter on success too
 *   result item will be array with 'name' and 'category' items. */
ZEND_FUNCTION(wb_xsdp_parse)
{
	WBLSM_DECLARE
	zval ** xsdp_zval,**source_zval,**result_zval;
	struct xsdp_t *xsdp;

	wblogs("wb_xsdp_parse enter\n");
	if( ZEND_NUM_ARGS()!=3 || zend_get_parameters_ex(3,&xsdp_zval,&source_zval,&result_zval)!=SUCCESS ) WRONG_PARAM_COUNT;
	xsdp=get_xsdp_from_resource(INTERNAL_FUNCTION_PARAM_PASSTHRU,xsdp_zval);
	if( !xsdp )
	{
		wblogs("wb_xsdp_parse leave, FALSE, error in get_xsdp_from_resource()\n");
		RETURN_FALSE;
	}
	WBLSM_START("wb_xsdp_parse",CDP_FROM_XSDP_P(xsdp));
	if( xsdp->status==XSDP_STATUS_CLOSED )
	{
		zend_error(E_WARNING,"602SQL: Can't parse XSD document or XML form by that XSD parser, parser is already closed.");
		wblogs("wb_xsdp_parse leave, FALSE, parser is closed.\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	if( xsdp->status==XSDP_STATUS_PARSED )
	{
		zend_error(E_WARNING,"602SQL: Can't parse XSD document or XML form by that XSD parser, parser already parsed XSD/XML document.");
		wblogs("wb_xsdp_parse leave, FALSE, parser already parsed.\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	if( xsdp->status==XSDP_STATUS_OBJECTS_CREATED )
	{
		zend_error(E_WARNING,"602SQL: Can't parse XSD document or XML form by that XSD parser, parser already parsed XSD/XML document and created database objects too.");
		wblogs("wb_xsdp_parse leave, FALSE, parser already parsed and created db. objects.\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	if( Z_TYPE_PP(result_zval)!=IS_ARRAY && array_init(*result_zval)!=SUCCESS )
	{
		zend_error(E_WARNING, "602SQL: Can not create output array");
		wblogs("wb_xsdp_parse leave, FALSE, array_init() failed\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	convert_to_string_ex(source_zval);
	wblog("first char of source: %c\n",*Z_STRVAL_PP(source_zval));
	if( *(DWORD *)Z_STRVAL_PP(source_zval) == ('P' + ('K' << 8) + (3 << 16) + (4 << 24)) )
	{
		if( DSParserParseM(xsdp->xsdp_handle,Z_STRVAL_PP(source_zval),Z_STRLEN_PP(source_zval)) )
		{ 
			internal_xsdp_log_error(xsdp->xsdp_handle,"Can't parse XSD/XML document.");
			wblog("wb_xsdp_parse leave, FALSE\n");
			WBLSM_END;
			RETURN_FALSE;
		}
	}
	else
	{
		if( DSParserParse(xsdp->xsdp_handle,Z_STRVAL_PP(source_zval)) )
		{ 
			internal_xsdp_log_error(xsdp->xsdp_handle,"Can't parse XSD/XML document.");
			wblog("wb_xsdp_parse leave, FALSE\n");
			WBLSM_END;
			RETURN_FALSE;
		}
	}
	// retrieve object names and categories
	{
		int index=0;
		tname object_name;
		tcateg object_categ;
		for(;;)
		{
			zval *elem;
			*object_name='\0';
			object_categ=0;
		  if( DSParserGetObjectName(xsdp->xsdp_handle,index,object_name,&object_categ) ) break;
			wblog("%d. object \"%s\", category %d\n",index+1,object_name,object_categ);
			MAKE_STD_ZVAL(elem);
			if( array_init(elem)!=SUCCESS )
			{
				zend_error(E_WARNING,"602SQL: Can't create output array element.");
				wblogs("wb_xsdp_create_objects leave, FALSE, array_init() failed\n");
				WBLSM_END;
				RETURN_FALSE;
			}
			add_assoc_long(elem,"category",object_categ);
			add_assoc_string(elem,"name",object_name,1);
			add_index_zval(*result_zval,index,elem);
			index++;
		}
	}
	// change status
	xsdp->status=XSDP_STATUS_PARSED;
	wblogs("wb_xsdp_parse leave, OK\n");
	WBLSM_END;
	RETURN_TRUE;
}
/*}}}*/
/* wb_xsdp_get_columns {{{
 * boolean wb_xsdp_get_columns(resource xsdp,string table_name,array &result) */
ZEND_FUNCTION(wb_xsdp_get_columns)
{
	WBLSM_DECLARE
	zval ** xsdp_zval,**table_name_zval,**result_zval;
	struct xsdp_t *xsdp;

	wblogs("wb_xsdp_get_columns enter\n");
	if( ZEND_NUM_ARGS()!=3 || zend_get_parameters_ex(3,&xsdp_zval,&table_name_zval,&result_zval)!=SUCCESS ) WRONG_PARAM_COUNT;
	xsdp=get_xsdp_from_resource(INTERNAL_FUNCTION_PARAM_PASSTHRU,xsdp_zval);
	if( !xsdp )
	{
		wblogs("wb_xsdp_get_columns leave, FALSE, error in get_xsdp_from_resource()\n");
		RETURN_FALSE;
	}
	WBLSM_START("wb_xsdp_get_columns",CDP_FROM_XSDP_P(xsdp));
	if( xsdp->status==XSDP_STATUS_CLOSED )
	{
		zend_error(E_WARNING,"602SQL: Can't get column list for given table, XSD parser is already closed.");
		wblogs("wb_xsdp_get_columns leave, FALSE, parser is closed.\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	if( xsdp->status==XSDP_STATUS_OPENED )
	{
		zend_error(E_WARNING,"602SQL: Can't get column list for given table, XSD parser didn't parsed XSD/XML document yet.");
		wblogs("wb_xsdp_get_columns leave, FALSE, parser didn't parsed.\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	if( xsdp->status==XSDP_STATUS_OBJECTS_CREATED )
	{
		zend_error(E_WARNING,"602SQL: Can't get column list for given table, XSD parser already created database object for this XSD/XML document.");
		wblogs("wb_xsdp_get_columns leave, FALSE, parser already created db. objects.\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	if( Z_TYPE_PP(result_zval)!=IS_ARRAY && array_init(*result_zval)!=SUCCESS )
	{
		zend_error(E_WARNING, "602SQL: Can not create output array");
		wblogs("wb_xsdp_get_columns leave, FALSE, array_init() failed\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	convert_to_string_ex(table_name_zval);

	{
		tname column_name;
		int column_type,counter;
		union t_specif column_specif;
		int exclude,unspec_type;
		for( counter=0;;counter++ )
		{
			zval *elem;
			exclude=unspec_type=0;
			wblog("wb_xsdp_get_columns: before DSParserGetColumnProps(), table_name_zval=%s, counter=%d\n",Z_STRVAL_PP(table_name_zval),counter);
			if( DSParserGetColumnProps(xsdp->xsdp_handle,Z_STRVAL_PP(table_name_zval),counter,column_name,&column_type,&column_specif,&exclude
#if WBVERS>=1100
						,&unspec_type
#endif
						) )
			{
				int errnum,result_of_DSParserGetLastError;
				const char *errmsg;
				wblogs("wb_xsdp_get_columns(): after DSParserGetColumnProps()\n");
				if( !DSParserGetLastError(xsdp->xsdp_handle,&errnum,&errmsg) )
				{
					result_of_DSParserGetLastError=0;
					if( errnum==0 )
					{
						wblogs("wb_xsdp_get_columns(): all column props fetched\n");
						break; // no more columns
					} 
					else wblog("inside error handling code for DSParserGetColumnProps() function - DSParserGetLastError() returned error number %d\n",errnum);
				}
				else
				{
					result_of_DSParserGetLastError=1;
					wblogs("inside error handling code for DSParserGetColumnProps() function - DSParserGetLastError() returned TRUE\n");
				}
				{
					char buf[1000];
					sprintf(buf,"Can't retrieve properties of the %d-th column from `%s` table. (%d)",counter+1,Z_STRVAL_PP(table_name_zval),result_of_DSParserGetLastError);
					internal_xsdp_log_error(xsdp->xsdp_handle,buf);
				}
				WBLSM_END;
				RETURN_FALSE;
			}
			wblog("%d. column \"%s\", type %d, exclude %d, unspec_type %d\n",counter+1,column_name,column_type,exclude,unspec_type);
			// save into result
			MAKE_STD_ZVAL(elem);
			if( array_init(elem)!=SUCCESS )
			{
				zend_error(E_WARNING,"602SQL: Can't create output array element.");
				wblogs("wb_xsdp_get_columns leave, FALSE, array_init() failed\n");
				WBLSM_END;
				RETURN_FALSE;
			}
			// column name
			add_assoc_string(elem,"name",column_name,1);
			// column type
			add_assoc_long(elem,"type",column_type);
			// exclude flag
			add_assoc_bool(elem,"exclude",exclude);
			// unspec_type flag
			add_assoc_bool(elem,"unspec_type",unspec_type);
			// additional type information
			switch( column_type )
			{
				case ATT_INT8:
				case ATT_INT16:
				case ATT_INT32:
				case ATT_INT64:
					add_assoc_long(elem,"scale",column_specif.scale);
					break;
				case ATT_STRING:
				case ATT_TEXT:
#if WBVERS>=1100
				case ATT_EXTCLOB:
#endif
					add_assoc_bool(elem,"wide_char",(column_specif.wide_char!=0)?1:0);
					/* no break! */
				case ATT_RASTER:
				  if( column_type!=ATT_TEXT
#if WBVERS>=1100
						&& column_type!=ATT_EXTCLOB
#endif
					) 
					{
					  add_assoc_long(elem,"length",column_specif.length); // length is valid for ATT_STRING and ATT_RASTER only (not for ATT_TEXT, ATT_EXTCLOB and ATT_NOSPEC)
					}
					break;
			}
			add_index_zval(*result_zval,counter,elem);
		} 
	}
	wblogs("wb_xsdp_get_columns leave, OK\n");
	WBLSM_END;
	RETURN_TRUE;
} 
/*}}}*/
/* wb_xsdp_set_column_type {{{
 * boolean wb_xsdp_set_column_type(resource xsdp,string table_name,string column_name,array type_description,boolean exclude_column) */
ZEND_FUNCTION(wb_xsdp_set_column_type)
{
	WBLSM_DECLARE
	zval ** xsdp_zval,**table_name_zval,**column_name_zval,**type_zval,**exclude_column_zval;
	struct xsdp_t *xsdp;

	wblogs("wb_xsdp_set_column_type enter\n");
	switch( ZEND_NUM_ARGS() )
	{
		case 4:
			if( zend_get_parameters_ex(4,&xsdp_zval,&table_name_zval,&column_name_zval,&type_zval)!=SUCCESS ) WRONG_PARAM_COUNT;
			exclude_column_zval=NULL;
			break;
		case 5:
			if( zend_get_parameters_ex(5,&xsdp_zval,&table_name_zval,&column_name_zval,&type_zval,&exclude_column_zval)!=SUCCESS ) WRONG_PARAM_COUNT;
			break;
		default:
			WRONG_PARAM_COUNT;
	}
	xsdp=get_xsdp_from_resource(INTERNAL_FUNCTION_PARAM_PASSTHRU,xsdp_zval);
	if( !xsdp )
	{
		wblogs("wb_xsdp_set_column_type leave, FALSE, error in get_xsdp_from_resource()\n");
		RETURN_FALSE;
	}
	WBLSM_START("wb_xsdp_set_column_type",CDP_FROM_XSDP_P(xsdp));
	if( xsdp->status==XSDP_STATUS_CLOSED )
	{
		zend_error(E_WARNING,"602SQL: Can't set column type, XSD parser is already closed.");
		wblogs("wb_xsdp_set_column_type leave, FALSE, parser is closed.\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	if( xsdp->status==XSDP_STATUS_OPENED )
	{
		zend_error(E_WARNING,"602SQL: Can't set column type, XSD parser didn't parsed XSD/XML document yet.");
		wblogs("wb_xsdp_set_column_type leave, FALSE, parser didn't parsed.\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	if( xsdp->status==XSDP_STATUS_OBJECTS_CREATED )
	{
		zend_error(E_WARNING,"602SQL: Can't set column type, XSD parser already created database object for this XSD/XML document.");
		wblogs("wb_xsdp_set_column_type leave, FALSE, parser already created db. objects.\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	if( Z_TYPE_PP(type_zval)!=IS_ARRAY )
	{
		zend_error(E_WARNING, "602SQL: Type parameter of the function isn't array.");
		wblogs("wb_xsdp_set_column_type leave, FALSE, type_zval isn't array\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	convert_to_string_ex(table_name_zval);
	convert_to_string_ex(column_name_zval);
	if( exclude_column_zval!=NULL ) convert_to_boolean_ex(exclude_column_zval);

	{
		union t_specif specif;
		int type=0;
		zval **tmp;
		memset(&specif,0,sizeof(specif));
		if( zend_hash_find(Z_ARRVAL_PP(type_zval),"type",5,(void**)&tmp)!=SUCCESS )
		{
			zend_error(E_WARNING,"602SQL: type_description parameter array doesn't contain \"type\" item.");
			wblogs("wb_xsdp_set_column_type leave, FALSE, type_description doens't contain \"type\" item\n");
			WBLSM_END;
			RETURN_FALSE;
		}
		convert_to_long_ex(tmp);
		type=Z_LVAL_PP(tmp);
		switch( type )
		{
			case ATT_INT8:
			case ATT_INT16:
			case ATT_INT32:
			case ATT_INT64:
				if( zend_hash_find(Z_ARRVAL_PP(type_zval),"scale",6,(void**)&tmp)==SUCCESS )
				{
					convert_to_long_ex(tmp);
					specif.scale=(sig8)Z_LVAL_PP(tmp);
				}
				break;
			case ATT_STRING:
			case ATT_TEXT:
#if WBVERS>=1100
			case ATT_EXTCLOB:
#endif
				if( zend_hash_find(Z_ARRVAL_PP(type_zval),"wide_char",10,(void**)&tmp)==SUCCESS )
				{
					convert_to_boolean_ex(tmp);
					specif.wide_char=Z_BVAL_PP(tmp);
				}
				/* no break! */
			case ATT_RASTER:
				if(
#if WBVERS>=1100
						(type!=ATT_TEXT && type!=ATT_EXTCLOB)
#else
					 	type!=ATT_TEXT
#endif
					&& zend_hash_find(Z_ARRVAL_PP(type_zval),"length",7,(void**)&tmp)==SUCCESS
				)
				{
					convert_to_long_ex(tmp);
					specif.length=(uns16)Z_LVAL_PP(tmp);
				}
				break;
		}
		if( DSParserSetColumnProps(xsdp->xsdp_handle,Z_STRVAL_PP(table_name_zval),Z_STRVAL_PP(column_name_zval),type,specif,(exclude_column_zval==NULL)?false:Z_BVAL_PP(exclude_column_zval)) )
		{
			internal_xsdp_log_error(xsdp->xsdp_handle,"Can't set column type.");
			wblogs("wb_xsdp_set_column_type leave, FALSE, DSParserSetColumnProps() failed\n");
			WBLSM_END;
			RETURN_FALSE;
		}
	}
	wblogs("wb_xsdp_set_column_type leave, OK\n");
	WBLSM_END;
	RETURN_TRUE;
} 
/*}}}*/
/*{{{ wb_xsdp_exclude_table
 * boolean wb_xsdp_exclude_table(resource xsdp,string table_name)
 * excludes given table from creating, i.e. the table won't be created by wb_xsdp_create_objects() function, returns TRUE on success or FALSE on error */
ZEND_FUNCTION(wb_xsdp_exclude_table)
{
	WBLSM_DECLARE
	zval **xsdp_zval,**table_name_zval;
	struct xsdp_t *xsdp;

	wblogs("wb_xsdp_exclude_table enter\n");
	if( ZEND_NUM_ARGS()!=2 || zend_get_parameters_ex(2,&xsdp_zval,&table_name_zval)!=SUCCESS ) WRONG_PARAM_COUNT;
	xsdp=get_xsdp_from_resource(INTERNAL_FUNCTION_PARAM_PASSTHRU,xsdp_zval);
	if( !xsdp )
	{
		wblogs("wb_xsdp_exclude_table leave, FALSE, error in get_xsdp_from_resource()\n");
		RETURN_FALSE;
	}
	WBLSM_START("wb_xsdp_exclude_table",CDP_FROM_XSDP_P(xsdp));
	if( xsdp->status==XSDP_STATUS_CLOSED )
	{
		zend_error(E_WARNING,"602SQL: Can't exclude table, XSD parser is already closed.");
		wblogs("wb_xsdp_exclude_table leave, FALSE, parser is closed.\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	convert_to_string_ex(table_name_zval);
	wblog("table_name_zval=%s\n",Z_STRVAL_PP(table_name_zval));
	if( DSParserSetTableExcludeFlag(xsdp->xsdp_handle,Z_STRVAL_PP(table_name_zval),TRUE) )
	{ // DSParserSetTableExcludeFlag() returns TRUE on error!!!
		internal_xsdp_log_error(xsdp->xsdp_handle,"Failed to exclude table.");
		wblogs("wb_xsdp_exclude_table leave, FALSE, DSParserSetTableExcludeFlag() failed\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	else
	{
		wblogs("wb_xsdp_exclude_table leave, TRUE\n");
		WBLSM_END;
		RETURN_TRUE;
	}
}
/*}}}*/
/* wb_xsdp_create_objects {{{
 * boolean wb_xsdp_create_objects(resource xsdp,array &result)
 * creates db. objects according to xsdp parser
 * returns TRUE on success (and db. object list in result array output parameter) or FALSE on error
 * result will be array of array of 'name' (database object name)
 * and 'category' (database object category number one of CATEG_xxx constants) items
 * XSD parser have to be parsed */
ZEND_FUNCTION(wb_xsdp_create_objects)
{
	WBLSM_DECLARE
	zval ** xsdp_zval,**result_zval;
	struct xsdp_t *xsdp;

	wblogs("wb_xsdp_create_objects enter\n");
	if( ZEND_NUM_ARGS()!=2 || zend_get_parameters_ex(2,&xsdp_zval,&result_zval)!=SUCCESS ) WRONG_PARAM_COUNT;
	xsdp=get_xsdp_from_resource(INTERNAL_FUNCTION_PARAM_PASSTHRU,xsdp_zval);
	if( !xsdp )
	{
		wblogs("wb_xsdp_create_objects leave, FALSE, error in get_xsdp_from_resource()\n");
		RETURN_FALSE;
	}
	WBLSM_START("wb_xsdp_create_objects",CDP_FROM_XSDP_P(xsdp));
	if( xsdp->status==XSDP_STATUS_CLOSED )
	{
		zend_error(E_WARNING,"602SQL: Can't create database objects for XSD document or XML form by that XSD parser, parser is already closed.");
		wblogs("wb_xsdp_create_objects leave, FALSE, parser is closed.\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	if( xsdp->status==XSDP_STATUS_OPENED )
	{
		zend_error(E_WARNING,"602SQL: Can't create database objects for XSD document or XML form by that XSD parser, parser didn't parsed XSD/XML document yet.");
		wblogs("wb_xsdp_create_objects leave, FALSE, parser didn't parsed.\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	if( xsdp->status==XSDP_STATUS_OBJECTS_CREATED )
	{
		zend_error(E_WARNING,"602SQL: Can't create database objects for XSD document or XML form by that XSD parser, parser already created database object for this XSD/XML document.");
		wblogs("wb_xsdp_create_objects leave, FALSE, parser already created db. objects.\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	if( Z_TYPE_PP(result_zval)!=IS_ARRAY && array_init(*result_zval)!=SUCCESS )
	{
		zend_error(E_WARNING, "602SQL: Can not create output array");
		wblogs("wb_xsdp_create_objects leave, FALSE, array_init() failed\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	if( DSParserCreateObjects(xsdp->xsdp_handle) )
	{
		internal_xsdp_log_error(xsdp->xsdp_handle,"Can't create database objects.");
		wblogs("wb_xsdp_create_objects leave, FALSE\n");
		WBLSM_END;
		RETURN_FALSE;
	}
	// retrieve created object names and categories
	{
		int index=0;
		tname object_name;
		tcateg object_categ;
		while( !DSParserGetObjectName(xsdp->xsdp_handle,index,object_name,&object_categ) )
		{
			zval *elem;
			MAKE_STD_ZVAL(elem);
			if( array_init(elem)!=SUCCESS )
			{
				zend_error(E_WARNING,"602SQL: Can't create output array element.");
				wblogs("wb_xsdp_create_objects leave, FALSE, array_init() failed\n");
				WBLSM_END;
				RETURN_FALSE;
			}
			wblog("wb_xsdp_create_objects: index=%d categ=%d name=\"%s\"\n",index,object_categ,object_name);
			add_assoc_long(elem,"category",object_categ);
			add_assoc_string(elem,"name",object_name,1);
			add_index_zval(*result_zval,index,elem);
			index++;
		}
	}
	// change status
	xsdp->status=XSDP_STATUS_OBJECTS_CREATED;
	wblogs("wb_xsdp_create_objects leave, TRUE\n");
	WBLSM_END;
	RETURN_TRUE;
}
/*}}}*/
/*}}}*/
#endif
/*{{{ Following enables find the compile-time information for the module using "ident" command. */
#ifndef WIN32
static const char vers_id[] = " $" VERSION " for PHP " PHP_VERSION " $";
#endif
/*}}}*/

#endif /* HAVE_602SQL */

/*
 * Local variables:
 * tab-width: 2
 * c-basic-offset: 2
 * End:
 * vim600: noet sw=2 ts=2 fdm=marker
 * vim<600: noet sw=2 ts=2
 */

